@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) 2020-2022, Matthew Broadway
+# Copyright (c) 2020-2023, Matthew Broadway
 # License: MIT License
 import argparse
 import signal
@@ -9,17 +9,18 @@ from ezdxf.addons.xqt import QtWidgets as qw
 
 import ezdxf
 from ezdxf import recover
-from ezdxf.addons.drawing.qtviewer import CadViewer
+from ezdxf.addons.drawing.qtviewer import CADViewer
 from ezdxf.addons.drawing.config import Configuration
 
+# ------------------------------------------------------------------------------
 # IMPORTANT: This example is just a remaining skeleton, the implementation
 # details moved into module: ezdxf.addon.drawing.qtviewer
 #
 # The CAD viewer can be executed by the new ezdxf command line launcher:
 # C:\> ezdxf view FILE
-
-# Load and draw proxy graphic:
-ezdxf.options.preserve_proxy_graphics()
+#
+# # docs: https://ezdxf.mozman.at/docs/addons/drawing.html
+# ------------------------------------------------------------------------------
 
 
 def _main():
@@ -31,14 +32,13 @@ def _main():
     args = parser.parse_args()
 
     # setup drawing add-on configuration
-    config = Configuration.defaults()
-    config = config.with_changes(
+    config = Configuration(
         lineweight_scaling=args.lineweight_scaling,
     )
 
     signal.signal(signal.SIGINT, signal.SIG_DFL)  # handle Ctrl+C properly
     app = qw.QApplication(sys.argv)
-    v = CadViewer(config=config)
+    v = CADViewer.from_config(config)
     if args.cad_file is not None:
         try:
             doc, auditor = recover.readfile(args.cad_file)
