@@ -1,10 +1,20 @@
-# Copyright (c) 2015-2021 Manfred Moitzi
+# Copyright (c) 2015-2022 Manfred Moitzi
 # License: MIT License
+import pathlib
 import ezdxf
 
+CWD = pathlib.Path("~/Desktop/Outbox").expanduser()
+if not CWD.exists():
+    CWD = pathlib.Path(".")
+
+# ------------------------------------------------------------------------------
+# This example shows how to use true-color and transparency.
+# ------------------------------------------------------------------------------
+
 
 def lwpolyline_with_true_color():
-    # for true color and transparency is DXF version AC1018 (ACAD R2004) or newer necessary
+    # for true-color and transparency is DXF version AC1018 (ACAD R2004)
+    # or newer necessary
     doc = ezdxf.new("AC1018")
     msp = doc.modelspace()
 
@@ -18,12 +28,13 @@ def lwpolyline_with_true_color():
             ),  # true color has higher priority than the color attribute
         },
     )
-    doc.saveas("true_color_lwpolyline.dxf")
+    doc.saveas(CWD / "true_color_lwpolyline.dxf")
 
 
-# Another way to set true color values for DXF entities: Property DXFEntity.rgb
+# Another way to set true-color values for DXF entities: Property DXFEntity.rgb
 def lines_with_true_color():
-    # for true color and transparency is DXF version AC1018 (ACAD R2004) or newer necessary
+    # for true-color and transparency is DXF version AC1018 (ACAD R2004)
+    # or newer necessary
     doc = ezdxf.new("AC1018")
     msp = doc.modelspace()
     for y in range(10):
@@ -31,7 +42,7 @@ def lines_with_true_color():
         line.rgb = (50, y * 20, 50)  # set true color as RGB tuple
         # IMPORTANT: as you see it is not in the line.dxf namespace!
     # getting RGB values by r, g, b = line.rgb also works
-    doc.saveas("true_color_lines.dxf")
+    doc.saveas(CWD / "true_color_lines.dxf")
 
 
 def rect(x, y, width=10, height=10):
@@ -39,20 +50,22 @@ def rect(x, y, width=10, height=10):
 
 
 def solids_with_true_color():
-    # for true color and transparency is DXF version AC1018 (ACAD R2004) or newer necessary
+    # for true-color and transparency is DXF version AC1018 (ACAD R2004)
+    # or newer necessary
     doc = ezdxf.new("AC1018")
     msp = doc.modelspace()
     for i in range(10):
         x = i * 7
         y = i * 7
         solid = msp.add_solid(rect(x, y))
-        solid.rgb = (50, i * 20, 50)  # set true color as RGB tuple
+        solid.rgb = (50, i * 20, 50)  # set true-color as RGB tuple
         # IMPORTANT: as you see it is not in the solid.dxf namespace!
-    doc.saveas("true_color_solids.dxf")
+    doc.saveas(CWD / "true_color_solids.dxf")
 
 
 def solids_with_transparency():
-    # for true color and transparency is DXF version AC1018 (ACAD R2004) or newer necessary
+    # for true-color and transparency is DXF version AC1018 (ACAD R2004)
+    # or newer necessary
     doc = ezdxf.new("AC1018")
     msp = doc.modelspace()
     for i in range(10):
@@ -63,11 +76,13 @@ def solids_with_transparency():
             i / 10.0
         )  # set transparency as float between 0.0 (opaque) and 1.0 (100% transparent)
         # transparency: 0.0, 0.1, 0.2, ...
-        # IMPORTANT: as you see it is not in the sold.dxf namespace and better never use the raw DXF data for transparency
-    doc.saveas("transparent_solids.dxf")
+        # IMPORTANT: as you see it is not in the sold.dxf namespace and better
+        # never use the raw DXF data for transparency
+    doc.saveas(CWD / "transparent_solids.dxf")
 
 
-lwpolyline_with_true_color()
-lines_with_true_color()
-solids_with_true_color()
-solids_with_transparency()
+if __name__ == '__main__':
+    lwpolyline_with_true_color()
+    lines_with_true_color()
+    solids_with_true_color()
+    solids_with_transparency()
