@@ -13,18 +13,23 @@ def drawing(request):
 
 def add_line_entities(layout, offset):
     for color in range(1, 256):
-        layout.add_line((offset+0, color), (offset+50, color), {
-            'color': color,
-            'layer': 'Träger'
-        })
+        layout.add_line(
+            (offset + 0, color),
+            (offset + 50, color),
+            {"color": color, "layer": "Träger"},
+        )
 
 
 def test_basic_graphics(drawing, tmpdir):
     add_line_entities(drawing.modelspace(), 0)
     add_line_entities(drawing.layout(), 70)
-    filename = str(tmpdir.join('basic_graphics_%s.dxf' % drawing.dxfversion))
+    filename = str(tmpdir.join("basic_graphics_%s.dxf" % drawing.dxfversion))
     try:
         drawing.saveas(filename)
     except ezdxf.DXFError as e:
-        pytest.fail("DXFError: {0} for DXF version {1}".format(str(e), drawing.dxfversion))
+        pytest.fail(
+            "DXFError: {0} for DXF version {1}".format(
+                str(e), drawing.dxfversion
+            )
+        )
     assert os.path.exists(filename) is True
