@@ -1,19 +1,18 @@
 # Purpose: read and write AutoCAD CTB files
-# Copyright (c) 2010-2021, Manfred Moitzi
+# Copyright (c) 2010-2023, Manfred Moitzi
 # License: MIT License
 # IMPORTANT: use only standard 7-Bit ascii code
-
+from __future__ import annotations
 from typing import (
     Union,
-    Tuple,
     Optional,
     BinaryIO,
     TextIO,
     Iterable,
-    List,
+    Iterator,
     Any,
-    Dict,
 )
+import os
 from abc import abstractmethod
 from io import StringIO
 from array import array
@@ -118,7 +117,10 @@ def get_bool(value: Union[str, bool]) -> bool:
 
 class PlotStyle:
     def __init__(
-        self, index: int, data: dict = None, parent: "PlotStyleTable" = None
+        self,
+        index: int,
+        data: Optional[dict] = None,
+        parent: Optional[PlotStyleTable] = None,
     ):
         data = data or {}
         self.parent = parent
@@ -133,9 +135,7 @@ class PlotStyle:
         if self._color != OBJECT_COLOR:
             self._mode_color = int(data.get("mode_color", self._color))
         self._color_policy = int(data.get("color_policy", DITHERING_ON))
-        self.physical_pen_number = int(
-            data.get("physical_pen_number", AUTOMATIC)
-        )
+        self.physical_pen_number = int(data.get("physical_pen_number", AUTOMATIC))
         self.virtual_pen_number = int(data.get("virtual_pen_number", AUTOMATIC))
         self.screen = int(data.get("screen", 100))
         self.linepattern_size = float(data.get("linepattern_size", 0.5))
@@ -149,7 +149,7 @@ class PlotStyle:
         self.fill_style = int(data.get("fill_style", FILL_STYLE_OBJECT))
 
     @property
-    def color(self) -> Optional[Tuple[int, int, int]]:
+    def color(self) -> Optional[tuple[int, int, int]]:
         """Get style color as ``(r, g, b)`` tuple or ``None``, if style has
         object color.
         """
@@ -159,7 +159,7 @@ class PlotStyle:
             return int2color(self._mode_color)[:3]
 
     @color.setter
-    def color(self, rgb: Tuple[int, int, int]) -> None:
+    def color(self, rgb: tuple[int, int, int]) -> None:
         """Set color as RGB values."""
         r, g, b = rgb
         # when defining a user-color, `mode_color` represents the real
@@ -272,8 +272,7 @@ class PlotStyle:
         stream.write("  linepattern_size=%s\n" % str(self.linepattern_size))
         stream.write("  linetype=%d\n" % self.linetype)
         stream.write(
-            "  adaptive_linetype=%s\n"
-            % str(bool(self.adaptive_linetype)).upper()
+            "  adaptive_linetype=%s\n" % str(bool(self.adaptive_linetype)).upper()
         )
         stream.write("  lineweight=%s\n" % str(self.lineweight))
         stream.write("  fill_style=%d\n" % self.fill_style)
@@ -295,8 +294,8 @@ class PlotStyleTable:
         self.scale_factor = scale_factor
         self.apply_factor = apply_factor
 
-        # set custom_lineweight_display_units to 1 for showing lineweight in inch in AutoCAD CTB editor window, but
-        # lineweight is always defined in mm
+        # set custom_lineweight_display_units to 1 for showing lineweight in inch in
+        # AutoCAD CTB editor window, but lineweight is always defined in mm
         self.custom_lineweight_display_units = 0
         self.lineweights = array("f", DEFAULT_LINE_WEIGHTS)
 
@@ -337,7 +336,7 @@ class PlotStyleTable:
         """
         return self.lineweights[index]
 
-    def save(self, filename: str) -> None:
+    def save(self, filename: str | os.PathLike) -> None:
         """Save CTB or STB file as `filename` to the file system."""
         with open(filename, "wb") as stream:
             self.write(stream)
@@ -395,9 +394,9 @@ class ColorDependentPlotStyles(PlotStyleTable):
         apply_factor: bool = False,
     ):
         super().__init__(description, scale_factor, apply_factor)
-        self._styles = [
+        self._styles: list[PlotStyle] = [
             PlotStyle(index, parent=self) for index in range(STYLE_COUNT)
-        ]  # type: List[PlotStyle]
+        ]
         self._styles.insert(
             0, PlotStyle(256)
         )  # 1-based array: insert dummy value for index 0
@@ -421,7 +420,7 @@ class ColorDependentPlotStyles(PlotStyleTable):
         """Iterable of all plot styles."""
         return iter(self._styles[1:])
 
-    def new_style(self, aci: int, data: dict = None) -> PlotStyle:
+    def new_style(self, aci: int, data: Optional[dict] = None) -> PlotStyle:
         """Set `aci` to new attributes defined by `data` dict.
 
         Args:
@@ -507,7 +506,7 @@ class NamedPlotStyles(PlotStyleTable):
                 "localized_name": "Normal",
             },
         )
-        self._styles = {"Normal": normal}  # type: Dict[str, PlotStyle]
+        self._styles: dict[str, PlotStyle] = {"Normal": normal}
 
     def __iter__(self) -> Iterable[str]:
         """Iterable of all plot style names."""
@@ -532,7 +531,7 @@ class NamedPlotStyles(PlotStyleTable):
         result.extend(sorted(keys))
         return iter(result)
 
-    def items(self) -> Iterable[Tuple[str, PlotStyle]]:
+    def items(self) -> Iterator[tuple[str, PlotStyle]]:
         """Iterable of all plot styles as (``name``, class:`PlotStyle`) tuples."""
         for key in self.keys():
             yield key, self._styles[key]
@@ -543,7 +542,10 @@ class NamedPlotStyles(PlotStyleTable):
             yield value
 
     def new_style(
-        self, name: str, data: dict = None, localized_name: str = None
+        self,
+        name: str,
+        data: Optional[dict] = None,
+        localized_name: Optional[str] = None,
     ) -> PlotStyle:
         """Create new class:`PlotStyle` `name` by attribute dict `data`, replaces
         existing class:`PlotStyle` objects.
@@ -629,9 +631,11 @@ def _read_stb(stream: BinaryIO) -> NamedPlotStyles:
     return styles
 
 
-def load(filename: str) -> Union[ColorDependentPlotStyles, NamedPlotStyles]:
+def load(
+    filename: str | os.PathLike,
+) -> Union[ColorDependentPlotStyles, NamedPlotStyles]:
     """Load the CTB or STB file `filename` from file system."""
-
+    filename = str(filename)
     with open(filename, "rb") as stream:
         if filename.lower().endswith(".ctb"):
             return _read_ctb(stream)
@@ -642,14 +646,7 @@ def load(filename: str) -> Union[ColorDependentPlotStyles, NamedPlotStyles]:
 
 
 def new_ctb() -> ColorDependentPlotStyles:
-    """
-    Create a new CTB file.
-
-    .. versionchanged:: 0.10
-
-        renamed from :func:`new`
-
-    """
+    """Create a new CTB file."""
     return ColorDependentPlotStyles()
 
 
@@ -680,10 +677,6 @@ class PlotStyleFileParser:
     """
 
     def __init__(self, text: str):
-        """
-        :param str text: ctb content as string
-
-        """
         self.data = {}
         for element, value in PlotStyleFileParser.iteritems(text):
             self.data[element] = value
@@ -745,14 +738,14 @@ class PlotStyleFileParser:
         while line_index < len(lines):
             name = get_name()
             value = get_value()
-            yield (name, value)
+            yield name, value
             skip_empty_lines()
 
     def get(self, name: str, default: Any) -> Any:
         return self.data.get(name, default)
 
 
-def int2color(color: int) -> Tuple[int, int, int, int]:
+def int2color(color: int) -> tuple[int, int, int, int]:
     """Convert color integer value from CTB-file to ``(r, g, b, color_type)
     tuple.
     """
@@ -764,15 +757,11 @@ def int2color(color: int) -> Tuple[int, int, int, int]:
     return red, green, blue, color_type
 
 
-def mode_color2int(
-    red: int, green: int, blue: int, color_type=COLOR_RGB
-) -> int:
+def mode_color2int(red: int, green: int, blue: int, color_type=COLOR_RGB) -> int:
     """Convert mode_color (r, g, b, color_type) tuple to integer."""
     return -color2int(red, green, blue, color_type)
 
 
 def color2int(red: int, green: int, blue: int, color_type: int) -> int:
     """Convert color (r, g, b, color_type) to integer."""
-    return (
-        -((color_type << 24) + (red << 16) + (green << 8) + blue) & 0xFFFFFFFF
-    )
+    return -((color_type << 24) + (red << 16) + (green << 8) + blue) & 0xFFFFFFFF
