@@ -1,28 +1,28 @@
 # Copyright (c) 2020-2021, Matthew Broadway
-# Copyright (c) 2020-2022, Manfred Moitzi
+# Copyright (c) 2020-2023, Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
 from typing import (
     TYPE_CHECKING,
-    Dict,
     Optional,
-    Tuple,
     Union,
-    List,
-    Set,
     cast,
     Sequence,
     Callable,
     Iterable,
+    NamedTuple,
 )
 import re
 import copy
+import logging
+import sys
 
 from ezdxf import options
+from ezdxf.colors import RGB
 from ezdxf.addons import acadctb
 from ezdxf.addons.drawing.config import Configuration
-from ezdxf.addons.drawing.type_hints import Color, RGB
-from ezdxf.colors import luminance, DXF_DEFAULT_COLORS, int2rgb
+from ezdxf.addons.drawing.type_hints import Color
+from ezdxf.colors import DXF_DEFAULT_COLORS, int2rgb
 from ezdxf.entities import (
     Attrib,
     Insert,
@@ -40,7 +40,7 @@ from ezdxf.enums import InsertUnits, Measurement
 from ezdxf.filemanagement import find_support_file
 from ezdxf.lldxf import const
 from ezdxf.lldxf.validator import make_table_key as layer_key
-from ezdxf.tools import fonts
+from ezdxf.fonts import fonts
 from ezdxf.tools.pattern import scale_pattern, HatchPatternType
 
 if TYPE_CHECKING:
@@ -50,6 +50,7 @@ if TYPE_CHECKING:
 
 __all__ = [
     "Properties",
+    "BackendProperties",
     "LayerProperties",
     "LayoutProperties",
     "RenderContext",
@@ -70,11 +71,11 @@ MODEL_SPACE_BG_COLOR = "#212830"
 PAPER_SPACE_BG_COLOR = "#ffffff"
 VIEWPORT_COLOR = "#aaaaaa"  # arbitrary choice
 OLE2FRAME_COLOR = "#89adba"  # arbitrary choice
+logger = logging.getLogger("ezdxf")
 
 
 def is_dark_color(color: Color, dark: float = 0.2) -> bool:
-    luma = luminance(hex_to_rgb(color[:7]))
-    return luma <= dark
+    return RGB.from_hex(color).luminance <= dark
 
 
 class Filling:
@@ -82,7 +83,7 @@ class Filling:
     PATTERN = 1
     GRADIENT = 2
 
-    def __init__(self):
+    def __init__(self) -> None:
         # Solid fill color is stored in Properties.color attribute
         self.type = Filling.SOLID
         # Gradient- or pattern name
@@ -92,34 +93,26 @@ class Filling:
         self.gradient_color1: Optional[Color] = None
         self.gradient_color2: Optional[Color] = None
         self.gradient_centered: float = 0.0  # todo: what's the meaning?
-        # TODO: remove HATCH pattern definition, backends do not
-        #   render hatch patterns since v0.18.1:
         self.pattern_scale: float = 1.0
         self.pattern: HatchPatternType = []
 
 
 class Properties:
-    """An implementation agnostic representation of entity properties like
-    color and linetype.
+    """An implementation agnostic representation of DXF entity properties like
+    color and linetype. These properties represent the actual values after
+    resolving all DXF specific rules like "by layer", "by block" and so on.
     """
 
-    def __init__(self):
-        self.color: str = "#ffffff"  # format #RRGGBB or #RRGGBBAA
-        # Color names should be resolved into a actual color value
+    def __init__(self) -> None:
+        # color string format "#RRGGBB" or "#RRGGBBAA"
+        # alpha channel AA: 0x00 = transparent; 0xff = opaque
+        self.color: str = "#ffffff"
+        self.pen = 7  # equals the ACI (1-255), for pen based backends like plotters
 
-        # Store linetype name for backends which don't have the ability to use
-        # user-defined linetypes, but have some predefined linetypes, maybe
-        # matching most common AutoCAD linetypes is possible.
-        # Store linetype names in UPPERCASE.
+        # Linetype rendering is done by the frontend, the backend receives only solid
+        # lines.
         self.linetype_name: str = "CONTINUOUS"
 
-        # Linetypes: Complex DXF linetypes are not supported:
-        # 1. Don't know if there are any backends which can use linetypes
-        #    including text or shapes
-        # 2. No decoder for SHX files available, which are the source for
-        #    shapes in linetypes
-        # 3. SHX files are copyrighted - including in ezdxf not possible
-        #
         # Simplified DXF linetype definition:
         # all line elements >= 0.0, 0.0 = point
         # all gap elements > 0.0
@@ -133,10 +126,6 @@ class Properties:
         # DXF: ("DASHDOTX2", "Dash dot (2x) ____  .  ____  .  ____  .  ____",
         #      [2.4, 2.0, -0.2, 0.0, -0.2])
         # linetype_pattern: [2.0, 0.2, 0.0, 0.2] = line-gap-point-gap
-        # Stored as tuple, so pattern could be used as key for caching.
-        # SVG dash-pattern does not support points, so a minimal line length
-        # (maybe inferred from linewidth?) has to be used, which may alter the
-        # overall line appearance - but linetype mapping will never be perfect.
         # The continuous pattern is an empty tuple ()
         self.linetype_pattern: Sequence[float] = CONTINUOUS_PATTERN
         self.linetype_scale: float = 1.0
@@ -151,10 +140,14 @@ class Properties:
         self.layer: str = "0"
 
         # Font definition object for text entities:
-        # `None` is for the default font
-        self.font: Optional[fonts.FontFace] = None
+        # Font rendering is done by the frontend, backends receive paths for
+        # stroke-fonts (.shx) and filled paths for outline fonts (.ttf).
+        self.font: Optional[fonts.FontFace] = None  # use default font
 
         # Filling properties: Solid, Pattern, Gradient
+        # Pattern rendering is done by the frontend, backends receive only solid lines,
+        # and the fill color for filled polygons and filled paths is passed as the
+        # color attribute in the BackendProperties, gradients are not supported.
         self.filling: Optional[Filling] = None
 
         self.units = InsertUnits.Unitless
@@ -168,12 +161,32 @@ class Properties:
     @property
     def rgb(self) -> RGB:
         """Returns color as RGB tuple."""
-        return hex_to_rgb(self.color[:7])  # ignore alpha if present
+        return RGB.from_hex(self.color)
 
     @property
     def luminance(self) -> float:
         """Returns perceived color luminance in range [0, 1] from dark to light."""
-        return luminance(self.rgb)
+        return self.rgb.luminance
+
+
+class BackendProperties(NamedTuple):
+    """The backend receives a condensed version of the entity properties."""
+
+    color: Color = "#000000"
+    lineweight: float = 0.25  # in mm
+    layer: str = "0"  # maybe useful to group entities (SVG, PDF)
+    pen: int = 1  # equals the ACI (1-255), for pen based backends like plotters
+    handle: str = ""  # top level entity handle
+
+    @property
+    def rgb(self) -> RGB:
+        """Returns color as RGB tuple."""
+        return RGB.from_hex(self.color)
+
+    @property
+    def luminance(self) -> float:
+        """Returns perceived color luminance in range [0, 1] from dark to light."""
+        return self.rgb.luminance
 
 
 class LayerProperties(Properties):
@@ -252,9 +265,7 @@ class LayoutProperties:
         )
 
     @staticmethod
-    def from_layout(
-        layout: Layout, units: Optional[int] = None
-    ) -> LayoutProperties:
+    def from_layout(layout: Layout, units: Optional[int] = None) -> LayoutProperties:
         """Setup default layout properties."""
         if layout.name == "Model":
             bg = MODEL_SPACE_BG_COLOR
@@ -264,7 +275,7 @@ class LayoutProperties:
             units = layout.units
         return LayoutProperties(layout.name, bg, units=units)
 
-    def set_colors(self, bg: Color, fg: Color = None) -> None:
+    def set_colors(self, bg: Color, fg: Optional[Color] = None) -> None:
         """Setup default layout colors.
 
         Required color format "#RRGGBB" or including alpha transparency
@@ -281,15 +292,31 @@ class LayoutProperties:
                 raise ValueError(f"Invalid foreground color: {fg}")
             self._default_color = fg
         else:
-            self._default_color = (
-                "#ffffff" if self._has_dark_background else "#000000"
-            )
+            self._default_color = "#ffffff" if self._has_dark_background else "#000000"
 
 
 LayerPropsOverride = Callable[[Sequence[LayerProperties]], None]
 
 
 class RenderContext:
+    """The render context for the given DXF document. The
+    :class:`RenderContext` resolves the properties of DXF entities from the
+    context they reside in to actual values like RGB colors, transparency,
+    linewidth and so on.
+
+    A given `ctb` file (plot style file) overrides the default properties for
+    all layouts, which means the plot style table stored in the layout is
+    always ignored.
+
+    Args:
+        doc: DXF document
+        ctb: path to a plot style table
+        export_mode: Whether to render the document as it would look when
+            exported (plotted) by a CAD application to a file such as pdf,
+            or whether to render the document as it would appear inside a
+            CAD application.
+    """
+
     def __init__(
         self,
         doc: Optional[Drawing] = None,
@@ -297,35 +324,25 @@ class RenderContext:
         ctb: str = "",
         export_mode: bool = False,
     ):
-        """Represents the render context for the DXF document `doc`.
-        A given `ctb` file (plot style file) overrides the default properties of
-        all layout, which means the plot style table stored in the layout is
-        always ignored.
-
-        Args:
-            doc: DXF document
-            ctb: path to a plot style table
-            export_mode: Whether to render the document as it would look when
-                exported (plotted) by a CAD application to a file such as pdf,
-                or whether to render the document as it would appear inside a
-                CAD application.
-        """
-        self._saved_states: List[Properties] = []
-        self.line_pattern: Dict[str, Sequence[float]] = dict()
+        self._saved_states: list[Properties] = []
+        self.line_pattern: dict[str, Sequence[float]] = dict()
         self.current_block_reference_properties: Optional[Properties] = None
         self.export_mode = export_mode
         self.override_ctb = ctb
-        self.layers: Dict[str, LayerProperties] = dict()
-        self.fonts: Dict[str, fonts.FontFace] = dict()
+        self.layers: dict[str, LayerProperties] = dict()
+        self.fonts: dict[str, fonts.FontFace] = dict()
         self.units = InsertUnits.Unitless  # modelspace units
         self.linetype_scale: float = 1.0  # overall modelspace linetype scaling
         self.measurement = Measurement.Imperial
         self.pdsize: float = 0
         self.pdmode: int = 0
-        self._hatch_pattern_cache: Dict[str, HatchPatternType] = dict()
+        self._hatch_pattern_cache: dict[str, HatchPatternType] = dict()
         self.current_layout_properties = LayoutProperties.modelspace()
         self.plot_styles = self._load_plot_style_table(self.override_ctb)
-
+        # Order for resolving SHX fonts: 1. "t"=TrueType; 2. "s"=SHX; 3. "l"=LFF
+        self.shx_resolve_order = options.get(
+            "drawing-addon", "shx_resolve_order", "tsl"
+        )
         # callable to override layer properties:
         self._layer_properties_override: Optional[LayerPropsOverride] = None
 
@@ -341,9 +358,7 @@ class RenderContext:
             except ValueError:
                 self.units = InsertUnits.Unitless
             try:
-                self.measurement = Measurement(
-                    doc.header.get("$MEASUREMENT", 0)
-                )
+                self.measurement = Measurement(doc.header.get("$MEASUREMENT", 0))
             except ValueError:
                 self.measurement = Measurement.Imperial
             if self.units == InsertUnits.Unitless:
@@ -353,7 +368,7 @@ class RenderContext:
                     self.units = InsertUnits.Inches
         self.current_layout_properties.units = self.units  # default modelspace
 
-    def set_layer_properties_override(self, func: LayerPropsOverride = None):
+    def set_layer_properties_override(self, func: Optional[LayerPropsOverride] = None):
         """The function `func` is called with the current layer properties as
         argument after resetting them, so the function can override the layer
         properties.
@@ -365,6 +380,7 @@ class RenderContext:
             self._layer_properties_override(layers)
 
     def set_current_layout(self, layout: Layout, ctb: str = ""):
+        """Set the current layout and update layout specific properties."""
         # the given ctb has the highest priority
         if ctb == "":
             # next is the override ctb
@@ -396,8 +412,8 @@ class RenderContext:
             changes["measurement"] = self.measurement
         return config.with_changes(**changes)
 
-    def _setup_layers(self, doc: Drawing) -> Dict[str, LayerProperties]:
-        layers: Dict[str, LayerProperties] = dict()
+    def _setup_layers(self, doc: Drawing) -> dict[str, LayerProperties]:
+        layers: dict[str, LayerProperties] = dict()
         for layer in doc.layers:
             layer_properties = self.resolve_layer_properties(cast(Layer, layer))
             layers[layer_key(layer_properties.layer)] = layer_properties
@@ -407,12 +423,12 @@ class RenderContext:
         for text_style in doc.styles:
             self.add_text_style(cast("Textstyle", text_style))
 
-    def _setup_vp_layers(self, vp: Viewport) -> Dict[str, LayerProperties]:
+    def _setup_vp_layers(self, vp: Viewport) -> dict[str, LayerProperties]:
         doc = vp.doc
         assert doc is not None
         frozen_layer_names = {layer_key(name) for name in vp.frozen_layers}
         vp_handle = vp.dxf.handle
-        layers: Dict[str, LayerProperties] = dict()
+        layers: dict[str, LayerProperties] = dict()
         for layer in doc.layers:
             layer = cast(Layer, layer)
             overrides = layer.get_vp_overrides()
@@ -459,6 +475,7 @@ class RenderContext:
         properties = LayerProperties()
         # Store real layer name (mixed case):
         properties.layer = layer.dxf.name
+        properties.pen = layer.dxf.color
         properties.color = self._true_layer_color(layer)
 
         # set layer transparency
@@ -477,9 +494,7 @@ class RenderContext:
         properties.linetype_pattern = self.line_pattern.get(
             properties.linetype_name, CONTINUOUS_PATTERN
         )
-        properties.lineweight = self._true_layer_lineweight(
-            layer.dxf.lineweight
-        )
+        properties.lineweight = self._true_layer_lineweight(layer.dxf.lineweight)
         properties.is_visible = layer.is_on() and not layer.is_frozen()
         if self.export_mode:
             properties.is_visible &= bool(layer.dxf.plot)
@@ -493,9 +508,16 @@ class RenderContext:
         if font_file == "":  # Font family stored in XDATA?
             family, italic, bold = text_style.get_extended_font_data()
             if family:
-                font_face = fonts.find_font_face_by_family(family, italic, bold)
+                font_face = fonts.find_best_match(
+                    family=family, weight=700 if bold else 400, italic=italic
+                )
         else:
-            font_face = fonts.get_font_face(font_file, map_shx=True)
+            try:
+                font_face = fonts.resolve_font_face(
+                    font_file, order=self.shx_resolve_order
+                )
+            except fonts.FontNotFoundError:
+                logger.warning("no fonts available, not even fallback fonts")
 
         if font_face is None:  # fall back to default font
             font_face = fonts.FontFace()
@@ -560,16 +582,13 @@ class RenderContext:
         resolved_layer = layer_key(p.layer)
         p.units = self.resolve_units()
         p.color = self.resolve_color(entity, resolved_layer=resolved_layer)
+        p.pen = self.resolve_pen(entity, resolved_layer=resolved_layer)
         p.linetype_name, p.linetype_pattern = self.resolve_linetype(
             entity, resolved_layer=resolved_layer
         )
-        p.lineweight = self.resolve_lineweight(
-            entity, resolved_layer=resolved_layer
-        )
+        p.lineweight = self.resolve_lineweight(entity, resolved_layer=resolved_layer)
         p.linetype_scale = self.resolve_linetype_scale(entity)
-        p.is_visible = self.resolve_visible(
-            entity, resolved_layer=resolved_layer
-        )
+        p.is_visible = self.resolve_visible(entity, resolved_layer=resolved_layer)
         if entity.is_supported_dxf_attrib("style"):
             p.font = self.resolve_font(entity)
         if isinstance(entity, DXFPolygon):
@@ -631,9 +650,7 @@ class RenderContext:
             aci = entity.dxf.color  # defaults to BYLAYER
 
         entity_layer = resolved_layer or layer_key(self.resolve_layer(entity))
-        layer_properties = self.layers.get(
-            entity_layer, DEFAULT_LAYER_PROPERTIES
-        )
+        layer_properties = self.layers.get(entity_layer, DEFAULT_LAYER_PROPERTIES)
 
         if aci == const.BYLAYER:
             color = layer_properties.get_entity_color_from_layer(
@@ -651,6 +668,25 @@ class RenderContext:
         )
         return color[:7] + alpha
 
+    def resolve_pen(
+        self, entity: DXFGraphic, *, resolved_layer: Optional[str] = None
+    ) -> int:
+        """Resolve the aci-color of `entity` as pen number in the range of [1..255]."""
+        pen = entity.dxf.color  # defaults to BYLAYER
+        entity_layer = resolved_layer or layer_key(self.resolve_layer(entity))
+        layer_properties = self.layers.get(entity_layer, DEFAULT_LAYER_PROPERTIES)
+
+        if pen == const.BYLAYER:
+            pen = layer_properties.pen
+        elif pen == const.BYBLOCK:
+            if not self.inside_block_reference:
+                pen = 7
+            else:
+                pen = self.current_block_reference_properties.pen  # type: ignore
+        elif pen == const.BYOBJECT:
+            pen = 7  # ???
+        return pen
+
     def _entity_alpha_str(
         self, raw_transparency: Optional[int], layer_color: Color
     ) -> str:
@@ -678,9 +714,7 @@ class RenderContext:
     def resolve_aci_color(self, aci: int, resolved_layer: str) -> Color:
         """Resolve the `aci` color as hex color string: "#RRGGBB" """
         if aci == const.BYLAYER:
-            layer = self.layers.get(
-                layer_key(resolved_layer), DEFAULT_LAYER_PROPERTIES
-            )
+            layer = self.layers.get(layer_key(resolved_layer), DEFAULT_LAYER_PROPERTIES)
             color = layer.get_entity_color_from_layer(
                 self.current_layout_properties.default_color
             )
@@ -694,7 +728,7 @@ class RenderContext:
         return color
 
     def _true_entity_color(
-        self, true_color: Optional[Tuple[int, int, int]], aci: int
+        self, true_color: Optional[tuple[int, int, int]], aci: int
     ) -> Color:
         """Returns rgb color in hex format: "#RRGGBB".
 
@@ -705,25 +739,20 @@ class RenderContext:
         elif 0 < aci < 256:
             return self._aci_to_true_color(aci)
         else:
-            return (
-                self.current_layout_properties.default_color
-            )  # unknown / invalid
+            return self.current_layout_properties.default_color  # unknown / invalid
 
     def _aci_to_true_color(self, aci: int) -> Color:
         """Returns the `aci` value (AutoCAD Color Index) as rgb value in
         hex format: "#RRGGBB".
         """
-        if aci == 7:  # black/white; todo: this bypasses the plot style table
-            if self.current_layout_properties.has_dark_background:
-                return "#ffffff"
-            else:
-                return "#000000"
+        if aci == 7:  # black/white
+            return self.current_layout_properties.default_color
         else:
             return rgb_to_hex(self.plot_styles[aci].color)
 
     def resolve_linetype(
-        self, entity: DXFGraphic, *, resolved_layer: str = None
-    ) -> Tuple[str, Sequence[float]]:
+        self, entity: DXFGraphic, *, resolved_layer: Optional[str] = None
+    ) -> tuple[str, Sequence[float]]:
         """Resolve the linetype of `entity`. Returns a tuple of the linetype
         name as upper-case string and the simplified linetype pattern as tuple
         of floats.
@@ -738,9 +767,7 @@ class RenderContext:
             pass
         name = entity.dxf.linetype.upper()  # default is 'BYLAYER'
         if name == "BYLAYER":
-            entity_layer = resolved_layer or layer_key(
-                self.resolve_layer(entity)
-            )
+            entity_layer = resolved_layer or layer_key(self.resolve_layer(entity))
             layer = self.layers.get(entity_layer, DEFAULT_LAYER_PROPERTIES)
             name = layer.linetype_name
             pattern = layer.linetype_pattern
@@ -760,7 +787,7 @@ class RenderContext:
         return name, pattern
 
     def resolve_lineweight(
-        self, entity: DXFGraphic, *, resolved_layer: str = None
+        self, entity: DXFGraphic, *, resolved_layer: Optional[str] = None
     ) -> float:
         """Resolve the lineweight of `entity` in mm.
 
@@ -785,9 +812,7 @@ class RenderContext:
                 return self.plot_styles.get_lineweight(aci)
             lineweight = entity.dxf.lineweight  # default is BYLAYER
             if lineweight == const.LINEWEIGHT_BYLAYER:
-                entity_layer = resolved_layer or layer_key(
-                    self.resolve_layer(entity)
-                )
+                entity_layer = resolved_layer or layer_key(self.resolve_layer(entity))
                 return self.layers.get(
                     entity_layer, DEFAULT_LAYER_PROPERTIES
                 ).lineweight
@@ -898,9 +923,7 @@ def is_valid_color(color: Color) -> bool:
     return False
 
 
-def rgb_to_hex(
-    rgb: Union[Tuple[int, int, int], Tuple[float, float, float]]
-) -> Color:
+def rgb_to_hex(rgb: Union[tuple[int, int, int], tuple[float, float, float]]) -> Color:
     """Returns color in hex format: "#RRGGBB"."""
     assert all(0 <= x <= 255 for x in rgb), f"invalid RGB color: {rgb}"
     r, g, b = rgb
@@ -914,7 +937,7 @@ def hex_to_rgb(hex_string: Color) -> RGB:
     r = int(hex_string[0:2], 16)
     g = int(hex_string[2:4], 16)
     b = int(hex_string[4:6], 16)
-    return r, g, b
+    return RGB(r, g, b)
 
 
 def set_color_alpha(color: Color, alpha: int) -> Color:
@@ -939,12 +962,12 @@ def transparency_to_alpha(value: float) -> int:
     return int(round((1.0 - value) * 255))
 
 
-def _load_line_pattern(linetypes: Table) -> Dict[str, Sequence[float]]:
+def _load_line_pattern(linetypes: Table) -> dict[str, Sequence[float]]:
     """Load linetypes defined in a DXF document into  as dictionary,
     key is the upper case linetype name, value is the simplified line pattern,
     see :func:`compile_line_pattern`.
     """
-    pattern: Dict[str, Sequence[float]] = dict()
+    pattern: dict[str, Sequence[float]] = dict()
     for linetype in linetypes:
         assert isinstance(linetype, Linetype)
         name = linetype.dxf.name.upper()
