@@ -1,18 +1,23 @@
-# Copyright (c) 2020-2021, Matthew Broadway
+# Copyright (c) 2020-2023, Matthew Broadway
 # License: MIT License
+from __future__ import annotations
+from typing import Union, Tuple, Iterable, Optional, Callable
+from typing_extensions import TypeAlias
 import enum
 from math import radians
-from typing import Union, Tuple, Dict, Iterable, List, Optional, Callable
 
 import ezdxf.lldxf.const as DXFConstants
-from ezdxf.enums import TextEntityAlignment, MAP_TEXT_ENUM_TO_ALIGN_FLAGS, MTextEntityAlignment
-from ezdxf.addons.drawing.backend import BackendInterface
-from ezdxf.addons.drawing.debug_utils import draw_rect
+from ezdxf.enums import (
+    TextEntityAlignment,
+    MAP_TEXT_ENUM_TO_ALIGN_FLAGS,
+    MTextEntityAlignment,
+)
 from ezdxf.entities import MText, Text, Attrib, AttDef
 from ezdxf.math import Matrix44, Vec3, sign
-from ezdxf.tools import fonts
-from ezdxf.tools.fonts import FontMeasurements
+from ezdxf.fonts import fonts
+from ezdxf.fonts.font_measurements import FontMeasurements
 from ezdxf.tools.text import plain_text, text_wrap
+from .text_renderer import TextRenderer
 
 """
 Search google for 'typography' or 'font anatomy' for explanations of terms like 
@@ -36,19 +41,17 @@ class VAlignment(enum.Enum):
     LOWER_CASE_CENTER = 1  # the midpoint between the baseline and the x-height
     BASELINE = 2  # the line which text rests on, characters with descenders (like 'p') are partially below this line
     BOTTOM = 3  # the lowest point on a character with a descender (like 'p')
-    UPPER_CASE_CENTER = (
-        4  # the midpoint between the baseline and the cap-height
-    )
+    UPPER_CASE_CENTER = 4  # the midpoint between the baseline and the cap-height
 
 
-Alignment = Tuple[HAlignment, VAlignment]
-AnyText = Union[Text, MText, Attrib, AttDef]
+Alignment: TypeAlias = Tuple[HAlignment, VAlignment]
+AnyText: TypeAlias = Union[Text, MText, Attrib, AttDef]
 
 # multiple of cap_height between the baseline of the previous line and the
 # baseline of the next line
 DEFAULT_LINE_SPACING = 5 / 3
 
-DXF_TEXT_ALIGNMENT_TO_ALIGNMENT: Dict[TextEntityAlignment, Alignment] = {
+DXF_TEXT_ALIGNMENT_TO_ALIGNMENT: dict[TextEntityAlignment, Alignment] = {
     TextEntityAlignment.LEFT: (HAlignment.LEFT, VAlignment.BASELINE),
     TextEntityAlignment.CENTER: (HAlignment.CENTER, VAlignment.BASELINE),
     TextEntityAlignment.RIGHT: (HAlignment.RIGHT, VAlignment.BASELINE),
@@ -77,12 +80,9 @@ DXF_TEXT_ALIGNMENT_TO_ALIGNMENT: Dict[TextEntityAlignment, Alignment] = {
     TextEntityAlignment.TOP_CENTER: (HAlignment.CENTER, VAlignment.TOP),
     TextEntityAlignment.TOP_RIGHT: (HAlignment.RIGHT, VAlignment.TOP),
 }
-assert (
-    DXF_TEXT_ALIGNMENT_TO_ALIGNMENT.keys()
-    == MAP_TEXT_ENUM_TO_ALIGN_FLAGS.keys()
-)
+assert DXF_TEXT_ALIGNMENT_TO_ALIGNMENT.keys() == MAP_TEXT_ENUM_TO_ALIGN_FLAGS.keys()
 
-DXF_MTEXT_ALIGNMENT_TO_ALIGNMENT: Dict[int, Alignment] = {
+DXF_MTEXT_ALIGNMENT_TO_ALIGNMENT: dict[int, Alignment] = {
     DXFConstants.MTEXT_TOP_LEFT: (HAlignment.LEFT, VAlignment.TOP),
     DXFConstants.MTEXT_TOP_CENTER: (HAlignment.CENTER, VAlignment.TOP),
     DXFConstants.MTEXT_TOP_RIGHT: (HAlignment.RIGHT, VAlignment.TOP),
@@ -161,7 +161,7 @@ def _split_into_lines(
     entity: AnyText,
     box_width: Optional[float],
     get_text_width: Callable[[str], float],
-) -> List[str]:
+) -> list[str]:
     if isinstance(entity, AttDef):
         # ATTDEF outside of an Insert renders the tag rather than the value
         text = plain_text(entity.dxf.tag)
@@ -228,18 +228,17 @@ def _get_extra_transform(text: AnyText, line_width: float) -> Matrix44:
 
 def _apply_alignment(
     alignment: Alignment,
-    line_widths: List[float],
+    line_widths: list[float],
     line_spacing: float,
     box_width: Optional[float],
     font_measurements: FontMeasurements,
-) -> Tuple[Tuple[float, float], List[float], List[float]]:
+) -> tuple[tuple[float, float], list[float], list[float]]:
     if not line_widths:
         return (0, 0), [], []
 
     halign, valign = alignment
     line_ys = [
-        -font_measurements.baseline
-        - (font_measurements.cap_height + i * line_spacing)
+        -font_measurements.baseline - (font_measurements.cap_height + i * line_spacing)
         for i in range(len(line_widths))
     ]
 
@@ -286,7 +285,10 @@ def _get_wcs_insert(text: AnyText) -> Vec3:
         if alignment == TextEntityAlignment.LEFT:
             # LEFT/BASELINE is always located at the insert point.
             pass
-        elif alignment in (TextEntityAlignment.FIT, TextEntityAlignment.ALIGNED):
+        elif alignment in (
+            TextEntityAlignment.FIT,
+            TextEntityAlignment.ALIGNED,
+        ):
             # Interpolate insertion location between insert and align point:
             insert = insert.lerp(align_point, factor=0.5)
         else:
@@ -300,11 +302,10 @@ def _get_wcs_insert(text: AnyText) -> Vec3:
 # Simple but fast MTEXT renderer:
 def simplified_text_chunks(
     text: AnyText,
-    out: BackendInterface,
+    render_engine: TextRenderer,
     *,
-    font: fonts.FontFace = None,
-    debug_draw_rect: bool = False
-) -> Iterable[Tuple[str, Matrix44, float]]:
+    font_face: fonts.FontFace,
+) -> Iterable[tuple[str, Matrix44, float]]:
     """Splits a complex text entity into simple chunks of text which can all be
     rendered the same way:
     render the string (which will not contain any newlines) with the given
@@ -318,13 +319,13 @@ def simplified_text_chunks(
     lines = _split_into_lines(
         text,
         box_width,
-        lambda s: out.get_text_line_width(s, cap_height, font=font),
+        lambda s: render_engine.get_text_line_width(s, font_face, cap_height),
     )
     line_spacing = _get_line_spacing(text, cap_height)
     line_widths = [
-        out.get_text_line_width(line, cap_height, font=font) for line in lines
+        render_engine.get_text_line_width(line, font_face, cap_height) for line in lines
     ]
-    font_measurements = out.get_font_measurements(cap_height, font=font)
+    font_measurements = render_engine.get_font_measurements(font_face, cap_height)
     anchor, line_xs, line_ys = _apply_alignment(
         alignment, line_widths, line_spacing, box_width, font_measurements
     )
@@ -348,18 +349,3 @@ def simplified_text_chunks(
     for i, (line, line_x, line_y) in enumerate(zip(lines, line_xs, line_ys)):
         transform = Matrix44.translate(line_x, line_y, 0) @ whole_text_transform
         yield line, transform, cap_height
-
-        if debug_draw_rect:
-            width = out.get_text_line_width(line, cap_height, font)
-            ps = list(
-                transform.transform_vertices(
-                    [
-                        Vec3(0, 0, 0),
-                        Vec3(width, 0, 0),
-                        Vec3(width, cap_height, 0),
-                        Vec3(0, cap_height, 0),
-                        Vec3(0, 0, 0),
-                    ]
-                )
-            )
-            draw_rect(ps, "#ff0000", out)
