@@ -1,4 +1,4 @@
-#  Copyright (c) 2020-2021, Manfred Moitzi
+#  Copyright (c) 2020-2023, Manfred Moitzi
 #  License: MIT License
 """
 Implementation of the `__geo_interface__`: https://gist.github.com/sgillies/2217756
@@ -8,29 +8,35 @@ Which is also supported by Shapely: https://pypi.org/project/Shapely/
 Type definitions see GeoJson Standard: https://tools.ietf.org/html/rfc7946
 and examples : https://tools.ietf.org/html/rfc7946#appendix-A
 
+GeoJSON Linter: https://geojsonlint.com/
+
 """
+from __future__ import annotations
 from typing import (
-    Dict,
     Iterable,
-    List,
+    Iterator,
     Union,
     cast,
     Callable,
     Sequence,
-    Tuple,
+    Optional,
+    Any,
+    MutableMapping,
 )
+from typing_extensions import TypeAlias, Self
 import numbers
 import copy
 import math
+import enum
 from ezdxf.math import Vec3, has_clockwise_orientation, Matrix44
-from ezdxf.path import make_path, from_hatch_boundary_path, fast_bbox_detection
-from ezdxf.entities import DXFGraphic, LWPolyline, Point, Polyline, Line, Solid
+from ezdxf.path import make_path, from_hatch_boundary_path, make_polygon_structure
+from ezdxf.entities import DXFGraphic, LWPolyline, Point, Polyline, Line
 from ezdxf.entities.polygon import DXFPolygon
 from ezdxf.lldxf import const
 from ezdxf.entities import factory
 
 
-__all__ = ["proxy", "dxf_entities", "gfilter", "GeoProxy"]
+__all__ = ["proxy", "dxf_entities", "gfilter", "GeoProxy", "PolygonConversion"]
 
 TYPE = "type"
 COORDINATES = "coordinates"
@@ -45,6 +51,7 @@ GEOMETRIES = "geometries"
 GEOMETRY = "geometry"
 FEATURES = "features"
 FEATURE = "Feature"
+PROPERTIES = "properties"
 FEATURE_COLLECTION = "FeatureCollection"
 MAX_FLATTENING_DISTANCE = 0.1
 SUPPORTED_DXF_TYPES = {
@@ -63,12 +70,32 @@ SUPPORTED_DXF_TYPES = {
     "SPLINE",
 }
 
+GeoMapping: TypeAlias = MutableMapping[str, Any]
+PostProcessFunc: TypeAlias = Callable[[DXFGraphic, GeoMapping], None]
+
+
+class PolygonConversion(enum.IntEnum):
+    """Polygon conversion types as :class:`IntEnum`.
+
+    Attributes:
+        HATCH:
+        POLYLINE:
+        HATCH_AND_POLYLINE:
+        MPOLYGON:
+
+    """
+
+    HATCH = 1
+    POLYLINE = 2
+    HATCH_AND_POLYLINE = 3
+    MPOLYGON = 4
+
 
 def proxy(
     entity: Union[DXFGraphic, Iterable[DXFGraphic]],
     distance: float = MAX_FLATTENING_DISTANCE,
     force_line_string: bool = False,
-) -> "GeoProxy":
+) -> GeoProxy:
     """Returns a :class:`GeoProxy` object.
 
     Args:
@@ -84,24 +111,29 @@ def proxy(
 
 
 def dxf_entities(
-    geo_mapping, polygon: int = 1, dxfattribs = None
-) -> Iterable[DXFGraphic]:
+    geo_mapping: GeoMapping,
+    polygon=PolygonConversion.HATCH,
+    dxfattribs=None,
+    *,
+    post_process: Optional[PostProcessFunc] = None,
+) -> Iterator[DXFGraphic]:
     """Returns ``__geo_interface__`` mappings as DXF entities.
 
-    The `polygon` argument determines the method to convert polygons,
-    use 1 for :class:`~ezdxf.entities.Hatch` entity, 2 for
-    :class:`~ezdxf.entities.LWPolyline` or 3 for both.
-    Option 2 returns for the exterior path and each hole a separated
-    :class:`LWPolyline` entity. The :class:`Hatch` entity supports holes,
-    but has no explicit border line.
+    The enum `polygon` determines the method to convert polygons,
+    use :attr:`PolygonConversion.HATCH` for :class:`~ezdxf.entities.Hatch` entity,
+    :attr:`PolygonConversion.POLYLINE` for :class:`~ezdxf.entities.LWPolyline` or
+    :attr:`PolygonConversion.HATCH_AND_POLYLINE` for both.
+    Option :attr:`PolygonConversion.POLYLINE` returns for the exterior path and each hole
+    a separated :class:`LWPolyline` entity. The :class:`Hatch` entity supports holes,
+    but has no explicit borderline.
 
     Yields :class:`Hatch` always before :class:`LWPolyline` entities.
 
-    :class:`~ezdxf.entities.MPolygon` support was added in v0.16.6, which is
-    like a :class:`~ezdxf.entities.Hatch` entity  with additional border
-    lines, but the MPOLYGON entity is not a core DXF entity and DXF viewers,
+    :attr:`PolygonConversion.MPOLYGON` support was added in v0.16.6, which is
+    like a :class:`~ezdxf.entities.Hatch` entity  with additional borderlines,
+    but the MPOLYGON entity is not a core DXF entity and DXF viewers,
     applications and libraries my not support this entity. The DXF attribute
-    `color` defines the border line color and `fill_color` the color of the
+    `color` defines the borderline color and `fill_color` the color of the
     solid filling.
 
     The returned DXF entities can be added to a layout by the
@@ -110,14 +142,19 @@ def dxf_entities(
     Args:
         geo_mapping: ``__geo__interface__`` mapping as :class:`dict` or a Python
             object with a :attr:`__geo__interface__` property
-        polygon: method to convert polygons (1-2-3-4)
+        polygon: see :class:`PolygonConversion`
         dxfattribs: dict with additional DXF attributes
+        post_process: post process function of type :class:`PostProcesFunc` that get the
+            created DXF entity and the geo mapping as input, see reference implementation
+            :func:`assign_layers`
 
     """
-    return GeoProxy.parse(geo_mapping).to_dxf_entities(polygon, dxfattribs)
+    return GeoProxy.parse(geo_mapping).to_dxf_entities(
+        polygon, dxfattribs, post_process=post_process
+    )
 
 
-def gfilter(entities: Iterable[DXFGraphic]) -> Iterable[DXFGraphic]:
+def gfilter(entities: Iterable[DXFGraphic]) -> Iterator[DXFGraphic]:
     """Filter DXF entities from iterable `entities`, which are incompatible to
     the ``__geo_reference__`` interface.
     """
@@ -129,7 +166,7 @@ def gfilter(entities: Iterable[DXFGraphic]) -> Iterable[DXFGraphic]:
             yield e
 
 
-TFunc = Callable[[Vec3], Vec3]
+TFunc: TypeAlias = Callable[[Vec3], Vec3]
 
 
 class GeoProxy:
@@ -149,12 +186,12 @@ class GeoProxy:
 
     """
 
-    def __init__(self, geo_mapping: Dict, places: int = 6):
+    def __init__(self, geo_mapping: GeoMapping, places: int = 6):
         self._root = geo_mapping
         self.places = places
 
     @classmethod
-    def parse(cls, geo_mapping) -> "GeoProxy":
+    def parse(cls, geo_mapping: GeoMapping) -> Self:
         """Parse and compile a ``__geo_interface__`` mapping as :class:`dict`
         or a Python object with a ``__geo_interface__`` property, does some
         basic syntax checks, converts all coordinates into :class:`Vec3`
@@ -167,7 +204,7 @@ class GeoProxy:
         return cls(parse(geo_mapping))
 
     @property
-    def root(self) -> Dict:
+    def root(self) -> GeoMapping:
         return self._root
 
     @property
@@ -175,21 +212,21 @@ class GeoProxy:
         """Property returns the top level entity type or ``None``."""
         return self._root.get("type")
 
-    def __copy__(self) -> "GeoProxy":
+    def __copy__(self) -> GeoProxy:
         """Returns a deep copy."""
         return copy.deepcopy(self)
 
     copy = __copy__
 
     @property
-    def __geo_interface__(self) -> Dict:
+    def __geo_interface__(self) -> GeoMapping:
         """Returns the ``__geo_interface__`` compatible mapping as
         :class:`dict`.
         """
         return _rebuild(self._root, self.places)
 
-    def __iter__(self) -> Iterable[Dict]:
-        """Iterate over all geo content objects.
+    def __iter__(self) -> Iterator[GeoMapping]:
+        """Iterate over all geometry entities.
 
         Yields only "Point", "LineString", "Polygon", "MultiPoint",
         "MultiLineString" and "MultiPolygon" objects, returns the content of
@@ -198,22 +235,26 @@ class GeoProxy:
 
         """
 
-        def _iter(root):
-            type_ = root[TYPE]
+        def _iter(node: GeoMapping) -> Iterator[GeoMapping]:
+            type_ = node[TYPE]
             if type_ == FEATURE_COLLECTION:
-                for feature in root[FEATURES]:
+                for feature in node[FEATURES]:
                     yield from _iter(feature)
             elif type_ == GEOMETRY_COLLECTION:
-                for geometry in root[GEOMETRIES]:
+                for geometry in node[GEOMETRIES]:
                     yield from _iter(geometry)
             elif type_ == FEATURE:
-                yield root[GEOMETRY]
+                geometry = node[GEOMETRY]
+                if geometry[TYPE] == GEOMETRY_COLLECTION:
+                    yield from _iter(geometry)
+                else:
+                    yield geometry
             else:
-                yield root
+                yield node
 
         yield from _iter(self._root)
 
-    def filter(self, func: Callable[["GeoProxy"], bool]) -> None:
+    def filter(self, func: Callable[[GeoProxy], bool]) -> None:
         """Removes all mappings for which `func()` returns ``False``.
         The function only has to handle Point, LineString and Polygon entities,
         other entities like MultiPolygon are divided into separate entities
@@ -256,7 +297,7 @@ class GeoProxy:
         if not check(self._root):
             self._root = {}
 
-    def globe_to_map(self, func: TFunc = None) -> None:
+    def globe_to_map(self, func: Optional[TFunc] = None) -> None:
         """Transform all coordinates recursive from globe representation
         in longitude and latitude in decimal degrees into 2D map representation
         in meters.
@@ -278,7 +319,7 @@ class GeoProxy:
             func = wgs84_4326_to_3395
         self.apply(func)
 
-    def map_to_globe(self, func: TFunc = None) -> None:
+    def map_to_globe(self, func: Optional[TFunc] = None) -> None:
         """Transform all coordinates recursive from 2D map representation in
         meters into globe representation as longitude and latitude in decimal
         degrees.
@@ -349,7 +390,7 @@ class GeoProxy:
 
         """
 
-        def process(entity: Dict):
+        def process(entity: GeoMapping):
             def transform(coords):
                 if isinstance(coords, Vec3):
                     return func(coords)
@@ -367,7 +408,7 @@ class GeoProxy:
         entity: Union[DXFGraphic, Iterable[DXFGraphic]],
         distance: float = MAX_FLATTENING_DISTANCE,
         force_line_string: bool = False,
-    ) -> "GeoProxy":
+    ) -> GeoProxy:
         """Constructor from a single DXF entity or an iterable of DXF entities.
 
         Args:
@@ -386,8 +427,12 @@ class GeoProxy:
         return cls(m)
 
     def to_dxf_entities(
-        self, polygon: int = 1, dxfattribs = None
-    ) -> Iterable[DXFGraphic]:
+        self,
+        polygon=PolygonConversion.HATCH,
+        dxfattribs=None,
+        *,
+        post_process: Optional[PostProcessFunc] = None,
+    ) -> Iterator[DXFGraphic]:
         """Returns stored ``__geo_interface__`` mappings as DXF entities.
 
         The `polygon` argument determines the method to convert polygons,
@@ -395,27 +440,26 @@ class GeoProxy:
         :class:`~ezdxf.entities.LWPolyline` or 3 for both.
         Option 2 returns for the exterior path and each hole a separated
         :class:`LWPolyline` entity. The :class:`Hatch` entity supports holes,
-        but has no explicit border line.
+        but has no explicit borderline.
 
         Yields :class:`Hatch` always before :class:`LWPolyline` entities.
 
         :class:`~ezdxf.entities.MPolygon` support was added in v0.16.6, which is
-        like a :class:`~ezdxf.entities.Hatch` entity  with additional border
-        lines, but the MPOLYGON entity is not a core DXF entity and DXF viewers,
+        like a :class:`~ezdxf.entities.Hatch` entity  with additional borderlines,
+        but the MPOLYGON entity is not a core DXF entity and DXF viewers,
         applications and libraries my not support this entity. The DXF attribute
-        `color` defines the border line color and `fill_color` the color of the
+        `color` defines the borderline color and `fill_color` the color of the
         solid filling.
 
         The returned DXF entities can be added to a layout by the
         :meth:`Layout.add_entity` method.
 
         Args:
-            polygon: method to convert polygons (1-2-3-4)
+            polygon: see :class:`PolygonConversion`
             dxfattribs: dict with additional DXF attributes
-
-        .. versionadded:: 0.16.6
-
-            MPOLYGON support
+            post_process: post process function of type :class:`PostProcesFunc` that get the
+                created DXF entity and the geo mapping as input, see reference implementation
+                :func:`assign_layers`
 
         """
 
@@ -431,24 +475,22 @@ class GeoProxy:
             polyline.append_points(vertices, format="xy")
             return polyline
 
-        def polygon_(exterior: List, holes: List) -> Iterable[DXFGraphic]:
-            if polygon & 4:  # MPOLYGON
+        def polygon_(exterior: list, holes: list) -> Iterator[DXFGraphic]:
+            if polygon == PolygonConversion.MPOLYGON:
                 yield mpolygon_(exterior, holes)
                 # the following DXF entities do not support the
                 # "fill_color" attribute
                 return
-            if polygon & 1:  # HATCH
+            if polygon & PolygonConversion.HATCH:
                 yield hatch_(exterior, holes)
-            if polygon & 2:  # LWPOLYLINE
+            if polygon & PolygonConversion.POLYLINE:
                 for path in [exterior] + holes:
                     yield lwpolyline(path)
 
         def dxf_polygon_(
             dxftype: str, exterior: Sequence, holes: Sequence
         ) -> DXFPolygon:
-            dxf_polygon = cast(
-                DXFPolygon, factory.new(dxftype, dxfattribs=dxfattribs)
-            )
+            dxf_polygon = cast(DXFPolygon, factory.new(dxftype, dxfattribs=dxfattribs))
             dxf_polygon.dxf.hatch_style = const.HATCH_STYLE_OUTERMOST
             dxf_polygon.paths.add_polyline_path(
                 exterior, flags=const.BOUNDARY_PATH_EXTERNAL
@@ -465,7 +507,7 @@ class GeoProxy:
         def mpolygon_(exterior: Sequence, holes: Sequence) -> DXFPolygon:
             return dxf_polygon_("MPOLYGON", exterior, holes)
 
-        def entity(type_, coordinates) -> Iterable[DXFGraphic]:
+        def entity(type_, coordinates) -> Iterator[DXFGraphic]:
             if type_ == POINT:
                 yield point(coordinates)
             elif type_ == LINE_STRING:
@@ -488,11 +530,48 @@ class GeoProxy:
             raise ValueError(f"invalid value for polygon: {polygon}")
 
         dxfattribs = dict(dxfattribs or {})
-        for _mapping in self.__iter__():
-            yield from entity(_mapping.get(TYPE), _mapping.get(COORDINATES))
+        for feature, geometry in iter_features(self._root):
+            type_ = geometry.get(TYPE)
+            for e in entity(type_, geometry.get(COORDINATES)):
+                if post_process:
+                    post_process(e, feature)
+                yield e
 
 
-def parse(geo_mapping: Dict) -> Dict:
+def iter_features(geo_mapping: GeoMapping) -> Iterator[tuple[GeoMapping, GeoMapping]]:
+    """Yields all geometries of a ``__geo_mapping__`` as (`feature`, `geometry`) tuples.
+
+    If no feature is defined the `feature` value is an empty ``dict``. When a `feature`
+    contains `GeometryCollections`, the function yields for each sub-geometry a separate
+    (`feature`, `geometry`) tuple.
+
+    """
+    current_feature: GeoMapping = {}
+
+    def features(node: GeoMapping) -> Iterator[tuple[GeoMapping, GeoMapping]]:
+        nonlocal current_feature
+
+        type_ = node[TYPE]
+        if type_ == FEATURE_COLLECTION:
+            for feature in node[FEATURES]:
+                yield from features(feature)
+        elif type_ == GEOMETRY_COLLECTION:
+            for geometry in node[GEOMETRIES]:
+                yield from features(geometry)
+        elif type_ == FEATURE:
+            current_feature = node
+            geometry = node[GEOMETRY]
+            if geometry[TYPE] == GEOMETRY_COLLECTION:
+                yield from features(geometry)
+            else:
+                yield current_feature, geometry
+        else:
+            yield current_feature, node
+
+    yield from features(geo_mapping)
+
+
+def parse(geo_mapping: GeoMapping) -> GeoMapping:
     """Parse ``__geo_interface__`` convert all coordinates into
     :class:`Vec3` objects, Polygon['coordinates'] is always a
     tuple (exterior, holes), holes maybe an empty list.
@@ -516,9 +595,7 @@ def parse(geo_mapping: Dict) -> Dict:
         if geometries:
             geo_mapping[GEOMETRIES] = [parse(g) for g in geometries]
         else:
-            raise ValueError(
-                f'Missing key "{GEOMETRIES}" in GeometryCollection.'
-            )
+            raise ValueError(f'Missing key "{GEOMETRIES}" in GeometryCollection.')
     elif type_ == FEATURE:
         # The value of the geometry member SHALL be either a Geometry object
         # or, in the case that the Feature is unlocated, a JSON null value.
@@ -580,13 +657,13 @@ def _parse_polygon(coordinates: Sequence) -> Sequence:
     return Vec3.list(exterior), [Vec3.list(h) for h in holes]
 
 
-def _rebuild(geo_mapping: Dict, places: int = 6) -> Dict:
+def _rebuild(geo_mapping: GeoMapping, places: int = 6) -> GeoMapping:
     """Returns ``__geo_interface__`` compatible mapping as :class:`dict` from
     compiled internal representation.
 
     """
 
-    def pnt(v: Vec3) -> Tuple[float, float]:
+    def pnt(v: Vec3) -> tuple[float, float]:
         return round(v.x, places), round(v.y, places)
 
     def _polygon(exterior, holes):
@@ -599,9 +676,7 @@ def _rebuild(geo_mapping: Dict, places: int = 6) -> Dict:
     if type_ == FEATURE_COLLECTION:
         geo_interface[FEATURES] = [_rebuild(f) for f in geo_interface[FEATURES]]
     elif type_ == GEOMETRY_COLLECTION:
-        geo_interface[GEOMETRIES] = [
-            _rebuild(g) for g in geo_interface[GEOMETRIES]
-        ]
+        geo_interface[GEOMETRIES] = [_rebuild(g) for g in geo_interface[GEOMETRIES]]
     elif type_ == FEATURE:
         geo_interface[GEOMETRY] = _rebuild(geo_interface[GEOMETRY])
     elif type_ == POINT:
@@ -618,8 +693,7 @@ def _rebuild(geo_mapping: Dict, places: int = 6) -> Dict:
         geo_interface[COORDINATES] = _polygon(*geo_interface[COORDINATES])
     elif type_ == MULTI_POLYGON:
         geo_interface[COORDINATES] = [
-            _polygon(exterior, holes)
-            for exterior, holes in geo_interface[COORDINATES]
+            _polygon(exterior, holes) for exterior, holes in geo_interface[COORDINATES]
         ]
     return geo_interface
 
@@ -628,7 +702,7 @@ def mapping(
     entity: DXFGraphic,
     distance: float = MAX_FLATTENING_DISTANCE,
     force_line_string: bool = False,
-) -> Dict:
+) -> GeoMapping:
     """Create the compiled ``__geo_interface__`` mapping as :class:`dict`
     for the given DXF `entity`, all coordinates are :class:`Vec3` objects and
     represents "Polygon" always as tuple (exterior, holes) even without holes.
@@ -678,9 +752,7 @@ def mapping(
         raise TypeError(dxftype)
 
 
-def _line_string_or_polygon_mapping(
-    points: List[Vec3], force_line_string: bool
-):
+def _line_string_or_polygon_mapping(points: list[Vec3], force_line_string: bool):
     len_ = len(points)
     if len_ < 2:
         raise ValueError("Invalid vertex count.")
@@ -695,12 +767,12 @@ def _line_string_or_polygon_mapping(
 
 def _hatch_as_polygon(
     hatch: DXFPolygon, distance: float, force_line_string: bool
-) -> Dict:
-    def boundary_to_vertices(boundary) -> List[Vec3]:
+) -> GeoMapping:
+    def boundary_to_vertices(boundary) -> list[Vec3]:
         path = from_hatch_boundary_path(boundary, ocs, elevation)
         return path_to_vertices(path)
 
-    def path_to_vertices(path) -> List[Vec3]:
+    def path_to_vertices(path) -> list[Vec3]:
         path.close()
         return list(path.flattening(distance))
 
@@ -726,9 +798,7 @@ def _hatch_as_polygon(
         if force_line_string:
             # Build a MultiString collection:
             points = boundary_to_vertices(exterior)
-            geometries = [
-                _line_string_or_polygon_mapping(points, force_line_string)
-            ]
+            geometries = [_line_string_or_polygon_mapping(points, force_line_string)]
             # All other boundary paths are treated as holes
             for hole in boundaries[1:]:
                 points = boundary_to_vertices(hole)
@@ -739,14 +809,10 @@ def _hatch_as_polygon(
         else:
             # Multiple separated polygons are possible in one HATCH entity:
             polygons = []
-            for exterior, holes in _boundaries_to_polygons(
-                boundaries, ocs, elevation
-            ):
+            for exterior, holes in _boundaries_to_polygons(boundaries, ocs, elevation):
                 points = path_to_vertices(exterior)
                 polygons.append(
-                    polygon_mapping(
-                        points, [path_to_vertices(hole) for hole in holes]
-                    )
+                    polygon_mapping(points, [path_to_vertices(hole) for hole in holes])
                 )
             if len(polygons) > 1:
                 return join_multi_single_type_mappings(polygons)
@@ -755,10 +821,9 @@ def _hatch_as_polygon(
 
 def _boundaries_to_polygons(boundaries, ocs, elevation):
     paths = (
-        from_hatch_boundary_path(boundary, ocs, elevation)
-        for boundary in boundaries
+        from_hatch_boundary_path(boundary, ocs, elevation) for boundary in boundaries
     )
-    for polygon in fast_bbox_detection(paths):
+    for polygon in make_polygon_structure(paths):
         exterior = polygon[0]
         # only take exterior path of level 1 holes, nested holes are ignored
         yield exterior, [hole[0] for hole in polygon[1:]]
@@ -768,7 +833,7 @@ def collection(
     entities: Iterable[DXFGraphic],
     distance: float = MAX_FLATTENING_DISTANCE,
     force_line_string: bool = False,
-) -> Dict:
+) -> GeoMapping:
     """Create the ``__geo_interface__`` mapping as :class:`dict` for the
     given DXF `entities`, see https://gist.github.com/sgillies/2217756
 
@@ -794,7 +859,7 @@ def collection(
         return join_multi_single_type_mappings(m)
 
 
-def line_string_mapping(points: List[Vec3]) -> Dict:
+def line_string_mapping(points: list[Vec3]) -> GeoMapping:
     """Returns a "LineString" mapping.
 
     .. code::
@@ -811,14 +876,14 @@ def line_string_mapping(points: List[Vec3]) -> Dict:
     return {TYPE: LINE_STRING, COORDINATES: points}
 
 
-def is_linear_ring(points: List[Vec3]):
+def is_linear_ring(points: list[Vec3]):
     return points[0].isclose(points[-1])
 
 
 # GeoJSON : A linear ring MUST follow the right-hand rule with respect
 # to the area it bounds, i.e., exterior rings are counterclockwise, and
 # holes are clockwise.
-def linear_ring(points: List[Vec3], ccw=True) -> List[Vec3]:
+def linear_ring(points: list[Vec3], ccw=True) -> list[Vec3]:
     """Return `points` as linear ring (last vertex == first vertex),
     argument `ccw` defines the winding orientation, ``True`` for counter-clock
     wise and ``False`` for clock wise.
@@ -839,7 +904,7 @@ def linear_ring(points: List[Vec3], ccw=True) -> List[Vec3]:
     return points
 
 
-def polygon_mapping(points: List[Vec3], holes: List[List[Vec3]]) -> Dict:
+def polygon_mapping(points: list[Vec3], holes: list[list[Vec3]]) -> GeoMapping:
     """Returns a "Polygon" mapping.
 
     .. code::
@@ -877,7 +942,7 @@ def polygon_mapping(points: List[Vec3], holes: List[List[Vec3]]) -> Dict:
     }
 
 
-def join_multi_single_type_mappings(geometries: Iterable[Dict]) -> Dict:
+def join_multi_single_type_mappings(geometries: Iterable[GeoMapping]) -> GeoMapping:
     """Returns multiple geometries as a "MultiPoint", "MultiLineString" or
     "MultiPolygon" mapping.
     """
@@ -895,7 +960,7 @@ def join_multi_single_type_mappings(geometries: Iterable[Dict]) -> Dict:
         return {TYPE: "Multi" + tuple(types)[0], COORDINATES: data}
 
 
-def geometry_collection_mapping(geometries: Iterable[Dict]) -> Dict:
+def geometry_collection_mapping(geometries: Iterable[GeoMapping]) -> GeoMapping:
     """Returns multiple geometries as a "GeometryCollection" mapping."""
     return {TYPE: GEOMETRY_COLLECTION, GEOMETRIES: list(geometries)}
 
@@ -908,7 +973,7 @@ def geometry_collection_mapping(geometries: Iterable[Dict]) -> Dict:
 WGS84_SEMI_MAJOR_AXIS = 6378137
 WGS84_SEMI_MINOR_AXIS = 6356752.3142
 WGS84_ELLIPSOID_ECCENTRIC = math.sqrt(
-    1.0 - WGS84_SEMI_MINOR_AXIS ** 2 / WGS84_SEMI_MAJOR_AXIS ** 2
+    1.0 - WGS84_SEMI_MINOR_AXIS**2 / WGS84_SEMI_MAJOR_AXIS**2
 )
 CONST_E2 = math.e / 2.0
 CONST_PI_2 = math.pi / 2.0
@@ -984,8 +1049,24 @@ def dms2dd(d: float, m: float = 0, s: float = 0) -> float:
     return dd
 
 
-def dd2dms(dd: float) -> Tuple[float, float, float]:
+def dd2dms(dd: float) -> tuple[float, float, float]:
     """Convert decimal degrees into degree, minutes, seconds."""
     m, s = divmod(dd * 3600, 60)
     d, m = divmod(m, 60)
     return d, m, s
+
+
+def assign_layers(entity: DXFGraphic, mapping: GeoMapping) -> None:
+    """Reference implementation for a :func:`post_process` function.
+
+    .. seealso::
+
+        :func:`dxf_entities`
+
+    """
+    properties = mapping.get(PROPERTIES)
+    if properties is None:
+        return
+    layer = properties.get("layer")
+    if layer:
+        entity.dxf.layer = layer
