@@ -1,6 +1,8 @@
-# Copyright (c) 2020-2022, Manfred Moitzi
+# Copyright (c) 2020-2023, Manfred Moitzi
 # License: MIT License
 # type: ignore
+from __future__ import annotations
+from typing import Optional
 import logging
 import os
 import platform
@@ -10,7 +12,6 @@ import tempfile
 import time
 from contextlib import contextmanager
 from pathlib import Path
-from typing import Optional, List, Union
 
 import ezdxf
 from ezdxf.document import Drawing
@@ -23,6 +24,7 @@ from ezdxf.lldxf.validator import (
 
 logger = logging.getLogger("ezdxf")
 win_exec_path = ezdxf.options.get("odafc-addon", "win_exec_path").strip('"')
+unix_exec_path = ezdxf.options.get("odafc-addon", "unix_exec_path").strip('"')
 
 
 class ODAFCError(IOError):
@@ -92,12 +94,10 @@ DARWIN = "Darwin"
 
 
 def is_installed() -> bool:
-    """Returns ``True`` if the ODAFileConverter is installed.
-
-    .. versionadded:: 0.18
-
-    """
+    """Returns ``True`` if the ODAFileConverter is installed."""
     if platform.system() in (LINUX, DARWIN):
+        if unix_exec_path and Path(unix_exec_path).is_file():
+            return True
         return shutil.which("ODAFileConverter") is not None
     # Windows:
     return os.path.exists(win_exec_path)
@@ -108,7 +108,7 @@ def map_version(version: str) -> str:
 
 
 def readfile(
-    filename: str, version: Optional[str] = None, *, audit: bool = False
+    filename: str | os.PathLike, version: Optional[str] = None, *, audit: bool = False
 ) -> Optional[Drawing]:
     """Uses an installed `ODA File Converter`_ to convert a DWG/DXB/DXF file
     into a temporary DXF file and load this file by `ezdxf`.
@@ -153,7 +153,7 @@ def readfile(
 
 def export_dwg(
     doc: Drawing,
-    filename: str,
+    filename: str | os.PathLike,
     version: Optional[str] = None,
     *,
     audit: bool = False,
@@ -211,14 +211,12 @@ def export_dwg(
             )
             _execute_odafc(arguments)
     else:
-        raise FileNotFoundError(
-            f"No such file or directory: '{str(out_folder)}'"
-        )
+        raise FileNotFoundError(f"No such file or directory: '{str(out_folder)}'")
 
 
 def convert(
-    source: Union[str, Path],
-    dest: Union[str, Path] = "",
+    source: str | os.PathLike,
+    dest: str | os.PathLike = "",
     *,
     version="R2018",
     audit=True,
@@ -226,8 +224,6 @@ def convert(
 ):
     """Convert `source` file to `dest` file.
 
-    .. versionadded::  0.18
-
     The file extension defines the target format
     e.g. :code:`convert("test.dxf", "Test.dwg")` converts the source file to a
     DWG file.
@@ -275,9 +271,7 @@ def convert(
     parent_dir = dest_path.parent
     if not parent_dir.exists() or not parent_dir.is_dir():
         # Cannot copy result to destination folder!
-        raise FileNotFoundError(
-            f"Destination folder does not exist: '{parent_dir}'"
-        )
+        raise FileNotFoundError(f"Destination folder does not exist: '{parent_dir}'")
     ext = dest_path.suffix
     fmt = ext.upper()[1:]
     if fmt not in ("DXF", "DWG"):
@@ -338,7 +332,7 @@ def _odafc_arguments(
     output_format: str = "DXF",
     version: str = "ACAD2013",
     audit: bool = False,
-) -> List[str]:
+) -> list[str]:
     """ODA File Converter command line format:
 
     ODAFileConverter "Input Folder" "Output Folder" version type recurse audit [filter]
@@ -370,6 +364,18 @@ def _get_odafc_path(system: str) -> str:
         odafc.ODAFCNotInstalledError: ODA File Converter not installed
 
     """
+    # on Linux and Darwin check if UNIX_EXEC_PATH is set and exist and
+    # return this path as exec path.
+    # This may help if the "which" command can not find the "ODAFileConverter"
+    # command and also adds support for AppImages provided by ODA.
+    if system != WINDOWS and unix_exec_path:
+        if Path(unix_exec_path).is_file():
+            return unix_exec_path
+        else:
+            logger.warning(
+                f"command '{unix_exec_path}' not found, using 'ODAFileConverter'"
+            )
+
     path = shutil.which("ODAFileConverter")
     if not path and system == WINDOWS:
         path = win_exec_path
@@ -402,14 +408,12 @@ def _linux_dummy_display():
         except OSError:
             pass
     else:
-        logger.warning(
-            f"Install xvfb to prevent the ODAFileConverter GUI from opening"
-        )
+        logger.warning(f"Install xvfb to prevent the ODAFileConverter GUI from opening")
         yield os.environ["DISPLAY"]
 
 
 def _run_with_no_gui(
-    system: str, command: str, arguments: List[str]
+    system: str, command: str, arguments: list[str]
 ) -> subprocess.Popen:
     """Execute ODAFC application without launching the GUI.
 
@@ -477,8 +481,8 @@ def _odafc_failed(system: str, proc: subprocess.Popen, stderr: str) -> bool:
         return stderr != ""
 
 
-def _execute_odafc(arguments: List[str]) -> Optional[bytes]:
-    """ Execute ODAFC application.
+def _execute_odafc(arguments: list[str]) -> Optional[bytes]:
+    """Execute ODAFC application.
 
     Args:
         arguments: ODAFC argument list
