@@ -1,13 +1,11 @@
-# Copyright (c) 2017-2021, Manfred Moitzi
+# Copyright (c) 2017-2023, Manfred Moitzi
 # License: MIT License
+from __future__ import annotations
 from typing import (
     TYPE_CHECKING,
     Iterable,
-    List,
-    Set,
     TextIO,
     Any,
-    Dict,
     Optional,
     Callable,
 )
@@ -18,12 +16,10 @@ from ezdxf.entities import factory, DXFEntity
 from ezdxf.math import NULLVEC
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        Drawing,
-        DXFGraphic,
-        BlocksSection,
-        EntityDB,
-    )
+    from ezdxf.document import Drawing
+    from ezdxf.entities import DXFGraphic
+    from ezdxf.sections.blocks import BlocksSection
+
 
 __all__ = ["Auditor", "AuditError", "audit", "BlockCycleDetector"]
 
@@ -44,6 +40,7 @@ class AuditError(IntEnum):
     REMOVED_INVALID_DXF_OBJECT = 12
     REMOVED_STANDALONE_ATTRIB_ENTITY = 13
     MISPLACED_ROOT_DICT = 14
+    ROOT_DICT_NOT_FOUND = 15
 
     UNDEFINED_LINETYPE = 100
     UNDEFINED_DIMENSION_STYLE = 101
@@ -63,7 +60,8 @@ class AuditError(IntEnum):
     UNDEFINED_BLOCK_NAME = 115
     INVALID_INTEGER_VALUE = 116
     INVALID_FLOATING_POINT_VALUE = 117
-    REMOVED_ENTITY_FROM_BLOCK_RECORD = 118
+    MISSING_PERSISTENT_REACTOR = 118
+    BLOCK_NAME_MISMATCH = 119
 
     # DXF entity property errors:
     INVALID_ENTITY_HANDLE = 201
@@ -101,7 +99,7 @@ class ErrorEntry:
         self,
         code: int,
         message: str = "",
-        dxf_entity: DXFEntity = None,
+        dxf_entity: Optional[DXFEntity] = None,
         data: Any = None,
     ):
         self.code: int = code  # error code AuditError()
@@ -111,18 +109,18 @@ class ErrorEntry:
 
 
 class Auditor:
-    def __init__(self, doc: "Drawing"):
-        assert doc is not None
+    def __init__(self, doc: Drawing) -> None:
+        assert doc is not None and doc.rootdict is not None and doc.entitydb is not None
         self.doc = doc
         self._rootdict_handle = doc.rootdict.dxf.handle
-        self.errors: List[ErrorEntry] = []
-        self.fixes: List[ErrorEntry] = []
+        self.errors: list[ErrorEntry] = []
+        self.fixes: list[ErrorEntry] = []
         self._trashcan = doc.entitydb.new_trashcan()
-        self._post_audit_jobs: List[Callable[[], None]] = []
+        self._post_audit_jobs: list[Callable[[], None]] = []
 
     def reset(self) -> None:
-        self.errors = []
-        self.fixes = []
+        self.errors.clear()
+        self.fixes.clear()
         self.empty_trashcan()
 
     def __len__(self) -> int:
@@ -146,20 +144,29 @@ class Auditor:
 
     @property
     def has_errors(self) -> bool:
+        """Returns ``True`` if any unrecoverable errors were detected."""
         return bool(self.errors)
 
     @property
     def has_fixes(self) -> bool:
+        """Returns ``True`` if any recoverable errors were fixed while auditing."""
         return bool(self.fixes)
 
+    @property
+    def has_issues(self) -> bool:
+        """Returns ``True`` if the DXF document has any errors or fixes."""
+        return self.has_fixes or self.has_errors
+
     def print_error_report(
-        self, errors: List[ErrorEntry] = None, stream: TextIO = None
+        self,
+        errors: Optional[list[ErrorEntry]] = None,
+        stream: Optional[TextIO] = None,
     ) -> None:
         def entity_str(count, code, entity):
-            if entity is not None:
-                return f"{count:4d}. Issue [{code}] in {str(entity)}."
+            if entity is not None and entity.is_alive:
+                return f"{count:4d}. Error [{code}] in {str(entity)}."
             else:
-                return f"{count:4d}. Issue [{code}]."
+                return f"{count:4d}. Error [{code}]."
 
         if errors is None:
             errors = self.errors
@@ -170,18 +177,16 @@ class Auditor:
             stream = sys.stdout
 
         if len(errors) == 0:
-            stream.write("No issues found.\n\n")
+            stream.write("No unrecoverable errors found.\n\n")
         else:
-            stream.write(f"{len(errors)} issues found.\n\n")
+            stream.write(f"{len(errors)} errors found.\n\n")
             for count, error in enumerate(errors):
-                stream.write(
-                    entity_str(count + 1, error.code, error.entity) + "\n"
-                )
+                stream.write(entity_str(count + 1, error.code, error.entity) + "\n")
                 stream.write("   " + error.message + "\n\n")
 
-    def print_fixed_errors(self, stream: TextIO = None) -> None:
+    def print_fixed_errors(self, stream: Optional[TextIO] = None) -> None:
         def entity_str(count, code, entity):
-            if entity is not None:
+            if entity is not None and entity.is_alive:
                 return f"{count:4d}. Issue [{code}] fixed in {str(entity)}."
             else:
                 return f"{count:4d}. Issue [{code}] fixed."
@@ -194,16 +199,14 @@ class Auditor:
         else:
             stream.write(f"{len(self.fixes)} issues fixed.\n\n")
             for count, error in enumerate(self.fixes):
-                stream.write(
-                    entity_str(count + 1, error.code, error.entity) + "\n"
-                )
+                stream.write(entity_str(count + 1, error.code, error.entity) + "\n")
                 stream.write("   " + error.message + "\n\n")
 
     def add_error(
         self,
         code: int,
         message: str = "",
-        dxf_entity: DXFEntity = None,
+        dxf_entity: Optional[DXFEntity] = None,
         data: Any = None,
     ) -> None:
         self.errors.append(ErrorEntry(code, message, dxf_entity, data))
@@ -212,12 +215,12 @@ class Auditor:
         self,
         code: int,
         message: str = "",
-        dxf_entity: DXFEntity = None,
+        dxf_entity: Optional[DXFEntity] = None,
         data: Any = None,
     ) -> None:
         self.fixes.append(ErrorEntry(code, message, dxf_entity, data))
 
-    def purge(self, codes: Set[int]):
+    def purge(self, codes: set[int]):
         """Remove error messages defined by integer error `codes`.
 
         This is useful to remove errors which are not important for a specific
@@ -226,16 +229,19 @@ class Auditor:
         """
         self.errors = [err for err in self.errors if err.code in codes]
 
-    def run(self) -> List[ErrorEntry]:
+    def run(self) -> list[ErrorEntry]:
+        if not self.check_root_dict():
+            # no root dict found: abort audit process
+            return self.errors
         self.doc.entitydb.audit(self)
-        self.check_root_dict()
+        self.check_root_dict_entries()
         self.check_tables()
-        self.audit_all_database_entities()
         self.doc.objects.audit(self)
         self.doc.blocks.audit(self)
         self.doc.groups.audit(self)
-        self.check_block_reference_cycles()
         self.doc.layouts.audit(self)
+        self.audit_all_database_entities()
+        self.check_block_reference_cycles()
         self.empty_trashcan()
         self.doc.objects.purge()
         return self.errors
@@ -259,14 +265,33 @@ class Auditor:
     def add_post_audit_job(self, job: Callable):
         self._post_audit_jobs.append(job)
 
-    def check_root_dict(self) -> None:
-        root_dict = self.doc.rootdict
+    def check_root_dict(self) -> bool:
+        rootdict = self.doc.rootdict
+        if rootdict.dxftype() != "DICTIONARY":
+            self.add_error(
+                AuditError.ROOT_DICT_NOT_FOUND,
+                f"Critical error - first object in OBJECTS section is not the expected "
+                f"root dictionary, found {str(rootdict)}.",
+            )
+            return False
+        if rootdict.dxf.get("owner") != "0":
+            rootdict.dxf.owner = "0"
+            self.fixed_error(
+                code=AuditError.INVALID_OWNER_HANDLE,
+                message=f"Fixed invalid owner handle in root {str(rootdict)}.",
+            )
+        return True
+
+    def check_root_dict_entries(self) -> None:
+        rootdict = self.doc.rootdict
+        if rootdict.dxftype() != "DICTIONARY":
+            return
         for name in REQUIRED_ROOT_DICT_ENTRIES:
-            if name not in root_dict:
+            if name not in rootdict:
                 self.add_error(
                     code=AuditError.MISSING_REQUIRED_ROOT_DICT_ENTRY,
-                    message=f"Missing root dict entry: {name}",
-                    dxf_entity=root_dict,
+                    message=f"Missing rootdict entry: {name}",
+                    dxf_entity=rootdict,
                 )
 
     def check_tables(self) -> None:
@@ -289,8 +314,7 @@ class Auditor:
         db = self.doc.entitydb
         db.locked = True
         # To create new entities while auditing, add a post audit job by calling
-        # Auditor.app_post_audit_job() with a callable object or
-        # function as argument.
+        # Auditor.app_post_audit_job() with a callable object or function as argument.
         self._post_audit_jobs = []
         for entity in db.values():
             if entity.is_alive:
@@ -342,7 +366,7 @@ class Auditor:
                 data=style,
             )
 
-    def check_dimension_style(self, entity: "DXFGraphic") -> None:
+    def check_dimension_style(self, entity: DXFGraphic) -> None:
         """Check for usage of undefined dimension styles."""
         assert self.doc is entity.doc, "Entity from different DXF document."
         if not entity.dxf.hasattr("dimstyle"):
@@ -371,7 +395,7 @@ class Auditor:
                 data=name,
             )
 
-    def check_entity_color_index(self, entity: "DXFGraphic") -> None:
+    def check_entity_color_index(self, entity: DXFGraphic) -> None:
         color = entity.dxf.color
         # 0 == BYBLOCK
         # 256 == BYLAYER
@@ -385,7 +409,7 @@ class Auditor:
                 data=color,
             )
 
-    def check_entity_lineweight(self, entity: "DXFGraphic") -> None:
+    def check_entity_lineweight(self, entity: DXFGraphic) -> None:
         weight = entity.dxf.lineweight
         if weight not in const.VALID_DXF_LINEWEIGHT_VALUES:
             entity.dxf.lineweight = validator.fix_lineweight(weight)
@@ -411,7 +435,7 @@ class Auditor:
                 entity.dxf.owner = "0"
                 self.fixed_error(
                     code=AuditError.INVALID_OWNER_HANDLE,
-                    message=f"Fixed invalid owner handle in root {str(self)}.",
+                    message=f"Fixed invalid owner handle in root {str(entity)}.",
                 )
             elif entity.dxftype() == "TABLE":
                 name = entity.dxf.get("name", "UNKNOWN")
@@ -426,14 +450,14 @@ class Auditor:
                     message=f"Deleted {str(entity)} entity with invalid owner "
                     f"handle #{owner_handle}.",
                 )
-                self.trash(doc.entitydb.get(handle))
+                self.trash(doc.entitydb.get(handle))  # type: ignore
 
     def check_extrusion_vector(self, entity: DXFEntity) -> None:
         if NULLVEC.isclose(entity.dxf.extrusion):
             entity.dxf.discard("extrusion")
             self.fixed_error(
                 code=AuditError.INVALID_EXTRUSION_VECTOR,
-                message=f"Fixed extrusion vector for entity: {str(self)}.",
+                message=f"Fixed extrusion vector for entity: {str(entity)}.",
                 dxf_entity=entity,
             )
 
@@ -445,7 +469,7 @@ class Auditor:
             entity.dxf.discard("transparency")
             self.fixed_error(
                 code=AuditError.INVALID_TRANSPARENCY,
-                message=f"Fixed invalid transparency for entity: {str(self)}.",
+                message=f"Fixed invalid transparency for entity: {str(entity)}.",
                 dxf_entity=entity,
             )
 
@@ -462,17 +486,15 @@ class Auditor:
 
 
 class BlockCycleDetector:
-    def __init__(self, doc: "Drawing"):
+    def __init__(self, doc: Drawing):
         self.key = doc.blocks.key
         self.blocks = self._build_block_ledger(doc.blocks)
 
-    def _build_block_ledger(
-        self, blocks: "BlocksSection"
-    ) -> Dict[str, Set[str]]:
+    def _build_block_ledger(self, blocks: BlocksSection) -> dict[str, set[str]]:
         ledger = dict()
         for block in blocks:
             inserts = {
-                self.key(insert.dxf.name) for insert in block.query("INSERT")
+                self.key(insert.dxf.get("name", "")) for insert in block.query("INSERT")
             }
             ledger[self.key(block.name)] = inserts
         return ledger
@@ -494,12 +516,12 @@ class BlockCycleDetector:
             path.pop()
             return False
 
-        path: List[str] = []
+        path: list[str] = []
         block_name = self.key(block_name)
         return check(block_name)
 
 
-def audit(entity: DXFEntity, doc: "Drawing") -> Auditor:
+def audit(entity: DXFEntity, doc: Drawing) -> Auditor:
     """Setup an :class:`Auditor` object, run the audit process for `entity`
     and return result as :class:`Auditor` object.
 
