@@ -1,6 +1,7 @@
-# Copyright (c) 2019-2021 Manfred Moitzi
+# Copyright (c) 2019-2022 Manfred Moitzi
 # License: MIT License
-from typing import TYPE_CHECKING, Iterable, Optional
+from __future__ import annotations
+from typing import TYPE_CHECKING, Iterable, Optional, Iterator
 import copy
 from ezdxf.math import Vec3, Matrix44
 from ezdxf.lldxf.tags import Tags
@@ -19,56 +20,11 @@ from .dxfobj import DXFObject
 from .objectcollection import ObjectCollection
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        TagWriter,
-        DXFNamespace,
-        Drawing,
-    )
+    from ezdxf.entities import DXFNamespace
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.document import Drawing
 
-__all__ = ["AcadTable", "AcadTableBlockContent"]
-
-
-@factory.register_entity
-class AcadTableBlockContent(DXFTagStorage):
-    DXFTYPE = "ACAD_TABLE"
-
-    def proxy_graphic_content(self) -> Iterable[DXFGraphic]:
-        return super().__virtual_entities__()
-
-    def _block_content(self) -> Iterable[DXFGraphic]:
-        tags = self._block_reference_tags()
-        block_name: str = tags.get_first_value(2, "*")
-        return self.doc.blocks.get(block_name, [])  # type: ignore
-
-    def _block_reference_tags(self) -> Tags:
-        try:
-            return self.xtags.get_subclass("AcDbBlockReference")
-        except const.DXFKeyError:
-            return Tags()
-
-    def _insert_location(self) -> Vec3:
-        return self._block_reference_tags().get_first_value(10, Vec3())
-
-    def __virtual_entities__(self) -> Iterable[DXFGraphic]:
-        """Implements the SupportsVirtualEntities protocol."""
-        insert: Vec3 = Vec3(self._insert_location())
-        m: Optional[Matrix44] = None
-        if insert:
-            # TODO: OCS transformation (extrusion) is ignored yet
-            m = Matrix44.translate(insert.x, insert.y, insert.z)
-
-        for entity in self._block_content():
-            try:
-                clone = entity.copy()
-            except const.DXFTypeError:
-                continue
-            if m is not None:
-                # noinspection PyUnboundLocalVariable
-                try:
-                    clone.transform(m)
-                except:  # skip entity at any transformation issue
-                    continue
-            yield clone
+__all__ = ["AcadTable", "AcadTableBlockContent", "acad_table_to_block"]
 
 
 acdb_block_reference = DefSubclass(
@@ -282,14 +238,14 @@ class AcadTable(DXFGraphic):
         super().__init__()
         self.data = None
 
-    def _copy_data(self, entity: "DXFEntity") -> None:
+    def copy_data(self, entity: DXFEntity) -> None:
         """Copy data."""
         assert isinstance(entity, AcadTable)
         entity.data = copy.deepcopy(self.data)
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             processor.fast_load_dxfattribs(
@@ -301,10 +257,10 @@ class AcadTable(DXFGraphic):
             self.load_table(tags)
         return dxf
 
-    def load_table(self, tags: "Tags"):
+    def load_table(self, tags: Tags):
         pass
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         super().export_entity(tagwriter)
         tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_block_reference.name)
@@ -312,7 +268,7 @@ class AcadTable(DXFGraphic):
         tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_table.name)
         self.export_table(tagwriter)
 
-    def export_table(self, tagwriter: "TagWriter"):
+    def export_table(self, tagwriter: AbstractTagWriter):
         pass
 
     def __referenced_blocks__(self) -> Iterable[str]:
@@ -382,7 +338,88 @@ class TableStyle(DXFObject):
 
 
 class TableStyleManager(ObjectCollection[TableStyle]):
-    def __init__(self, doc: "Drawing"):
-        super().__init__(
-            doc, dict_name="ACAD_TABLESTYLE", object_type="TABLESTYLE"
-        )
+    def __init__(self, doc: Drawing):
+        super().__init__(doc, dict_name="ACAD_TABLESTYLE", object_type="TABLESTYLE")
+
+
+@factory.register_entity
+class AcadTableBlockContent(DXFTagStorage):
+    DXFTYPE = "ACAD_TABLE"
+    DXFATTRIBS = DXFAttributes(
+        base_class, acdb_entity, acdb_block_reference, acdb_table
+    )
+
+    def load_dxf_attribs(
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
+        dxf = super().load_dxf_attribs(processor)
+        if processor:
+            processor.fast_load_dxfattribs(
+                dxf, acdb_block_reference_group_codes, subclass=2
+            )
+            processor.fast_load_dxfattribs(
+                dxf, acdb_table_group_codes, subclass=3, log=False
+            )
+        return dxf
+
+    def proxy_graphic_content(self) -> Iterable[DXFGraphic]:
+        return super().__virtual_entities__()
+
+    def _block_content(self) -> Iterator[DXFGraphic]:
+        block_name: str = self.get_block_name()
+        return self.doc.blocks.get(block_name, [])  # type: ignore
+
+    def get_block_name(self) -> str:
+        return self.dxf.get("geometry", "")
+
+    def get_insert_location(self) -> Vec3:
+        return self.dxf.get("insert", Vec3())
+
+    def __virtual_entities__(self) -> Iterator[DXFGraphic]:
+        """Implements the SupportsVirtualEntities protocol."""
+        insert: Vec3 = Vec3(self.get_insert_location())
+        m: Optional[Matrix44] = None
+        if insert:
+            # TODO: OCS transformation (extrusion) is ignored yet
+            m = Matrix44.translate(insert.x, insert.y, insert.z)
+
+        for entity in self._block_content():
+            try:
+                clone = entity.copy()
+            except const.DXFTypeError:
+                continue
+            if m is not None:
+                # noinspection PyUnboundLocalVariable
+                try:
+                    clone.transform(m)
+                except:  # skip entity at any transformation issue
+                    continue
+            yield clone
+
+
+def acad_table_to_block(table: DXFEntity) -> None:
+    """Converts the given ACAD_TABLE entity to a block references (INSERT entity).
+
+    The original ACAD_TABLE entity will be destroyed.
+
+    .. versionadded:: 1.1
+
+    """
+    if not isinstance(table, AcadTableBlockContent):
+        return
+    doc = table.doc
+    owner = table.dxf.owner
+    block_name = table.get_block_name()
+    if doc is None or block_name == "" or owner is None:
+        return
+    try:
+        layout = doc.layouts.get_layout_by_key(owner)
+    except const.DXFKeyError:
+        return
+    # replace ACAD_TABLE entity by INSERT entity
+    layout.add_blockref(
+        block_name,
+        insert=table.get_insert_location(),
+        dxfattribs={"layer": table.dxf.get("layer", "0")},
+    )
+    layout.delete_entity(table)  # type: ignore
