@@ -1,6 +1,7 @@
-# Copyright (c) 2019-2021 Manfred Moitzi
+# Copyright (c) 2019-2023 Manfred Moitzi
 # License: MIT License
-from typing import TYPE_CHECKING, Iterable, List, Union
+from __future__ import annotations
+from typing import TYPE_CHECKING, Iterable, Union, Optional, Sequence
 from ezdxf.lldxf.attributes import (
     DXFAttr,
     DXFAttributes,
@@ -24,7 +25,8 @@ from .dxfgfx import DXFGraphic, acdb_entity
 from .factory import register_entity
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import TagWriter, DXFNamespace
+    from ezdxf.entities import DXFNamespace
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
 
 __all__ = [
     "Body",
@@ -80,14 +82,16 @@ class Body(DXFGraphic):
     DXFATTRIBS = DXFAttributes(base_class, acdb_entity, acdb_modeler_geometry)
     MIN_DXF_VERSION_FOR_EXPORT = DXF2000
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
-        self._sat: List[str] = []
+        # Store SAT data as immutable sequence of strings, so the data can be shared
+        # across multiple copies of an ACIS entity.
+        self._sat: Sequence[str] = tuple()
         self._sab: bytes = b""
         self._update = False
 
     @property
-    def acis_data(self) -> Union[bytes, List[str]]:
+    def acis_data(self) -> Union[bytes, Sequence[str]]:
         """Returns :term:`SAT` data  for DXF R2000 up to R2010 and :term:`SAB`
         data for DXF R2013 and later
         """
@@ -96,18 +100,18 @@ class Body(DXFGraphic):
         return self.sat
 
     @property
-    def sat(self) -> List[str]:
-        """Get/Set :term:`SAT` data as list of strings. """
+    def sat(self) -> Sequence[str]:
+        """Get/Set :term:`SAT` data as sequence of strings."""
         return self._sat
 
     @sat.setter
-    def sat(self, data: List[str]) -> None:
-        """Set :term:`SAT` data as list of strings. """
-        self._sat = data
+    def sat(self, data: Sequence[str]) -> None:
+        """Set :term:`SAT` data as sequence of strings."""
+        self._sat = tuple(data)
 
     @property
     def sab(self) -> bytes:
-        """Get/Set :term:`SAB` data as bytes. """
+        """Get/Set :term:`SAB` data as bytes."""
         if (  # load SAB data on demand
             self.doc is not None
             and self.has_binary_data
@@ -118,7 +122,7 @@ class Body(DXFGraphic):
 
     @sab.setter
     def sab(self, data: bytes) -> None:
-        """Set :term:`SAB` data as bytes. """
+        """Set :term:`SAB` data as bytes."""
         self._update = True
         self._sab = data
 
@@ -132,13 +136,13 @@ class Body(DXFGraphic):
         else:
             return False
 
-    def copy(self):
+    def raw_copy(self):
         """Prevent copying. (internal interface)"""
         raise DXFTypeError("Copying of ACIS data not supported.")
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         """Loading interface. (internal API)"""
         dxf = super().load_dxf_attribs(processor)
         if processor:
@@ -152,9 +156,9 @@ class Body(DXFGraphic):
     def load_sat_data(self, tags: Tags):
         """Loading interface. (internal API)"""
         text_lines = tags2textlines(tag for tag in tags if tag.code in (1, 3))
-        self._sat = list(crypt.decode(text_lines))
+        self._sat = tuple(crypt.decode(text_lines))
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags. (internal API)"""
         super().export_entity(tagwriter)
         tagwriter.write_tag2(SUBCLASS_MARKER, acdb_modeler_geometry.name)
@@ -172,7 +176,7 @@ class Body(DXFGraphic):
             self.dxf.export_dxf_attribs(tagwriter, "version")
             self.export_sat_data(tagwriter)
 
-    def export_sat_data(self, tagwriter: "TagWriter") -> None:
+    def export_sat_data(self, tagwriter: AbstractTagWriter) -> None:
         """Export ACIS data as DXF tags. (internal API)"""
 
         def cleanup(lines):
@@ -196,6 +200,9 @@ class Body(DXFGraphic):
             self.doc.acdsdata.del_acis_data(self.dxf.handle)  # type: ignore
         super().destroy()
 
+    def transform(self, m: Matrix44) -> DXFGraphic:
+        raise NotImplementedError("cannot transform ACIS entities")
+
 
 def tags2textlines(tags: Iterable) -> Iterable[str]:
     """Yields text lines from code 1 and 3 tags, code 1 starts a line following
@@ -253,14 +260,14 @@ class Solid3d(Body):
     )
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             processor.fast_load_dxfattribs(dxf, acdb_3dsolid_group_codes, 3)
         return dxf
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         # base class export is done by parent class
         super().export_entity(tagwriter)
@@ -271,14 +278,16 @@ class Solid3d(Body):
             self.dxf.export_dxf_attribs(tagwriter, "history_handle")
 
 
-def load_matrix(subclass: "Tags", code: int) -> Matrix44:
+def load_matrix(subclass: Tags, code: int) -> Matrix44:
     values = [tag.value for tag in subclass.find_all(code)]
     if len(values) != 16:
         raise DXFStructureError("Invalid transformation matrix.")
     return Matrix44(values)
 
 
-def export_matrix(tagwriter: "TagWriter", code: int, matrix: Matrix44) -> None:
+def export_matrix(
+    tagwriter: AbstractTagWriter, code: int, matrix: Matrix44
+) -> None:
     for value in list(matrix):
         tagwriter.write_tag2(code, value)
 
@@ -303,14 +312,14 @@ class Surface(Body):
     )
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             processor.fast_load_dxfattribs(dxf, acdb_surface_group_codes, 3)
         return dxf
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         # base class export is done by parent class
         super().export_entity(tagwriter)
@@ -375,8 +384,8 @@ class ExtrudedSurface(Surface):
         self.path_entity_transformation_matrix = Matrix44()
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             processor.fast_load_dxfattribs(
@@ -390,7 +399,7 @@ class ExtrudedSurface(Surface):
         self.sweep_entity_transformation_matrix = load_matrix(tags, code=46)
         self.path_entity_transformation_matrix = load_matrix(tags, code=47)
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         # base class export is done by parent class
         super().export_entity(tagwriter)
@@ -477,8 +486,8 @@ class LoftedSurface(Surface):
         self.transformation_matrix_lofted_entity = Matrix44()
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             processor.fast_load_dxfattribs(
@@ -490,7 +499,7 @@ class LoftedSurface(Surface):
     def load_matrices(self, tags: Tags):
         self.transformation_matrix_lofted_entity = load_matrix(tags, code=40)
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         # base class export is done by parent class
         super().export_entity(tagwriter)
@@ -544,8 +553,8 @@ class RevolvedSurface(Surface):
         self.transformation_matrix_revolved_entity = Matrix44()
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             processor.fast_load_dxfattribs(
@@ -557,7 +566,7 @@ class RevolvedSurface(Surface):
     def load_matrices(self, tags: Tags):
         self.transformation_matrix_revolved_entity = load_matrix(tags, code=42)
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         # base class export is done by parent class
         super().export_entity(tagwriter)
@@ -655,8 +664,8 @@ class SweptSurface(Surface):
         self.path_entity_transformation_matrix = Matrix44()
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             processor.fast_load_dxfattribs(
@@ -671,7 +680,7 @@ class SweptSurface(Surface):
         self.sweep_entity_transformation_matrix = load_matrix(tags, code=46)
         self.path_entity_transformation_matrix = load_matrix(tags, code=47)
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         # base class export is done by parent class
         super().export_entity(tagwriter)
