@@ -1,16 +1,7 @@
-# Copyright (c) 2019-2021 Manfred Moitzi
+# Copyright (c) 2019-2023 Manfred Moitzi
 # License: MIT License
-from typing import (
-    TYPE_CHECKING,
-    List,
-    Iterable,
-    Set,
-    Sequence,
-    Optional,
-    Iterator,
-    Dict,
-)
-from collections import OrderedDict
+from __future__ import annotations
+from typing import TYPE_CHECKING, Iterable, Sequence, Optional, Iterator
 from ezdxf.lldxf.types import dxftag, uniform_appid
 from ezdxf.lldxf.tags import Tags
 from ezdxf.lldxf.const import DXFKeyError, DXFStructureError
@@ -21,7 +12,7 @@ from ezdxf.lldxf.const import (
 )
 
 if TYPE_CHECKING:
-    from ezdxf.lldxf.tagwriter import TagWriter
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
 
 __all__ = ["AppData", "Reactors"]
 
@@ -30,8 +21,8 @@ ERR_DXF_ATTRIB_NOT_EXITS = "DXF attribute {} does not exist"
 
 
 class AppData:
-    def __init__(self):
-        self.data: Dict[str, Tags] = OrderedDict()
+    def __init__(self) -> None:
+        self.data: dict[str, Tags] = dict()
 
     def __contains__(self, appid: str) -> bool:
         """Returns ``True`` if application-defined data exist for `appid`."""
@@ -41,6 +32,9 @@ class AppData:
         """Returns the count of AppData."""
         return len(self.data)
 
+    def tags(self) -> Iterable[Tags]:
+        return self.data.values()
+
     def get(self, appid: str) -> Tags:
         """Get application-defined data for `appid` as
         :class:`~ezdxf.lldxf.tags.Tags` container.
@@ -82,7 +76,7 @@ class AppData:
         if _appid in self.data:
             del self.data[_appid]
 
-    def export_dxf(self, tagwriter: "TagWriter") -> None:
+    def export_dxf(self, tagwriter: AbstractTagWriter) -> None:
         for data in self.data.values():
             tagwriter.write_tags(data)
 
@@ -95,8 +89,8 @@ class Reactors:
 
     """
 
-    def __init__(self, handles: Iterable[str] = None):
-        self.reactors: Set[str] = set(handles or [])
+    def __init__(self, handles: Optional[Iterable[str]] = None):
+        self.reactors: set[str] = set(handles or [])
 
     def __len__(self) -> int:
         """Returns count of registered handles."""
@@ -111,7 +105,7 @@ class Reactors:
         return iter(self.get())
 
     @classmethod
-    def from_tags(cls, tags: Tags = None) -> "Reactors":
+    def from_tags(cls, tags: Optional[Tags] = None) -> Reactors:
         """Create Reactors() instance from tags.
 
         Expected DXF structure:
@@ -128,7 +122,7 @@ class Reactors:
             raise DXFStructureError("ACAD_REACTORS error")
         return cls((handle.value for handle in tags[1:-1]))
 
-    def get(self) -> List[str]:
+    def get(self) -> list[str]:
         """Returns all registered handles as sorted list."""
         return sorted(self.reactors, key=lambda x: int(x, base=16))
 
@@ -144,7 +138,7 @@ class Reactors:
         """Discard a single `handle`."""
         self.reactors.discard(handle)
 
-    def export_dxf(self, tagwriter: "TagWriter") -> None:
+    def export_dxf(self, tagwriter: AbstractTagWriter) -> None:
         tagwriter.write_tag2(APP_DATA_MARKER, ACAD_REACTORS)
         for handle in self.get():
             tagwriter.write_tag2(REACTOR_HANDLE_CODE, handle)
