@@ -1,9 +1,11 @@
-# Copyright (c) 2019-2021 Manfred Moitzi
+# Copyright (c) 2019-2023 Manfred Moitzi
 # License: MIT License
+from __future__ import annotations
 from typing import TYPE_CHECKING, Optional
+from typing_extensions import Self
 import copy
 from ezdxf.lldxf import validator
-from ezdxf.math import NULLVEC, Vec3, Z_AXIS, OCS
+from ezdxf.math import NULLVEC, Vec3, Z_AXIS, OCS, Matrix44
 from ezdxf.lldxf.attributes import (
     DXFAttr,
     DXFAttributes,
@@ -35,7 +37,11 @@ from .mtext import (
 from .factory import register_entity
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import TagWriter, Tags, DXFEntity
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.lldxf.tags import Tags
+    from ezdxf.entities import DXFEntity
+    from ezdxf import xref
+
 
 __all__ = ["AttDef", "Attrib", "copy_attrib_as_text", "BaseAttrib"]
 
@@ -66,8 +72,13 @@ __all__ = ["AttDef", "Attrib", "copy_attrib_as_text", "BaseAttrib"]
 # from the 2nd AcDbText subclass of the TEXT entity is stored in the
 # AcDbAttribute subclass:
 attrib_fields = {
+    # "version": DXFAttr(280, default=0, dxfversion=const.DXF2010),
+    # The "version" tag has the same group code as the lock_position tag!!!!!
     # Version number: 0 = 2010
-    "version": DXFAttr(280, default=0, dxfversion=const.DXF2010),
+    # This tag is not really used (at least by BricsCAD) but there exists DXF files
+    # which do use this tag: "dxftest\attrib\attrib_with_mtext_R2018.dxf"
+    # ezdxf stores the last group code 280 as "lock_position" attribute and does
+    # not export a version tag for any DXF version.
     # Tag string (cannot contain spaces):
     "tag": DXFAttr(
         2,
@@ -95,7 +106,7 @@ attrib_fields = {
     "lock_position": DXFAttr(
         280,
         default=0,
-        dxfversion=const.DXF2010,
+        dxfversion=const.DXF2007,  # tested with BricsCAD 2023/TrueView 2023
         optional=True,
         validator=validator.is_integer_bool,
         fixer=RETURN_DEFAULT,
@@ -129,6 +140,11 @@ acdb_attdef_group_codes = group_code_mapping(acdb_attdef)
 acdb_attrib = DefSubclass("AcDbAttribute", attrib_fields)
 acdb_attrib_group_codes = group_code_mapping(acdb_attrib)
 
+# --------------------------------------------------------------------------------------
+# Does subclass AcDbXrecord really exist? Only the documentation in the DXF reference
+# exists, no real world examples seen so far - it wouldn't be the first error or misleading
+# information in the DXF reference.
+# --------------------------------------------------------------------------------------
 # For XRECORD the tag order is important and group codes appear multiple times,
 # therefore this attribute definition needs a special treatment!
 acdb_attdef_xrecord = DefSubclass(
@@ -156,7 +172,7 @@ acdb_attdef_xrecord = DefSubclass(
         # Number of secondary attributes or attribute definitions:
         ("secondary_attribs_count", DXFAttr(70, default=0)),
         # Hard-pointer id of secondary attribute(s) or attribute definition(s):
-        ("secondary_attribs_handle", DXFAttr(70, default=0)),
+        ("secondary_attribs_handle", DXFAttr(340, default="0")),
         # Alignment point of attribute or attribute definition:
         ("align_point", DXFAttr(10, xtype=XType.point3d, default=NULLVEC)),
         ("current_annotation_scale", DXFAttr(40, default=0)),
@@ -195,16 +211,15 @@ acdb_attdef_xrecord = DefSubclass(
 class BaseAttrib(Text):
     XRECORD_DEF = acdb_attdef_xrecord
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         # Does subclass AcDbXrecord really exist?
-        self._xrecord: Optional["Tags"] = None
-        self._embedded_mtext: Optional["EmbeddedMText"] = None
+        self._xrecord: Optional[Tags] = None
+        self._embedded_mtext: Optional[EmbeddedMText] = None
 
-    def _copy_data(self, entity: "DXFEntity") -> None:
+    def copy_data(self, entity: DXFEntity) -> None:
         """Copy entity data, xrecord data and embedded MTEXT are not stored
         in the entity database.
-
         """
         assert isinstance(entity, BaseAttrib)
         entity._xrecord = copy.deepcopy(self._xrecord)
@@ -219,7 +234,7 @@ class BaseAttrib(Text):
             mtext.load_dxf_tags(processor)
             self._embedded_mtext = mtext
 
-    def export_dxf_r2018_features(self, tagwriter: "TagWriter") -> None:
+    def export_dxf_r2018_features(self, tagwriter: AbstractTagWriter) -> None:
         tagwriter.write_tag2(71, self.dxf.attribute_type)
         tagwriter.write_tag2(72, 0)  # unknown tag
         if self.dxf.hasattr("align_point"):
@@ -233,63 +248,46 @@ class BaseAttrib(Text):
 
     @property
     def is_const(self) -> bool:
-        """This is a constant attribute."""
+        """This is a constant attribute if ``True``."""
         return bool(self.dxf.flags & const.ATTRIB_CONST)
 
     @is_const.setter
     def is_const(self, state: bool) -> None:
-        """This is a constant attribute."""
-        self.dxf.flags = set_flag_state(
-            self.dxf.flags, const.ATTRIB_CONST, state
-        )
+        self.dxf.flags = set_flag_state(self.dxf.flags, const.ATTRIB_CONST, state)
 
     @property
     def is_invisible(self) -> bool:
-        """Attribute is invisible (does not appear)."""
+        """Attribute is invisible if ``True``."""
         return bool(self.dxf.flags & const.ATTRIB_INVISIBLE)
 
     @is_invisible.setter
     def is_invisible(self, state: bool) -> None:
-        """Attribute is invisible (does not appear)."""
-        self.dxf.flags = set_flag_state(
-            self.dxf.flags, const.ATTRIB_INVISIBLE, state
-        )
+        self.dxf.flags = set_flag_state(self.dxf.flags, const.ATTRIB_INVISIBLE, state)
 
     @property
     def is_verify(self) -> bool:
-        """Verification is required on input of this attribute.
-        (CAD application feature)
-
+        """Verification is required on input of this attribute. (interactive CAD
+        application feature)
         """
         return bool(self.dxf.flags & const.ATTRIB_VERIFY)
 
     @is_verify.setter
     def is_verify(self, state: bool) -> None:
-        """Verification is required on input of this attribute.
-        (CAD application feature)
-
-        """
-        self.dxf.flags = set_flag_state(
-            self.dxf.flags, const.ATTRIB_VERIFY, state
-        )
+        self.dxf.flags = set_flag_state(self.dxf.flags, const.ATTRIB_VERIFY, state)
 
     @property
     def is_preset(self) -> bool:
-        """No prompt during insertion. (CAD application feature)"""
+        """No prompt during insertion. (interactive CAD application feature)"""
         return bool(self.dxf.flags & const.ATTRIB_IS_PRESET)
 
     @is_preset.setter
     def is_preset(self, state: bool) -> None:
-        """No prompt during insertion. (CAD application feature)"""
-        self.dxf.flags = set_flag_state(
-            self.dxf.flags, const.ATTRIB_IS_PRESET, state
-        )
+        self.dxf.flags = set_flag_state(self.dxf.flags, const.ATTRIB_IS_PRESET, state)
 
     @property
     def has_embedded_mtext_entity(self) -> bool:
-        """Returns ``True`` if the entity has an embedded MTEXT entity for multi
-        line support.
-
+        """Returns ``True`` if the entity has an embedded MTEXT entity for multi-line
+        support.
         """
         return bool(self._embedded_mtext)
 
@@ -297,7 +295,6 @@ class BaseAttrib(Text):
         """Returns the embedded MTEXT entity as a regular but virtual
         :class:`MText` entity with the same graphical properties as the
         host entity.
-
         """
         if not self._embedded_mtext:
             raise TypeError("no embedded MTEXT entity exist")
@@ -309,17 +306,17 @@ class BaseAttrib(Text):
         """Returns the embedded MTEXT content without formatting codes.
         Returns an empty string if no embedded MTEXT entity exist.
 
-        The "fast" mode is accurate if the DXF content was created by
+        The `fast` mode is accurate if the DXF content was created by
         reliable (and newer) CAD applications like AutoCAD or BricsCAD.
-        The "accurate" mode is for some rare cases where the content was
+        The `accurate` mode is for some rare cases where the content was
         created by older CAD applications or unreliable DXF libraries and CAD
         applications.
 
-        The "accurate" mode is **much** slower than the "fast" mode.
+        The `accurate` mode is **much** slower than the `fast` mode.
 
         Args:
-            fast: uses the "fast" mode to extract the plain MTEXT content if
-                ``True`` or the "accurate" mode if set to ``False``
+            fast: uses the `fast` mode to extract the plain MTEXT content if
+                ``True`` or the `accurate` mode if set to ``False``
 
         """
         if self._embedded_mtext:
@@ -372,6 +369,30 @@ class BaseAttrib(Text):
         self.set_mtext(mtext, graphic_properties)
         mtext.destroy()
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        super().register_resources(registry)
+        if self._embedded_mtext:
+            self._embedded_mtext.register_resources(registry)
+
+    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        assert isinstance(clone, BaseAttrib)
+        super().map_resources(clone, mapping)
+        if self._embedded_mtext and clone._embedded_mtext:
+            self._embedded_mtext.map_resources(clone._embedded_mtext, mapping)
+        # todo: map handles in embedded XRECORD if a real world example shows up
+
+    def transform(self, m: Matrix44) -> Self:
+        if self._embedded_mtext is None:
+            super().transform(m)
+        else:
+            mtext = self._embedded_mtext.virtual_mtext_entity()
+            mtext.transform(m)
+            self.set_mtext(mtext, graphic_properties=False)
+            self.post_transform(m)
+        return self
+
 
 def _update_content_from_mtext(text: Text, mtext: MText) -> None:
     content = mtext.plain_text(split=True, fast=True)
@@ -412,14 +433,12 @@ class AttDef(BaseAttrib):
     DXFATTRIBS = DXFAttributes(base_class, acdb_entity, acdb_text, acdb_attdef)
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super(Text, self).load_dxf_attribs(processor)
         # Do not call Text loader.
         if processor:
-            processor.fast_load_dxfattribs(
-                dxf, acdb_text_group_codes, 2, recover=True
-            )
+            processor.fast_load_dxfattribs(dxf, acdb_text_group_codes, 2, recover=True)
             processor.fast_load_dxfattribs(
                 dxf, acdb_attdef_group_codes, 3, recover=True
             )
@@ -431,7 +450,7 @@ class AttDef(BaseAttrib):
 
         return dxf
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         # Text() writes 2x AcDbText which is not suitable for AttDef()
         self.export_acdb_entity(tagwriter)
         self.export_acdb_text(tagwriter)
@@ -440,13 +459,13 @@ class AttDef(BaseAttrib):
             self.dxf.attribute_type = 4 if self.has_embedded_mtext_entity else 1
             self.export_dxf_r2018_features(tagwriter)
 
-    def export_acdb_attdef(self, tagwriter: "TagWriter") -> None:
+    def export_acdb_attdef(self, tagwriter: AbstractTagWriter) -> None:
         if tagwriter.dxfversion > const.DXF12:
             tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_attdef.name)
         self.dxf.export_dxf_attribs(
             tagwriter,
             [
-                "version",
+                # write version tag (280, 0) here, if required in the future
                 "prompt",
                 "tag",
                 "flags",
@@ -466,14 +485,12 @@ class Attrib(BaseAttrib):
     DXFATTRIBS = DXFAttributes(base_class, acdb_entity, acdb_text, acdb_attrib)
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super(Text, self).load_dxf_attribs(processor)
         # Do not call Text loader.
         if processor:
-            processor.fast_load_dxfattribs(
-                dxf, acdb_text_group_codes, 2, recover=True
-            )
+            processor.fast_load_dxfattribs(dxf, acdb_text_group_codes, 2, recover=True)
             processor.fast_load_dxfattribs(
                 dxf, acdb_attrib_group_codes, 3, recover=True
             )
@@ -484,7 +501,7 @@ class Attrib(BaseAttrib):
                 elevation_to_z_axis(dxf, ("insert", "align_point"))
         return dxf
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         # Text() writes 2x AcDbText which is not suitable for AttDef()
         self.export_acdb_entity(tagwriter)
         self.export_acdb_attrib_text(tagwriter)
@@ -493,7 +510,7 @@ class Attrib(BaseAttrib):
             self.dxf.attribute_type = 2 if self.has_embedded_mtext_entity else 1
             self.export_dxf_r2018_features(tagwriter)
 
-    def export_acdb_attrib_text(self, tagwriter: "TagWriter") -> None:
+    def export_acdb_attrib_text(self, tagwriter: AbstractTagWriter) -> None:
         # Despite the similarities to TEXT, it is different to
         # Text.export_acdb_text():
         if tagwriter.dxfversion > const.DXF12:
@@ -516,13 +533,13 @@ class Attrib(BaseAttrib):
             ],
         )
 
-    def export_acdb_attrib(self, tagwriter: "TagWriter") -> None:
+    def export_acdb_attrib(self, tagwriter: AbstractTagWriter) -> None:
         if tagwriter.dxfversion > const.DXF12:
             tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_attrib.name)
         self.dxf.export_dxf_attribs(
             tagwriter,
             [
-                "version",
+                # write version tag (280, 0) here, if required in the future
                 "tag",
                 "flags",
                 "field_length",
@@ -588,13 +605,13 @@ class EmbeddedMText:
 
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         # Attribute "dxf" contains the DXF attributes defined in subclass
         # "AcDbMText"
         self.dxf = EmbeddedMTextNS()
         self.text: str = ""
 
-    def copy(self) -> "EmbeddedMText":
+    def copy(self) -> EmbeddedMText:
         copy_ = EmbeddedMText()
         copy_.dxf = copy.deepcopy(self.dxf)
         return copy_
@@ -645,7 +662,7 @@ class EmbeddedMText:
             if not dxf.hasattr(key):
                 dxf.set(key, default)
 
-    def export_dxf_tags(self, tagwriter: "TagWriter") -> None:
+    def export_dxf_tags(self, tagwriter: AbstractTagWriter) -> None:
         """Export embedded MTEXT as "Embedded Object"."""
         tagwriter.write_tag2(EMBEDDED_OBJ_MARKER, EMBEDDED_OBJ_STR)
         self.set_required_dxf_attributes()
@@ -680,3 +697,13 @@ class EmbeddedMText:
                 "bg_fill_transparency",
             ],
         )
+
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        if self.dxf.hasattr("style"):
+            registry.add_text_style(self.dxf.style)
+
+    def map_resources(self, clone: EmbeddedMText, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        if clone.dxf.hasattr("style"):
+            clone.dxf.style = mapping.get_text_style(clone.dxf.style)
