@@ -1,8 +1,9 @@
-# Copyright (c) 2019-2022, Manfred Moitzi
+# Copyright (c) 2019-2023, Manfred Moitzi
 # License: MIT License
+from __future__ import annotations
 from typing import TYPE_CHECKING, Optional
 import logging
-from ezdxf.audit import AuditError
+
 from ezdxf.lldxf import validator
 from ezdxf.lldxf.attributes import (
     DXFAttr,
@@ -21,21 +22,17 @@ from ezdxf.entities.dxfentity import base_class, SubclassProcessor, DXFEntity
 from ezdxf.entities.layer import acdb_symbol_table_record
 from .factory import register_entity
 
-logger = logging.getLogger("ezdxf")
-
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        TagWriter,
-        DXFNamespace,
-        DXFGraphic,
-        EntitySpace,
-        BlockLayout,
-        Block,
-        EndBlk,
-        Auditor,
-    )
+    from ezdxf.audit import Auditor
+    from ezdxf.entities import DXFGraphic, Block, EndBlk
+    from ezdxf.entities import DXFNamespace
+    from ezdxf.entitydb import EntitySpace
+    from ezdxf.layouts import BlockLayout
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf import xref
 
 __all__ = ["BlockRecord"]
+logger = logging.getLogger("ezdxf")
 
 acdb_blockrec = DefSubclass(
     "AcDbBlockTableRecord",
@@ -91,29 +88,27 @@ class BlockRecord(DXFEntity):
     """
 
     DXFTYPE = "BLOCK_RECORD"
-    DXFATTRIBS = DXFAttributes(
-        base_class, acdb_symbol_table_record, acdb_blockrec
-    )
+    DXFATTRIBS = DXFAttributes(base_class, acdb_symbol_table_record, acdb_blockrec)
 
-    def __init__(self):
+    def __init__(self) -> None:
         from ezdxf.entitydb import EntitySpace
 
         super().__init__()
         # Store entities in the block_record instead of BlockLayout and Layout,
         # because BLOCK_RECORD is also the hard owner of all the entities.
         self.entity_space = EntitySpace()
-        self.block: Optional["Block"] = None
-        self.endblk: Optional["EndBlk"] = None
+        self.block: Optional[Block] = None
+        self.endblk: Optional[EndBlk] = None
         # stores also the block layout structure
         self.block_layout: Optional[BlockLayout] = None
 
-    def set_block(self, block: "Block", endblk: "EndBlk"):
+    def set_block(self, block: Block, endblk: EndBlk):
         self.block = block
         self.endblk = endblk
         self.block.dxf.owner = self.dxf.handle
         self.endblk.dxf.owner = self.dxf.handle
 
-    def set_entity_space(self, entity_space: "EntitySpace") -> None:
+    def set_entity_space(self, entity_space: EntitySpace) -> None:
         self.entity_space = entity_space
 
     def rename(self, name: str) -> None:
@@ -122,14 +117,14 @@ class BlockRecord(DXFEntity):
         self.block.dxf.name = name
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             processor.simple_dxfattribs_loader(dxf, acdb_blockrec_group_codes)  # type: ignore
         return dxf
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         super().export_entity(tagwriter)
         if tagwriter.dxfversion == DXF12:
             raise DXFInternalEzdxfError("Exporting BLOCK_RECORDS for DXF R12.")
@@ -147,10 +142,10 @@ class BlockRecord(DXFEntity):
             ],
         )
 
-    def export_block_definition(self, tagwriter: "TagWriter") -> None:
-        """Exports BLOCK, than all DXF entities and at last the ENDBLK entity,
-        except for *Model_space and *Paper_Pacer, their entities are stored
-        in the entities section.
+    def export_block_definition(self, tagwriter: AbstractTagWriter) -> None:
+        """Exports the BLOCK entity, followed by all content entities and finally the
+        ENDBLK entity, except for the *Model_Space and *Paper_Space blocks, their
+        entities are stored in the ENTITIES section.
 
         """
         assert self.block is not None
@@ -162,6 +157,43 @@ class BlockRecord(DXFEntity):
             self.entity_space.export_dxf(tagwriter)
         self.endblk.export_dxf(tagwriter)
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        assert self.doc is not None, "BLOCK_RECORD entity must be assigned to document"
+        assert self.doc.entitydb is not None, "entity database required"
+        super().register_resources(registry)
+        key = self.dxf.handle
+        assert key in self.doc.entitydb, "invalid BLOCK_RECORD handle"
+
+        if self.block is not None:
+            registry.add_entity(self.block, block_key=key)
+        else:
+            raise DXFInternalEzdxfError(
+                f"BLOCK entity in BLOCK_RECORD #{key} is invalid"
+            )
+        if self.endblk is not None:
+            registry.add_entity(self.endblk, block_key=key)
+        else:
+            raise DXFInternalEzdxfError(
+                f"ENDBLK entity in BLOCK_RECORD #{key} is invalid"
+            )
+        for e in self.entity_space:
+            registry.add_entity(e, block_key=key)
+
+    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        assert isinstance(clone, BlockRecord)
+        super().map_resources(clone, mapping)
+
+        assert self.block is not None
+        mapping.map_resources_of_copy(self.block)
+
+        assert self.endblk is not None
+        mapping.map_resources_of_copy(self.endblk)
+
+        for entity in self.entity_space:
+            mapping.map_resources_of_copy(entity)
+
     def destroy(self):
         """Destroy associated data:
 
@@ -222,7 +254,7 @@ class BlockRecord(DXFEntity):
             return bool(self.block.dxf.flags & 12)
         return False
 
-    def add_entity(self, entity: "DXFGraphic") -> None:
+    def add_entity(self, entity: DXFGraphic) -> None:
         """Add an existing DXF entity to BLOCK_RECORD.
 
         Args:
@@ -231,18 +263,14 @@ class BlockRecord(DXFEntity):
         """
         # assign layout
         try:
-            entity.set_owner(
-                self.dxf.handle, paperspace=int(self.is_any_paperspace)
-            )
+            entity.set_owner(self.dxf.handle, paperspace=int(self.is_any_paperspace))
         except AttributeError:
-            logger.debug(
-                f"Unexpected DXF entity {str(entity)} in {str(self.block)}"
-            )
+            logger.debug(f"Unexpected DXF entity {str(entity)} in {str(self.block)}")
         # Add unexpected entities also to the entity space - auditor should fix
         # errors!
         self.entity_space.add(entity)
 
-    def unlink_entity(self, entity: "DXFGraphic") -> None:
+    def unlink_entity(self, entity: DXFGraphic) -> None:
         """Unlink `entity` from BLOCK_RECORD.
 
         Removes `entity` just from  entity space but not from the drawing
@@ -259,7 +287,7 @@ class BlockRecord(DXFEntity):
             except AttributeError:
                 pass  # unsupported entities as DXFTagStorage
 
-    def delete_entity(self, entity: "DXFGraphic") -> None:
+    def delete_entity(self, entity: DXFGraphic) -> None:
         """Delete `entity` from BLOCK_RECORD entity space and drawing database.
 
         Args:
@@ -269,30 +297,9 @@ class BlockRecord(DXFEntity):
         self.unlink_entity(entity)  # 1. unlink from entity space
         entity.destroy()
 
-    def audit(self, auditor: "Auditor") -> None:
+    def audit(self, auditor: Auditor) -> None:
         """Validity check. (internal API)"""
         if not self.is_alive:
             return
         super().audit(auditor)
-        entitydb = auditor.entitydb
-        rec_name = self.dxf.name
-        trash = []
-        for entity in self.entity_space:
-            if entity.is_alive:
-                check = entitydb.get(entity.dxf.handle)
-                if check is not entity:
-                    # Different entity stored in the database for this handle,
-                    # scenario #604:
-                    # - document has entities without handles (invalid for DXF R13+)
-                    # - $HANDSEED is not the next usable handle (dubious, causes error #1)
-                    # - entity gets an already used handle (loading error #1)
-                    # - entity overwrites existing entity or will be
-                    #   overwritten by an entity loaded afterwards (loading error #2)
-                    trash.append(entity)
-                    auditor.fixed_error(
-                        code=AuditError.REMOVED_ENTITY_FROM_BLOCK_RECORD,
-                        message=f"Removed invalid database entry {str(entity)}"
-                        f' from BLOCK_RECORD "{rec_name}".',
-                    )
-        for e in trash:
-            self.entity_space.remove(e)
+        self.entity_space.audit(auditor)
