@@ -1,7 +1,8 @@
-# Copyright (c) 2019-2021 Manfred Moitzi
+# Copyright (c) 2019-2022 Manfred Moitzi
 # License: MIT License
+from __future__ import annotations
+from typing import TYPE_CHECKING, Optional, Union, Iterable, Iterator
 import math
-from typing import TYPE_CHECKING, Optional, Union, Iterable
 import logging
 from ezdxf.audit import AuditError
 from ezdxf.lldxf import validator, const
@@ -37,17 +38,13 @@ from .factory import register_entity
 from .dimstyleoverride import DimStyleOverride
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        TagWriter,
-        DimStyle,
-        DXFNamespace,
-        BlockLayout,
-        OCS,
-        BaseLayout,
-        EntityQuery,
-        DXFEntity,
-        Auditor,
-    )
+    from ezdxf.entities import DXFNamespace, DXFEntity, DimStyle
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.layouts import BaseLayout, BlockLayout
+    from ezdxf.audit import Auditor
+    from ezdxf.query import EntityQuery
+    from ezdxf.math import OCS
+    from ezdxf import xref
 
 logger = logging.getLogger("ezdxf")
 ADSK_CONSTRAINTS = "*ADSK_CONSTRAINTS"
@@ -58,6 +55,7 @@ __all__ = [
     "RadialDimensionLarge",
     "OverrideMixin",
     "DXF_DIMENSION_TYPES",
+    "register_override_handles",
 ]
 
 DXF_DIMENSION_TYPES = {"DIMENSION", "ARC_DIMENSION", "LARGE_RADIAL_DIMENSION"}
@@ -85,9 +83,7 @@ acdb_dimension = DefSubclass(
         "text_midpoint": DXFAttr(11, xtype=XType.point3d),
         # Insertion point for clones of a dimension (Baseline and Continue?) (in OCS)
         # located in AcDbDimension? Another error in the DXF reference?
-        "insert": DXFAttr(
-            12, xtype=XType.point3d, default=NULLVEC, optional=True
-        ),
+        "insert": DXFAttr(12, xtype=XType.point3d, default=NULLVEC, optional=True),
         # Dimension type:
         # Important: Dimensional constraints do not have group code 70
         # Values 0–6 are integer values that represent the dimension type.
@@ -166,8 +162,8 @@ acdb_dimension = DefSubclass(
         ),
         # Dimension text explicitly entered by the user
         # default is the measurement.
-        # If null or “<>”, the dimension measurement is drawn as the text,
-        # if “ “ (one blank space), the text is suppressed.
+        # If null or "<>", the dimension measurement is drawn as the text,
+        # if " " (one blank space), the text is suppressed.
         # Anything else is drawn as the text.
         "text": DXFAttr(1, default="", optional=True),
         # Linear dimension types with an oblique angle have an optional group
@@ -229,7 +225,7 @@ acdb_dimension_dummy_group_codes = group_code_mapping(acdb_dimension_dummy)
 
 # noinspection PyUnresolvedReferences
 class OverrideMixin:
-    def get_dim_style(self) -> "DimStyle":
+    def get_dim_style(self) -> DimStyle:
         """Returns the associated :class:`DimStyle` entity."""
         assert self.doc is not None, "valid DXF document required"  # type: ignore
 
@@ -238,14 +234,12 @@ class OverrideMixin:
         # exist!
         return self.doc.dimstyles.get(dim_style_name)  # type: ignore
 
-    def dim_style_attributes(self) -> "DXFAttributes":
+    def dim_style_attributes(self) -> DXFAttributes:
         """Returns all valid DXF attributes (internal API)."""
         return self.get_dim_style().DXFATTRIBS
 
-    def dim_style_attr_names_to_handles(
-        self, data: dict, dxfversion: str
-    ) -> dict:
-        """`ezdxf` uses internally only resource names for arrows, line types
+    def dim_style_attr_names_to_handles(self, data: dict, dxfversion: str) -> dict:
+        """`Ezdxf` uses internally only resource names for arrows, linetypes
         and text styles, but DXF 2000 and later requires handles for these
         resources, this method translates resource names into related handles.
         (e.g. 'dimtxsty': 'FancyStyle' -> 'dimtxsty_handle', <handle of FancyStyle>)
@@ -282,9 +276,7 @@ class OverrideMixin:
             try:
                 ltype = self.doc.linetypes.get(linetype_name)
             except DXFTableEntryError:
-                logger.warning(
-                    f'Required line type "{linetype_name}" does not exist.'
-                )
+                logger.warning(f'Required line type "{linetype_name}" does not exist.')
             else:
                 data[attrib_name + "_handle"] = ltype.dxf.handle
 
@@ -352,7 +344,8 @@ class OverrideMixin:
                 code = dxf_attr.code
                 tags.append((1070, code))
                 if code == 5:
-                    # DimStyle 'dimblk' has group code 5 but is not a handle
+                    # DimStyle 'dimblk' has group code 5 but is not a handle, only used
+                    # for DXF R12
                     tags.append((1000, value))
                 else:
                     tags.append((get_xcode_for(code), value))
@@ -361,7 +354,7 @@ class OverrideMixin:
             self.set_xdata_list("ACAD", "DSTYLE", tags)  # type: ignore
 
     def dim_style_attr_handles_to_names(self, data: dict) -> dict:
-        """`ezdxf` uses internally only resource names for arrows, line types
+        """`Ezdxf` uses internally only resource names for arrows, line types
         and text styles, but DXF 2000 and later requires handles for these
         resources, this method translates resource handles into related names.
         (e.g. 'dimtxsty_handle', <handle of FancyStyle> -> 'dimtxsty': 'FancyStyle')
@@ -441,7 +434,7 @@ class OverrideMixin:
                 set_ltype_name(attrib_name, handle)
         return data
 
-    def get_acad_dstyle(self, dim_style: "DimStyle") -> dict:
+    def get_acad_dstyle(self, dim_style: DimStyle) -> dict:
         """Get XDATA section ACAD:DSTYLE, to override DIMSTYLE attributes for
         this DIMENSION entity. Returns a ``dict`` with DIMSTYLE attribute names
         as keys.
@@ -484,25 +477,23 @@ class Dimension(DXFGraphic, OverrideMixin):
     # WARNING for destroy() method:
     # Do not destroy associated anonymous block, if DIMENSION is used in a
     # block, the same geometry block maybe used by multiple block references.
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         # store the content of the geometry block for virtual entities
         self.virtual_block_content: Optional[EntitySpace] = None
 
-    def copy(self) -> "Dimension":
+    def copy(self) -> Dimension:
         virtual_copy = super().copy()
         # The new virtual copy can not reference the same geometry block as the
         # original dimension entity:
         virtual_copy.dxf.discard("geometry")
-        return virtual_copy
+        return virtual_copy  # type: ignore
 
-    def _copy_data(self, entity: "DXFEntity") -> None:
+    def copy_data(self, entity: DXFEntity) -> None:
         assert isinstance(entity, Dimension)
         if self.virtual_block_content:
             # another copy of a virtual entity:
-            virtual_content = EntitySpace(
-                e.copy() for e in self.virtual_block_content
-            )
+            virtual_content = EntitySpace(e.copy() for e in self.virtual_block_content)
         else:
             # entity is a new virtual copy of self and can not share the same
             # geometry block to be independently transformable:
@@ -512,10 +503,14 @@ class Dimension(DXFGraphic, OverrideMixin):
             entity.dxf.discard("insert")
         entity.virtual_block_content = virtual_content
 
+    def copy_external(self) -> Dimension:
+        # virtual content is not required
+        return self.raw_copy()  # type: ignore
+
     def post_bind_hook(self):
         """Called after binding a virtual dimension entity to a document.
 
-        This method is not called at the loading stage, but virtual dimension
+        This method is not called at the loading stage and virtual dimension
         entities do not exist at the loading stage!
 
         """
@@ -531,8 +526,8 @@ class Dimension(DXFGraphic, OverrideMixin):
             self.virtual_block_content = None
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             processor.fast_load_dxfattribs(
@@ -545,7 +540,7 @@ class Dimension(DXFGraphic, OverrideMixin):
             # content.
         return dxf
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         super().export_entity(tagwriter)
         if tagwriter.dxfversion == DXF12:
@@ -608,16 +603,12 @@ class Dimension(DXFGraphic, OverrideMixin):
 
         if dim_type == 0:  # linear
             tagwriter.write_tag2(SUBCLASS_MARKER, "AcDbAlignedDimension")
-            self.dxf.export_dxf_attribs(
-                tagwriter, ["defpoint2", "defpoint3", "angle"]
-            )
+            self.dxf.export_dxf_attribs(tagwriter, ["defpoint2", "defpoint3", "angle"])
             # empty but required subclass
             tagwriter.write_tag2(SUBCLASS_MARKER, "AcDbRotatedDimension")
         elif dim_type == 1:  # aligned
             tagwriter.write_tag2(SUBCLASS_MARKER, "AcDbAlignedDimension")
-            self.dxf.export_dxf_attribs(
-                tagwriter, ["defpoint2", "defpoint3", "angle"]
-            )
+            self.dxf.export_dxf_attribs(tagwriter, ["defpoint2", "defpoint3", "angle"])
         elif dim_type == 2:  # angular & angular3p
             tagwriter.write_tag2(SUBCLASS_MARKER, "AcDb2LineAngularDimension")
             self.dxf.export_dxf_attribs(
@@ -625,14 +616,10 @@ class Dimension(DXFGraphic, OverrideMixin):
             )
         elif dim_type == 3:  # diameter
             tagwriter.write_tag2(SUBCLASS_MARKER, "AcDbDiametricDimension")
-            self.dxf.export_dxf_attribs(
-                tagwriter, ["defpoint4", "leader_length"]
-            )
+            self.dxf.export_dxf_attribs(tagwriter, ["defpoint4", "leader_length"])
         elif dim_type == 4:  # radius
             tagwriter.write_tag2(SUBCLASS_MARKER, "AcDbRadialDimension")
-            self.dxf.export_dxf_attribs(
-                tagwriter, ["defpoint4", "leader_length"]
-            )
+            self.dxf.export_dxf_attribs(tagwriter, ["defpoint4", "leader_length"])
         elif dim_type == 5:  # angular & angular3p
             tagwriter.write_tag2(SUBCLASS_MARKER, "AcDb3PointAngularDimension")
             self.dxf.export_dxf_attribs(
@@ -642,6 +629,40 @@ class Dimension(DXFGraphic, OverrideMixin):
             tagwriter.write_tag2(SUBCLASS_MARKER, "AcDbOrdinateDimension")
             self.dxf.export_dxf_attribs(tagwriter, ["defpoint2", "defpoint3"])
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        assert self.doc is not None
+        super().register_resources(registry)
+        registry.add_dim_style(self.dxf.dimstyle)
+        geometry = self.dxf.geometry
+        if self.doc.block_records.has_entry(geometry):
+            registry.add_block_name(geometry)
+
+        if not self.has_xdata_list("ACAD", "DSTYLE"):
+            return
+
+        if self.doc.dxfversion > const.DXF12:
+            # overridden resources are referenced by handle
+            register_override_handles(self, registry)
+        else:
+            # overridden resources are referenced by name
+            self.override().register_resources_r12(registry)
+
+    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+        super().map_resources(clone, mapping)
+        clone.dxf.dimstyle = mapping.get_dim_style(self.dxf.dimstyle)
+        clone.dxf.geometry = mapping.get_block_name(self.dxf.geometry)
+
+        # DXF R2000+ references overridden resources by group code 1005 handles in the
+        # XDATA section, which are automatically mapped by the parent class DXFEntity!
+        assert self.doc is not None
+        if self.doc.dxfversion > const.DXF12:
+            return
+        self_override = self.override()
+        if not self_override.dimstyle_attribs:
+            return  # has no overrides
+        assert isinstance(clone, Dimension)
+        self_override.map_resources_r12(clone, mapping)
+
     @property
     def dimtype(self) -> int:
         """:attr:`dxf.dimtype` without binary flags (32, 62, 128)."""
@@ -652,18 +673,17 @@ class Dimension(DXFGraphic, OverrideMixin):
     # No information in the DXF reference:
     # layer name is "*ADSK_CONSTRAINTS"
     # missing group code 2 - geometry block name
-    # missing group code 70 - dimension type
     # has reactor to ACDBASSOCDEPENDENCY object
     # Autodesk example: architectural_example-imperial.dxf
     @property
     def is_dimensional_constraint(self) -> bool:
         """Returns ``True`` if the DIMENSION entity is a dimensional
-        constrains object.
+        constraint object.
         """
         dxf = self.dxf
-        return not dxf.hasattr("dimtype") and dxf.layer == ADSK_CONSTRAINTS
+        return dxf.layer == ADSK_CONSTRAINTS and not dxf.hasattr("geometry")
 
-    def get_geometry_block(self) -> Optional["BlockLayout"]:
+    def get_geometry_block(self) -> Optional[BlockLayout]:
         """Returns :class:`~ezdxf.layouts.BlockLayout` of associated anonymous
         dimension block, which contains the entities that make up the dimension
         picture. Returns ``None`` if block name is not set or the BLOCK itself
@@ -686,12 +706,14 @@ class Dimension(DXFGraphic, OverrideMixin):
         else:
             raise TypeError(f"Unknown DIMENSION type {self.dimtype}.")
 
-    def override(self) -> "DimStyleOverride":
+    def override(self) -> DimStyleOverride:
         """Returns the :class:`~ezdxf.entities.DimStyleOverride` object."""
         return DimStyleOverride(self)
 
     def render(self) -> None:
-        """Render graphical representation as anonymous block."""
+        """Renders the graphical representation of the DIMENSION entity as DXF
+        primitives (TEXT, LINE, ARC, ...) into an anonymous content BLOCK.
+        """
         if self.is_virtual:
             raise DXFTypeError("can not render virtual entity")
         # Do not delete existing anonymous block, it is maybe referenced
@@ -699,7 +721,7 @@ class Dimension(DXFGraphic, OverrideMixin):
         # references share the same geometry block!
         self.override().render()
 
-    def transform(self, m: "Matrix44") -> "Dimension":
+    def transform(self, m: Matrix44) -> Dimension:
         """Transform the DIMENSION entity by transformation matrix `m` inplace.
 
         Raises ``NonUniformScalingError()`` for non uniform scaling.
@@ -731,7 +753,7 @@ class Dimension(DXFGraphic, OverrideMixin):
         self.post_transform(m)
         return self
 
-    def _block_content(self) -> Iterable["DXFGraphic"]:
+    def _block_content(self) -> Iterable[DXFGraphic]:
         if self.virtual_block_content or self.is_virtual:
             content = self.virtual_block_content
         else:
@@ -745,13 +767,46 @@ class Dimension(DXFGraphic, OverrideMixin):
             except (NotImplementedError, NonUniformScalingError):
                 pass  # ignore transformation errors
 
-    def __virtual_entities__(self) -> Iterable["DXFGraphic"]:
+    def __virtual_entities__(self) -> Iterator[DXFGraphic]:
         """Implements the SupportsVirtualEntities protocol."""
+
+        def ocs_to_wcs(e: DXFGraphic, elevation: float):
+            # - OCS entities have to get the extrusion vector and the
+            #   elevation of the DIMENSION entity
+            # - WCS entities have to be transformed to the WCS
+            dxftype = e.dxftype()
+            dxf = e.dxf
+            if dxftype == "LINE":
+                dxf.start = ocs.to_wcs(dxf.start.replace(z=elevation))
+                dxf.end = ocs.to_wcs(dxf.end.replace(z=elevation))
+            elif dxftype == "MTEXT":
+                e.convert_rotation_to_text_direction()  # type: ignore
+                dxf.extrusion = ocs.uz
+                dxf.text_direction = ocs.to_wcs(dxf.text_direction)
+                dxf.insert = ocs.to_wcs(dxf.insert.replace(z=elevation))
+            elif dxftype == "POINT":
+                dxf.location = ocs.to_wcs(dxf.location.replace(z=elevation))
+            else:  # OCS entities
+                dxf.extrusion = ocs.uz
+                # set elevation:
+                if dxf.hasattr("insert"):  # INSERT, TEXT
+                    dxf.insert = dxf.insert.replace(z=elevation)
+                elif dxf.hasattr("center"):  # ARC, CIRCLE
+                    dxf.center = dxf.center.replace(z=elevation)
+                elif dxftype == "SOLID":
+                    # AutoCAD uses the SOLID entity to render the "solid fill"
+                    # arrow directly without using a block reference as usual. >:(
+                    for vtx_name in const.VERTEXNAMES:
+                        point = dxf.get(vtx_name, NULLVEC).replace(z=elevation)
+                        dxf.set(vtx_name, point)
+
+        ocs = self.ocs()
+        dim_elevation = self.dxf.text_midpoint.z
         transform = False
         insert = self.dxf.get("insert", None)
         if insert:
             transform = True
-            insert = self.ocs().to_wcs(insert)
+            insert = ocs.to_wcs(insert)
             m = Matrix44.translate(insert.x, insert.y, insert.z)
 
         for entity in self._block_content():
@@ -759,35 +814,40 @@ class Dimension(DXFGraphic, OverrideMixin):
                 copy = entity.copy()
             except DXFTypeError:
                 continue
+
+            if ocs.transform:
+                # All block content entities are located in the OCS defined by
+                # the DIMENSION entity, even the WCS entities LINE, MTEXT and
+                # POINT:
+                ocs_to_wcs(copy, dim_elevation)
+
             if transform:
                 # noinspection PyUnboundLocalVariable
                 copy.transform(m)
             yield copy
 
-    def virtual_entities(self) -> Iterable["DXFGraphic"]:
-        """
-        Yields 'virtual' parts of DIMENSION as basic DXF entities like LINE, ARC
-        or TEXT.
+    def virtual_entities(self) -> Iterator[DXFGraphic]:
+        """Yields the graphical representation of the anonymous content BLOCK as virtual
+        DXF primitives (LINE, ARC, TEXT, ...).
 
-        This entities are located at the original positions, but are not stored
-        in the entity database, have no handle and are not assigned to any
-        layout.
+        These virtual entities are located at the original location of the DIMENSION entity,
+        but they are not stored in the entity database, have no handle and are not
+        assigned to any layout.
 
         """
         return self.__virtual_entities__()
 
-    def explode(self, target_layout: "BaseLayout" = None) -> "EntityQuery":
-        """
-        Explode parts of DIMENSION as basic DXF entities like LINE, ARC or TEXT
-        into target layout, if target layout is ``None``, the target layout is
-        the layout of the DIMENSION.
+    def explode(self, target_layout: Optional[BaseLayout] = None) -> EntityQuery:
+        """Explodes the graphical representation of the DIMENSION entity as DXF
+        primitives (LINE, ARC, TEXT, ...) into the target layout, ``None`` for the same
+        layout as the source DIMENSION entity.
 
-        Returns an :class:`~ezdxf.query.EntityQuery` container with all DXF
+        Returns an :class:`~ezdxf.query.EntityQuery` container containing all DXF
         primitives.
 
         Args:
-            target_layout: target layout for DXF parts, ``None`` for same layout
-                as source entity.
+            target_layout: target layout for the DXF primitives, ``None`` for same
+                layout as source DIMENSION entity.
 
         """
         return explode_entity(self, target_layout)
@@ -807,12 +867,15 @@ class Dimension(DXFGraphic, OverrideMixin):
                     return (block.block_record_handle,)
         return tuple()
 
-    def audit(self, auditor: "Auditor") -> None:
+    def audit(self, auditor: Auditor) -> None:
         super().audit(auditor)
         doc = auditor.doc
         dxf = self.dxf
 
-        if dxf.get("geometry", "*") not in doc.blocks:
+        if (
+            not self.is_dimensional_constraint
+            and dxf.get("geometry", "*") not in doc.blocks
+        ):
             auditor.fixed_error(
                 code=AuditError.UNDEFINED_BLOCK,
                 message=f"Removed {str(self)} without valid geometry block.",
@@ -831,6 +894,13 @@ class Dimension(DXFGraphic, OverrideMixin):
         # check or repair. Ezdxf also ignores invalid XDATA overrides.
 
 
+def register_override_handles(entity: DXFEntity, registry: xref.Registry) -> None:
+    override_tags = entity.get_xdata_list("ACAD", "DSTYLE")
+    for code, value in override_tags:
+        if code == 1005:
+            registry.add_handle(value)
+
+
 acdb_arc_dimension = DefSubclass(
     "AcDbArcDimension",
     {
@@ -865,8 +935,8 @@ class ArcDimension(Dimension):
     MIN_DXF_VERSION_FOR_EXPORT = DXF2000
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         # Skip Dimension loader:
         dxf = super(Dimension, self).load_dxf_attribs(processor)
         if processor:
@@ -884,7 +954,7 @@ class ArcDimension(Dimension):
         else:
             return (self.dxf.dimtype & 0xFFF0) | 5
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         dimtype = self.dxf.dimtype  # preserve original dimtype
         self.dxf.dimtype = self.versioned_dimtype(tagwriter.dxfversion)
@@ -906,7 +976,7 @@ class ArcDimension(Dimension):
         )
         self.dxf.dimtype = dimtype  # restore original dimtype
 
-    def transform(self, m: "Matrix44") -> "Dimension":
+    def transform(self, m: Matrix44) -> Dimension:
         """Transform the ARC_DIMENSION entity by transformation matrix `m` inplace.
 
         Raises ``NonUniformScalingError()`` for non uniform scaling.
@@ -953,8 +1023,8 @@ class RadialDimensionLarge(Dimension):
     MIN_DXF_VERSION_FOR_EXPORT = DXF2004
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         # Skip Dimension loader:
         dxf = super(Dimension, self).load_dxf_attribs(processor)
         if processor:
@@ -966,7 +1036,7 @@ class RadialDimensionLarge(Dimension):
             )
         return dxf
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         super().export_entity(tagwriter)
         tagwriter.write_tag2(SUBCLASS_MARKER, "AcDbRadialDimensionLarge")
@@ -975,7 +1045,7 @@ class RadialDimensionLarge(Dimension):
             ["chord_point", "override_center", "jog_point", "unknown2"],
         )
 
-    def transform(self, m: "Matrix44") -> "Dimension":
+    def transform(self, m: Matrix44) -> Dimension:
         """Transform the LARGE_RADIAL_DIMENSION entity by transformation matrix
         `m` inplace.
 
@@ -989,7 +1059,7 @@ class RadialDimensionLarge(Dimension):
 
         dxf = self.dxf
         super().transform(m)
-        # todo: are this WCS points?
+        # todo: are these WCS points?
         for vertex_name in ("chord_point", "override_center", "Jog_point"):
             transform_if_exist(vertex_name, m.transform)
 
@@ -1126,7 +1196,7 @@ def angle_between(v1: Vec3, v2: Vec3) -> float:
 
 
 def linear_measurement(
-    p1: Vec3, p2: Vec3, angle: float = 0, ocs: "OCS" = None
+    p1: Vec3, p2: Vec3, angle: float = 0, ocs: Optional[OCS] = None
 ) -> float:
     """Returns distance from `p1` to `p2` projected onto ray defined by
     `angle`, `angle` in radians in the xy-plane.
