@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021 Manfred Moitzi
+# Copyright (c) 2019-2022 Manfred Moitzi
 # License: MIT License
 """ :class:`DXFEntity` is the super class of all DXF entities.
 
@@ -14,16 +14,15 @@ Ezdxf does no conversion between different DXF versions, this package is
 still not a CAD application.
 
 """
+from __future__ import annotations
 from typing import (
     TYPE_CHECKING,
-    List,
-    Dict,
     Any,
     Iterable,
+    Iterator,
     Optional,
     Type,
     TypeVar,
-    Set,
     Callable,
 )
 import copy
@@ -32,6 +31,7 @@ import uuid
 from ezdxf import options
 from ezdxf.lldxf import const
 from ezdxf.lldxf.tags import Tags
+from ezdxf.lldxf.types import DXFTag
 from ezdxf.lldxf.extendedtags import ExtendedTags
 from ezdxf.lldxf.attributes import DXFAttr, DXFAttributes, DefSubclass
 from ezdxf.tools import set_flag_state
@@ -41,19 +41,17 @@ from .dxfns import DXFNamespace, SubclassProcessor
 from .xdata import XData
 from .xdict import ExtensionDict
 
-logger = logging.getLogger("ezdxf")
-
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        Auditor,
-        TagWriter,
-        Drawing,
-        DXFAttr,
-        DXFGraphic,
-        Insert,
-    )
+    from ezdxf.audit import Auditor
+    from ezdxf.document import Drawing
+    from ezdxf.entities import DXFGraphic, Insert
+    from ezdxf.lldxf.attributes import DXFAttr
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.math import Matrix44
+    from ezdxf import xref
 
 __all__ = ["DXFEntity", "DXFTagStorage", "base_class", "SubclassProcessor"]
+logger = logging.getLogger("ezdxf")
 
 # Dynamic attributes created only at request:
 # Source entity of a copy or None if not a copy:
@@ -94,10 +92,10 @@ class DXFEntity:
     # really set, this means there is an real object in the dxf namespace
     # defined, where default attribute values get returned on access without
     # an existing object in the dxf namespace.
-    DEFAULT_ATTRIBS: Dict[str, Any] = {}
+    DEFAULT_ATTRIBS: dict[str, Any] = {}
     MIN_DXF_VERSION_FOR_EXPORT = const.DXF12
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Default constructor. (internal API)"""
         # Public attributes for package users
         self.doc: Optional[Drawing] = None
@@ -119,7 +117,7 @@ class DXFEntity:
 
     @property
     def uuid(self) -> uuid.UUID:
-        """Returns an UUID, which allows to distinguish even
+        """Returns a UUID, which allows to distinguish even
         virtual entities without a handle.
 
         Dynamic attribute: this UUID will be created at the first request.
@@ -134,10 +132,10 @@ class DXFEntity:
     @classmethod
     def new(
         cls: Type[T],
-        handle: str = None,
-        owner: str = None,
-        dxfattribs: Dict = None,
-        doc: "Drawing" = None,
+        handle: Optional[str] = None,
+        owner: Optional[str] = None,
+        dxfattribs=None,
+        doc: Optional[Drawing] = None,
     ) -> T:
         """Constructor for building new entities from scratch by ezdxf.
 
@@ -171,7 +169,7 @@ class DXFEntity:
         return entity
 
     def post_new_hook(self):
-        """Post processing and integrity validation after entity creation.
+        """Post-processing and integrity validation after entity creation.
 
         Called only if created by ezdxf (see :meth:`DXFEntity.new`),
         not if loaded from an external source.
@@ -181,7 +179,7 @@ class DXFEntity:
         pass
 
     def post_bind_hook(self):
-        """Post processing and integrity validation after binding entity to a
+        """Post-processing and integrity validation after binding entity to a
         DXF Document. This method is triggered by the :func:`factory.bind`
         function only when the entity was created by ezdxf.
 
@@ -197,7 +195,7 @@ class DXFEntity:
         pass
 
     @classmethod
-    def load(cls: Type[T], tags: ExtendedTags, doc: "Drawing" = None) -> T:
+    def load(cls: Type[T], tags: ExtendedTags, doc: Optional[Drawing] = None) -> T:
         """Constructor to generate entities loaded from an external source.
 
         LOAD process:
@@ -226,7 +224,7 @@ class DXFEntity:
         entity.load_tags(tags, dxfversion=dxfversion)
         return entity
 
-    def load_tags(self, tags: ExtendedTags, dxfversion: str = None) -> None:
+    def load_tags(self, tags: ExtendedTags, dxfversion: Optional[str] = None) -> None:
         """Generic tag loading interface, called if DXF document is loaded
         from external sources.
 
@@ -245,20 +243,18 @@ class DXFEntity:
                     self.xdata = XData(tags.xdata)
                 except const.DXFValueError:  # contains invalid group codes
                     self.xdata = XData.safe_init(tags.xdata)
-                    logger.debug(
-                        f"removed invalid XDATA from {tags.entity_name()}"
-                    )
+                    logger.debug(f"removed invalid XDATA from {tags.entity_name()}")
 
             processor = SubclassProcessor(tags, dxfversion=dxfversion)
             self.dxf = self.load_dxf_attribs(processor)
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
+        self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
         """Load DXF attributes into DXF namespace."""
         return DXFNamespace(processor, self)
 
-    def post_load_hook(self, doc: "Drawing") -> Optional[Callable]:
+    def post_load_hook(self, doc: Drawing) -> Optional[Callable]:
         """The 2nd loading stage when loading DXF documents from an external
         source, for the 1st loading stage see :meth:`load_tags`.
 
@@ -283,12 +279,12 @@ class DXFEntity:
         return None
 
     @classmethod
-    def from_text(cls: Type[T], text: str, doc: "Drawing" = None) -> T:
+    def from_text(cls: Type[T], text: str, doc: Optional[Drawing] = None) -> T:
         """Load constructor from text for testing. (internal API)"""
         return cls.load(ExtendedTags.from_text(text), doc)
 
     @classmethod
-    def shallow_copy(cls: Type[T], other: "DXFEntity") -> T:
+    def shallow_copy(cls: Type[T], other: DXFEntity) -> T:
         """Copy constructor for type casting e.g. Polyface and Polymesh.
         (internal API)
         """
@@ -304,19 +300,10 @@ class DXFEntity:
         # Do not set copy state, this is not a real copy!
         return entity
 
-    def copy(self: T) -> T:
-        """Returns a copy of `self` but without handle, owner and reactors.
-        This copy is NOT stored in the entity database and does NOT reside
-        in any layout, block, table or objects section!
-        The extension dictionary will be copied for entities bound to a valid
-        DXF document. The reactors are not copied.
+    def raw_copy(self: T) -> T:
+        """Returns a raw copy of `self` but without handle, owner and reactors.
 
-        Don't use this function to duplicate DXF entities in drawing,
-        use :meth:`EntityDB.duplicate_entity` instead for this task.
-
-        Copying is not trivial, because of linked resources and the lack of
-        documentation how to handle this linked resources: extension dictionary,
-        handles in appdata, xdata or embedded objects.
+        This is the first stage of the copy process, see copy() method.
 
         (internal API)
         """
@@ -343,17 +330,70 @@ class DXFEntity:
 
         # if xdata contains handles, they are treated as shared resources
         entity.xdata = copy.deepcopy(self.xdata)
-        entity.set_source_of_copy(self)
+        return entity
+
+    def copy_data(self, entity: DXFEntity) -> None:
+        """Copy entity data like vertices or attribs to the copy of the entity.
+
+        This is the second stage of the copy process, see copy() method.
+
+        (internal API)
+        """
+        pass
+
+    def copy(self: T) -> T:
+        """Internal entity copy for usage in the same document or as virtual entity.
+
+        Returns a copy of `self` but without handle, owner and reactors.
+        This copy is NOT stored in the entity database and does NOT reside
+        in any layout, block, table or objects section!
+        The extension dictionary will be copied for entities bound to a valid
+        DXF document. The reactors are not copied.
+
+        (internal API)
+        """
+        clone = self.raw_copy()
+        clone.set_source_of_copy(self)
         # DO NOT COPY DYN_SOURCE_BLOCK_REFERENCE_ATTRIBUTE.
         # Copying an entity from a block reference, takes it out of context of
         # this block reference!
-        self._copy_data(entity)
-        return entity
+        self.copy_data(clone)
+        return clone
 
-    def set_source_of_copy(self, source: Optional["DXFEntity"]):
+    def copy_external(self: T) -> T:
+        """External entity copy for usage in another document, this copy mode requires
+        registering and mapping of resources, see also:
+
+            - DXFEntity.register_resources()
+            - DXFEntity.map_resources()
+
+        Introduced for usage by the xref module.
+
+        (internal API)
+        """
+        clone = self.raw_copy()
+        # source of copy is not required
+        self.copy_data(clone)
+        return clone
+
+    def __deepcopy__(self, memodict: Optional[dict] = None):
+        """Some entities maybe linked by more than one entity, to be safe use
+        `memodict` for bookkeeping.
+
+        (internal API)
+        """
+        memodict = memodict or {}
+        try:
+            return memodict[id(self)]
+        except KeyError:
+            copy = self.copy()
+            memodict[id(self)] = copy
+            return copy
+
+    def set_source_of_copy(self, source: Optional[DXFEntity]):
         """Set immediate source entity of a copy.
 
-        Also used from outside of DFXEntity to set the source of sub-entities
+        Also used from outside to set the source of sub-entities
         of disassembled entities (POLYLINE, LWPOLYLINE, ...).
 
         (Internal API)
@@ -377,7 +417,7 @@ class DXFEntity:
         return self.source_of_copy is not None
 
     @property
-    def source_of_copy(self) -> Optional["DXFEntity"]:
+    def source_of_copy(self) -> Optional[DXFEntity]:
         """The immediate source entity if this entity is a copy else
         ``None``. Never references a destroyed entity.
         """
@@ -388,42 +428,18 @@ class DXFEntity:
         return source
 
     @property
-    def origin_of_copy(self) -> Optional["DXFEntity"]:
+    def origin_of_copy(self) -> Optional[DXFEntity]:
         """The origin source entity if this entity is a copy else
-        ``None``. References the first non virtual source entity and never
+        ``None``. References the first non-virtual source entity and never
         references a destroyed entity.
         """
         source = self.source_of_copy
-        # follow source entities references until the first non virtual entity:
-        while (
-            isinstance(source, DXFEntity)
-            and source.is_alive
-            and source.is_virtual
-        ):
+        # follow source entities references until the first non-virtual entity:
+        while isinstance(source, DXFEntity) and source.is_alive and source.is_virtual:
             source = source.source_of_copy
         return source
 
-    def _copy_data(self, entity: "DXFEntity") -> None:
-        """Copy entity data like vertices or attribs and store the copies into
-        the entity database.
-        (internal API)
-        """
-        pass
-
-    def __deepcopy__(self, memodict: Dict = None):
-        """Some entities maybe linked by more than one entity, to be safe use
-        `memodict` for bookkeeping.
-        (internal API)
-        """
-        memodict = memodict or {}
-        try:
-            return memodict[id(self)]
-        except KeyError:
-            copy = self.copy()
-            memodict[id(self)] = copy
-            return copy
-
-    def update_dxf_attribs(self, dxfattribs: Dict) -> None:
+    def update_dxf_attribs(self, dxfattribs: dict) -> None:
         """Set DXF attributes by a ``dict`` like :code:`{'layer': 'test',
         'color': 4}`.
         """
@@ -431,7 +447,7 @@ class DXFEntity:
         for key, value in dxfattribs.items():
             setter(key, value)
 
-    def setup_app_data(self, appdata: List[Tags]) -> None:
+    def setup_app_data(self, appdata: list[Tags]) -> None:
         """Setup data structures from APP data. (internal API)"""
         for data in appdata:
             code, appid = data[0]
@@ -450,32 +466,28 @@ class DXFEntity:
 
     @property
     def is_alive(self) -> bool:
-        """Is ``False`` if entity has been deleted.
-        """
+        """Is ``False`` if entity has been deleted."""
         return hasattr(self, "dxf")
 
     @property
     def is_virtual(self) -> bool:
-        """Is ``True`` if entity is a virtual entity.
-        """
+        """Is ``True`` if entity is a virtual entity."""
         return self.doc is None or self.dxf.handle is None
 
     @property
     def is_bound(self) -> bool:
-        """Is ``True`` if entity is bound to DXF document.
-        """
+        """Is ``True`` if entity is bound to DXF document."""
         if self.is_alive and not self.is_virtual:
             return factory.is_bound(self, self.doc)  # type: ignore
         return False
 
     @property
     def has_source_block_reference(self) -> bool:
-        """Is ``True`` if this virtual entity was created by a block reference.
-        """
+        """Is ``True`` if this virtual entity was created by a block reference."""
         return hasattr(self, DYN_SOURCE_BLOCK_REFERENCE_ATTRIBUTE)
 
     @property
-    def source_block_reference(self) -> Optional["Insert"]:
+    def source_block_reference(self) -> Optional[Insert]:
         """The source block reference (INSERT) which created
         this virtual entity. The property is ``None`` if this entity was not
         created by a block reference.
@@ -485,7 +497,7 @@ class DXFEntity:
             return blockref  # type: ignore
         return None
 
-    def set_source_block_reference(self, blockref: "Insert") -> None:
+    def set_source_block_reference(self, blockref: Insert) -> None:
         """Set the immediate source block reference which created this virtual
         entity.
 
@@ -585,7 +597,7 @@ class DXFEntity:
         """Returns a simple string representation including the class."""
         return str(self.__class__) + " " + str(self)
 
-    def dxfattribs(self, drop: Set[str] = None) -> Dict:
+    def dxfattribs(self, drop: Optional[set[str]] = None) -> dict:
         """Returns a ``dict`` with all existing DXF attributes and their
         values and exclude all DXF attributes listed in set `drop`.
 
@@ -599,20 +611,20 @@ class DXFEntity:
     def set_flag_state(
         self, flag: int, state: bool = True, name: str = "flags"
     ) -> None:
-        """Set binary coded `flag` of DXF attribute `name` to ``1`` (on)
-        if `state` is ``True``, set `flag` to ``0`` (off)
+        """Set binary coded `flag` of DXF attribute `name` to 1 (on)
+        if `state` is ``True``, set `flag` to 0 (off)
         if `state` is ``False``.
         """
         flags = self.dxf.get(name, 0)
         self.dxf.set(name, set_flag_state(flags, flag, state=state))
 
     def get_flag_state(self, flag: int, name: str = "flags") -> bool:
-        """Returns ``True`` if any `flag` of DXF attribute is ``1`` (on), else
+        """Returns ``True`` if any `flag` of DXF attribute is 1 (on), else
         ``False``. Always check only one flag state at the time.
         """
         return bool(self.dxf.get(name, 0) & flag)
 
-    def remove_dependencies(self, other: "Drawing" = None):
+    def remove_dependencies(self, other: Optional[Drawing] = None):
         """Remove all dependencies from current document.
 
         Intended usage is to remove dependencies from the current document to
@@ -620,7 +632,7 @@ class DXFEntity:
 
         An error free call of this method does NOT guarantee that this entity
         can be moved/copied to the `other` document, some entities like
-        DIMENSION have too much dependencies to a document to move or copy
+        DIMENSION have too many dependencies to a document to move or copy
         them, but to check this is not the domain of this method!
 
         (internal API)
@@ -640,11 +652,7 @@ class DXFEntity:
         """Delete all data and references. Does not delete entity from
         structures like layouts or groups.
 
-        Starting with `ezdxf` v0.14 this method could be used to delete
-        entities.
-
         (internal API)
-
         """
         if not self.is_alive:
             return
@@ -661,16 +669,32 @@ class DXFEntity:
         self.del_source_of_copy()
         self.del_source_block_reference()
 
-    def preprocess_export(self, tagwriter: "TagWriter") -> bool:
-        """Pre requirement check and pre processing for export.
+    def _silent_kill(self):  # final - do not override this method!
+        """Delete entity but not the referenced content!
 
-        Returns False if entity should not be exported at all.
+        DANGER! DON'T USE THIS METHOD!
+
+        (internal API)
+        """
+        if not self.is_alive:
+            return
+        del self.extension_dict
+        del self.appdata
+        del self.reactors
+        del self.xdata
+        del self.doc
+        del self.dxf  # check mark for is_alive
+
+    def preprocess_export(self, tagwriter: AbstractTagWriter) -> bool:
+        """Pre requirement check and pre-processing for export.
+
+        Returns ``False``  if entity should not be exported at all.
 
         (internal API)
         """
         return True
 
-    def export_dxf(self, tagwriter: "TagWriter") -> None:
+    def export_dxf(self, tagwriter: AbstractTagWriter) -> None:
         """Export DXF entity by `tagwriter`.
 
         This is the first key method for exporting DXF entities:
@@ -696,7 +720,7 @@ class DXFEntity:
         # write xdata at the end of the entity
         self.export_xdata(tagwriter)
 
-    def export_base_class(self, tagwriter: "TagWriter") -> None:
+    def export_base_class(self, tagwriter: AbstractTagWriter) -> None:
         """Export base class DXF attributes and structures. (internal API)"""
         dxftype = self.DXFTYPE
         _handle_code = 105 if dxftype == "DIMSTYLE" else 5
@@ -717,7 +741,7 @@ class DXFEntity:
                 tagwriter.write_tag2(_handle_code, self.dxf.handle)
                 # do not write owner handle - not supported by DXF R12
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export DXF entity specific data by `tagwriter`.
 
         This is the second key method for exporting DXF entities:
@@ -732,12 +756,12 @@ class DXFEntity:
         pass
         # xdata and embedded objects  export is also done by parent
 
-    def export_xdata(self, tagwriter: "TagWriter") -> None:
+    def export_xdata(self, tagwriter: AbstractTagWriter) -> None:
         """Export DXF XDATA by `tagwriter`. (internal API)"""
         if self.xdata:
             self.xdata.export_dxf(tagwriter)
 
-    def audit(self, auditor: "Auditor") -> None:
+    def audit(self, auditor: Auditor) -> None:
         """Validity check. (internal API)"""
         # Important: do not check owner handle! -> DXFGraphic(), DXFObject()
         # check app data
@@ -756,7 +780,7 @@ class DXFEntity:
             return xdict.dictionary.is_alive
         return False
 
-    def get_extension_dict(self) -> "ExtensionDict":
+    def get_extension_dict(self) -> ExtensionDict:
         """Returns the existing :class:`~ezdxf.entities.xdict.ExtensionDict`
         instance.
 
@@ -769,7 +793,7 @@ class DXFEntity:
         else:
             raise AttributeError("Entity has no extension dictionary.")
 
-    def new_extension_dict(self) -> "ExtensionDict":
+    def new_extension_dict(self) -> ExtensionDict:
         """Create a new :class:`~ezdxf.entities.xdict.ExtensionDict` instance ."""
         assert self.doc is not None
         xdict = ExtensionDict.new(self.dxf.handle, self.doc)
@@ -932,7 +956,7 @@ class DXFEntity:
         """Returns ``True`` if entity has reactors."""
         return bool(self.reactors)
 
-    def get_reactors(self) -> List[str]:
+    def get_reactors(self) -> list[str]:
         """Returns associated reactors as list of handles."""
         return self.reactors.get() if self.reactors else []
 
@@ -955,22 +979,66 @@ class DXFEntity:
         if self.reactors:
             self.reactors.discard(handle)
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        if self.xdata:
+            for name in self.xdata.data.keys():
+                registry.add_appid(name)
+        if self.appdata:  # add hard owned entities
+            for tags in self.appdata.tags():
+                for tag in tags.get_hard_owner_handles():
+                    registry.add_handle(tag.value)
+
+    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+
+        def map_xdata_resources():
+            for index, (code, value) in enumerate(tags):
+                if code == 1005:  # map soft-pointer handles
+                    tags[index] = DXFTag(code, mapping.get_handle(value))
+                elif code == 1003:  # map layer name
+                    tags[index] = DXFTag(code, mapping.get_layer(value))
+
+        if clone.xdata:
+            for tags in clone.xdata.data.values():
+                map_xdata_resources()
+
+        if clone.appdata:
+            for tags in clone.appdata.tags():
+                mapping.map_pointers(tags, new_owner_handle=clone.dxf.handle)
+
+        if clone.extension_dict:
+            assert self.extension_dict is not None
+            self.extension_dict.dictionary.map_resources(
+                clone.extension_dict.dictionary, mapping
+            )
+        # reactors are not copied automatically, clone.reactors is always None:
+        if self.reactors:
+            # reactors are soft-pointers (group code 330)
+            mapped_handles = [mapping.get_handle(h) for h in self.reactors.reactors]
+            mapped_handles = [h for h in mapped_handles if h != "0"]
+            if mapped_handles:
+                clone.set_reactors(mapped_handles)
+
 
 @factory.set_default_class
 class DXFTagStorage(DXFEntity):
     """Just store all the tags as they are. (internal class)"""
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Default constructor"""
         super().__init__()
         self.xtags = ExtendedTags()
-        self.embedded_objects: Optional[List[Tags]] = None
+        self.embedded_objects: Optional[list[Tags]] = None
 
-    def copy(self: T) -> T:
+    def raw_copy(self: T) -> T:
         raise const.DXFTypeError(
             f"Cloning of tag storage {self.dxftype()} not supported."
         )
 
+    def transform(self, m: Matrix44) -> DXFGraphic:
+        raise NotImplementedError("cannot transform DXF tag storage")
+
     @property
     def base_class(self):
         return self.xtags.subclasses[0]
@@ -984,8 +1052,31 @@ class DXFTagStorage(DXFEntity):
         """
         return self.xtags.has_subclass("AcDbEntity")
 
+    def graphic_properties(self) -> dict[str, Any]:
+        """Returns the graphical properties like layer, color, linetype, ... as
+        `dxfattribs` dict if the :class:`TagStorage` object represents a graphical
+        entity otherwise returns an empty dictionary.
+
+        These are all the DXF attributes which are stored in the ``AcDbEntity``
+        subclass.
+
+        """
+        from ezdxf.entities.dxfgfx import acdb_entity_group_codes
+
+        attribs: dict[str, Any] = dict()
+        try:
+            tags = self.xtags.get_subclass("AcDbEntity")
+        except const.DXFKeyError:
+            return attribs
+
+        for code, value in tags:
+            attrib_name = acdb_entity_group_codes.get(code)
+            if isinstance(attrib_name, str):
+                attribs[attrib_name] = value
+        return attribs
+
     @classmethod
-    def load(cls, tags: ExtendedTags, doc: "Drawing" = None) -> "DXFTagStorage":
+    def load(cls, tags: ExtendedTags, doc: Optional[Drawing] = None) -> DXFTagStorage:
         assert isinstance(tags, ExtendedTags)
         entity = cls.new(doc=doc)
         dxfversion = doc.dxfversion if doc else None
@@ -1020,7 +1111,7 @@ class DXFTagStorage(DXFEntity):
     def store_embedded_objects(self, tags: ExtendedTags) -> None:
         self.embedded_objects = tags.embedded_objects
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Write subclass tags as they are."""
         for subclass in self.xtags.subclasses[1:]:
             tagwriter.write_tags(subclass)
@@ -1036,18 +1127,16 @@ class DXFTagStorage(DXFEntity):
         del self.xtags
         super().destroy()
 
-    def __virtual_entities__(self) -> Iterable["DXFGraphic"]:
+    def __virtual_entities__(self) -> Iterator[DXFGraphic]:
         """Implements the SupportsVirtualEntities protocol."""
         from ezdxf.proxygraphic import ProxyGraphic
 
         if self.proxy_graphic:
-            for e in ProxyGraphic(
-                self.proxy_graphic, self.doc
-            ).virtual_entities():
+            for e in ProxyGraphic(self.proxy_graphic, self.doc).virtual_entities():
                 e.set_source_of_copy(self)
                 yield e
         return []
 
-    def virtual_entities(self) -> Iterable["DXFGraphic"]:
+    def virtual_entities(self) -> Iterator[DXFGraphic]:
         """Yields proxy graphic as "virtual" entities."""
         return self.__virtual_entities__()
