@@ -1,16 +1,14 @@
-# Copyright (c) 2019-2022 Manfred Moitzi
+# Copyright (c) 2019-2023 Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
 from typing import (
     TYPE_CHECKING,
     Iterable,
+    Iterator,
     cast,
-    Tuple,
     Union,
     Optional,
     Callable,
-    Dict,
-    List,
 )
 import math
 from ezdxf.lldxf import validator
@@ -37,11 +35,13 @@ from ezdxf.math import (
     Y_AXIS,
     Z_AXIS,
     Matrix44,
-    OCS,
     UCS,
     NULLVEC,
 )
-from ezdxf.math.transformtools import OCSTransform, InsertTransformationError
+from ezdxf.math.transformtools import (
+    InsertTransformationError,
+    InsertCoordinateSystem,
+)
 from ezdxf.explode import (
     explode_block_reference,
     virtual_block_reference_entities,
@@ -60,15 +60,11 @@ from .subentity import LinkedEntities
 from .attrib import Attrib
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        TagWriter,
-        DXFNamespace,
-        AttDef,
-        BlockLayout,
-        BaseLayout,
-        Auditor,
-        BlockRecord,
-    )
+    from ezdxf.audit import Auditor
+    from ezdxf.entities import DXFNamespace, AttDef, DXFEntity
+    from ezdxf.layouts import BaseLayout, BlockLayout
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf import xref
 
 __all__ = ["Insert"]
 
@@ -146,11 +142,6 @@ merged_insert_group_codes = merge_group_code_mappings(
     acdb_entity_group_codes, acdb_block_reference_group_codes  # type: ignore
 )
 
-NON_ORTHO_MSG = (
-    "INSERT entity can not represent a non-orthogonal target "
-    "coordinate system."
-)
-
 
 # Notes to SEQEND:
 #
@@ -161,19 +152,31 @@ NON_ORTHO_MSG = (
 # ATTRIB entities, but the LinkedEntities.post_bind_hook() method creates
 # always a new SEQEND after binding the INSERT entity to a document.
 #
-# Nonetheless the Insert.add_attrib() method also creates a requires SEQEND if
+# Nonetheless, the Insert.add_attrib() method also creates the required SEQEND entity if
 # necessary.
 
 
 @factory.register_entity
 class Insert(LinkedEntities):
-    """DXF INSERT entity"""
+    """DXF INSERT entity
+
+    The INSERT entity is hard owner of its ATTRIB entities and the SEQEND entity:
+
+        ATTRIB.dxf.owner == INSERT.dxf.handle
+        SEQEND.dxf.owner == INSERT.dxf.handle
+
+    Note:
+
+        The ATTDEF entity in block definitions is owned by the BLOCK_RECORD like
+        all graphical entities.
+
+    """
 
     DXFTYPE = "INSERT"
     DXFATTRIBS = DXFAttributes(base_class, acdb_entity, acdb_block_reference)
 
     @property
-    def attribs(self) -> List[Attrib]:
+    def attribs(self) -> list[Attrib]:
         return self._sub_entities  # type: ignore
 
     @property
@@ -181,7 +184,7 @@ class Insert(LinkedEntities):
         return bool(len(self.attribs))
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
+        self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
         """Loading interface. (internal API)"""
         # bypass DXFGraphic, loading proxy graphic is skipped!
@@ -193,7 +196,7 @@ class Insert(LinkedEntities):
                 elevation_to_z_axis(dxf, ("insert",))
         return dxf
 
-    def export_entity(self, tagwriter: TagWriter) -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         super().export_entity(tagwriter)
         if tagwriter.dxfversion > DXF12:
@@ -220,15 +223,25 @@ class Insert(LinkedEntities):
             ],
         )
 
-    def export_dxf(self, tagwriter: TagWriter):
+    def export_dxf(self, tagwriter: AbstractTagWriter):
         super().export_dxf(tagwriter)
         # Do no export SEQEND if no ATTRIBS attached:
         if self.attribs_follow:
             self.process_sub_entities(lambda e: e.export_dxf(tagwriter))
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        # The attached ATTRIB entities are registered by the parent class LinkedEntities
+        super().register_resources(registry)
+        registry.add_block_name(self.dxf.name)
+
+    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+        # The attached ATTRIB entities are mapped by the parent class LinkedEntities
+        super().map_resources(clone, mapping)
+        clone.dxf.name = mapping.get_block_name(self.dxf.name)
+
     @property
     def has_scaling(self) -> bool:
-        """Returns ``True`` if any axis scaling is applied."""
+        """Returns ``True`` if scaling is applied to any axis."""
         if self.dxf.hasattr("xscale") and self.dxf.xscale != 1:
             return True
         if self.dxf.hasattr("yscale") and self.dxf.yscale != 1:
@@ -239,31 +252,29 @@ class Insert(LinkedEntities):
 
     @property
     def has_uniform_scaling(self) -> bool:
-        """Returns ``True`` if scaling is uniform in x-, y- and z-axis ignoring
-        reflections e.g. (1, 1, -1) is uniform scaling.
+        """Returns ``True`` if the scale factor is uniform for x-, y- and z-axis,
+        ignoring reflections e.g. (1, 1, -1) is uniform scaling.
 
         """
-        return (
-            abs(self.dxf.xscale) == abs(self.dxf.yscale) == abs(self.dxf.zscale)
-        )
+        return abs(self.dxf.xscale) == abs(self.dxf.yscale) == abs(self.dxf.zscale)
 
     def set_scale(self, factor: float):
-        """Set uniform scaling."""
+        """Set a uniform scale factor."""
         if factor == 0:
-            raise ValueError("Invalid scaling factor.")
+            raise ValueError("Invalid scale factor.")
         self.dxf.xscale = factor
         self.dxf.yscale = factor
         self.dxf.zscale = factor
         return self
 
     def block(self) -> Optional[BlockLayout]:
-        """Returns associated :class:`~ezdxf.layouts.BlockLayout`."""
+        """Returns the associated :class:`~ezdxf.layouts.BlockLayout`."""
         if self.doc:
             return self.doc.blocks.get(self.dxf.name)
         return None
 
     def is_xref(self) -> bool:
-        """Return ``True`` if XREF or XREF_OVERLAY."""
+        """Return ``True`` if the INSERT entity represents a XREF or XREF_OVERLAY."""
         block = self.block()
         if block is not None:
             return block.block_record.is_xref
@@ -271,17 +282,17 @@ class Insert(LinkedEntities):
 
     def place(
         self,
-        insert: UVec = None,
-        scale: Tuple[float, float, float] = None,
-        rotation: float = None,
+        insert: Optional[UVec] = None,
+        scale: Optional[tuple[float, float, float]] = None,
+        rotation: Optional[float] = None,
     ) -> Insert:
         """
-        Set block reference placing location `insert`, scaling and rotation
-        attributes. Parameters which are ``None`` will not be altered.
+        Set the location, scaling and rotation attributes. Arguments which are ``None``
+        will be ignored.
 
         Args:
-            insert: insert location as ``(x, y [,z])`` tuple
-            scale: ``(x-scale, y-scale, z-scale)`` tuple
+            insert: insert location as (x, y [,z]) tuple
+            scale: (x-scale, y-scale, z-scale) tuple
             rotation : rotation angle in degrees
 
         """
@@ -289,9 +300,7 @@ class Insert(LinkedEntities):
             self.dxf.insert = insert
         if scale is not None:
             if len(scale) != 3:
-                raise DXFValueError(
-                    "Parameter scale has to be a (x, y, z)-tuple."
-                )
+                raise DXFValueError("Argument scale has to be a (x, y[, z]) tuple.")
             x, y, z = scale
             self.dxf.xscale = x
             self.dxf.yscale = y
@@ -301,33 +310,28 @@ class Insert(LinkedEntities):
         return self
 
     def grid(
-        self,
-        size: Tuple[int, int] = (1, 1),
-        spacing: Tuple[float, float] = (1, 1),
+        self, size: tuple[int, int] = (1, 1), spacing: tuple[float, float] = (1, 1)
     ) -> Insert:
         """Place block reference in a grid layout, grid `size` defines the
         row- and column count, `spacing` defines the distance between two block
         references.
 
         Args:
-            size: grid size as ``(row_count, column_count)`` tuple
-            spacing: distance between placing as
-                ``(row_spacing, column_spacing)`` tuple
+            size: grid size as (row_count, column_count) tuple
+            spacing: distance between placing as (row_spacing, column_spacing) tuple
 
         """
         try:
             rows, cols = size
         except ValueError:
-            raise DXFValueError(
-                "Size has to be a 2-tuple: (row_count, column_count)."
-            )
+            raise DXFValueError("Size has to be a (row_count, column_count) tuple.")
         self.dxf.row_count = rows
         self.dxf.column_count = cols
         try:
             row_spacing, col_spacing = spacing
         except ValueError:
             raise DXFValueError(
-                "Spacing has to be a 2-tuple: (row_spacing, column_spacing)."
+                "Spacing has to be a (row_spacing, column_spacing) tuple."
             )
         self.dxf.row_spacing = row_spacing
         self.dxf.column_spacing = col_spacing
@@ -336,13 +340,13 @@ class Insert(LinkedEntities):
     def get_attrib(
         self, tag: str, search_const: bool = False
     ) -> Optional[Union[Attrib, AttDef]]:
-        """Get attached :class:`Attrib` entity with :code:`dxf.tag == tag`,
-        returns ``None`` if not found. Some applications may not attach constant
+        """Get an attached :class:`Attrib` entity with the given `tag`,
+        returns ``None`` if not found.  Some applications do not attach constant
         ATTRIB entities, set `search_const` to ``True``, to get at least the
         associated :class:`AttDef` entity.
 
         Args:
-            tag: tag name
+            tag: tag name of the ATTRIB entity
             search_const: search also const ATTDEF entities
 
         """
@@ -359,14 +363,14 @@ class Insert(LinkedEntities):
     def get_attrib_text(
         self, tag: str, default: str = "", search_const: bool = False
     ) -> str:
-        """Get content text of attached :class:`Attrib` entity with
-        :code:`dxf.tag == tag`, returns `default` if not found.
-        Some applications may not attach constant ATTRIB entities, set
+        """Get content text of an attached :class:`Attrib` entity with
+        the given `tag`, returns the `default` value if not found.
+        Some applications do not attach constant ATTRIB entities, set
         `search_const` to ``True``, to get content text of the
         associated :class:`AttDef` entity.
 
         Args:
-            tag: tag name
+            tag: tag name of the ATTRIB entity
             default: default value if ATTRIB `tag` is absent
             search_const: search also const ATTDEF entities
 
@@ -377,36 +381,34 @@ class Insert(LinkedEntities):
         return attrib.dxf.text
 
     def has_attrib(self, tag: str, search_const: bool = False) -> bool:
-        """Returns ``True`` if ATTRIB `tag` exist, for `search_const` doc see
-        :meth:`get_attrib`.
+        """Returns ``True`` if the INSERT entity has an attached ATTRIB entity with the
+        given `tag`.  Some applications do not attach constant ATTRIB entities, set
+        `search_const` to ``True``, to check for an associated :class:`AttDef` entity
+        with constant content.
+
 
         Args:
-            tag: tag name as string
+            tag: tag name fo the ATTRIB entity
             search_const: search also const ATTDEF entities
 
         """
         return self.get_attrib(tag, search_const) is not None
 
     def add_attrib(
-        self,
-        tag: str,
-        text: str,
-        insert: UVec = (0, 0),
-        dxfattribs=None,
-    ) -> "Attrib":
+        self, tag: str, text: str, insert: UVec = (0, 0), dxfattribs=None
+    ) -> Attrib:
         """Attach an :class:`Attrib` entity to the block reference.
 
-        Example for appending an attribute to an INSERT entity with none
-        standard alignment::
+        Example for appending an attribute to an INSERT entity::
 
             e.add_attrib('EXAMPLETAG', 'example text').set_placement(
                 (3, 7), align=TextEntityAlignment.MIDDLE_CENTER
             )
 
         Args:
-            tag: tag name as string
+            tag: tag name of the ATTRIB entity
             text: content text as string
-            insert: insert location as tuple ``(x, y[, z])`` in :ref:`WCS`
+            insert: insert location as (x, y[, z]) tuple in :ref:`OCS`
             dxfattribs: additional DXF attributes for the ATTRIB entity
 
         """
@@ -417,24 +419,24 @@ class Insert(LinkedEntities):
         attrib = cast("Attrib", self._new_compound_entity("ATTRIB", dxfattribs))
         self.attribs.append(attrib)
 
-        # This case is only possible if INSERT is read from file without
-        # attached ATTRIBS:
+        # This case is only possible if the INSERT was read from a file without
+        # attached ATTRIB entities:
         if self.seqend is None:
             self.new_seqend()
         return attrib
 
     def delete_attrib(self, tag: str, ignore=False) -> None:
-        """Delete an attached :class:`Attrib` entity from INSERT. If `ignore`
-        is ``False``, an :class:`DXFKeyError` exception is raised, if
-        ATTRIB `tag` does not exist.
+        """Delete an attached :class:`Attrib` entity from INSERT. Raises an
+        :class:`DXFKeyError` exception, if no ATTRIB for the given `tag` exist if
+        `ignore` is ``False``.
 
         Args:
-            tag: ATTRIB name
+            tag: tag name of the ATTRIB entity
             ignore: ``False`` for raising :class:`DXFKeyError` if ATTRIB `tag`
                 does not exist.
 
         Raises:
-            DXFKeyError: if ATTRIB `tag` does not exist.
+            DXFKeyError: no ATTRIB for the given `tag` exist
 
         """
         for index, attrib in enumerate(self.attribs):
@@ -458,47 +460,29 @@ class Insert(LinkedEntities):
         """Transform INSERT entity by transformation matrix `m` inplace.
 
         Unlike the transformation matrix `m`, the INSERT entity can not
-        represent a non-orthogonal target coordinate system, for this case an
-        :class:`InsertTransformationError` will be raised.
+        represent a non-orthogonal target coordinate system and an
+        :class:`InsertTransformationError` will be raised in that case.
 
         """
-
         dxf = self.dxf
-        ocs = self.ocs()
-
-        # Transform source OCS axis into the target coordinate system:
-        ux, uy, uz = m.transform_directions((ocs.ux, ocs.uy, ocs.uz))
-
-        # Calculate new axis scaling factors:
-        x_scale = ux.magnitude * dxf.xscale
-        y_scale = uy.magnitude * dxf.yscale
-        z_scale = uz.magnitude * dxf.zscale
-
-        ux = ux.normalize()
-        uy = uy.normalize()
-        uz = uz.normalize()
-        # check for orthogonal x-, y- and z-axis
-        if (
-            abs(ux.dot(uz)) > ABS_TOL
-            or abs(ux.dot(uy)) > ABS_TOL
-            or abs(uz.dot(uy)) > ABS_TOL
-        ):
-            raise InsertTransformationError(NON_ORTHO_MSG)
-
-        # expected y-axis for an orthogonal right handed coordinate system:
-        expected_uy = uz.cross(ux)
-        if not expected_uy.isclose(uy, abs_tol=ABS_TOL):
-            # new y-axis points into opposite direction:
-            y_scale = -y_scale
-
-        ocs_transform = OCSTransform.from_ocs(OCS(dxf.extrusion), OCS(uz), m)
-        dxf.insert = ocs_transform.transform_vertex(dxf.insert)
-        dxf.rotation = ocs_transform.transform_deg_angle(dxf.rotation)
-
-        dxf.extrusion = uz
-        dxf.xscale = x_scale
-        dxf.yscale = y_scale
-        dxf.zscale = z_scale
+        source_system = InsertCoordinateSystem(
+            insert=Vec3(dxf.insert),
+            scale=(dxf.xscale, dxf.yscale, dxf.zscale),
+            rotation=dxf.rotation,
+            extrusion=dxf.extrusion,
+        )
+        try:
+            target_system = source_system.transform(m, ABS_TOL)
+        except InsertTransformationError:
+            raise InsertTransformationError(
+                "INSERT entity can not represent a non-orthogonal target coordinate system."
+            )
+        dxf.insert = target_system.insert
+        dxf.rotation = target_system.rotation
+        dxf.extrusion = target_system.extrusion
+        dxf.xscale = target_system.scale_factor_x
+        dxf.yscale = target_system.scale_factor_y
+        dxf.zscale = target_system.scale_factor_z
 
         for attrib in self.attribs:
             attrib.transform(m)
@@ -511,17 +495,14 @@ class Insert(LinkedEntities):
 
         """
         ocs = self.ocs()
-        self.dxf.insert = ocs.from_wcs(
-            Vec3(dx, dy, dz) + ocs.to_wcs(self.dxf.insert)
-        )
+        self.dxf.insert = ocs.from_wcs(Vec3(dx, dy, dz) + ocs.to_wcs(self.dxf.insert))
         for attrib in self.attribs:
             attrib.translate(dx, dy, dz)
         return self
 
     def matrix44(self) -> Matrix44:
-        """Returns a transformation matrix of type :class:`Matrix44` to
-        transform the block entities into :ref:`WCS`.
-
+        """Returns a transformation matrix to transform the block entities from the
+        block reference coordinate system into the :ref:`WCS`.
         """
         dxf = self.dxf
         sx = dxf.xscale
@@ -551,8 +532,8 @@ class Insert(LinkedEntities):
         return m
 
     def ucs(self):
-        """Returns the block reference coordinate system as
-        :class:`ezdxf.math.UCS` object.
+        """Returns the block reference coordinate system as :class:`ezdxf.math.UCS`
+        object.
         """
         m = self.matrix44()
         ucs = UCS()
@@ -560,8 +541,8 @@ class Insert(LinkedEntities):
         return ucs
 
     def reset_transformation(self) -> None:
-        """Reset block reference parameters `location`, `rotation` and
-        `extrusion` vector.
+        """Reset block reference attributes location, rotation angle and
+        the extrusion vector but preserves the scale factors.
 
         """
         self.dxf.insert = NULLVEC
@@ -569,71 +550,65 @@ class Insert(LinkedEntities):
         self.dxf.discard("extrusion")
 
     def explode(
-        self, target_layout: BaseLayout = None, *, redraw_order=False
+        self, target_layout: Optional[BaseLayout] = None, *, redraw_order=False
     ) -> EntityQuery:
-        """Explode block reference entities into target layout, if target
-        layout is ``None``, the target layout is the layout of the block
-        reference. This method destroys the source block reference entity.
+        """Explodes the block reference entities into the target layout, if target
+        layout is ``None``, the layout of the block reference will be used.
+        This method destroys the source block reference entity.
 
         Transforms the block entities into the required :ref:`WCS` location by
         applying the block reference attributes `insert`, `extrusion`,
-        `rotation` and the scaling values `xscale`, `yscale` and `zscale`.
+        `rotation` and the scale factors `xscale`, `yscale` and `zscale`.
 
         Attached ATTRIB entities are converted to TEXT entities, this is the
         behavior of the BURST command of the AutoCAD Express Tools.
 
-        Returns an :class:`~ezdxf.query.EntityQuery` container with all
-        "exploded" DXF entities.
-
         .. warning::
 
-            **Non uniform scaling** may lead to incorrect results for text
-            entities (TEXT, MTEXT, ATTRIB) and maybe some other entities.
+            **Non-uniform scale factors** may lead to incorrect results some entities
+            (TEXT, MTEXT, ATTRIB).
 
         Args:
             target_layout: target layout for exploded entities, ``None`` for
                 same layout as source entity.
             redraw_order: create entities in ascending redraw order if ``True``
 
+        Returns:
+            :class:`~ezdxf.query.EntityQuery` container referencing all exploded
+            DXF entities.
+
         """
         if target_layout is None:
             target_layout = self.get_layout()
             if target_layout is None:
                 raise DXFStructureError(
-                    "INSERT without layout assignment, specify target layout."
+                    "INSERT without layout assignment, specify target layout"
                 )
         return explode_block_reference(
             self, target_layout=target_layout, redraw_order=redraw_order
         )
 
-    def __virtual_entities__(self) -> Iterable[DXFGraphic]:
+    def __virtual_entities__(self) -> Iterator[DXFGraphic]:
         """Implements the SupportsVirtualEntities protocol.
 
         This protocol is for consistent internal usage and does not replace
-        the method :meth:`virtual_entities`! Ignores the redraw order!
+        the method :meth:`virtual_entities`! Ignores the redraw-order!
         """
         return self.virtual_entities()
 
     def virtual_entities(
         self,
         *,
-        skipped_entity_callback: Optional[
-            Callable[[DXFGraphic, str], None]
-        ] = None,
+        skipped_entity_callback: Optional[Callable[[DXFGraphic, str], None]] = None,
         redraw_order=False,
-    ) -> Iterable[DXFGraphic]:
+    ) -> Iterator[DXFGraphic]:
         """
-        Yields "virtual" entities of a block reference. This method is meant to
-        examine the block reference entities at the "exploded" location without
-        really "exploding" the block reference. The`skipped_entity_callback()`
-        will be called for all entities which are not processed, signature:
-        :code:`skipped_entity_callback(entity: DXFEntity, reason: str)`,
-        `entity` is the original (untransformed) DXF entity of the block
-        definition, the `reason` string is an explanation why the entity was
-        skipped.
+        Yields the transformed referenced block content as virtual entities.
 
-        This entities are not stored in the entity database, have no handle and
-        are not assigned to any layout. It is possible to convert this entities
+        This method is meant to examine the block reference entities at the target
+        location without exploding the block reference.
+        These entities are not stored in the entity database, have no handle and
+        are not assigned to any layout. It is possible to convert these entities
         into regular drawing entities by adding the entities to the entities
         database and a layout of the same DXF document as the block reference::
 
@@ -641,16 +616,22 @@ class Insert(LinkedEntities):
             msp = doc.modelspace()
             msp.add_entity(entity)
 
-        This method does not resolve the MINSERT attributes, only the
-        sub-entities of the base INSERT will be returned. To resolve MINSERT
-        entities check if multi insert processing is required, that's the case
-        if property :attr:`Insert.mcount` > 1, use the :meth:`Insert.multi_insert`
-        method to resolve the MINSERT entity into single INSERT entities.
-
         .. warning::
 
-            **Non uniform scaling** may return incorrect results for text
-            entities (TEXT, MTEXT, ATTRIB) and maybe some other entities.
+            **Non-uniform scale factors** may return incorrect results for some entities
+            (TEXT, MTEXT, ATTRIB).
+
+        This method does not resolve the MINSERT attributes, only the
+        sub-entities of the first INSERT will be returned. To resolve MINSERT
+        entities check if multi insert processing is required, that's the case
+        if the property :attr:`Insert.mcount` > 1, use the :meth:`Insert.multi_insert`
+        method to resolve the MINSERT entity into multiple INSERT entities.
+
+        The `skipped_entity_callback()` will be called for all entities which are not
+        processed, signature:
+        :code:`skipped_entity_callback(entity: DXFEntity, reason: str)`,
+        `entity` is the original (untransformed) DXF entity of the block definition, the
+        `reason` string is an explanation why the entity was skipped.
 
         Args:
             skipped_entity_callback: called whenever the transformation of an
@@ -676,7 +657,7 @@ class Insert(LinkedEntities):
             self.dxf.column_count if self.dxf.column_spacing else 1
         )
 
-    def multi_insert(self) -> Iterable[Insert]:
+    def multi_insert(self) -> Iterator[Insert]:
         """Yields a virtual INSERT entity for each grid element of a MINSERT
         entity (multi-insert).
         """
@@ -712,7 +693,7 @@ class Insert(LinkedEntities):
                     transform_attached_attrib_entities(insert, offset)
                     yield insert
 
-    def add_auto_attribs(self, values: Dict[str, str]) -> Insert:
+    def add_auto_attribs(self, values: dict[str, str]) -> Insert:
         """
         Attach for each :class:`~ezdxf.entities.Attdef` entity, defined in the
         block definition, automatically an :class:`Attrib` entity to the block
@@ -724,7 +705,7 @@ class Insert(LinkedEntities):
         This method avoids the wrapper block of the
         :meth:`~ezdxf.layouts.BaseLayout.add_auto_blockref` method, but the
         visual results may not match the results of CAD applications, especially
-        for non uniform scaling. If the visual result is very important to you,
+        for non-uniform scaling. If the visual result is very important to you,
         use the :meth:`add_auto_blockref` method.
 
         Args:
@@ -733,7 +714,7 @@ class Insert(LinkedEntities):
 
         """
 
-        def unpack(dxfattribs) -> Tuple[str, str, UVec]:
+        def unpack(dxfattribs) -> tuple[str, str, UVec]:
             tag = dxfattribs.pop("tag")
             text = values.get(tag, "")
             location = dxfattribs.pop("insert")
