@@ -1,6 +1,7 @@
 # Copyright (c) 2019-2022, Manfred Moitzi
 # License: MIT License
-from typing import TYPE_CHECKING, Optional, Tuple, cast, Dict, List, Any
+from __future__ import annotations
+from typing import TYPE_CHECKING, Optional, cast, Any
 import logging
 from dataclasses import dataclass
 from ezdxf.lldxf import validator
@@ -28,18 +29,15 @@ from ezdxf.lldxf.const import (
 from ezdxf.entities.dxfentity import base_class, SubclassProcessor, DXFEntity
 from .factory import register_entity
 
-logger = logging.getLogger("ezdxf")
-
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        TagWriter,
-        DXFNamespace,
-        Viewport,
-        XRecord,
-        EntityDB,
-    )
+    from ezdxf.entities import DXFNamespace, Viewport, XRecord
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.entitydb import EntityDB
+    from ezdxf import xref
+
 
 __all__ = ["Layer", "acdb_symbol_table_record", "LayerOverrides"]
+logger = logging.getLogger("ezdxf")
 
 
 def is_valid_layer_color_index(aci: int) -> bool:
@@ -133,8 +131,8 @@ class Layer(DXFEntity):
     UNLOCK = 0b11111011
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             processor.simple_dxfattribs_loader(
@@ -142,7 +140,7 @@ class Layer(DXFEntity):
             )
         return dxf
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         super().export_entity(tagwriter)
         if tagwriter.dxfversion > DXF12:
             tagwriter.write_tag2(SUBCLASS_MARKER, acdb_symbol_table_record.name)
@@ -165,7 +163,7 @@ class Layer(DXFEntity):
         )
 
     def set_required_attributes(self):
-        if not self.dxf.hasattr("material"):
+        if not self.dxf.hasattr("material_handle"):
             global_ = self.doc.materials["Global"]
             if isinstance(global_, DXFEntity):
                 handle = global_.dxf.handle
@@ -226,19 +224,19 @@ class Layer(DXFEntity):
 
     def get_color(self) -> int:
         """Get layer color, safe method for getting the layer color, because
-        dxf.color is negative for layer status `off`.
+        :attr:`dxf.color` is negative for layer status `off`.
         """
         return abs(self.dxf.color)
 
     def set_color(self, color: int) -> None:
         """Set layer color, safe method for setting the layer color, because
-        dxf.color is negative for layer status `off`.
+        :attr:`dxf.color` is negative for layer status `off`.
         """
         color = abs(color) if self.is_on() else -abs(color)
         self.dxf.color = color
 
     @property
-    def rgb(self) -> Optional[Tuple[int, int, int]]:
+    def rgb(self) -> Optional[tuple[int, int, int]]:
         """Returns RGB true color as (r, g, b)-tuple or None if attribute
         dxf.true_color is not set.
         """
@@ -248,21 +246,21 @@ class Layer(DXFEntity):
             return None
 
     @rgb.setter
-    def rgb(self, rgb: Tuple[int, int, int]) -> None:
+    def rgb(self, rgb: tuple[int, int, int]) -> None:
         """Set RGB true color as (r, g, b)-tuple e.g. (12, 34, 56)."""
         self.dxf.set("true_color", clr.rgb2int(rgb))
 
     @property
     def color(self) -> int:
         """Get layer color, safe method for getting the layer color, because
-        dxf.color is negative for layer status `off`.
+        :attr:`dxf.color` is negative for layer status `off`.
         """
         return self.get_color()
 
     @color.setter
     def color(self, value: int) -> None:
         """Set layer color, safe method for setting the layer color, because
-        dxf.color is negative for layer status `off`.
+        :attr:`dxf.color` is negative for layer status `off`.
         """
         self.set_color(value)
 
@@ -311,7 +309,7 @@ class Layer(DXFEntity):
                 AcCmTransparency, [(1071, clr.float2transparency(value))]
             )
         else:
-            raise ValueError("Value out of range (0 .. 1).")
+            raise ValueError("Value out of range [0, 1].")
 
     def rename(self, name: str) -> None:
         """
@@ -319,7 +317,17 @@ class Layer(DXFEntity):
 
         .. warning::
 
-            Renaming layers may damage the DXF file in some circumstances!
+            The DXF format is not consistent in storing layer references, the
+            layers are mostly referenced by their case-insensitive name,
+            some later introduced entities do reference layers by handle, which
+            is the safer way in the context of renaming layers.
+
+            There is no complete overview of where layer references are
+            stored, third-party entities are black-boxes with unknown content
+            and layer names could be stored in the extended data section of any
+            DXF entity or in XRECORD entities.
+            Which means that in some rare cases references to the old layer name
+            can persist, at least this does not invalidate the DXF document.
 
         Args:
              name: new layer name
@@ -355,7 +363,7 @@ class Layer(DXFEntity):
                 e.dxf.layer = new_name
             entity_type = e.dxftype()
             if entity_type == "VIEWPORT":
-                e.rename_frozen_layer(old_name, new_name)
+                e.rename_frozen_layer(old_name, new_name)  # type: ignore
             elif entity_type == "LAYER_FILTER":
                 # todo: if LAYER_FILTER implemented, add support for
                 #  renaming layers
@@ -371,10 +379,35 @@ class Layer(DXFEntity):
                     f"LAYER_INDEX"
                 )
 
-    def get_vp_overrides(self) -> "LayerOverrides":
+    def get_vp_overrides(self) -> LayerOverrides:
         """Returns the :class:`LayerOverrides` object for this layer."""
         return LayerOverrides(self)
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        assert self.doc is not None, "LAYER entity must be assigned to a document"
+        super().register_resources(registry)
+        registry.add_linetype(self.dxf.linetype)
+        registry.add_handle(self.dxf.get("material_handle"))
+        # current plot style will be replaced by default plot style "Normal"
+
+    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        assert isinstance(clone, Layer)
+        super().map_resources(clone, mapping)
+        self.dxf.linetype = mapping.get_linetype(self.dxf.linetype)
+
+        mapping.map_existing_handle(self, clone, "material_handle", optional=True)
+        # remove handles pointing to the source document:
+        clone.dxf.discard("plotstyle_handle")  # replaced by plot style "Normal"
+        clone.dxf.discard("unknown1")
+
+        # create required handles to resources in the target document
+        clone.set_required_attributes()
+        # todo: map layer overrides
+        # remove layer overrides
+        clone.discard_extension_dict()
+
 
 @dataclass
 class OverrideAttributes:
@@ -404,7 +437,7 @@ class LayerOverrides:
         self._layer = layer
         self._overrides = load_layer_overrides(layer)
 
-    def has_overrides(self, vp_handle: str = None) -> bool:
+    def has_overrides(self, vp_handle: Optional[str] = None) -> bool:
         """Returns ``True`` if attribute overrides exist for the given
         :class:`Viewport` handle.
         Returns ``True`` if `any` attribute overrides exist if the given
@@ -539,7 +572,7 @@ class LayerOverrides:
         vp_overrides = self._acquire_overrides(vp_handle)
         vp_overrides.lineweight = value
 
-    def discard(self, vp_handle: str = None) -> None:
+    def discard(self, vp_handle: Optional[str] = None) -> None:
         """Discard all attribute overrides for the given :class:`Viewport`
         handle or for all :class:`Viewport` entities if the handle is ``None``.
         """
@@ -571,7 +604,7 @@ def is_layer_frozen_in_vp(layer, vp_handle) -> bool:
     return False
 
 
-def load_layer_overrides(layer: Layer) -> Dict[str, OverrideAttributes]:
+def load_layer_overrides(layer: Layer) -> dict[str, OverrideAttributes]:
     """Load all VIEWPORT overrides from the layer extension dictionary."""
 
     def get_ovr(vp_handle: str):
@@ -620,7 +653,7 @@ def load_layer_overrides(layer: Layer) -> Dict[str, OverrideAttributes]:
     entitydb: EntityDB = layer.doc.entitydb
     assert entitydb is not None, "valid entity database required"
 
-    overrides: Dict[str, OverrideAttributes] = dict()
+    overrides: dict[str, OverrideAttributes] = dict()
     if not layer.has_extension_dict:
         return overrides
 
@@ -628,7 +661,7 @@ def load_layer_overrides(layer: Layer) -> Dict[str, OverrideAttributes]:
     return overrides
 
 
-def _load_ovr_values(xrec: "XRecord", group_code):
+def _load_ovr_values(xrec: XRecord, group_code):
     tags = xrec.tags
     handles = [value for code, value in tags.find_all(const.OVR_VP_HANDLE_CODE)]
     values = [value for code, value in tags.find_all(group_code)]
@@ -636,7 +669,7 @@ def _load_ovr_values(xrec: "XRecord", group_code):
 
 
 def store_layer_overrides(
-    layer: Layer, overrides: Dict[str, OverrideAttributes]
+    layer: Layer, overrides: dict[str, OverrideAttributes]
 ) -> None:
     """Store all VIEWPORT overrides in the layer extension dictionary.
     Replaces all existing overrides!
@@ -649,7 +682,7 @@ def store_layer_overrides(
         else:
             return layer.new_extension_dict()
 
-    def set_xdict_tags(key: str, tags: List[DXFTag]):
+    def set_xdict_tags(key: str, tags: list[DXFTag]):
         xdict = get_xdict()
         xrec = xdict.get(key)
         if xrec is None:
@@ -667,9 +700,9 @@ def store_layer_overrides(
             xdict.discard(key)
 
     def make_tags(
-        data: List[Tuple[Any, str]], name: str, code: int
-    ) -> List[DXFTag]:
-        tags: List[DXFTag] = []
+        data: list[tuple[Any, str]], name: str, code: int
+    ) -> list[DXFTag]:
+        tags: list[DXFTag] = []
         for value, vp_handle in data:
             tags.extend(
                 [
