@@ -1,10 +1,10 @@
-# Copyright (c) 2019-2021, Manfred Moitzi
+# Copyright (c) 2019-2022, Manfred Moitzi
 # License: MIT License
+from __future__ import annotations
 from typing import (
     TYPE_CHECKING,
     Union,
     Iterable,
-    cast,
     Sequence,
     Optional,
 )
@@ -25,7 +25,9 @@ from ezdxf.tools.complex_ltype import lin_compiler
 from .factory import register_entity
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import TagWriter, DXFNamespace, Drawing
+    from ezdxf.entities import DXFNamespace
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf import xref
 
 __all__ = ["Linetype", "compile_line_pattern", "CONTINUOUS_PATTERN"]
 
@@ -51,13 +53,13 @@ class LinetypePattern:
     def __len__(self):
         return len(self.tags)
 
-    def export_dxf(self, tagwriter: "TagWriter"):
+    def export_dxf(self, tagwriter: AbstractTagWriter):
         if tagwriter.dxfversion <= DXF12:
             self.export_r12_dxf(tagwriter)
         else:
             tagwriter.write_tags(self.tags)
 
-    def export_r12_dxf(self, tagwriter: "TagWriter"):
+    def export_r12_dxf(self, tagwriter: AbstractTagWriter):
         tags49 = Tags(tag for tag in self.tags if tag.code == 49)
         tagwriter.write_tag2(72, 65)
         tagwriter.write_tag2(73, len(tags49))
@@ -159,14 +161,14 @@ class Linetype(DXFEntity):
         super().__init__()
         self.pattern_tags = LinetypePattern(Tags())
 
-    def _copy_data(self, entity: "DXFEntity") -> None:
+    def copy_data(self, entity: DXFEntity) -> None:
         """Copy pattern_tags."""
         assert isinstance(entity, Linetype)
         entity.pattern_tags = deepcopy(self.pattern_tags)
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             tags = processor.fast_load_dxfattribs(
@@ -175,7 +177,7 @@ class Linetype(DXFEntity):
             self.pattern_tags = LinetypePattern(tags)
         return dxf
 
-    def preprocess_export(self, tagwriter: "TagWriter"):
+    def preprocess_export(self, tagwriter: AbstractTagWriter):
         if len(self.pattern_tags) == 0:
             return False
         # Do not export complex linetypes for DXF12
@@ -183,7 +185,7 @@ class Linetype(DXFEntity):
             return not self.pattern_tags.is_complex_type()
         return True
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         super().export_entity(tagwriter)
         # AcDbEntity export is done by parent class
         if tagwriter.dxfversion > DXF12:
@@ -245,3 +247,22 @@ class Linetype(DXFEntity):
         or shapes are not supported and return a continuous line pattern.
         """
         return self.pattern_tags.compile()
+
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        assert self.doc is not None, "LTYPE entity must be assigned to a document"
+        super().register_resources(registry)
+        # register text styles and shape files for complex linetypes
+        style_handle = self.pattern_tags.get_style_handle()
+        style = self.doc.styles.get_entry_by_handle(style_handle)
+        if style is not None:
+            registry.add_entity(style)
+
+    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+        """Translate registered resources from self to the copied entity."""
+        assert isinstance(clone, Linetype)
+        super().map_resources(clone, mapping)
+        style_handle = self.pattern_tags.get_style_handle()
+        if style_handle != "0":
+            # map text style or shape file handle of complex linetype
+            clone.pattern_tags.set_style_handle(mapping.get_handle(style_handle))
