@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021 Manfred Moitzi
+# Copyright (c) 2019-2022 Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
 from typing import (
@@ -6,14 +6,15 @@ from typing import (
     Tuple,
     Sequence,
     Iterable,
-    List,
     Union,
     Iterator,
+    Optional,
 )
+from typing_extensions import TypeAlias
 import array
 import copy
 from contextlib import contextmanager
-from ezdxf.math import Vec3, UVec, Matrix44, Z_AXIS
+from ezdxf.math import Vec3, Matrix44, Z_AXIS
 from ezdxf.math.transformtools import OCSTransform, NonUniformScalingError
 from ezdxf.lldxf import validator
 from ezdxf.lldxf.attributes import (
@@ -41,18 +42,13 @@ from .dxfgfx import DXFGraphic, acdb_entity
 from .factory import register_entity
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        TagWriter,
-        DXFNamespace,
-        Line,
-        Arc,
-        BaseLayout,
-        DXFEntity,
-    )
+    from ezdxf.entities import DXFNamespace, Line, Arc, DXFEntity
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.layouts import BaseLayout
 
 __all__ = ["LWPolyline", "FORMAT_CODES"]
 
-LWPointType = Tuple[float, float, float, float, float]
+LWPointType: TypeAlias = Tuple[float, float, float, float, float]
 
 FORMAT_CODES = frozenset("xysebv")
 DEFAULT_FORMAT = "xyseb"
@@ -106,13 +102,13 @@ class LWPolyline(DXFGraphic):
         super().__init__()
         self.lwpoints = LWPolylinePoints()
 
-    def _copy_data(self, entity: DXFEntity) -> None:
+    def copy_data(self, entity: DXFEntity) -> None:
         """Copy lwpoints."""
         assert isinstance(entity, LWPolyline)
         entity.lwpoints = copy.deepcopy(self.lwpoints)
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
+        self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
         """
         Adds subclass processing for AcDbPolyline, requires previous base class
@@ -139,12 +135,12 @@ class LWPolyline(DXFGraphic):
         self.lwpoints, unprocessed_tags = LWPolylinePoints.from_tags(tags)
         return unprocessed_tags
 
-    def preprocess_export(self, tagwriter: TagWriter) -> bool:
+    def preprocess_export(self, tagwriter: AbstractTagWriter) -> bool:
         # Returns True if entity should be exported
         # Do not export polylines without vertices
         return len(self.lwpoints) > 0
 
-    def export_entity(self, tagwriter: TagWriter) -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         super().export_entity(tagwriter)
         tagwriter.write_tag2(SUBCLASS_MARKER, acdb_lwpolyline.name)
@@ -157,9 +153,8 @@ class LWPolyline(DXFGraphic):
 
     @property
     def closed(self) -> bool:
-        """Get/set closed state of polyline.
-        A closed polyline has a connection from the last vertex to the first
-        vertex.
+        """Get/set closed state of polyline.  A closed polyline has a connection
+        segment from the last vertex to the first vertex.
         """
         return self.get_flag_state(LWPOLYLINE_CLOSED)
 
@@ -169,13 +164,13 @@ class LWPolyline(DXFGraphic):
 
     @property
     def is_closed(self) -> bool:
-        """Returns ``True`` if LWPOLYLINE is closed.
+        """Get closed state of LWPOLYLINE.
         Compatibility interface to :class:`Polyline`
         """
         return self.get_flag_state(LWPOLYLINE_CLOSED)
 
     def close(self, state: bool = True) -> None:
-        """Get/set closed state of LWPOLYLINE.
+        """Set closed state of LWPOLYLINE.
         Compatibility interface to :class:`Polyline`
         """
         self.closed = state
@@ -188,9 +183,7 @@ class LWPolyline(DXFGraphic):
     @property
     def has_width(self) -> bool:
         """Returns ``True`` if LWPOLYLINE has any segment with width attributes
-        or DXF attribute const_width != 0.
-
-        .. versionadded:: 0.14
+        or the DXF attribute const_width is not 0.
 
         """
         if self.dxf.hasattr("const_width"):
@@ -209,8 +202,8 @@ class LWPolyline(DXFGraphic):
 
     def __getitem__(self, index: int) -> LWPointType:
         """Returns point at position `index` as (x, y, start_width, end_width,
-        bulge) tuple. start_width, end_width and bulge is ``0`` if not present,
-        supports extended slicing. Point format is fixed as ``'xyseb'``.
+        bulge) tuple. start_width, end_width and bulge is 0 if not present,
+        supports extended slicing. Point format is fixed as "xyseb".
 
         All coordinates in :ref:`OCS`.
 
@@ -220,10 +213,10 @@ class LWPolyline(DXFGraphic):
     def __setitem__(self, index: int, value: Sequence[float]) -> None:
         """
         Set point at position `index` as (x, y, [start_width, [end_width,
-        [bulge]]]) tuple. If start_width or end_width is ``0`` or left off the
-        default value is used. If the bulge value is left off, bulge is ``0``
+        [bulge]]]) tuple. If start_width or end_width is 0 or left off the
+        default width value is used. If the bulge value is left off, bulge is 0
         by default (straight line).
-        Does NOT support extend slicing. Point format is fixed as ``'xyseb'``.
+        Does NOT support extend slicing. Point format is fixed as "xyseb".
 
         All coordinates in :ref:`OCS`.
 
@@ -238,7 +231,7 @@ class LWPolyline(DXFGraphic):
         """Delete point at position `index`, supports extended slicing."""
         del self.lwpoints[index]
 
-    def vertices(self) -> Iterator[Tuple[float, float]]:
+    def vertices(self) -> Iterator[tuple[float, float]]:
         """
         Returns iterable of all polyline points as (x, y) tuples in :ref:`OCS`
         (:attr:`dxf.elevation` is the z-axis value).
@@ -247,14 +240,14 @@ class LWPolyline(DXFGraphic):
         for point in self:
             yield point[0], point[1]
 
-    def vertices_in_wcs(self) -> Iterable[Vec3]:
+    def vertices_in_wcs(self) -> Iterator[Vec3]:
         """Returns iterable of all polyline points as Vec3(x, y, z) in :ref:`WCS`."""
         ocs = self.ocs()
         elevation = self.get_dxf_attrib("elevation", default=0.0)
         for x, y in self.vertices():
             yield ocs.to_wcs(Vec3(x, y, elevation))
 
-    def vertices_in_ocs(self) -> Iterable[Vec3]:
+    def vertices_in_ocs(self) -> Iterator[Vec3]:
         """Returns iterable of all polyline points as Vec3(x, y, z) in :ref:`OCS`."""
         elevation = self.get_dxf_attrib("elevation", default=0.0)
         for x, y in self.vertices():
@@ -263,14 +256,14 @@ class LWPolyline(DXFGraphic):
     def append(
         self, point: Sequence[float], format: str = DEFAULT_FORMAT
     ) -> None:
-        """Append `point` to polyline, `format`` specifies a user defined
+        """Append `point` to polyline, `format` specifies a user defined
         point format.
 
         All coordinates in :ref:`OCS`.
 
         Args:
             point: (x, y, [start_width, [end_width, [bulge]]]) tuple
-            format: format string, default is ``'xyseb'``, see: `format codes`_
+            format: format string, default is "xyseb", see: `format codes`_
 
         """
         self.lwpoints.append(point, format=format)
@@ -286,7 +279,7 @@ class LWPolyline(DXFGraphic):
         Args:
             pos: insert position
             point: point data
-            format: format string, default is 'xyseb', see: `format codes`_
+            format: format string, default is "xyseb", see: `format codes`_
 
         """
         data = compile_array(point, format=format)
@@ -304,7 +297,7 @@ class LWPolyline(DXFGraphic):
         Args:
             points: iterable of point, point is (x, y, [start_width, [end_width,
                 [bulge]]]) tuple
-            format: format string, default is ``'xyseb'``, see: `format codes`_
+            format: format string, default is "xyseb", see: `format codes`_
 
         """
         for point in points:
@@ -313,7 +306,7 @@ class LWPolyline(DXFGraphic):
     @contextmanager
     def points(
         self, format: str = DEFAULT_FORMAT
-    ) -> Iterator[List[Sequence[float]]]:
+    ) -> Iterator[list[Sequence[float]]]:
         """Context manager for polyline points. Returns a standard Python list
         of points, according to the format string.
 
@@ -327,7 +320,7 @@ class LWPolyline(DXFGraphic):
         yield points
         self.set_points(points, format=format)
 
-    def get_points(self, format: str = DEFAULT_FORMAT) -> List[Sequence[float]]:
+    def get_points(self, format: str = DEFAULT_FORMAT) -> list[Sequence[float]]:
         """Returns all points as list of tuples, format specifies a user
         defined point format.
 
@@ -335,7 +328,7 @@ class LWPolyline(DXFGraphic):
         the z-axis value).
 
         Args:
-            format: format string, default is ``'xyseb'``, see `format codes`_
+            format: format string, default is "xyseb", see `format codes`_
 
         """
         return [format_point(p, format=format) for p in self.lwpoints]
@@ -350,7 +343,7 @@ class LWPolyline(DXFGraphic):
         Args:
             points: iterable of point, point is (x, y, [start_width, [end_width,
                 [bulge]]]) tuple
-            format: format string, default is ``'xyseb'``, see `format codes`_
+            format: format string, default is "xyseb", see `format codes`_
 
         """
         self.lwpoints.clear()
@@ -370,7 +363,7 @@ class LWPolyline(DXFGraphic):
             m: transformation :class:`~ezdxf.math.Matrix44`
 
         Raises:
-            NonUniformScalingError: for non uniform scaling of entity containing
+            NonUniformScalingError: for non-uniform scaling of entity containing
                 circular arc segments (bulges)
 
         """
@@ -406,7 +399,7 @@ class LWPolyline(DXFGraphic):
         self.post_transform(m)
         return self
 
-    def virtual_entities(self) -> Iterable[Union[Line, Arc]]:
+    def virtual_entities(self) -> Iterator[Union[Line, Arc]]:
         """Yields the graphical representation of LWPOLYLINE as virtual DXF
         primitives (LINE or ARC).
 
@@ -419,12 +412,12 @@ class LWPolyline(DXFGraphic):
             e.set_source_of_copy(self)
             yield e
 
-    def explode(self, target_layout: BaseLayout = None) -> EntityQuery:
+    def explode(self, target_layout: Optional[BaseLayout] = None) -> EntityQuery:
         """Explode the LWPOLYLINE entity as DXF primitives (LINE or ARC) into
         the target layout, if the target layout is ``None``, the target layout
-        is the layout of the source entity.
+        is the layout of the source entity. This method destroys the source entity.
 
-        Returns an :class:`~ezdxf.query.EntityQuery` container of all DXF
+        Returns an :class:`~ezdxf.query.EntityQuery` container referencing all DXF
         primitives.
 
         Args:
@@ -478,7 +471,7 @@ class LWPolylinePoints(VertexArray):
     ) -> None:
         super().append(compile_array(point, format=format))
 
-    def dxftags(self) -> Iterable[DXFTag]:
+    def dxftags(self) -> Iterator[DXFTag]:
         for point in self:
             x, y, start_width, end_width, bulge = point
             yield DXFVertex(self.VERTEX_CODE, (x, y))
@@ -507,7 +500,7 @@ def format_point(
 
     Args:
         point: list or tuple of (x, y, start_width, end_width, bulge)
-        format: format string, default is 'xyseb'
+        format: format string, default is "xyseb"
 
     Returns:
         Sequence[float]: tuple of selected components
@@ -533,7 +526,7 @@ def compile_array(data: Sequence[float], format="xyseb") -> array.array:
 
     Args:
         data: list or tuple of point components
-        format: format string, default is 'xyseb'
+        format: format string, default is "xyseb"
 
     Returns:
         array.array: array.array('d', (x, y, start_width, end_width, bulge))
