@@ -1,8 +1,10 @@
-# Copyright (c) 2018-2021, Manfred Moitzi
+# Copyright (c) 2018-2022, Manfred Moitzi
 # License: MIT License
-from typing import TYPE_CHECKING, List, Union, Optional, Iterable, Dict, Any
+from __future__ import annotations
+from typing import TYPE_CHECKING, Union, Optional, Iterable, Any, Iterator
 import copy
 import logging
+import math
 from collections import namedtuple
 
 from ezdxf.lldxf import const
@@ -29,7 +31,7 @@ from ezdxf.math import (
 )
 from ezdxf import colors
 from ezdxf.proxygraphic import ProxyGraphicError
-from ezdxf.tools.text import safe_string
+from ezdxf.tools.text import safe_string, scale_mtext_inline_commands
 from ezdxf.tools.handle import safe_handle
 
 from .dxfentity import base_class, SubclassProcessor
@@ -40,16 +42,14 @@ from .factory import register_entity
 from .objectcollection import ObjectCollection
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        TagWriter,
-        Drawing,
-        DXFNamespace,
-        DXFTag,
-        DXFEntity,
-        BaseLayout,
-        EntityQuery,
-        Auditor,
-    )
+    from ezdxf.audit import Auditor
+    from ezdxf.document import Drawing
+    from ezdxf.entities import DXFNamespace, DXFEntity
+    from ezdxf.layouts import BaseLayout
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.lldxf.types import DXFTag
+    from ezdxf.query import EntityQuery
+    from ezdxf import xref
 
 __all__ = [
     "MultiLeader",
@@ -64,7 +64,6 @@ __all__ = [
     "ArrowHeadData",
     "AttribData",
 ]
-
 logger = logging.getLogger("ezdxf")
 
 
@@ -161,12 +160,8 @@ acdb_mleader = DefSubclass(
         "has_text_frame": DXFAttr(292, default=0),
         # Block Content:
         "block_record_handle": DXFAttr(344),
-        "block_color": DXFAttr(
-            93, default=colors.BY_BLOCK_RAW_VALUE
-        ),  # raw color
-        "block_scale_vector": DXFAttr(
-            10, xtype=XType.point3d, default=Vec3(1, 1, 1)
-        ),
+        "block_color": DXFAttr(93, default=colors.BY_BLOCK_RAW_VALUE),  # raw color
+        "block_scale_vector": DXFAttr(10, xtype=XType.point3d, default=Vec3(1, 1, 1)),
         "block_rotation": DXFAttr(43, default=0),  # in radians!!!
         "block_connection_type": DXFAttr(176, default=0),
         # 0 = center extents
@@ -183,13 +178,9 @@ acdb_mleader = DefSubclass(
         # attrib_text: 302, collision with group code (302, "LEADER{") in context data
         # END "block attribs"
         # Text Content:
-        "is_text_direction_negative": DXFAttr(
-            294, default=0, dxfversion=const.DXF2007
-        ),
+        "is_text_direction_negative": DXFAttr(294, default=0, dxfversion=const.DXF2007),
         "text_IPE_align": DXFAttr(178, default=0, dxfversion=const.DXF2007),
-        "text_attachment_point": DXFAttr(
-            179, default=1, dxfversion=const.DXF2007
-        ),
+        "text_attachment_point": DXFAttr(179, default=1, dxfversion=const.DXF2007),
         # 1 = left
         # 2 = center
         # 3 = right
@@ -198,9 +189,7 @@ acdb_mleader = DefSubclass(
         # block/text, or attach to the top/bottom:
         # 0 = horizontal
         # 1 = vertical
-        "text_attachment_direction": DXFAttr(
-            271, default=0, dxfversion=const.DXF2010
-        ),
+        "text_attachment_direction": DXFAttr(271, default=0, dxfversion=const.DXF2010),
         # like 173, but
         # 9 = center
         # 10= underline and center
@@ -210,12 +199,8 @@ acdb_mleader = DefSubclass(
         # like 173, but
         # 9 = center
         # 10= overline and center
-        "text_top_attachment_type": DXFAttr(
-            273, default=9, dxfversion=const.DXF2010
-        ),
-        "leader_extend_to_text": DXFAttr(
-            295, default=0, dxfversion=const.DXF2013
-        ),
+        "text_top_attachment_type": DXFAttr(273, default=9, dxfversion=const.DXF2010),
+        "leader_extend_to_text": DXFAttr(295, default=0, dxfversion=const.DXF2013),
     },
 )
 # The text frame shape is stored in XDATA except for the default rectangle:
@@ -244,18 +229,16 @@ END_LEADER_LINE = 305
 
 
 def compile_context_tags(
-    data: List["DXFTag"], stop_code: int
-) -> List[Union["DXFTag", List]]:
-    def build_structure(
-        tag: "DXFTag", stop: int
-    ) -> List[Union["DXFTag", List]]:
+    data: list[DXFTag], stop_code: int
+) -> list[Union[DXFTag, list]]:
+    def build_structure(tag: DXFTag, stop: int) -> list[Union[DXFTag, list]]:
         collector = [tag]
         tag = next(tags)
         while tag.code != stop:
             if tag.code == START_LEADER:
                 collector.append(build_structure(tag, END_LEADER))  # type: ignore
             # Group code 304 is used also for MTEXT content, therefore always
-            # test for group code AND and value string:
+            # test for group code and value string:
             elif tag.code == START_LEADER_LINE and tag.value == LEADER_LINE_STR:
                 collector.append(build_structure(tag, END_LEADER_LINE))  # type: ignore
             else:
@@ -277,14 +260,15 @@ class MultiLeader(DXFGraphic):
     DXFATTRIBS = DXFAttributes(base_class, acdb_entity, acdb_mleader)
     MIN_DXF_VERSION_FOR_EXPORT = const.DXF2000
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         self.context = MLeaderContext()
-        self.arrow_heads: List[ArrowHeadData] = []
-        self.block_attribs: List[AttribData] = []
+        self.arrow_heads: list[ArrowHeadData] = []
+        self.block_attribs: list[AttribData] = []
 
     @property
     def has_mtext_content(self) -> bool:
+        """``True`` if MULTILEADER has MTEXT content."""
         return self.context.mtext is not None
 
     def get_mtext_content(self) -> str:
@@ -306,28 +290,29 @@ class MultiLeader(DXFGraphic):
 
     @property
     def has_block_content(self) -> bool:
+        """``True`` if MULTILEADER has BLOCK content."""
         return self.context.block is not None
 
-    def get_block_content(self) -> Dict[str, str]:
+    def get_block_content(self) -> dict[str, str]:
         """Get BLOCK attributes as dictionary of (tag, value) pairs.
         Returns an empty dictionary if MULTILEADER has MTEXT content.
         """
         assert self.doc is not None, "valid DXF document required"
         entitydb = self.doc.entitydb
-        tags: Dict[str, str] = dict()
+        tags: dict[str, str] = dict()
         for attr in self.block_attribs:
             attdef = entitydb.get(attr.handle)
             if attdef is not None:
                 tags[attdef.dxf.tag] = attr.text
         return tags
 
-    def set_block_content(self, content: Dict[str, str]):
+    def set_block_content(self, content: dict[str, str]):
         """Set BLOCK attributes by a dictionary of (tag, value) pairs.
         Does nothing if MULTILEADER has MTEXT content.
         """
         assert self.doc is not None, "valid DXF document required"
         entitydb = self.doc.entitydb
-        tags: Dict[str, str] = dict()
+        tags: dict[str, str] = dict()
         block_attribs = self.block_attribs
         for index, attr in enumerate(block_attribs):
             attdef = entitydb.get(attr.handle)
@@ -338,7 +323,7 @@ class MultiLeader(DXFGraphic):
                     block_attribs[index] = attr._replace(text=new_text)
         return tags
 
-    def _copy_data(self, entity: "DXFEntity") -> None:
+    def copy_data(self, entity: DXFEntity) -> None:
         """Copy leaders"""
         assert isinstance(entity, MultiLeader)
         entity.context = copy.deepcopy(self.context)
@@ -346,8 +331,8 @@ class MultiLeader(DXFGraphic):
         entity.block_attribs = copy.deepcopy(self.block_attribs)
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor is None:
             return dxf
@@ -375,7 +360,7 @@ class MultiLeader(DXFGraphic):
         return dxf
 
     @staticmethod
-    def extract_context_data(tags: Tags) -> List["DXFTag"]:
+    def extract_context_data(tags: Tags) -> list[DXFTag]:
         start, end = None, None
         context_data = []
         for index, tag in enumerate(tags):
@@ -391,7 +376,7 @@ class MultiLeader(DXFGraphic):
         return context_data
 
     @staticmethod
-    def load_context(data: List["DXFTag"]) -> "MLeaderContext":
+    def load_context(data: list[DXFTag]) -> MLeaderContext:
         try:
             context = compile_context_tags(data, END_CONTEXT_DATA)
         except StopIteration:
@@ -400,7 +385,7 @@ class MultiLeader(DXFGraphic):
             return MLeaderContext.load(context)
 
     @staticmethod
-    def extract_arrow_heads(data: Tags) -> List[ArrowHeadData]:
+    def extract_arrow_heads(data: Tags) -> list[ArrowHeadData]:
         def store_head():
             heads.append(
                 ArrowHeadData(
@@ -410,7 +395,7 @@ class MultiLeader(DXFGraphic):
             )
             collector.clear()
 
-        heads: List[ArrowHeadData] = []
+        heads: list[ArrowHeadData] = []
         try:
             start = data.tag_index(94)
         except const.DXFValueError:
@@ -429,7 +414,7 @@ class MultiLeader(DXFGraphic):
         return heads
 
     @staticmethod
-    def extract_block_attribs(data: Tags) -> List[AttribData]:
+    def extract_block_attribs(data: Tags) -> list[AttribData]:
         def store_attrib():
             attribs.append(
                 AttribData(
@@ -441,17 +426,15 @@ class MultiLeader(DXFGraphic):
             )
             collector.clear()
 
-        attribs: List[AttribData] = []
+        attribs: list[AttribData] = []
         try:
             start = data.tag_index(330)
         except const.DXFValueError:
             return attribs
 
         end = start
-        collector: Dict[int, Any] = dict()
-        for code, value in data.collect_consecutive_tags(
-            {330, 177, 44, 302}, start
-        ):
+        collector: dict[int, Any] = dict()
+        for code, value in data.collect_consecutive_tags({330, 177, 44, 302}, start):
             end += 1
             if code == 330 and len(collector):
                 store_attrib()
@@ -463,16 +446,14 @@ class MultiLeader(DXFGraphic):
         del data[start:end]
         return attribs
 
-    def preprocess_export(self, tagwriter: "TagWriter") -> bool:
+    def preprocess_export(self, tagwriter: AbstractTagWriter) -> bool:
         if self.context.is_valid:
             return True
         else:
-            logger.debug(
-                f"Ignore {str(self)} at DXF export, invalid context data."
-            )
+            logger.debug(f"Ignore {str(self)} at DXF export, invalid context data.")
             return False
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         def write_handle_if_exist(code: int, name: str):
             handle = dxf.get(name)
             if handle is not None:
@@ -533,30 +514,73 @@ class MultiLeader(DXFGraphic):
         if version >= const.DXF2013:
             write_tag2(295, dxf.leader_extend_to_text)
 
-    def export_arrow_heads(self, tagwriter: "TagWriter") -> None:
+    def export_arrow_heads(self, tagwriter: AbstractTagWriter) -> None:
         for index, handle in self.arrow_heads:
             tagwriter.write_tag2(94, index)
             tagwriter.write_tag2(345, handle)
 
-    def export_block_attribs(self, tagwriter: "TagWriter") -> None:
+    def export_block_attribs(self, tagwriter: AbstractTagWriter) -> None:
         for attrib in self.block_attribs:
             tagwriter.write_tag2(330, attrib.handle)
             tagwriter.write_tag2(177, attrib.index)
             tagwriter.write_tag2(44, attrib.width)
             tagwriter.write_tag2(302, safe_string(attrib.text, EXT_MAX_STR_LEN))
 
-    def virtual_entities(self) -> Iterable["DXFGraphic"]:
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        super().register_resources(registry)
+        dxf = self.dxf
+        registry.add_handle(dxf.style_handle)
+        registry.add_handle(dxf.leader_linetype_handle)
+        registry.add_handle(dxf.arrow_head_handle)
+        registry.add_handle(dxf.text_style_handle)
+        registry.add_handle(dxf.block_record_handle)
+        for arrow_head in self.arrow_heads:
+            registry.add_handle(arrow_head.handle)
+        # block attdef entities are included in the block definition!
+        self.context.register_resources(registry)
+
+    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        assert isinstance(clone, MultiLeader)
+        super().map_resources(clone, mapping)
+        dxf = self.dxf
+        clone_dxf = clone.dxf
+        clone_dxf.style_handle = mapping.get_handle(dxf.style_handle)
+        clone_dxf.leader_linetype_handle = mapping.get_handle(
+            dxf.leader_linetype_handle
+        )
+        clone_dxf.arrow_head_handle = mapping.get_handle(dxf.arrow_head_handle)
+        clone_dxf.text_style_handle = mapping.get_handle(dxf.text_style_handle)
+        clone_dxf.block_record_handle = mapping.get_handle(dxf.block_record_handle)
+        clone.map_arrow_head_handles(mapping)
+        clone.map_block_attrib_handles(mapping)
+        clone.context.map_resources(mapping)
+
+    def map_arrow_head_handles(self, mapping: xref.ResourceMapper):
+        self.arrow_heads = [
+            arrow._replace(handle=mapping.get_handle(arrow.handle))
+            for arrow in self.arrow_heads
+        ]
+
+    def map_block_attrib_handles(self, mapping: xref.ResourceMapper):
+        self.block_attribs = [
+            attrib._replace(handle=mapping.get_handle(attrib.handle))
+            for attrib in self.block_attribs
+        ]
+
+    def virtual_entities(self) -> Iterator[DXFGraphic]:
         """Yields the graphical representation of MULTILEADER as virtual DXF
         primitives.
 
-        This entities are located at the original location, but are not stored
+        These entities are located at the original location, but are not stored
         in the entity database, have no handle and are not assigned to any
         layout.
 
         """
         return self.__virtual_entities__()
 
-    def explode(self, target_layout: "BaseLayout" = None) -> "EntityQuery":
+    def explode(self, target_layout: Optional[BaseLayout] = None) -> EntityQuery:
         """Explode MULTILEADER as DXF primitives into target layout,
         if target layout is ``None``, the target layout is the layout of the
         source entity.
@@ -573,7 +597,7 @@ class MultiLeader(DXFGraphic):
 
         return explode_entity(self, target_layout)
 
-    def __virtual_entities__(self) -> Iterable["DXFGraphic"]:
+    def __virtual_entities__(self) -> Iterator[DXFGraphic]:
         """Support for "VirtualEntities" protocol."""
         from ezdxf.render import mleader
 
@@ -600,16 +624,16 @@ class MultiLeader(DXFGraphic):
             if handle is not None:
                 yield handle
 
-    def transform(self, m: "Matrix44") -> "MultiLeader":
+    def transform(self, m: Matrix44) -> MultiLeader:
         """Transform the MULTILEADER entity by transformation matrix `m` inplace.
 
-        Non uniform scaling is not supported.
+        Non-uniform scaling is not supported.
 
         Args:
             m: transformation :class:`~ezdxf.math.Matrix44`
 
         Raises:
-            NonUniformScalingError: for non uniform scaling
+            NonUniformScalingError: for non-uniform scaling
 
         """
         dxf = self.dxf
@@ -619,17 +643,14 @@ class MultiLeader(DXFGraphic):
         if not wcs.has_uniform_xy_scaling:
             # caller has to catch this exception and explode the MULTILEADER
             raise NonUniformScalingError(
-                "MULTILEADER does not support non uniform scaling"
+                "MULTILEADER does not support non-uniform scaling"
             )
-        if (
-            abs(context.plane_x_axis.z) > 1e-12
-            or abs(context.plane_y_axis.z) > 1e-12
-        ):
+        if abs(context.plane_x_axis.z) > 1e-12 or abs(context.plane_y_axis.z) > 1e-12:
             # check only if not parallel to xy-plane
             if not wcs.has_uniform_xyz_scaling:
                 # caller has to catch this exception and explode the MULTILEADER
                 raise NonUniformScalingError(
-                    "MULTILEADER does not support non uniform scaling"
+                    "MULTILEADER does not support non-uniform scaling"
                 )
         context.transform(wcs)
         # copy redundant attributes from sub-structures:
@@ -678,8 +699,8 @@ class MLeaderContext:
         273: "bottom_attachment",
     }
 
-    def __init__(self):
-        self.leaders: List["LeaderData"] = []
+    def __init__(self) -> None:
+        self.leaders: list[LeaderData] = []
         self.scale: float = 1.0  # overall scale
 
         # MTEXT base point: is not the MTEXT insertion point!
@@ -714,7 +735,7 @@ class MLeaderContext:
         self.bottom_attachment = 9
 
     @classmethod
-    def load(cls, context: List[Union["DXFTag", List]]) -> "MLeaderContext":
+    def load(cls, context: list[Union[DXFTag, list]]) -> MLeaderContext:
         assert context[0] == (START_CONTEXT_DATA, CONTEXT_STR)
         ctx = cls()
         content = None
@@ -742,6 +763,20 @@ class MLeaderContext:
                     ctx.__setattr__(name, cast_value(code, value))
         return ctx
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        if self.mtext is not None:
+            self.mtext.register_resources(registry)
+        if self.block is not None:
+            self.block.register_resources(registry)
+
+    def map_resources(self, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        if self.mtext is not None:
+            self.mtext.map_resources(mapping)
+        if self.block is not None:
+            self.block.map_resources(mapping)
+
     @property
     def is_valid(self) -> bool:
         return True
@@ -753,7 +788,7 @@ class MLeaderContext:
             z_axis = -z_axis
         return z_axis
 
-    def export_dxf(self, tagwriter: "TagWriter") -> None:
+    def export_dxf(self, tagwriter: AbstractTagWriter) -> None:
         write_tag2 = tagwriter.write_tag2
         write_vertex = tagwriter.write_vertex
         write_tag2(START_CONTEXT_DATA, CONTEXT_STR)
@@ -803,15 +838,9 @@ class MLeaderContext:
         self.char_height *= scale
         self.arrow_head_size *= scale
         self.landing_gap_size *= scale
-        self.plane_origin = m.transform(
-            self.plane_origin
-        )  # confirmed by BricsCAD
-        self.plane_x_axis = m.transform_direction(
-            self.plane_x_axis, normalize=True
-        )
-        self.plane_y_axis = m.transform_direction(
-            self.plane_y_axis, normalize=True
-        )
+        self.plane_origin = m.transform(self.plane_origin)  # confirmed by BricsCAD
+        self.plane_x_axis = m.transform_direction(self.plane_x_axis, normalize=True)
+        self.plane_y_axis = m.transform_direction(self.plane_y_axis, normalize=True)
         self.plane_normal_reversed = 0
         z_axis = m.transform_direction(Z_AXIS, normalize=True)
         if z_axis.isclose(-self.plane_z_axis):  # reversed z-axis?
@@ -864,7 +893,7 @@ class MTextData:
         295: "use_word_break",
     }
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.default_content: str = ""
         self.extrusion: Vec3 = Z_AXIS
         self.style_handle: str = "0"  # handle of TextStyle() table entry
@@ -888,9 +917,17 @@ class MTextData:
         self.column_width: float = 0.0  # not scaled
         self.column_gutter_width: float = 0.0  # not scaled
         self.column_flow_reversed: int = 0
-        self.column_sizes: List[float] = []  # heights?, not scaled
+        self.column_sizes: list[float] = []  # heights?, not scaled
         self.use_word_break: int = 1
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        registry.add_handle(self.style_handle)
+
+    def map_resources(self, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        self.style_handle = mapping.get_handle(self.style_handle)
+
     def parse(self, code: int, value) -> bool:
         # return True if data belongs to mtext else False (end of mtext section)
         if code == 144:
@@ -901,7 +938,7 @@ class MTextData:
             self.__setattr__(attrib, cast_value(code, value))
         return bool(attrib)
 
-    def export_dxf(self, tagwriter: "TagWriter") -> None:
+    def export_dxf(self, tagwriter: AbstractTagWriter) -> None:
         write_tag2 = tagwriter.write_tag2
         write_vertex = tagwriter.write_vertex
         write_tag2(304, safe_string(self.default_content, EXT_MAX_STR_LEN))
@@ -943,19 +980,29 @@ class MTextData:
         # don't use rotation ;)
         self.rotation = ocs.transform_angle(self.rotation)
         scale = wcs.uniform_scale
+        if math.isclose(scale, 1.0) or abs(scale) <= 1e-12:
+            return
         self.width *= scale
         self.defined_height *= scale
         self.column_width *= scale
         self.column_gutter_width *= scale
         self.column_sizes = [size * scale for size in self.column_sizes]
+        self.default_content = scale_mtext_inline_commands(
+            self.default_content, scale
+        )
 
     def apply_conversion_factor(self, conversion_factor: float):
         # conversion_factor: convert from an old scaling to a new scaling
+        if math.isclose(conversion_factor, 1.0) or abs(conversion_factor) <= 1e-12:
+            return
         self.width *= conversion_factor
         self.defined_height *= conversion_factor
         self.column_width *= conversion_factor
         self.column_gutter_width *= conversion_factor
         self.column_sizes = [h * conversion_factor for h in self.column_sizes]
+        self.default_content = scale_mtext_inline_commands(
+            self.default_content, conversion_factor
+        )
 
 
 class BlockData:
@@ -968,8 +1015,8 @@ class BlockData:
         93: "color",
     }
 
-    def __init__(self):
-        self.block_record_handle = None
+    def __init__(self) -> None:
+        self.block_record_handle: Optional[str] = None
         self.extrusion: Vec3 = Z_AXIS
         self.insert: Vec3 = NULLVEC
         self.scale: Vec3 = Vec3(1, 1, 1)
@@ -977,7 +1024,16 @@ class BlockData:
         self.color: int = colors.BY_BLOCK_RAW_VALUE
         # The transformation matrix is stored in transposed order
         # of ezdxf.math.Matrix44()!
-        self._matrix: List[float] = []  # group code 47 x 16
+        self._matrix: list[float] = []  # group code 47 x 16
+
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        registry.add_handle(self.block_record_handle)
+
+    def map_resources(self, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        assert self.block_record_handle is not None
+        self.block_record_handle = mapping.get_handle(self.block_record_handle)
 
     @property
     def matrix44(self) -> Matrix44:
@@ -1005,7 +1061,7 @@ class BlockData:
         # return True if data belongs to block else False (end of block section)
         return True
 
-    def export_dxf(self, tagwriter: "TagWriter") -> None:
+    def export_dxf(self, tagwriter: AbstractTagWriter) -> None:
         write_tag2 = tagwriter.write_tag2
         write_vertex = tagwriter.write_vertex
         if self.block_record_handle:
@@ -1045,8 +1101,8 @@ class BlockData:
 
 
 class LeaderData:
-    def __init__(self):
-        self.lines: List["LeaderLine"] = []
+    def __init__(self) -> None:
+        self.lines: list[LeaderLine] = []
         # has_last_leader_line:
         # in AutoCAD the leader is invisible if set to 0
         # BricsCAD ignores this flag
@@ -1059,16 +1115,14 @@ class LeaderData:
 
         # 0=horizontal; 1=vertical
         self.attachment_direction: int = 0  # group code 271, R2010+
-        self.breaks: List[
-            Vec3
-        ] = []  # group code 12, 13 - multiple breaks possible!
+        self.breaks: list[Vec3] = []  # group code 12, 13 - multiple breaks possible!
 
     @property
     def has_horizontal_attachment(self) -> bool:
         return not bool(self.attachment_direction)
 
     @classmethod
-    def load(cls, context: List[Union["DXFTag", List]]):
+    def load(cls, context: list[Union[DXFTag, list]]):
         assert context[0] == (START_LEADER, LEADER_STR)
         leader = cls()
         for tag in context:
@@ -1096,7 +1150,7 @@ class LeaderData:
 
         return leader
 
-    def export_dxf(self, tagwriter: "TagWriter") -> None:
+    def export_dxf(self, tagwriter: AbstractTagWriter) -> None:
         write_tag2 = tagwriter.write_tag2
         write_vertex = tagwriter.write_vertex
 
@@ -1139,9 +1193,9 @@ class LeaderData:
 
 
 class LeaderLine:
-    def __init__(self):
-        self.vertices: List[Vec3] = []  # WCS coordinates
-        self.breaks: List[Union[int, Vec3]] = []
+    def __init__(self) -> None:
+        self.vertices: list[Vec3] = []  # WCS coordinates
+        self.breaks: list[Union[int, Vec3]] = []
         # Breaks: 90, 11, 12, [11, 12, ...] [, 90, 11, 12 [11, 12, ...]]
         # group code 90 = break index
         # group code 11 = start vertex of break
@@ -1152,7 +1206,7 @@ class LeaderLine:
         # R2010+: override properties see ODA DWG pg. 214-215
 
     @classmethod
-    def load(cls, tags: List["DXFTag"]):
+    def load(cls, tags: list[DXFTag]):
         assert tags[0] == (START_LEADER_LINE, LEADER_LINE_STR)
         line = LeaderLine()
         vertices = line.vertices
@@ -1170,7 +1224,7 @@ class LeaderLine:
             line.breaks = breaks
         return line
 
-    def export_dxf(self, tagwriter: "TagWriter") -> None:
+    def export_dxf(self, tagwriter: AbstractTagWriter) -> None:
         write_tag2 = tagwriter.write_tag2
         write_vertex = tagwriter.write_vertex
 
@@ -1196,7 +1250,7 @@ class LeaderLine:
         m = wcs.m
         self.vertices = list(m.transform_vertices(self.vertices))
         if self.breaks:
-            breaks: List[Union[int, Vec3]] = []
+            breaks: list[Union[int, Vec3]] = []
             for value in self.breaks:
                 if isinstance(value, Vec3):
                     breaks.append(m.transform(value))
@@ -1266,6 +1320,13 @@ acdb_mleader_style = DefSubclass(
 )
 acdb_mleader_style_group_codes = group_code_mapping(acdb_mleader_style)
 
+MLEADER_STYLE_HANDLE_ATTRIBS = (
+    "leader_linetype_handle",
+    "arrow_head_handle",
+    "text_style_handle",
+    "block_record_handle",
+)
+
 
 @register_entity
 class MLeaderStyle(DXFObject):
@@ -1274,8 +1335,8 @@ class MLeaderStyle(DXFObject):
     MIN_DXF_VERSION_FOR_EXPORT = const.DXF2000
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             processor.fast_load_dxfattribs(
@@ -1283,6 +1344,21 @@ class MLeaderStyle(DXFObject):
             )
         return dxf
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        super().register_resources(registry)
+        dxf = self.dxf
+        for attrib_name in MLEADER_STYLE_HANDLE_ATTRIBS:
+            registry.add_handle(dxf.get(attrib_name, "0"))
+
+    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+        super().map_resources(clone, mapping)
+        dxf = self.dxf
+        for attrib_name in MLEADER_STYLE_HANDLE_ATTRIBS:
+            if dxf.hasattr(attrib_name):
+                clone.dxf.set(attrib_name, mapping.get_handle(dxf.get(attrib_name)))
+            else:
+                clone.dxf.discard(attrib_name)
+
     def set_mtext_style(self, name: str) -> None:
         assert self.doc is not None, "valid DXF document required"
         style = self.doc.styles.get(name)
@@ -1312,20 +1388,16 @@ class MLeaderStyle(DXFObject):
 
         assert self.doc is not None, "valid DXF document required"
         if name:
-            self.dxf.arrow_head_handle = ARROWS.arrow_handle(
-                self.doc.blocks, name
-            )
+            self.dxf.arrow_head_handle = ARROWS.arrow_handle(self.doc.blocks, name)
         else:
             # empty string is the default "closed filled" arrow,
             # no handle needed
             del self.dxf.arrow_head_handle
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         super().export_entity(tagwriter)
         tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_mleader_style.name)
-        self.dxf.export_dxf_attribs(
-            tagwriter, acdb_mleader_style.attribs.keys()
-        )
+        self.dxf.export_dxf_attribs(tagwriter, acdb_mleader_style.attribs.keys())
 
     def __referenced_blocks__(self) -> Iterable[str]:
         """Support for "ReferencedBlocks" protocol."""
@@ -1334,7 +1406,7 @@ class MLeaderStyle(DXFObject):
             if handle is not None:
                 yield handle
 
-    def audit(self, auditor: "Auditor") -> None:
+    def audit(self, auditor: Auditor) -> None:
         from ezdxf.audit import AuditError
 
         if not self.is_alive:
@@ -1352,7 +1424,7 @@ class MLeaderStyle(DXFObject):
                     AuditError.UNDEFINED_TEXT_STYLE,
                     f"{name}: text_style_handle={handle} is not valid, replaced by "
                     f"'Standard' text style",
-                    self
+                    self,
                 )
             else:
                 logger.warning("required text style 'Standard' does not exist")
@@ -1360,7 +1432,7 @@ class MLeaderStyle(DXFObject):
                 auditor.fixed_error(
                     AuditError.UNDEFINED_TEXT_STYLE,
                     f"{name}: removed invalid text_style_handle={handle}",
-                    self
+                    self,
                 )
         for attrib in ("arrow_head_handle", "block_record_handle"):
             handle = dxf.get(attrib)
@@ -1371,15 +1443,13 @@ class MLeaderStyle(DXFObject):
                 auditor.fixed_error(
                     AuditError.UNDEFINED_BLOCK,
                     f"{name}: removed invalid {attrib}={handle}",
-                    self
+                    self,
                 )
 
 
 class MLeaderStyleCollection(ObjectCollection[MLeaderStyle]):
-    def __init__(self, doc: "Drawing"):
-        super().__init__(
-            doc, dict_name="ACAD_MLEADERSTYLE", object_type="MLEADERSTYLE"
-        )
+    def __init__(self, doc: Drawing):
+        super().__init__(doc, dict_name="ACAD_MLEADERSTYLE", object_type="MLEADERSTYLE")
         self.create_required_entries()
 
     def create_required_entries(self) -> None:
