@@ -1,7 +1,13 @@
 # Copyright (c) 2018-2022, Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
-from typing import TYPE_CHECKING, Dict, Iterable, List, Optional, Sequence, Iterator
+from typing import (
+    TYPE_CHECKING,
+    Iterable,
+    Optional,
+    Sequence,
+    Iterator,
+)
 from collections import OrderedDict, namedtuple
 import math
 
@@ -27,23 +33,20 @@ from .objectcollection import ObjectCollection
 import logging
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        TagWriter,
-        Drawing,
-        DXFNamespace,
-        EntityQuery,
-        BaseLayout,
-        Matrix44,
-        Auditor,
-        DXFEntity,
-    )
+    from ezdxf.audit import Auditor
+    from ezdxf.document import Drawing
+    from ezdxf.entities import DXFNamespace, DXFEntity
+    from ezdxf.layouts import BaseLayout
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.math import Matrix44
+    from ezdxf.query import EntityQuery
+    from ezdxf import xref
 
 __all__ = ["MLine", "MLineVertex", "MLineStyle", "MLineStyleCollection"]
+logger = logging.getLogger("ezdxf")
 
 # Usage example: CADKitSamples\Lock-Off.dxf
 
-logger = logging.getLogger("ezdxf")
-
 
 def filter_close_vertices(
     vertices: Iterable[Vec3], abs_tol: float = 1e-12
@@ -160,7 +163,7 @@ acdb_mline_group_codes = group_code_mapping(acdb_mline)
 
 
 class MLineVertex:
-    def __init__(self):
+    def __init__(self) -> None:
         self.location: Vec3 = NULLVEC
         self.line_direction: Vec3 = X_AXIS
         self.miter_direction: Vec3 = Y_AXIS
@@ -188,7 +191,7 @@ class MLineVertex:
         # 1. line element: [miter-offset, line-start-offset, dash, gap, dash, ...]
         # 2. line element: [...]
         # ...
-        self.line_params: List[Sequence[float]] = []
+        self.line_params: list[Sequence[float]] = []
         """ The line parameterization is a list of float values.
         The list may contain zero or more items.
         """
@@ -211,9 +214,9 @@ class MLineVertex:
         # start.
         #
         # [dash-length, gap-length, ...]?
-        self.fill_params: List[Sequence[float]] = []
+        self.fill_params: list[Sequence[float]] = []
 
-    def __copy__(self) -> "MLineVertex":
+    def __copy__(self) -> MLineVertex:
         vtx = self.__class__()
         vtx.location = self.location
         vtx.line_direction = self.line_direction
@@ -227,9 +230,9 @@ class MLineVertex:
     @classmethod
     def load(cls, tags: Tags) -> MLineVertex:
         vtx = MLineVertex()
-        line_params: List[float] = []
+        line_params: list[float] = []
         line_params_count = 0
-        fill_params: List[float] = []
+        fill_params: list[float] = []
         fill_params_count = 0
         for code, value in tags:
             if code == 11:
@@ -263,7 +266,7 @@ class MLineVertex:
                     vtx.fill_params.append(tuple(fill_params))
         return vtx
 
-    def export_dxf(self, tagwriter: TagWriter):
+    def export_dxf(self, tagwriter: AbstractTagWriter):
         tagwriter.write_vertex(11, self.location)
         tagwriter.write_vertex(12, self.line_direction)
         tagwriter.write_vertex(13, self.miter_direction)
@@ -281,9 +284,9 @@ class MLineVertex:
         start: UVec,
         line_direction: UVec,
         miter_direction: UVec,
-        line_params: Iterable = None,
-        fill_params: Iterable = None,
-    ) -> "MLineVertex":
+        line_params: Optional[Iterable[Sequence[float]]] = None,
+        fill_params: Optional[Iterable[Sequence[float]]] = None,
+    ) -> MLineVertex:
         vtx = MLineVertex()
         vtx.location = Vec3(start)
         vtx.line_direction = Vec3(line_direction)
@@ -291,9 +294,7 @@ class MLineVertex:
         vtx.line_params = list(line_params or [])
         vtx.fill_params = list(fill_params or [])
         if len(vtx.line_params) != len(vtx.fill_params):
-            raise const.DXFValueError(
-                "Count mismatch of line- and fill parameters"
-            )
+            raise const.DXFValueError("Count mismatch of line- and fill parameters")
         return vtx
 
     def transform(self, m: Matrix44) -> MLineVertex:
@@ -308,6 +309,7 @@ class MLineVertex:
 class MLine(DXFGraphic):
     DXFTYPE = "MLINE"
     DXFATTRIBS = DXFAttributes(base_class, acdb_entity, acdb_mline)
+    MIN_DXF_VERSION_FOR_EXPORT = const.DXF2000
     TOP = const.MLINE_TOP
     ZERO = const.MLINE_ZERO
     BOTTOM = const.MLINE_BOTTOM
@@ -316,24 +318,24 @@ class MLine(DXFGraphic):
     SUPPRESS_START_CAPS = const.MLINE_SUPPRESS_START_CAPS
     SUPPRESS_END_CAPS = const.MLINE_SUPPRESS_END_CAPS
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         # The MLINE geometry stored in vertices, is the final geometry,
         # scaling factor, justification and MLineStyle settings are already
         # applied. This is why the geometry has to be updated every time a
         # change is applied.
-        self.vertices: List[MLineVertex] = []
+        self.vertices: list[MLineVertex] = []
 
     def __len__(self):
         """Count of MLINE vertices."""
         return len(self.vertices)
 
-    def _copy_data(self, entity: DXFEntity) -> None:
+    def copy_data(self, entity: DXFEntity) -> None:
         assert isinstance(entity, MLine)
         entity.vertices = [v.copy() for v in self.vertices]
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
+        self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
@@ -348,13 +350,13 @@ class MLine(DXFGraphic):
             MLineVertex.load(tags) for tags in group_tags(tags, splitcode=11)
         )
 
-    def preprocess_export(self, tagwriter: TagWriter) -> bool:
+    def preprocess_export(self, tagwriter: AbstractTagWriter) -> bool:
         # Do not export MLines without vertices
         return len(self.vertices) > 1
         # todo: check if line- and fill parametrization is compatible with
         #  MLINE style, requires same count of elements!
 
-    def export_entity(self, tagwriter: TagWriter) -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         # ezdxf does not export MLINE entities without vertices,
         # see method preprocess_export()
         self.set_flag_state(self.HAS_VERTICES, True)
@@ -363,10 +365,29 @@ class MLine(DXFGraphic):
         self.dxf.export_dxf_attribs(tagwriter, acdb_mline.attribs.keys())
         self.export_vertices(tagwriter)
 
-    def export_vertices(self, tagwriter: TagWriter) -> None:
+    def export_vertices(self, tagwriter: AbstractTagWriter) -> None:
         for vertex in self.vertices:
             vertex.export_dxf(tagwriter)
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        super().register_resources(registry)
+        registry.add_handle(self.dxf.style_handle)
+
+    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        super().map_resources(clone, mapping)
+        style = mapping.get_reference_of_copy(self.dxf.style_handle)
+        if not isinstance(style, MLineStyle):
+            assert clone.doc is not None
+            style = clone.doc.mline_styles.get("Standard")
+        if isinstance(style, MLineStyle):
+            clone.dxf.style_handle = style.dxf.handle
+            clone.dxf.style_name = style.dxf.name
+        else:
+            clone.dxf.style_handle = "0"
+            clone.dxf.style_name = "Standard"
+
     @property
     def is_closed(self) -> bool:
         """Returns ``True`` if MLINE is closed.
@@ -429,7 +450,7 @@ class MLine(DXFGraphic):
         _style = self.doc.entitydb.get(self.dxf.style_handle)
         if _style is None:
             _style = self.doc.mline_styles.get(self.dxf.style_name)
-        return _style
+        return _style  # type: ignore
 
     def set_style(self, name: str) -> None:
         """Set MLINESTYLE by name and update geometry accordingly.
@@ -468,7 +489,7 @@ class MLine(DXFGraphic):
         else:
             return NULLVEC
 
-    def get_locations(self) -> List[Vec3]:
+    def get_locations(self) -> list[Vec3]:
         """Returns the vertices of the reference line."""
         return [v.location for v in self.vertices]
 
@@ -493,7 +514,7 @@ class MLine(DXFGraphic):
         """Regenerate the MLINE geometry based on current settings."""
         self.generate_geometry(self.get_locations())
 
-    def generate_geometry(self, vertices: List[Vec3]) -> None:
+    def generate_geometry(self, vertices: list[Vec3]) -> None:
         """Regenerate the MLINE geometry for new reference line defined by
         `vertices`.
         """
@@ -508,9 +529,7 @@ class MLine(DXFGraphic):
         style = self.style
         assert style is not None, "valid MLINE style required"
         if len(style.elements) == 0:
-            raise const.DXFStructureError(
-                f"No line elements defined in {str(style)}."
-            )
+            raise const.DXFStructureError(f"No line elements defined in {str(style)}.")
 
         def miter(dir1: Vec3, dir2: Vec3):
             return ((dir1 + dir2) * 0.5).normalize().orthogonal()
@@ -522,7 +541,7 @@ class MLine(DXFGraphic):
         )
         # Transform given vertices into UCS and project them into the
         # UCS-xy-plane by setting the z-axis to 0:
-        vertices = [v.replace(z=0) for v in ucs.points_from_wcs(vertices)]
+        vertices = [v.replace(z=0.) for v in ucs.points_from_wcs(vertices)]
         start_angle = style.dxf.start_angle
         end_angle = style.dxf.end_angle
 
@@ -579,8 +598,7 @@ class MLine(DXFGraphic):
             except ZeroDivisionError:
                 stretch = 1.0
             vertex.line_params = [
-                ((element.offset + shift) * stretch, 0.0)
-                for element in style.elements
+                ((element.offset + shift) * stretch, 0.0) for element in style.elements
             ]
             vertex.fill_params = [tuple() for _ in style.elements]
 
@@ -588,7 +606,7 @@ class MLine(DXFGraphic):
         """Remove all MLINE vertices."""
         self.vertices.clear()
 
-    def remove_dependencies(self, other: Drawing = None) -> None:
+    def remove_dependencies(self, other: Optional[Drawing] = None) -> None:
         """Remove all dependencies from current document.
 
         (internal API)
@@ -612,9 +630,9 @@ class MLine(DXFGraphic):
         self.dxf.extrusion = m.transform_direction(self.dxf.extrusion)
         scale = self.dxf.scale_factor
         scale_vec = m.transform_direction(Vec3(scale, scale, scale))
-        if math.isclose(
-            scale_vec.x, scale_vec.y, abs_tol=1e-6
-        ) and math.isclose(scale_vec.y, scale_vec.z, abs_tol=1e-6):
+        if math.isclose(scale_vec.x, scale_vec.y, abs_tol=1e-6) and math.isclose(
+            scale_vec.y, scale_vec.z, abs_tol=1e-6
+        ):
             self.dxf.scale_factor = sum(scale_vec) / 3  # average error
         # None uniform scaling will not be applied to the scale_factor!
         self.update_geometry()
@@ -628,29 +646,32 @@ class MLine(DXFGraphic):
         the method :meth:`virtual_entities`!
         """
         from ezdxf.render.mline import virtual_entities
+
         for e in virtual_entities(self):
             e.set_source_of_copy(self)
             yield e
 
     def virtual_entities(self) -> Iterator[DXFGraphic]:
-        """Yields 'virtual' parts of MLINE as LINE, ARC and HATCH entities.
+        """Yields virtual DXF primitives of the MLINE entity  as LINE, ARC and HATCH
+        entities.
 
-        This entities are located at the original positions, but are not stored
+        These entities are located at the original positions, but are not stored
         in the entity database, have no handle and are not assigned to any
         layout.
 
         """
         return self.__virtual_entities__()
 
-    def explode(self, target_layout: BaseLayout = None) -> EntityQuery:
-        """Explode parts of MLINE as LINE, ARC and HATCH entities into target
+    def explode(self, target_layout: Optional[BaseLayout] = None) -> EntityQuery:
+        """Explode the MLINE entity as LINE, ARC and HATCH entities into target
         layout, if target layout is ``None``, the target layout is the layout
-        of the MLINE.
+        of the MLINE. This method destroys the source entity.
 
-        Returns an :class:`~ezdxf.query.EntityQuery` container with all DXF parts.
+        Returns an :class:`~ezdxf.query.EntityQuery` container referencing all DXF
+        primitives.
 
         Args:
-            target_layout: target layout for DXF parts, ``None`` for same layout
+            target_layout: target layout for DXF primitives, ``None`` for same layout
                 as source entity.
         """
         from ezdxf.explode import explode_entity
@@ -677,7 +698,7 @@ class MLine(DXFGraphic):
 
         # Audit associated MLINESTYLE name and handle:
         style = doc.entitydb.get(self.dxf.style_handle)
-        if style is None:  # handle is invalid, get style by name
+        if not isinstance(style, MLineStyle):  # handle is invalid, get style by name
             style = doc.mline_styles.get(self.dxf.style_name, None)
             if style is None:
                 reset_mline_style()
@@ -766,8 +787,8 @@ MLineStyleElement = namedtuple("MLineStyleElement", "offset color linetype")
 
 
 class MLineStyleElements:
-    def __init__(self, tags: Tags = None):
-        self.elements: List[MLineStyleElement] = []
+    def __init__(self, tags: Optional[Tags] = None):
+        self.elements: list[MLineStyleElement] = []
         if tags:
             for e in self.parse_tags(tags):
                 data = MLineStyleElement(
@@ -777,6 +798,12 @@ class MLineStyleElements:
                 )
                 self.elements.append(data)
 
+    def copy(self) -> MLineStyleElements:
+        elements = MLineStyleElements()
+        # new list of immutable data
+        elements.elements = list(self.elements)
+        return elements
+
     def __len__(self):
         return len(self.elements)
 
@@ -786,7 +813,7 @@ class MLineStyleElements:
     def __iter__(self):
         return iter(self.elements)
 
-    def export_dxf(self, tagwriter: TagWriter):
+    def export_dxf(self, tagwriter: AbstractTagWriter):
         write_tag = tagwriter.write_tag2
         write_tag(71, len(self.elements))
         for offset, color, linetype in self.elements:
@@ -794,9 +821,7 @@ class MLineStyleElements:
             write_tag(62, color)
             write_tag(6, linetype)
 
-    def append(
-        self, offset: float, color: int = 0, linetype: str = "BYLAYER"
-    ) -> None:
+    def append(self, offset: float, color: int = 0, linetype: str = "BYLAYER") -> None:
         """Append a new line element.
 
         Args:
@@ -812,7 +837,7 @@ class MLineStyleElements:
         )
 
     @staticmethod
-    def parse_tags(tags: Tags) -> Iterator[Dict]:
+    def parse_tags(tags: Tags) -> Iterator[dict]:
         collector = None
         for code, value in tags:
             if code == 49:
@@ -826,7 +851,7 @@ class MLineStyleElements:
         if collector is not None:
             yield collector
 
-    def ordered_indices(self) -> List[int]:
+    def ordered_indices(self) -> list[int]:
         offsets = [e.offset for e in self.elements]
         return [offsets.index(value) for value in sorted(offsets)]
 
@@ -848,11 +873,12 @@ class MLineStyle(DXFObject):
         super().__init__()
         self.elements = MLineStyleElements()
 
-    def copy(self):
-        raise const.DXFTypeError("Copying of MLINESTYLE not supported.")
+    def copy_data(self, entity: DXFEntity) -> None:
+        assert isinstance(entity, MLineStyle)
+        entity.elements = self.elements.copy()
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
+        self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
@@ -872,12 +898,10 @@ class MLineStyle(DXFObject):
                 self.elements = MLineStyleElements(tags[index71 + 1 :])  # type: ignore
                 # Remove processed tags:
                 del tags[index71:]
-            processor.fast_load_dxfattribs(
-                dxf, acdb_mline_style_group_codes, tags
-            )
+            processor.fast_load_dxfattribs(dxf, acdb_mline_style_group_codes, tags)
         return dxf
 
-    def export_entity(self, tagwriter: TagWriter) -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         super().export_entity(tagwriter)
         tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_mline_style.name)
         self.dxf.export_dxf_attribs(tagwriter, acdb_mline_style.attribs.keys())
@@ -892,14 +916,12 @@ class MLineStyle(DXFObject):
         """
         if self.doc:
             handle = self.dxf.handle
-            mlines = (
-                e for e in self.doc.entitydb.values() if e.dxftype() == "MLINE"
-            )
+            mlines = (e for e in self.doc.entitydb.values() if e.dxftype() == "MLINE")
             for mline in mlines:
                 if mline.dxf.style_handle == handle:
                     mline.update_geometry()
 
-    def ordered_indices(self) -> List[int]:
+    def ordered_indices(self) -> list[int]:
         return self.elements.ordered_indices()
 
     def audit(self, auditor: Auditor) -> None:
@@ -911,12 +933,29 @@ class MLineStyle(DXFObject):
                 dxf_entity=self,
             )
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        super().register_resources(registry)
+        for element in self.elements:
+            registry.add_linetype(element.linetype)
+
+    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        assert isinstance(clone, MLineStyle)
+        super().map_resources(clone, mapping)
+        self.elements.elements = [
+            MLineStyleElement(
+                element.offset,
+                element.color,
+                mapping.get_linetype(element.linetype),
+            )
+            for element in self.elements
+        ]
+
 
 class MLineStyleCollection(ObjectCollection[MLineStyle]):
     def __init__(self, doc: Drawing):
-        super().__init__(
-            doc, dict_name="ACAD_MLINESTYLE", object_type="MLINESTYLE"
-        )
+        super().__init__(doc, dict_name="ACAD_MLINESTYLE", object_type="MLINESTYLE")
         self.create_required_entries()
 
     def create_required_entries(self) -> None:
