@@ -1,20 +1,18 @@
 # Copyright (c) 2019-2022 Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
-import enum
-import math
-import logging
 from typing import (
     TYPE_CHECKING,
     Union,
-    Tuple,
-    List,
     Iterable,
     Iterator,
     Optional,
     Callable,
     cast,
 )
+import enum
+import math
+import logging
 
 from ezdxf.lldxf import const, validator
 from ezdxf.lldxf.attributes import (
@@ -42,6 +40,7 @@ from ezdxf.tools.text import (
     escape_dxf_line_endings,
     fast_plain_mtext,
     plain_mtext,
+    scale_mtext_inline_commands,
 )
 from . import factory
 from .dxfentity import base_class, SubclassProcessor
@@ -49,14 +48,12 @@ from .dxfgfx import DXFGraphic, acdb_entity
 from .xdata import XData
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        TagWriter,
-        DXFNamespace,
-        DXFEntity,
-        Auditor,
-        Drawing,
-        EntityDB,
-    )
+    from ezdxf.audit import Auditor
+    from ezdxf.document import Drawing
+    from ezdxf.entities import DXFNamespace, DXFEntity
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.entitydb import EntityDB
+    from ezdxf import xref
 
 __all__ = [
     "MText",
@@ -66,7 +63,6 @@ __all__ = [
     "acdb_mtext_group_codes",
     "export_mtext_content",
 ]
-
 logger = logging.getLogger("ezdxf")
 
 BG_FILL_MASK = 1 + 2 + 16
@@ -242,7 +238,7 @@ class MTextColumns:
 
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.column_type: ColumnType = ColumnType.STATIC
         # The embedded object in R2018 does not store the column count for
         # column type DYNAMIC and auto_height is True!
@@ -257,11 +253,11 @@ class MTextColumns:
         self.total_width: float = 0.0
         self.total_height: float = 0.0
         # Storage for handles of linked MTEXT entities at loading stage:
-        self.linked_handles: Optional[List[str]] = None
+        self.linked_handles: Optional[list[str]] = None
         # Storage for linked MTEXT entities for DXF versions < R2018:
-        self.linked_columns: List["MText"] = []
+        self.linked_columns: list[MText] = []
         # R2018+: heights of all columns if auto_height is False
-        self.heights: List[float] = []
+        self.heights: list[float] = []
 
     def deep_copy(self) -> MTextColumns:
         columns = self.shallow_copy()
@@ -331,9 +327,7 @@ class MTextColumns:
     def update_total_width(self):
         count = self.count
         if count > 0:
-            self.total_width = (
-                count * self.width + (count - 1) * self.gutter_width
-            )
+            self.total_width = count * self.width + (count - 1) * self.gutter_width
         else:
             self.total_width = 0.0
 
@@ -415,7 +409,7 @@ class MTextColumns:
         tags.append(DXFTag(1000, "ACAD_MTEXT_COLUMNS_END"))
         return tags
 
-    def mtext_handles(self) -> List[str]:
+    def mtext_handles(self) -> list[str]:
         """Returns a list of all linked MTEXT handles."""
         if self.linked_handles:
             return self.linked_handles
@@ -424,9 +418,7 @@ class MTextColumns:
             if column.is_alive:
                 handle = column.dxf.handle
                 if handle is None:
-                    raise const.DXFStructureError(
-                        "Linked MTEXT column has no handle."
-                    )
+                    raise const.DXFStructureError("Linked MTEXT column has no handle.")
                 handles.append(handle)
             else:
                 raise const.DXFStructureError("Linked MTEXT column deleted!")
@@ -545,8 +537,8 @@ def load_mtext_column_info(tags: Tags) -> Optional[MTextColumns]:
     return columns
 
 
-def load_mtext_linked_column_handles(tags: Tags) -> List[str]:
-    handles: List[str] = []
+def load_mtext_linked_column_handles(tags: Tags) -> list[str]:
+    handles: list[str] = []
     try:
         start, end = find_begin_and_end_of_encoded_xdata_tags(
             "ACAD_MTEXT_COLUMNS", tags
@@ -584,9 +576,7 @@ def load_mtext_defined_height(tags: Tags) -> float:
     return height
 
 
-def load_columns_from_xdata(
-    dxf: DXFNamespace, xdata: XData
-) -> Optional[MTextColumns]:
+def load_columns_from_xdata(dxf: DXFNamespace, xdata: XData) -> Optional[MTextColumns]:
     # The ACAD section in XDATA of the main MTEXT entity stores all column
     # related information:
     if "ACAD" in xdata:
@@ -624,11 +614,11 @@ def load_columns_from_xdata(
     return columns
 
 
-def extract_mtext_text_frame_handles(xdata: XData) -> List[str]:
+def extract_mtext_text_frame_handles(xdata: XData) -> list[str]:
     # Stores information about the text frame until DXF R2018.
     # Newer CAD applications do not need that information nor the separated
     # LWPOLYLINE as text frame entity.
-    handles: List[str] = []
+    handles: list[str] = []
     if "ACAD" in xdata:
         acad = xdata.get("ACAD")
     else:
@@ -661,7 +651,7 @@ class MText(DXFGraphic):
     DXFATTRIBS = DXFAttributes(base_class, acdb_entity, acdb_mtext)
     MIN_DXF_VERSION_FOR_EXPORT = DXF2000
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         self.text: str = ""
         # Linked MText columns do not have a MTextColumns() object!
@@ -678,7 +668,7 @@ class MText(DXFGraphic):
         return self.text
 
     def _set_text(self, value):
-        """Setter for virtual Mtext.dxf.text attribute. """
+        """Setter for virtual Mtext.dxf.text attribute."""
         self.text = str(value)
 
     @property
@@ -692,7 +682,7 @@ class MText(DXFGraphic):
     def has_columns(self) -> bool:
         return self._columns is not None
 
-    def _copy_data(self, entity: DXFEntity) -> None:
+    def copy_data(self, entity: DXFEntity) -> None:
         assert isinstance(entity, MText)
         entity.text = self.text
         if self.has_columns:
@@ -700,7 +690,7 @@ class MText(DXFGraphic):
             entity._columns = self._columns.deep_copy()  # type: ignore
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
+        self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
@@ -751,7 +741,7 @@ class MText(DXFGraphic):
             return unlink_mtext_columns_from_layout
         return None
 
-    def preprocess_export(self, tagwriter: TagWriter) -> bool:
+    def preprocess_export(self, tagwriter: AbstractTagWriter) -> bool:
         """Pre requirement check and pre-processing for export.
 
         Returns False if MTEXT should not be exported at all.
@@ -761,9 +751,7 @@ class MText(DXFGraphic):
         columns = self._columns
         if columns and tagwriter.dxfversion < const.DXF2018:
             if columns.count != len(columns.linked_columns) + 1:
-                logger.debug(
-                    f"{str(self)}: column count does not match linked columns"
-                )
+                logger.debug(f"{str(self)}: column count does not match linked columns")
                 # just log for debugging, because AutoCAD accept this!
             if not all(column.is_alive for column in columns.linked_columns):
                 logger.debug(f"{str(self)}: contains destroyed linked columns")
@@ -771,13 +759,13 @@ class MText(DXFGraphic):
             self.sync_common_attribs_of_linked_columns()
         return True
 
-    def export_dxf(self, tagwriter: TagWriter) -> None:
+    def export_dxf(self, tagwriter: AbstractTagWriter) -> None:
         super().export_dxf(tagwriter)
         # Linked MTEXT entities are not stored in the layout entity space!
         if self.has_columns and tagwriter.dxfversion < const.DXF2018:
             self.export_linked_entities(tagwriter)
 
-    def export_entity(self, tagwriter: TagWriter) -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         super().export_entity(tagwriter)
         tagwriter.write_tag2(SUBCLASS_MARKER, acdb_mtext.name)
@@ -835,7 +823,7 @@ class MText(DXFGraphic):
         parts.append(tail)
         self.text = escape_dxf_line_endings("".join(parts))
 
-    def export_embedded_object(self, tagwriter: TagWriter):
+    def export_embedded_object(self, tagwriter: AbstractTagWriter):
         dxf = self.dxf
         cols = self._columns
         assert cols is not None
@@ -863,12 +851,10 @@ class MText(DXFGraphic):
         for height in cols.heights:
             tagwriter.write_tag2(46, height)
 
-    def export_linked_entities(self, tagwriter: TagWriter):
+    def export_linked_entities(self, tagwriter: AbstractTagWriter):
         for mtext in self._columns.linked_columns:  # type: ignore
             if mtext.dxf.handle is None:
-                raise const.DXFStructureError(
-                    "Linked MTEXT column has no handle."
-                )
+                raise const.DXFStructureError("Linked MTEXT column has no handle.")
             # Export linked columns as separated DXF entities:
             mtext.export_dxf(tagwriter)
 
@@ -904,10 +890,7 @@ class MText(DXFGraphic):
                 column.set_xdata("ACAD", tags)
 
     def get_rotation(self) -> float:
-        """Get text rotation in degrees, independent if it is defined by
-        :attr:`dxf.rotation` or :attr:`dxf.text_direction`.
-
-        """
+        """Returns the text rotation in degrees."""
         if self.dxf.hasattr("text_direction"):
             vector = self.dxf.text_direction
             radians = math.atan2(vector[1], vector[0])  # ignores z-axis
@@ -917,9 +900,8 @@ class MText(DXFGraphic):
         return rotation
 
     def set_rotation(self, angle: float) -> MText:
-        """Set attribute :attr:`rotation` to `angle` (in degrees) and deletes
+        """Sets the attribute :attr:`rotation` to `angle` (in degrees) and deletes
         :attr:`dxf.text_direction` if present.
-
         """
         # text_direction has higher priority than rotation, therefore delete it
         self.dxf.discard("text_direction")
@@ -929,10 +911,10 @@ class MText(DXFGraphic):
     def set_location(
         self,
         insert: UVec,
-        rotation: float = None,
-        attachment_point: int = None,
+        rotation: Optional[float] = None,
+        attachment_point: Optional[int] = None,
     ) -> MText:
-        """Set attributes :attr:`dxf.insert`, :attr:`dxf.rotation` and
+        """Sets the attributes :attr:`dxf.insert`, :attr:`dxf.rotation` and
         :attr:`dxf.attachment_point`, ``None`` for :attr:`dxf.rotation` or
         :attr:`dxf.attachment_point` preserves the existing value.
 
@@ -950,19 +932,15 @@ class MText(DXFGraphic):
         scale: float = 1.5,
         text_frame=False,
     ):
-        """Set background color as :ref:`ACI` value or as name string or as RGB
-        tuple ``(r, g, b)``.
+        """Sets the background color as :ref:`ACI` value, as name string or as
+        (r, g, b) tuple.
 
-        Use special color name ``canvas``, to set background color to canvas
-        background color.
-
-        Use `color` = ``None`` to remove the background filling.
-
-        Setting only a text border is supported (`color`=``None``), but in this
-        case the scaling is always 1.5.
+        Use the special color name ``canvas``, to set the background color to the canvas
+        background color.  Remove the background filling by setting argument `color` to
+        ``None``.
 
         Args:
-            color: color as :ref:`ACI`, string, RGB tuple or ``None``
+            color: color as :ref:`ACI`, string, (r, g, b) tuple or ``None``
             scale: determines how much border there is around the text, the
                 value is based on the text height, and should be in the range
                 of [1, 5], where 1 fits exact the MText entity.
@@ -999,7 +977,7 @@ class MText(DXFGraphic):
                 self.dxf.bg_fill_true_color = rgb2int(color)
         return self  # fluent interface
 
-    def __iadd__(self, text: str) -> "MText":
+    def __iadd__(self, text: str) -> MText:
         """Append `text` to existing content (:attr:`text` attribute)."""
         self.text += text
         return self
@@ -1054,10 +1032,19 @@ class MText(DXFGraphic):
         new_text_direction = m.transform_direction(old_text_direction)
 
         old_vertical_direction = old_extrusion.cross(old_text_direction)
-        old_char_height_vec = old_vertical_direction.normalize(dxf.char_height)
+        old_char_height = float(dxf.char_height)
+        old_char_height_vec = old_vertical_direction.normalize(old_char_height)
         new_char_height_vec = m.transform_direction(old_char_height_vec)
         oblique = new_text_direction.angle_between(new_char_height_vec)
-        dxf.char_height = new_char_height_vec.magnitude * math.sin(oblique)
+        new_char_height = new_char_height_vec.magnitude * math.sin(oblique)
+        dxf.char_height = new_char_height
+        if (
+            not math.isclose(old_char_height, new_char_height)
+            and abs(old_char_height) > 1e-12
+        ):
+            factor = new_char_height / old_char_height
+            # Column content is transformed by the sub-entities itself!
+            self.text = scale_mtext_inline_commands(self.text, factor)
 
         if dxf.hasattr("width"):
             width_vec = old_text_direction.normalize(dxf.width)
@@ -1067,17 +1054,13 @@ class MText(DXFGraphic):
         dxf.text_direction = new_text_direction
         dxf.extrusion = new_extrusion
         if self.has_columns:
-            hscale = m.transform_direction(
-                old_text_direction.normalize()
-            ).magnitude
-            vscale = m.transform_direction(
-                old_vertical_direction.normalize()
-            ).magnitude
+            hscale = m.transform_direction(old_text_direction.normalize()).magnitude
+            vscale = m.transform_direction(old_vertical_direction.normalize()).magnitude
             self._columns.transform(m, hscale, vscale)  # type: ignore
         self.post_transform(m)
         return self
 
-    def plain_text(self, split=False, fast=True) -> Union[List[str], str]:
+    def plain_text(self, split=False, fast=True) -> Union[list[str], str]:
         """Returns the text content without inline formatting codes.
 
         The "fast" mode is accurate if the DXF content was created by
@@ -1092,16 +1075,13 @@ class MText(DXFGraphic):
             fast: uses the "fast" mode to extract the plain MTEXT content if
                 ``True`` or the "accurate" mode if set to ``False``
 
-        .. versionadded:: 0.16.6
-            `fast` argument
-
         """
         if fast:
             return fast_plain_mtext(self.text, split=split)
         else:
             return plain_mtext(self.text, split=split)
 
-    def all_columns_plain_text(self, split=False) -> Union[List[str], str]:
+    def all_columns_plain_text(self, split=False) -> Union[list[str], str]:
         """Returns the text content of all columns without inline formatting
         codes.
 
@@ -1109,8 +1089,6 @@ class MText(DXFGraphic):
             split: split content text at line breaks if ``True`` and
                 returns a list of strings without line endings
 
-        .. versionadded:: 0.17
-
         """
 
         def merged_content():
@@ -1140,8 +1118,6 @@ class MText(DXFGraphic):
         """Returns the text content of all columns as a single string
         including the inline formatting codes.
 
-        .. versionadded:: 0.17
-
         """
         content = [self.text]
         if self.has_columns:
@@ -1194,9 +1170,7 @@ class MText(DXFGraphic):
                 if entity.is_alive:
                     func(entity)
 
-    def setup_columns(
-        self, columns: MTextColumns, linked: bool = False
-    ) -> None:
+    def setup_columns(self, columns: MTextColumns, linked: bool = False) -> None:
         assert columns.column_type != ColumnType.NONE
         assert columns.count > 0, "one or more columns required"
         assert columns.width > 0, "column width has to be > 0"
@@ -1234,7 +1208,7 @@ class MText(DXFGraphic):
             column.dxf.insert = insert
             linked_columns.append(column)
 
-    def remove_dependencies(self, other: Drawing = None) -> None:
+    def remove_dependencies(self, other: Optional[Drawing] = None) -> None:
         if not self.is_alive:
             return
 
@@ -1252,8 +1226,30 @@ class MText(DXFGraphic):
         # different way!
         return OCS()
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        super().register_resources(registry)
+        if self.dxf.hasattr("style"):
+            registry.add_text_style(self.dxf.style)
+        if self._columns:
+            for mtext in self._columns.linked_columns:
+                mtext.register_resources(registry)
 
-def export_mtext_content(text, tagwriter: TagWriter) -> None:
+    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        assert isinstance(clone, MText)
+        super().map_resources(clone, mapping)
+
+        if clone.dxf.hasattr("style"):
+            clone.dxf.style = mapping.get_text_style(clone.dxf.style)
+        if self._columns and clone._columns:
+            for col_self, col_clone in zip(
+                self._columns.linked_columns, clone._columns.linked_columns
+            ):
+                col_self.map_resources(col_clone, mapping)
+
+
+def export_mtext_content(text, tagwriter: AbstractTagWriter) -> None:
     txt = escape_dxf_line_endings(text)
     str_chunks = split_mtext_string(txt, size=250)
     if len(str_chunks) == 0:
