@@ -6,11 +6,9 @@ from typing import (
     Iterable,
     Iterator,
     Union,
-    List,
     cast,
-    Tuple,
     Sequence,
-    Dict,
+    Optional,
 )
 from itertools import chain
 from ezdxf.lldxf import validator
@@ -38,16 +36,11 @@ from .lwpolyline import FORMAT_CODES
 from .subentity import LinkedEntities
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        TagWriter,
-        FaceType,
-        DXFNamespace,
-        Line,
-        Arc,
-        Face3d,
-        BaseLayout,
-        Auditor,
-    )
+    from ezdxf.audit import Auditor
+    from ezdxf.entities import DXFNamespace, Line, Arc, Face3d
+    from ezdxf.layouts import BaseLayout
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.eztypes import FaceType
 
 __all__ = ["Polyline", "Polyface", "Polymesh"]
 
@@ -131,7 +124,14 @@ merged_polyline_group_codes = merge_group_code_mappings(
 
 @factory.register_entity
 class Polyline(LinkedEntities):
-    """DXF POLYLINE entity"""
+    """DXF POLYLINE entity
+
+    The POLYLINE entity is hard owner of its VERTEX entities and the SEQEND entity:
+
+       VERTEX.dxf.owner == POLYLINE.dxf.handle
+       SEQEND.dxf.owner == POLYLINE.dxf.handle
+
+    """
 
     DXFTYPE = "POLYLINE"
     DXFATTRIBS = DXFAttributes(base_class, acdb_entity, acdb_polyline)
@@ -153,11 +153,11 @@ class Polyline(LinkedEntities):
     ANY3D = POLYLINE_3D | POLYMESH | POLYFACE
 
     @property
-    def vertices(self) -> List[DXFVertex]:
+    def vertices(self) -> list[DXFVertex]:
         return self._sub_entities  # type: ignore
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
+        self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
         """Loading interface. (internal API)"""
         # bypass DXFGraphic, loading proxy graphic is skipped!
@@ -166,13 +166,13 @@ class Polyline(LinkedEntities):
             processor.simple_dxfattribs_loader(dxf, merged_polyline_group_codes)
         return dxf
 
-    def export_dxf(self, tagwriter: TagWriter):
+    def export_dxf(self, tagwriter: AbstractTagWriter):
         """Export POLYLINE entity and all linked entities: VERTEX, SEQEND."""
         super().export_dxf(tagwriter)
         # export sub-entities
         self.process_sub_entities(lambda e: e.export_dxf(tagwriter))
 
-    def export_entity(self, tagwriter: TagWriter) -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export POLYLINE specific data as DXF tags."""
         super().export_entity(tagwriter)
         if tagwriter.dxfversion > DXF12:
@@ -223,10 +223,10 @@ class Polyline(LinkedEntities):
     def get_mode(self) -> str:
         """Returns POLYLINE type as string:
 
-        - 'AcDb2dPolyline'
-        - 'AcDb3dPolyline'
-        - 'AcDbPolygonMesh'
-        - 'AcDbPolyFaceMesh'
+        - "AcDb2dPolyline"
+        - "AcDb3dPolyline"
+        - "AcDbPolygonMesh"
+        - "AcDbPolyFaceMesh"
 
         """
         if self.is_3d_polyline:
@@ -282,8 +282,7 @@ class Polyline(LinkedEntities):
         """Returns ``True`` if 2D POLYLINE has an arc segment."""
         if self.is_2d_polyline:
             return any(
-                v.dxf.hasattr("bulge") and bool(v.dxf.bulge)
-                for v in self.vertices
+                v.dxf.hasattr("bulge") and bool(v.dxf.bulge) for v in self.vertices
             )
         else:
             return False
@@ -292,9 +291,6 @@ class Polyline(LinkedEntities):
     def has_width(self) -> bool:
         """Returns ``True`` if 2D POLYLINE has default width values or any
         segment with width attributes.
-
-        .. versionadded:: 0.14
-
         """
         if self.is_2d_polyline:
             if self.dxf.hasattr("default_start_width") and bool(
@@ -336,13 +332,12 @@ class Polyline(LinkedEntities):
         return len(self.vertices)
 
     def __getitem__(self, pos) -> DXFVertex:
-        """Get :class:`Vertex` entity at position `pos`, supports ``list``
-        slicing.
+        """Get :class:`Vertex` entity at position `pos`, supports list-like slicing.
         """
         return self.vertices[pos]
 
     def points(self) -> Iterator[Vec3]:
-        """Returns iterable of all polyline vertices as ``(x, y, z)`` tuples,
+        """Returns iterable of all polyline vertices as (x, y, z) tuples,
         not as :class:`Vertex` objects.
         """
         return (vertex.dxf.location for vertex in self.vertices)
@@ -350,13 +345,11 @@ class Polyline(LinkedEntities):
     def _append_vertex(self, vertex: DXFVertex) -> None:
         self.vertices.append(vertex)
 
-    def append_vertices(
-        self, points: Iterable[UVec], dxfattribs=None
-    ) -> None:
+    def append_vertices(self, points: Iterable[UVec], dxfattribs=None) -> None:
         """Append multiple :class:`Vertex` entities at location `points`.
 
         Args:
-            points: iterable of ``(x, y[, z])`` tuples
+            points: iterable of (x, y[, z]) tuples
             dxfattribs: dict of DXF attributes for the VERTEX objects
 
         """
@@ -375,14 +368,12 @@ class Polyline(LinkedEntities):
         Args:
             points: iterable of (x, y, [start_width, [end_width, [bulge]]])
                     tuple
-            format: format string, default is ``'xy'``, see: :ref:`format codes`
+            format: format string, default is "xy", see: :ref:`format codes`
             dxfattribs: dict of DXF attributes for the VERTEX objects
 
         """
         dxfattribs = dict(dxfattribs or {})
-        dxfattribs["flags"] = (
-            dxfattribs.get("flags", 0) | self.get_vertex_flags()
-        )
+        dxfattribs["flags"] = dxfattribs.get("flags", 0) | self.get_vertex_flags()
 
         # same DXF attributes for VERTEX entities as for POLYLINE
         dxfattribs["owner"] = self.dxf.owner
@@ -393,16 +384,14 @@ class Polyline(LinkedEntities):
         for point in points:
             attribs = vertex_attribs(point, format)
             attribs.update(dxfattribs)
-            vertex = cast(
-                DXFVertex, self._new_compound_entity("VERTEX", attribs)
-            )
+            vertex = cast(DXFVertex, self._new_compound_entity("VERTEX", attribs))
             self._append_vertex(vertex)
 
     def append_vertex(self, point: UVec, dxfattribs=None) -> None:
         """Append a single :class:`Vertex` entity at location `point`.
 
         Args:
-            point: as ``(x, y[, z])`` tuple
+            point: as (x, y[, z]) tuple
             dxfattribs: dict of DXF attributes for :class:`Vertex` class
 
         """
@@ -418,38 +407,32 @@ class Polyline(LinkedEntities):
 
         Args:
             pos: insertion position of list :attr:`Polyline.vertices`
-            points: list of ``(x, y[, z])`` tuples
+            points: list of (x, y[, z]) tuples
             dxfattribs: dict of DXF attributes for :class:`Vertex` class
 
         """
         dxfattribs = dict(dxfattribs or {})
-        self.vertices[pos:pos] = list(
-            self._build_dxf_vertices(points, dxfattribs)
-        )
+        self.vertices[pos:pos] = list(self._build_dxf_vertices(points, dxfattribs))
 
     def _build_dxf_vertices(
         self, points: Iterable[UVec], dxfattribs: dict
     ) -> Iterator[DXFVertex]:
-        """Converts point (x, y, z)-tuples into DXFVertex objects.
+        """Converts point (x, y, z) tuples into DXFVertex objects.
 
         Args:
             points: list of (x, y, z)-tuples
             dxfattribs: dict of DXF attributes
         """
-        dxfattribs["flags"] = (
-            dxfattribs.get("flags", 0) | self.get_vertex_flags()
-        )
+        dxfattribs["flags"] = dxfattribs.get("flags", 0) | self.get_vertex_flags()
 
         # same DXF attributes for VERTEX entities as for POLYLINE
-        dxfattribs["owner"] = self.dxf.owner
+        dxfattribs["owner"] = self.dxf.handle
         dxfattribs["layer"] = self.dxf.layer
         if self.dxf.hasattr("linetype"):
             dxfattribs["linetype"] = self.dxf.linetype
         for point in points:
             dxfattribs["location"] = Vec3(point)
-            yield cast(
-                DXFVertex, self._new_compound_entity("VERTEX", dxfattribs)
-            )
+            yield cast(DXFVertex, self._new_compound_entity("VERTEX", dxfattribs))
 
     def cast(self) -> Union[Polyline, Polymesh, Polyface]:
         mode = self.get_mode()
@@ -498,14 +481,13 @@ class Polyline(LinkedEntities):
             else:
                 z_axis = None
             vertices = [
-                ocs.transform_vertex(vertex)
-                for vertex in _ocs_locations(z_axis)
+                ocs.transform_vertex(vertex) for vertex in _ocs_locations(z_axis)
             ]
 
             # All vertices of a 2D polyline must have the same z-axis, which is
             # the elevation of the polyline:
             if vertices:
-                dxf.elevation = vertices[0].replace(x=0, y=0)
+                dxf.elevation = vertices[0].replace(x=0.0, y=0.0)
 
             for vertex, location in zip(self.vertices, vertices):
                 vdxf = vertex.dxf
@@ -516,13 +498,9 @@ class Polyline(LinkedEntities):
                     vdxf.end_width = ocs.transform_width(vdxf.end_width)
 
             if dxf.hasattr("default_start_width"):
-                dxf.default_start_width = ocs.transform_width(
-                    dxf.default_start_width
-                )
+                dxf.default_start_width = ocs.transform_width(dxf.default_start_width)
             if dxf.hasattr("default_end_width"):
-                dxf.default_end_width = ocs.transform_width(
-                    dxf.default_end_width
-                )
+                dxf.default_end_width = ocs.transform_width(dxf.default_end_width)
             if dxf.hasattr("thickness"):
                 dxf.thickness = ocs.transform_thickness(dxf.thickness)
 
@@ -533,12 +511,12 @@ class Polyline(LinkedEntities):
         self.post_transform(m)
         return self
 
-    def explode(self, target_layout: BaseLayout = None) -> EntityQuery:
+    def explode(self, target_layout: Optional[BaseLayout] = None) -> EntityQuery:
         """Explode the POLYLINE entity as DXF primitives (LINE, ARC or 3DFACE)
         into the target layout, if the target layout is ``None``, the target
         layout is the layout of the POLYLINE entity.
 
-        Returns an :class:`~ezdxf.query.EntityQuery` container of all
+        Returns an :class:`~ezdxf.query.EntityQuery` container referencing all
         DXF primitives.
 
         Args:
@@ -622,50 +600,44 @@ class Polyface(Polyline):
         return polyface
 
     def append_face(self, face: FaceType, dxfattribs=None) -> None:
-        """Append a single face. A `face` is a list of ``(x, y, z)`` tuples.
+        """Append a single face. A `face` is a sequence of (x, y, z) tuples.
 
         Args:
-            face: List[``(x, y, z)`` tuples]
-            dxfattribs: dict of DXF attributes for VERTEX objects
+            face: sequence of (x, y, z) tuples
+            dxfattribs: dict of DXF attributes for the VERTEX objects
 
         """
         self.append_faces([face], dxfattribs)
 
     def _points_to_dxf_vertices(
-        self, points: Iterable[UVec], dxfattribs: Dict
-    ) -> List[DXFVertex]:
-        """Convert (x, y, z)-tuples into DXFVertex objects.
+        self, points: Iterable[UVec], dxfattribs
+    ) -> list[DXFVertex]:
+        """Convert (x, y, z) tuples into DXFVertex objects.
 
         Args:
-            points: List[``(x, y, z)`` tuples]
-            dxfattribs: dict of DXF attributes for :class:`Vertex` entity
+            points: sequence of (x, y, z) tuples
+            dxfattribs: dict of DXF attributes for the VERTEX entity
 
         """
-        dxfattribs["flags"] = (
-            dxfattribs.get("flags", 0) | self.get_vertex_flags()
-        )
+        dxfattribs["flags"] = dxfattribs.get("flags", 0) | self.get_vertex_flags()
 
         # All vertices have to be on the same layer as the POLYLINE entity:
         dxfattribs["layer"] = self.get_dxf_attrib("layer", "0")
-        vertices: List[DXFVertex] = []
+        vertices: list[DXFVertex] = []
         for point in points:
             dxfattribs["location"] = point
             vertices.append(
-                cast(
-                    "DXFVertex", self._new_compound_entity("VERTEX", dxfattribs)
-                )
+                cast("DXFVertex", self._new_compound_entity("VERTEX", dxfattribs))
             )
         return vertices
 
-    def append_faces(
-        self, faces: Iterable[FaceType], dxfattribs=None
-    ) -> None:
+    def append_faces(self, faces: Iterable[FaceType], dxfattribs=None) -> None:
         """Append multiple `faces`. `faces` is a list of single faces and a
-        single face is a list of ``(x, y, z)`` tuples.
+        single face is a sequence of (x, y, z) tuples.
 
         Args:
-            faces: list of List[``(x, y, z)`` tuples]
-            dxfattribs: dict of DXF attributes for the VERTEX objects
+            faces: iterable of sequences of (x, y, z) tuples
+            dxfattribs: dict of DXF attributes for the VERTEX entity
 
         """
 
@@ -673,14 +645,12 @@ class Polyface(Polyline):
             dxfattribs["flags"] = const.VTX_3D_POLYFACE_MESH_VERTEX
             # location of face record vertex is always (0, 0, 0)
             dxfattribs["location"] = Vec3()
-            return cast(
-                DXFVertex, self._new_compound_entity("VERTEX", dxfattribs)
-            )
+            return cast(DXFVertex, self._new_compound_entity("VERTEX", dxfattribs))
 
         dxfattribs = dict(dxfattribs or {})
 
         existing_vertices, existing_faces = self.indexed_faces()
-        new_faces: List[FaceProxy] = []
+        new_faces: list[FaceProxy] = []
         for face in faces:
             face_mesh_vertices = self._points_to_dxf_vertices(face, {})
             # Index of first new vertex
@@ -689,15 +659,11 @@ class Polyface(Polyline):
             face_record = FaceProxy(new_face_record(), existing_vertices)
 
             # Set VERTEX indices:
-            face_record.indices = tuple(
-                range(index, index + len(face_mesh_vertices))
-            )
+            face_record.indices = tuple(range(index, index + len(face_mesh_vertices)))
             new_faces.append(face_record)
         self._rebuild(chain(existing_faces, new_faces))
 
-    def _rebuild(
-        self, faces: Iterable[FaceProxy], precision: int = 6
-    ) -> None:
+    def _rebuild(self, faces: Iterable[FaceProxy], precision: int = 6) -> None:
         """Build a valid POLYFACE structure from `faces`.
 
         Args:
@@ -705,7 +671,7 @@ class Polyface(Polyline):
 
         """
         polyface_builder = PolyfaceBuilder(faces, precision=precision)
-        # Why is List[DXFGraphic] incompatible to List[DXFVertex] when DXFVertex
+        # Why is list[DXFGraphic] incompatible to list[DXFVertex] when DXFVertex
         # is a subclass of DXFGraphic?
         self._sub_entities = polyface_builder.get_vertices()  # type: ignore
         self.update_count(polyface_builder.nvertices, polyface_builder.nfaces)
@@ -715,8 +681,8 @@ class Polyface(Polyline):
         self.dxf.n_count = nfaces
 
     def optimize(self, precision: int = 6) -> None:
-        """Rebuilds :class:`Polyface` including vertex optimization by merging
-        vertices with nearly same vertex locations.
+        """Rebuilds the :class:`Polyface` by merging vertices with nearly same vertex
+        locations.
 
         Args:
             precision: floating point precision for determining identical
@@ -726,11 +692,11 @@ class Polyface(Polyline):
         vertices, faces = self.indexed_faces()
         self._rebuild(faces, precision)
 
-    def faces(self) -> Iterator[List[DXFVertex]]:
+    def faces(self) -> Iterator[list[DXFVertex]]:
         """Iterable of all faces, a face is a tuple of vertices.
 
         Returns:
-             list: [vertex, vertex, vertex, [vertex,] face_record]
+             list of [vertex, vertex, vertex, [vertex,] face_record]
 
         """
         _, faces = self.indexed_faces()
@@ -739,22 +705,20 @@ class Polyface(Polyline):
             face_vertices.append(face.face_record)
             yield face_vertices
 
-    def indexed_faces(self) -> Tuple[List[DXFVertex], Iterator[FaceProxy]]:
+    def indexed_faces(self) -> tuple[list[DXFVertex], Iterator[FaceProxy]]:
         """Returns a list of all vertices and a generator of FaceProxy()
         objects.
 
         (internal API)
         """
-        vertices: List[DXFVertex] = []
-        face_records: List[DXFVertex] = []
+        vertices: list[DXFVertex] = []
+        face_records: list[DXFVertex] = []
         for vertex in self.vertices:
-            (
-                vertices if vertex.is_poly_face_mesh_vertex else face_records
-            ).append(vertex)
+            (vertices if vertex.is_poly_face_mesh_vertex else face_records).append(
+                vertex
+            )
 
-        faces = (
-            FaceProxy(face_record, vertices) for face_record in face_records
-        )
+        faces = (FaceProxy(face_record, vertices) for face_record in face_records)
         return vertices, faces
 
 
@@ -781,9 +745,7 @@ class FaceProxy:
 
     __slots__ = ("vertices", "face_record", "indices")
 
-    def __init__(
-        self, face_record: DXFVertex, vertices: Sequence[DXFVertex]
-    ):
+    def __init__(self, face_record: DXFVertex, vertices: Sequence[DXFVertex]):
         """Returns iterable of all face vertices as :class:`Vertex` entities."""
         self.vertices: Sequence[DXFVertex] = vertices
         self.face_record: DXFVertex = face_record
@@ -810,15 +772,10 @@ class FaceProxy:
         return (vertex.dxf.location for vertex in self)
 
     def _raw_indices(self) -> Iterable[int]:
-        return (
-            self.face_record.get_dxf_attrib(name, 0)
-            for name in const.VERTEXNAMES
-        )
+        return (self.face_record.get_dxf_attrib(name, 0) for name in const.VERTEXNAMES)
 
     def _indices(self) -> Sequence[int]:
-        return tuple(
-            abs(index) - 1 for index in self._raw_indices() if index != 0
-        )
+        return tuple(abs(index) - 1 for index in self._raw_indices() if index != 0)
 
     def is_edge_visible(self, pos: int) -> bool:
         """Returns ``True`` if edge starting at vertex `pos` is visible.
@@ -836,9 +793,9 @@ class PolyfaceBuilder:
 
     def __init__(self, faces: Iterable[FaceProxy], precision: int = 6):
         self.precision: int = precision
-        self.faces: List[DXFVertex] = []
-        self.vertices: List[DXFVertex] = []
-        self.index_mapping: Dict[Tuple[float, ...], int] = {}
+        self.faces: list[DXFVertex] = []
+        self.vertices: list[DXFVertex] = []
+        self.index_mapping: dict[tuple[float, ...], int] = {}
         self.build(faces)
 
     @property
@@ -849,7 +806,7 @@ class PolyfaceBuilder:
     def nfaces(self) -> int:
         return len(self.faces)
 
-    def get_vertices(self) -> List[DXFVertex]:
+    def get_vertices(self) -> list[DXFVertex]:
         vertices = self.vertices[:]
         vertices.extend(self.faces)
         return vertices
@@ -898,14 +855,12 @@ class Polymesh(Polyline):
         polymesh.seqend = polyline.seqend
         return polymesh
 
-    def set_mesh_vertex(
-        self, pos: Tuple[int, int], point: UVec, dxfattribs=None
-    ):
+    def set_mesh_vertex(self, pos: tuple[int, int], point: UVec, dxfattribs=None):
         """Set location and DXF attributes of a single mesh vertex.
 
         Args:
-            pos: 0-based (row, col)-tuple, position of mesh vertex
-            point: (x, y, z)-tuple, new 3D coordinates of the mesh vertex
+            pos: 0-based (row, col) tuple, position of mesh vertex
+            point: (x, y, z) tuple, new 3D coordinates of the mesh vertex
             dxfattribs: dict of DXF attributes
 
         """
@@ -914,11 +869,11 @@ class Polymesh(Polyline):
         vertex = self.get_mesh_vertex(pos)
         vertex.update_dxf_attribs(dxfattribs)
 
-    def get_mesh_vertex(self, pos: Tuple[int, int]) -> DXFVertex:
+    def get_mesh_vertex(self, pos: tuple[int, int]) -> DXFVertex:
         """Get location of a single mesh vertex.
 
         Args:
-            pos: 0-based ``(row, col)`` tuple, position of mesh vertex
+            pos: 0-based (row, col) tuple, position of mesh vertex
 
         """
         m_count = self.dxf.m_count
@@ -950,19 +905,19 @@ class MeshVertexCache:
     __slots__ = ("vertices",)
 
     def __init__(self, mesh: Polyline):
-        self.vertices: Dict[Tuple[int, int], DXFVertex] = self._setup(
+        self.vertices: dict[tuple[int, int], DXFVertex] = self._setup(
             mesh, mesh.dxf.m_count, mesh.dxf.n_count
         )
 
     def _setup(self, mesh: Polyline, m_count: int, n_count: int) -> dict:
-        cache: Dict[Tuple[int, int], DXFVertex] = {}
+        cache: dict[tuple[int, int], DXFVertex] = {}
         vertices = iter(mesh.vertices)
         for m in range(m_count):
             for n in range(n_count):
                 cache[(m, n)] = next(vertices)
         return cache
 
-    def __getitem__(self, pos: Tuple[int, int]) -> UVec:
+    def __getitem__(self, pos: tuple[int, int]) -> UVec:
         """Get mesh vertex location as (x, y, z)-tuple.
 
         Args:
@@ -974,7 +929,7 @@ class MeshVertexCache:
         except KeyError:
             raise const.DXFIndexError(repr(pos))
 
-    def __setitem__(self, pos: Tuple[int, int], location: UVec) -> None:
+    def __setitem__(self, pos: tuple[int, int], location: UVec) -> None:
         """Get mesh vertex location as (x, y, z)-tuple.
 
         Args:
@@ -1042,7 +997,7 @@ class DXFVertex(DXFGraphic):
     VTX3D = POLYLINE_3D_VERTEX + POLYGON_MESH_VERTEX + POLYFACE_MESH_VERTEX
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
+        self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
         """Loading interface. (internal API)"""
         # bypass DXFGraphic, loading proxy graphic is skipped!
@@ -1051,7 +1006,7 @@ class DXFVertex(DXFGraphic):
             processor.simple_dxfattribs_loader(dxf, merged_vertex_group_codes)
         return dxf
 
-    def export_entity(self, tagwriter: TagWriter) -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         # VERTEX can have 3 subclasses if representing a `face record` or
         # 4 subclasses if representing a vertex location, just the last
@@ -1063,17 +1018,11 @@ class DXFVertex(DXFGraphic):
             else:
                 tagwriter.write_tag2(SUBCLASS_MARKER, "AcDbVertex")
                 if self.is_3d_polyline_vertex:
-                    tagwriter.write_tag2(
-                        SUBCLASS_MARKER, "AcDb3dPolylineVertex"
-                    )
+                    tagwriter.write_tag2(SUBCLASS_MARKER, "AcDb3dPolylineVertex")
                 elif self.is_poly_face_mesh_vertex:
-                    tagwriter.write_tag2(
-                        SUBCLASS_MARKER, "AcDbPolyFaceMeshVertex"
-                    )
+                    tagwriter.write_tag2(SUBCLASS_MARKER, "AcDbPolyFaceMeshVertex")
                 elif self.is_polygon_mesh_vertex:
-                    tagwriter.write_tag2(
-                        SUBCLASS_MARKER, "AcDbPolygonMeshVertex"
-                    )
+                    tagwriter.write_tag2(SUBCLASS_MARKER, "AcDbPolygonMeshVertex")
                 else:
                     tagwriter.write_tag2(SUBCLASS_MARKER, "AcDb2dVertex")
 
@@ -1137,8 +1086,6 @@ class DXFVertex(DXFGraphic):
         Args:
             format: format string, default is "xyz"
 
-        .. versionadded:: 0.14
-
         """
         dxf = self.dxf
         v = Vec3(dxf.location)
@@ -1179,14 +1126,14 @@ def vertex_attribs(data: Sequence[float], format="xyseb") -> dict:
         if code == "v":
             location = Vec3(value)
         elif code == "b":
-            attribs["bulge"] = value
+            attribs["bulge"] = float(value)
         elif code == "s":
-            attribs["start_width"] = value
+            attribs["start_width"] = float(value)
         elif code == "e":
-            attribs["end_width"] = value
+            attribs["end_width"] = float(value)
         elif code == "x":
-            location = location.replace(x=value)
+            location = location.replace(x=float(value))
         elif code == "y":
-            location = location.replace(y=value)
+            location = location.replace(y=float(value))
     attribs["location"] = location
     return attribs
