@@ -1,16 +1,20 @@
-# Copyright (c) 2020-2021, Manfred Moitzi
+# Copyright (c) 2020-2022, Manfred Moitzi
 # License: MIT License
-from typing import TYPE_CHECKING, Iterable, Callable, List, Optional
+from __future__ import annotations
+from typing import TYPE_CHECKING, Iterable, Callable, Optional
 
 from ezdxf.entities import factory, DXFGraphic, SeqEnd, DXFEntity
 from ezdxf.lldxf import const
 import logging
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import DXFEntity, EntityDB, Drawing
+    from ezdxf.document import Drawing
+    from ezdxf.entities import DXFEntity
+    from ezdxf.entitydb import EntityDB
+    from ezdxf import xref
+
 
 __all__ = ["entity_linker", "LinkedEntities"]
-
 logger = logging.getLogger("ezdxf")
 
 
@@ -21,27 +25,29 @@ class LinkedEntities(DXFGraphic):
 
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
-        self._sub_entities: List[DXFGraphic] = []
-        self.seqend: Optional["SeqEnd"] = None
+        self._sub_entities: list[DXFGraphic] = []
+        self.seqend: Optional[SeqEnd] = None
 
-    def _copy_data(self, entity: "DXFEntity") -> None:
+    def copy_data(self, entity: DXFEntity) -> None:
         """Copy all sub-entities ands SEQEND. (internal API)"""
         assert isinstance(entity, LinkedEntities)
         entity._sub_entities = [e.copy() for e in self._sub_entities]
         if self.seqend:
             entity.seqend = self.seqend.copy()
 
-    def link_entity(self, entity: "DXFEntity") -> None:
+    def link_entity(self, entity: DXFEntity) -> None:
         """Link VERTEX to ATTRIB entities."""
         assert isinstance(entity, DXFGraphic)
-        entity.set_owner(self.dxf.owner, self.dxf.paperspace)
+        entity.set_owner(self.dxf.handle, self.dxf.paperspace)
         self._sub_entities.append(entity)
 
-    def link_seqend(self, seqend: "DXFEntity") -> None:
+    def link_seqend(self, seqend: DXFEntity) -> None:
         """Link SEQEND entity. (internal API)"""
         seqend.dxf.owner = self.dxf.owner
+        if self.seqend is not None:  # destroy existing seqend
+            self.seqend.destroy()
         self.seqend = seqend  # type: ignore
 
     def post_bind_hook(self):
@@ -49,24 +55,24 @@ class LinkedEntities(DXFGraphic):
         if self.seqend is None:
             self.new_seqend()
 
-    def all_sub_entities(self) -> Iterable["DXFEntity"]:
+    def all_sub_entities(self) -> Iterable[DXFEntity]:
         """Yields all sub-entities and SEQEND. (internal API)"""
         yield from self._sub_entities
         if self.seqend:
             yield self.seqend
 
-    def process_sub_entities(self, func: Callable[["DXFEntity"], None]):
+    def process_sub_entities(self, func: Callable[[DXFEntity], None]):
         """Call `func` for all sub-entities and SEQEND. (internal API)"""
         for entity in self.all_sub_entities():
             if entity.is_alive:
                 func(entity)
 
-    def add_sub_entities_to_entitydb(self, db: "EntityDB") -> None:
+    def add_sub_entities_to_entitydb(self, db: EntityDB) -> None:
         """Add sub-entities (VERTEX, ATTRIB, SEQEND) to entity database `db`,
         called from EntityDB. (internal API)
         """
 
-        def add(entity: "DXFEntity"):
+        def add(entity: DXFEntity):
             entity.doc = self.doc  # grant same document
             db.add(entity)
 
@@ -89,16 +95,18 @@ class LinkedEntities(DXFGraphic):
         # vertices/attrib entities are linked, so set_owner() of POLYLINE does
         # not set owner of vertices at loading time.
         super().set_owner(owner, paperspace)
+        self.take_ownership()
 
-        def set_owner(entity):
-            if isinstance(entity, DXFGraphic):
-                entity.set_owner(owner, paperspace)
-            else:  # SEQEND
-                entity.dxf.owner = owner
+    def take_ownership(self):
+        """Take ownership of all sub-entities and SEQEND. (internal API)"""
+        handle = self.dxf.handle
+        paperspace = self.dxf.paperspace
+        for entity in self.all_sub_entities():
+            if entity.is_alive:
+                entity.dxf.owner = handle
+                entity.dxf.paperspace = paperspace
 
-        self.process_sub_entities(set_owner)
-
-    def remove_dependencies(self, other: "Drawing" = None):
+    def remove_dependencies(self, other: Optional[Drawing] = None):
         """Remove all dependencies from current document to bind entity to
         `other` document. (internal API)
         """
@@ -115,6 +123,18 @@ class LinkedEntities(DXFGraphic):
         del self.seqend
         super().destroy()
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        super().register_resources(registry)
+        self.process_sub_entities(lambda e: e.register_resources(registry))
+
+    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        assert isinstance(clone, LinkedEntities)
+        super().map_resources(clone, mapping)
+        for source, _clone in zip(self.all_sub_entities(), clone.all_sub_entities()):
+            source.map_resources(_clone, mapping)
+
 
 LINKED_ENTITIES = {"INSERT": "ATTRIB", "POLYLINE": "VERTEX"}
 
