@@ -1,6 +1,7 @@
-# Copyright (c) 2019-2021, Manfred Moitzi
+# Copyright (c) 2019-2022, Manfred Moitzi
 # License: MIT-License
-from typing import TYPE_CHECKING
+from __future__ import annotations
+from typing import TYPE_CHECKING, Optional
 from ezdxf.lldxf import validator
 from ezdxf.lldxf.const import SUBCLASS_MARKER, DXF2007
 from ezdxf.lldxf.attributes import (
@@ -15,7 +16,8 @@ from .dxfobj import DXFObject
 from .factory import register_entity
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import TagWriter, DXFNamespace
+    from ezdxf.entities import DXFNamespace
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
 
 __all__ = ["Sun"]
 
@@ -77,14 +79,14 @@ class Sun(DXFObject):
     MIN_DXF_VERSION_FOR_EXPORT = DXF2007
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             processor.fast_load_dxfattribs(dxf, acdb_sun_group_codes, 1)
         return dxf
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         super().export_entity(tagwriter)
         tagwriter.write_tag2(SUBCLASS_MARKER, acdb_sun.name)
@@ -134,19 +136,19 @@ acdb_sunstudy = DefSubclass(
         # 95 Interval in seconds. If range of dates flag is true.
         "hours_count": DXFAttr(73),
         # 290 Hour. One entry for every hour as specified by the number of hours entry above.
-        "page_setup_wizard_id": DXFAttr(
+        "page_setup_wizard_handle": DXFAttr(
             340
         ),  # Page setup wizard hard pointer ID
-        "view_id": DXFAttr(341),  # View hard pointer ID
-        "visual_style_id": DXFAttr(342),  # Visual Style ID
+        "view_handle": DXFAttr(341),  # View hard pointer ID
+        "visual_style_handle": DXFAttr(342),  # Visual Style ID
         "shade_plot_type": DXFAttr(74),
         "viewports_per_page": DXFAttr(75),
-        "nrows": DXFAttr(76),  # Number of rows for viewport distribution
-        "ncols": DXFAttr(77),  # Number of columns for viewport distribution
+        "row_count": DXFAttr(76),  # Number of rows for viewport distribution
+        "column_count": DXFAttr(77),  # Number of columns for viewport distribution
         "spacing": DXFAttr(40),
         "lock_viewports": DXFAttr(293),
         "label_viewports": DXFAttr(294),
-        "text_style_id": DXFAttr(343),
+        "text_style_handle": DXFAttr(343),
     },
 )
 acdb_sunstudy_group_codes = group_code_mapping(acdb_sunstudy)
