@@ -1,13 +1,10 @@
-# Copyright (c) 2019-2021, Manfred Moitzi
+# Copyright (c) 2019-2023, Manfred Moitzi
 # License: MIT License
+from __future__ import annotations
 from typing import (
     Optional,
     Iterable,
-    Tuple,
     TYPE_CHECKING,
-    Dict,
-    Set,
-    List,
     Iterator,
 )
 from contextlib import contextmanager
@@ -21,7 +18,7 @@ from ezdxf.entities import factory
 from ezdxf.query import EntityQuery
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import TagWriter
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
 
 DATABASE_EXCLUDE = {
     "SECTION",
@@ -39,7 +36,7 @@ class EntityDB:
     """A simple key/entity database.
 
     Every entity/object, except tables and sections, are represented as
-    DXFEntity or inherited types, this entities are stored in the
+    DXFEntity or inherited types, these entities are stored in the
     DXF document database, database-key is the `handle` as string.
 
     """
@@ -47,9 +44,9 @@ class EntityDB:
     class Trashcan:
         """Store handles to entities which should be deleted later."""
 
-        def __init__(self, db: "EntityDB"):
+        def __init__(self, db: EntityDB):
             self._database = db._database
-            self._handles: Set[str] = set()
+            self._handles: set[str] = set()
 
         def add(self, handle: str):
             """Put handle into trashcan to delete the entity later, this is
@@ -72,8 +69,8 @@ class EntityDB:
 
             self._handles.clear()
 
-    def __init__(self):
-        self._database: Dict[str, DXFEntity] = {}
+    def __init__(self) -> None:
+        self._database: dict[str, DXFEntity] = {}
         # DXF handles of entities to delete later:
         self.handles = HandleGenerator()
         self.locked: bool = False  # used only for debugging
@@ -142,7 +139,7 @@ class EntityDB:
         """Iterable of all entities, does filter destroyed entities."""
         return (entity for handle, entity in self.items())
 
-    def items(self) -> Iterable[Tuple[str, DXFEntity]]:
+    def items(self) -> Iterable[tuple[str, DXFEntity]]:
         """Iterable of all (handle, entities) pairs, does filter destroyed
         entities.
         """
@@ -219,7 +216,7 @@ class EntityDB:
             doc.objects.add_object(new_entity)
         return new_entity
 
-    def audit(self, auditor: "Auditor"):
+    def audit(self, auditor: Auditor):
         """Restore database integrity:
 
         - restore database entries with modified handles (key != entity.dxf.handle)
@@ -269,14 +266,14 @@ class EntityDB:
                 continue
             self[handle] = entity
 
-    def new_trashcan(self) -> "EntityDB.Trashcan":
+    def new_trashcan(self) -> EntityDB.Trashcan:
         """Returns a new trashcan, empty trashcan manually by: :
         func:`Trashcan.clear()`.
         """
         return EntityDB.Trashcan(self)
 
     @contextmanager  # type: ignore
-    def trashcan(self) -> "EntityDB.Trashcan":  # type: ignore
+    def trashcan(self) -> EntityDB.Trashcan:  # type: ignore
         """Returns a new trashcan in context manager mode, trashcan will be
         emptied when leaving context.
         """
@@ -292,13 +289,11 @@ class EntityDB:
         """
         # Important: operate on underlying data structure:
         db = self._database
-        dead_handles = [
-            handle for handle, entity in db.items() if not entity.is_alive
-        ]
+        dead_handles = [handle for handle, entity in db.items() if not entity.is_alive]
         for handle in dead_handles:
             del db[handle]
 
-    def dxf_types_in_use(self) -> Set[str]:
+    def dxf_types_in_use(self) -> set[str]:
         return set(entity.dxftype() for entity in self.values())
 
     def reset_handle(self, entity: DXFEntity, handle: str) -> bool:
@@ -324,9 +319,7 @@ class EntityDB:
             :ref:`entity query string` and :ref:`entity queries`
 
         """
-        return EntityQuery(
-            (e for e in self._database.values() if e.is_alive), query
-        )
+        return EntityQuery((e for e in self._database.values() if e.is_alive), query)
 
 
 class EntitySpace:
@@ -340,8 +333,8 @@ class EntitySpace:
 
     """
 
-    def __init__(self, entities: Iterable[DXFEntity] = None):
-        self.entities: List[DXFEntity] = (
+    def __init__(self, entities: Optional[Iterable[DXFEntity]] = None):
+        self.entities: list[DXFEntity] = (
             list(e for e in entities if e.is_alive) if entities else []
         )
 
@@ -355,8 +348,8 @@ class EntitySpace:
         :class:`EntitySpace` has a standard Python list like interface,
         therefore `index` can be any valid list indexing or slicing term, like
         a single index ``layout[-1]`` to get the last entity, or an index slice
-        ``layout[:10]`` to get the first 10 or less entities as
-        ``List[DXFEntity]``. Does not filter destroyed entities.
+        ``layout[:10]`` to get the first 10 or fewer entities as
+        ``list[DXFEntity]``. Does not filter destroyed entities.
 
         """
         return self.entities[index]
@@ -385,7 +378,7 @@ class EntitySpace:
         for entity in entities:
             self.add(entity)
 
-    def export_dxf(self, tagwriter: "TagWriter") -> None:
+    def export_dxf(self, tagwriter: AbstractTagWriter) -> None:
         """Export all entities into DXF file by `tagwriter`.
 
         (internal API)
@@ -401,3 +394,38 @@ class EntitySpace:
         """Remove all entities."""
         # Do not destroy entities!
         self.entities = list()
+
+    def pop(self, index: int = -1) -> DXFEntity:
+        return self.entities.pop(index)
+
+    def insert(self, index: int, entity: DXFEntity) -> None:
+        self.entities.insert(index, entity)
+
+    def audit(self, auditor: Auditor) -> None:
+        db_get = auditor.entitydb.get
+        purge: list[DXFEntity] = []
+        # Check if every entity is the entity that is stored for this handle in the
+        # entity database.
+        for entity in self:
+            handle = entity.dxf.handle
+            if entity is not db_get(handle):
+                # A different entity is stored in the database for this handle,
+                # see issues #604 and #833:
+                # - document has entities without handles (invalid for DXF R13+)
+                # - $HANDSEED is not the next usable handle
+                # - entity gets an already used handle
+                # - entity overwrites existing entity or will be overwritten by an entity
+                #   loaded afterwards
+                auditor.fixed_error(
+                    AuditError.REMOVED_INVALID_DXF_OBJECT,
+                    f"Removed entity {entity} with a conflicting handle and without a "
+                    f"database entry.",
+                )
+                purge.append(entity)
+        if not purge:
+            return
+        for entity in purge:
+            self.entities.remove(entity)
+            # These are invalid entities do not call destroy() on them, because
+            # this method relies on well-defined entities!
+            entity._silent_kill()
