@@ -1,5 +1,6 @@
-# Copyright (c) 2020-2021, Manfred Moitzi
+# Copyright (c) 2020-2022, Manfred Moitzi
 # License: MIT License
+from __future__ import annotations
 import logging
 from typing import (
     TYPE_CHECKING,
@@ -7,11 +8,8 @@ from typing import (
     Callable,
     Optional,
     cast,
-    Dict,
-    List,
-    Any,
 )
-
+from ezdxf.lldxf import const
 from ezdxf.entities import factory
 from ezdxf.entities.boundary_paths import (
     PolylinePath,
@@ -21,7 +19,6 @@ from ezdxf.entities.boundary_paths import (
     EllipseEdge,
     SplineEdge,
 )
-from ezdxf.lldxf.const import DXFStructureError, DXFTypeError
 from ezdxf.math import OCS, Vec3, ABS_TOL
 from ezdxf.math.transformtools import (
     NonUniformScalingError,
@@ -29,18 +26,18 @@ from ezdxf.math.transformtools import (
 )
 from ezdxf.query import EntityQuery
 
-logger = logging.getLogger("ezdxf")
-
 if TYPE_CHECKING:
-    from ezdxf.entities.polygon import DXFPolygon
-    from ezdxf.eztypes import (
-        Insert,
-        BaseLayout,
+    from ezdxf.entities import (
         DXFGraphic,
+        Insert,
         Attrib,
         Text,
         LWPolyline,
     )
+    from ezdxf.entities.polygon import DXFPolygon
+    from ezdxf.layouts import BaseLayout
+
+logger = logging.getLogger("ezdxf")
 
 __all__ = [
     "virtual_block_reference_entities",
@@ -58,8 +55,8 @@ def default_logging_callback(entity, reason):
 
 
 def explode_block_reference(
-    block_ref: "Insert",
-    target_layout: "BaseLayout",
+    block_ref: Insert,
+    target_layout: BaseLayout,
     *,
     redraw_order=False,
 ) -> EntityQuery:
@@ -88,10 +85,10 @@ def explode_block_reference(
 
     """
     if target_layout is None:
-        raise DXFStructureError("Target layout is None.")
+        raise const.DXFStructureError("Target layout is None.")
 
     if block_ref.doc is None:
-        raise DXFStructureError(
+        raise const.DXFStructureError(
             "Block reference has to be assigned to a DXF document."
         )
 
@@ -120,7 +117,7 @@ def explode_block_reference(
         entitydb is not None
     ), "Exploding a block reference requires an entity database."
 
-    entities: List["DXFGraphic"] = []
+    entities: list[DXFGraphic] = []
     if block_ref.mcount > 1:
         for virtual_insert in block_ref.multi_insert():
             _explode_single_block_ref(virtual_insert)
@@ -147,7 +144,7 @@ IGNORE_FROM_ATTRIB = {
 }
 
 
-def attrib_to_text(attrib: "Attrib") -> "Text":
+def attrib_to_text(attrib: Attrib) -> Text:
     dxfattribs = attrib.dxfattribs(drop=IGNORE_FROM_ATTRIB)
     # ATTRIB has same owner as INSERT but does not reside in any EntitySpace()
     # and must not deleted from any layout.
@@ -160,13 +157,11 @@ def attrib_to_text(attrib: "Attrib") -> "Text":
 
 
 def virtual_block_reference_entities(
-    block_ref: "Insert",
+    block_ref: Insert,
     *,
-    skipped_entity_callback: Optional[
-        Callable[["DXFGraphic", str], None]
-    ] = None,
+    skipped_entity_callback: Optional[Callable[[DXFGraphic, str], None]] = None,
     redraw_order=False,
-) -> Iterable["DXFGraphic"]:
+) -> Iterable[DXFGraphic]:
     """Yields 'virtual' parts of block reference `block_ref`. This method is meant
     to examine the block reference entities without the need to explode the
     block reference. The `skipped_entity_callback()` will be called for all
@@ -199,7 +194,7 @@ def virtual_block_reference_entities(
         skipped_entity_callback or default_logging_callback
     )
 
-    def disassemble(layout) -> Iterable["DXFGraphic"]:
+    def disassemble(layout) -> Iterable[DXFGraphic]:
         for entity in (
             layout.entities_in_redraw_order() if redraw_order else layout
         ):
@@ -208,7 +203,7 @@ def virtual_block_reference_entities(
                 continue
             try:
                 copy = entity.copy()
-            except DXFTypeError:
+            except const.DXFTypeError:
                 if hasattr(entity, "virtual_entities"):
                     yield from entity.virtual_entities()
                 else:
@@ -240,12 +235,12 @@ def virtual_block_reference_entities(
                         entity, "unsupported non-uniform scaling"
                     )
             except InsertTransformationError:
-                # INSERT entity can not represented in the target coordinate
+                # INSERT entity can not be represented in the target coordinate
                 # system defined by transformation matrix `m`.
                 # Yield transformed sub-entities of the INSERT entity:
                 yield from transform(
                     virtual_block_reference_entities(
-                        entity, skipped_entity_callback
+                        entity, skipped_entity_callback=skipped_entity_callback
                     )
                 )
             else:
@@ -254,7 +249,7 @@ def virtual_block_reference_entities(
     m = block_ref.matrix44()
     block_layout = block_ref.block()
     if block_layout is None:
-        raise DXFStructureError(
+        raise const.DXFStructureError(
             f'Required block definition for "{block_ref.dxf.name}" does not exist.'
         )
 
@@ -265,8 +260,8 @@ EXCLUDE_FROM_EXPLODE = {"POINT"}
 
 
 def explode_entity(
-    entity: "DXFGraphic", target_layout: "BaseLayout" = None
-) -> "EntityQuery":
+    entity: DXFGraphic, target_layout: Optional[BaseLayout] = None
+) -> EntityQuery:
     """Explode parts of an entity as primitives into target layout, if target
     layout is ``None``, the target layout is the layout of the source entity.
 
@@ -284,23 +279,23 @@ def explode_entity(
     dxftype = entity.dxftype()
     virtual_entities = getattr(entity, "virtual_entities")
     if virtual_entities is None or dxftype in EXCLUDE_FROM_EXPLODE:
-        raise DXFTypeError(f"Can not explode entity {dxftype}.")
+        raise const.DXFTypeError(f"Can not explode entity {dxftype}.")
 
     if entity.doc is None:
-        raise DXFStructureError(
+        raise const.DXFStructureError(
             f"{dxftype} has to be assigned to a DXF document."
         )
 
     entitydb = entity.doc.entitydb
     if entitydb is None:
-        raise DXFStructureError(
+        raise const.DXFStructureError(
             f"Exploding {dxftype} requires an entity database."
         )
 
     if target_layout is None:
         target_layout = entity.get_layout()
         if target_layout is None:
-            raise DXFStructureError(
+            raise const.DXFStructureError(
                 f"{dxftype} without layout assignment, specify target layout."
             )
 
@@ -318,8 +313,8 @@ def explode_entity(
 
 
 def virtual_boundary_path_entities(
-    polygon: "DXFPolygon",
-) -> List[List["DXFGraphic"]]:
+    polygon: DXFPolygon,
+) -> list[list[DXFGraphic]]:
     from ezdxf.entities import LWPolyline
 
     def polyline():
@@ -345,8 +340,8 @@ def virtual_boundary_path_entities(
 
 
 def _virtual_edge_path(
-    path: EdgePath, dxfattribs: Dict, ocs: OCS, elevation: float
-) -> List["DXFGraphic"]:
+    path: EdgePath, dxfattribs, ocs: OCS, elevation: float
+) -> list[DXFGraphic]:
     from ezdxf.entities import Line, Arc, Ellipse, Spline
 
     def pnt_to_wcs(v):
@@ -355,7 +350,7 @@ def _virtual_edge_path(
     def dir_to_wcs(v):
         return ocs.to_wcs(v)
 
-    edges: List["DXFGraphic"] = []
+    edges: list[DXFGraphic] = []
     for edge in path.edges:
         attribs = dict(dxfattribs)
         if isinstance(edge, LineEdge):
@@ -366,7 +361,7 @@ def _virtual_edge_path(
             attribs["center"] = edge.center
             attribs["radius"] = edge.radius
             attribs["elevation"] = elevation
-            # Arcs angles are always stored in counter clockwise orientation
+            # Arcs angles are always stored in counter-clockwise orientation
             # around the extrusion vector!
             attribs["start_angle"] = edge.start_angle
             attribs["end_angle"] = edge.end_angle
@@ -376,7 +371,7 @@ def _virtual_edge_path(
             attribs["center"] = pnt_to_wcs(edge.center)
             attribs["major_axis"] = dir_to_wcs(edge.major_axis)
             attribs["ratio"] = edge.ratio
-            # Ellipse angles are always stored in counter clockwise orientation
+            # Ellipse angles are always stored in counter-clockwise orientation
             # around the extrusion vector!
             attribs["start_param"] = edge.start_param
             attribs["end_param"] = edge.end_param
