@@ -1,17 +1,18 @@
-# Copyright (C) 2018-2022, Manfred Moitzi
+# Copyright (C) 2018-2023, Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
-from typing import TextIO, TYPE_CHECKING, Union, Sequence
+from typing import TextIO, TYPE_CHECKING, Union, Sequence, Optional
 import base64
 import io
 import pathlib
+import os
 
 from ezdxf.tools.standards import setup_drawing
 from ezdxf.lldxf.const import DXF2013
 from ezdxf.document import Drawing
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import DXFInfo
+    from ezdxf.lldxf.validator import DXFInfo
 
 
 def new(
@@ -95,8 +96,8 @@ def read(stream: TextIO) -> Drawing:
 
 
 def readfile(
-    filename: Union[str, pathlib.Path],
-    encoding: str = None,
+    filename: str | os.PathLike,
+    encoding: Optional[str] = None,
     errors: str = "surrogateescape",
 ) -> Drawing:
     """Read the DXF document `filename` from the file-system.
@@ -161,11 +162,12 @@ def readfile(
     return doc
 
 
-def dxf_file_info(filename: str) -> DXFInfo:
+def dxf_file_info(filename: str | os.PathLike) -> DXFInfo:
     """Reads basic file information from a DXF document: DXF version, encoding
     and handle seed.
 
     """
+    filename = str(filename)
     with open(filename, mode="rt", encoding="utf-8", errors="ignore") as fp:
         return dxf_stream_info(fp)
 
@@ -185,8 +187,10 @@ def dxf_stream_info(stream: TextIO) -> DXFInfo:
 
 
 def readzip(
-    zipfile: str, filename: str = None, errors: str = "surrogateescape"
-) -> "Drawing":
+    zipfile: str | os.PathLike,
+    filename: Optional[str] = None,
+    errors: str = "surrogateescape",
+) -> Drawing:
     """Load a DXF document specified by `filename` from a zip archive, or if
     `filename` is ``None`` the first DXF document in the zip archive.
 
@@ -209,13 +213,13 @@ def readzip(
     """
     from ezdxf.tools.zipmanager import ctxZipReader
 
-    with ctxZipReader(zipfile, filename, errors=errors) as zipstream:  # type: ignore
-        doc = read(zipstream)
+    with ctxZipReader(str(zipfile), filename, errors=errors) as zipstream:
+        doc = read(zipstream)  # type: ignore
         doc.filename = zipstream.dxf_file_name
     return doc
 
 
-def decode_base64(data: bytes, errors: str = "surrogateescape") -> "Drawing":
+def decode_base64(data: bytes, errors: str = "surrogateescape") -> Drawing:
     """Load a DXF document from base64 encoded binary data, like uploaded data
     to web applications.
 
@@ -259,7 +263,7 @@ def decode_base64(data: bytes, errors: str = "surrogateescape") -> "Drawing":
 
 
 def find_support_file(
-    filename: str, support_dirs: Sequence[str] = None
+    filename: str, support_dirs: Optional[Sequence[str]] = None
 ) -> str:
     """Find file `filename` in the support directories`."""
     if pathlib.Path(filename).exists():
@@ -267,6 +271,7 @@ def find_support_file(
     if support_dirs is None:
         support_dirs = []
     for directory in support_dirs:
+        directory = directory.strip("\"'")
         filepath = pathlib.Path(directory).expanduser() / filename
         if filepath.exists():
             return str(filepath)
