@@ -5,8 +5,6 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Union,
-    List,
-    Tuple,
     cast,
     Optional,
 )
@@ -16,21 +14,14 @@ from ezdxf.lldxf import const
 from .base import BaseLayout
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        GeoData,
-        Viewport,
-        Drawing,
-        DXFLayout,
-        DXFGraphic,
-        BlockLayout,
-    )
+    from ezdxf.document import Drawing
+    from ezdxf.layouts import BlockLayout
+    from ezdxf.entities import GeoData, Viewport, DXFLayout, DXFGraphic
 
 
-def get_block_entity_space(
-    doc: "Drawing", block_record_handle: str
-) -> "EntitySpace":
+def get_block_entity_space(doc: Drawing, block_record_handle: str) -> EntitySpace:
     block_record = doc.entitydb[block_record_handle]
-    return block_record.entity_space
+    return block_record.entity_space  # type: ignore
 
 
 class Layout(BaseLayout):
@@ -77,17 +68,15 @@ class Layout(BaseLayout):
     INITIALIZING = 8192
     PREV_PLOT_INIT = 16384
 
-    def __init__(self, layout: "DXFLayout", doc: "Drawing"):
+    def __init__(self, layout: DXFLayout, doc: Drawing):
         self.dxf_layout = layout
         block_record = doc.entitydb[layout.dxf.block_record_handle]
         # link maybe broken
         block_record.dxf.layout = layout.dxf.handle
-        super().__init__(block_record)
+        super().__init__(block_record)  # type: ignore
 
     @classmethod
-    def new(
-        cls, name: str, block_name: str, doc: "Drawing", dxfattribs: dict = None
-    ) -> "Layout":
+    def new(cls, name: str, block_name: str, doc: Drawing, dxfattribs=None) -> Layout:
         """Returns the required structures for a new layout:
 
             - a :class:`BlockLayout` with BLOCK_RECORD, BLOCK and ENDBLK entities
@@ -102,7 +91,7 @@ class Layout(BaseLayout):
         (internal API)
 
         """
-        block_layout = doc.blocks.new(block_name)  # type: BlockLayout
+        block_layout: BlockLayout = doc.blocks.new(block_name)
         dxfattribs = dxfattribs or {}
         dxfattribs.update(
             {
@@ -114,7 +103,7 @@ class Layout(BaseLayout):
         return cls(dxf_layout, doc)  # type: ignore
 
     @classmethod
-    def load(cls, layout: "DXFLayout", doc: "Drawing"):
+    def load(cls, layout: DXFLayout, doc: Drawing):
         """Loading interface. (internal API)"""
         _layout = cls(layout, doc)
         _layout._repair_owner_tags()
@@ -153,7 +142,7 @@ class Layout(BaseLayout):
             if entity.dxf.paperspace != paperspace:
                 entity.dxf.paperspace = paperspace
 
-    def __contains__(self, entity: Union["DXFGraphic", str]) -> bool:
+    def __contains__(self, entity: Union[DXFGraphic, str]) -> bool:
         """Returns ``True`` if `entity` is stored in this layout.
 
         Args:
@@ -208,8 +197,8 @@ class Layout(BaseLayout):
         the paper size settings.
 
         Args:
-             extmin: minimum extents or (0, 0) as default
-             extmax: maximum extents or (paper width, paper height) as default value
+             limmin: minimum limits or (0, 0) as default
+             limmax: maximum limits or (paper width, paper height) as default value
 
         """
         dxf = self.dxf_layout.dxf
@@ -244,9 +233,7 @@ class Layout(BaseLayout):
         else:
             raise const.DXFValueError("Plot type value out of range (0-5).")
 
-    def set_plot_style(
-        self, name: str = "ezdxf.ctb", show: bool = False
-    ) -> None:
+    def set_plot_style(self, name: str = "ezdxf.ctb", show: bool = False) -> None:
         """Set plot style file of type `.ctb`.
 
         Args:
@@ -263,8 +250,8 @@ class Layout(BaseLayout):
 
     def set_plot_window(
         self,
-        lower_left: Tuple[float, float] = (0, 0),
-        upper_right: Tuple[float, float] = (0, 0),
+        lower_left: tuple[float, float] = (0, 0),
+        upper_right: tuple[float, float] = (0, 0),
     ) -> None:
         """Set plot window size in (scaled) paper space units.
 
@@ -282,6 +269,17 @@ class Layout(BaseLayout):
         dxf.plot_window_y2 = y2
         self.set_plot_type(4)
 
+    def get_plot_unit_scale_factor(self) -> float:
+        denom = self.dxf.scale_denominator
+        numerator = self.dxf.scale_numerator
+        scale = 1.0
+        if denom:
+            scale = numerator / denom
+        if self.dxf.plot_paper_units == 1:
+            return scale  # mm
+        else:
+            return scale * 25.4  # inch
+
     # plot layout flags setter
     def plot_viewport_borders(self, state: bool = True) -> None:
         self.set_plot_flags(self.PLOT_VIEWPORT_BORDERS, state)
@@ -326,16 +324,14 @@ class Layout(BaseLayout):
         self.set_plot_flags(self.PREV_PLOT_INIT, state)
 
     def set_plot_flags(self, flag, state: bool = True) -> None:
-        self.dxf_layout.set_flag_state(
-            flag, state=state, name="plot_layout_flags"
-        )
+        self.dxf_layout.set_flag_state(flag, state=state, name="plot_layout_flags")
 
 
 class Modelspace(Layout):
     """:class:`Modelspace` - not deletable, all entities of this layout are
     stored in the ENTITIES section of the DXF file, the associated
     "*Model_Space" block is empty, block name is fixed as "*Model_Space",
-    the name is fixed as ""Model".
+    the name is fixed as "Model".
 
     """
 
@@ -344,7 +340,7 @@ class Modelspace(Layout):
         """Name of modelspace is fixed as "Model"."""
         return "Model"
 
-    def new_geodata(self, dxfattribs: dict = None) -> "GeoData":
+    def new_geodata(self, dxfattribs=None) -> GeoData:
         """Creates a new :class:`GeoData` entity and replaces existing ones.
         The GEODATA entity resides in the OBJECTS section and not in the
         modelspace, it is linked to the modelspace by an
@@ -361,9 +357,7 @@ class Modelspace(Layout):
 
         """
         if self.doc.dxfversion < const.DXF2010:
-            raise const.DXFValueError(
-                "GEODATA entity requires DXF R2010 or later."
-            )
+            raise const.DXFValueError("GEODATA entity requires DXF R2010 or later.")
 
         if dxfattribs is None:
             dxfattribs = {}
@@ -375,7 +369,7 @@ class Modelspace(Layout):
         xdict["ACAD_GEOGRAPHICDATA"] = geodata
         return geodata
 
-    def get_geodata(self) -> Optional["GeoData"]:
+    def get_geodata(self) -> Optional[GeoData]:
         """Returns the :class:`~ezdxf.entities.GeoData` entity associated to
         the modelspace or ``None``.
         """
@@ -419,48 +413,53 @@ class Paperspace(Layout):
         """
         self.dxf_layout.dxf.name = name
 
-    def viewports(self) -> List["Viewport"]:
-        """Get all VIEWPORT entities defined in this paperspace layout.
-        Returns a list of :class:`~ezdxf.entities.Viewport` objects, sorted by
-        id, the first entity is always the main viewport with an id of 1.
+    def viewports(self) -> list[Viewport]:
+        """Get all VIEWPORT entities defined in this paperspace layout."""
+        return [e for e in self if e.is_alive and e.dxftype() == "VIEWPORT"]  # type: ignore
 
-        """
-        vports = [entity for entity in self if entity.dxftype() == "VIEWPORT"]
-        vports.sort(key=lambda e: e.dxf.id)
-        return vports  # type: ignore
-
-    def main_viewport(self) -> Optional["Viewport"]:
+    def main_viewport(self) -> Optional[Viewport]:
         """Returns the main viewport of this paper space layout, or ``None``
         if no main viewport exist.
 
         """
-        viewports = self.viewports()
-        if len(viewports):
-            return viewports[0]
-        else:
-            return None
-
-    def renumber_viewports(self) -> None:
-        """Reassign viewport ids. (internal API)"""
-        for num, viewport in enumerate(self.viewports(), start=1):
-            viewport.dxf.id = num
+        # Theory: the first VP found is the main VP of the layout, the attributes status
+        # and id are ignored by BricsCAD and AutoCAD!?
+        for viewport in self.viewports():
+            dxf = viewport.dxf
+            if dxf.hasattr("status") and dxf.status == 1:
+                return viewport
+            if dxf.id == 1:
+                return viewport
+        return None
 
     def add_viewport(
         self,
         center: UVec,
-        size: Tuple[float, float],
+        size: tuple[float, float],
         view_center_point: UVec,
         view_height: float,
-        dxfattribs: dict = None,
-    ) -> "Viewport":
-        """Add a new :class:`~ezdxf.entities.Viewport` entity."""
+        status: int = 2,
+        dxfattribs=None,
+    ) -> Viewport:
+        """Add a new :class:`~ezdxf.entities.Viewport` entity.
+
+        Viewport :attr:`status`:
+
+            - -1 is on, but is fully off-screen, or is one of the viewports that is not
+              active because the $MAXACTVP count is currently being exceeded.
+            - 0 is off
+            - any value>0 is on and active. The value indicates the order of
+              stacking for the viewports, where 1 is the "active viewport", 2 is the
+              next, ...
+
+        """
         dxfattribs = dxfattribs or {}
         width, height = size
         attribs = {
             "center": center,  # center in paperspace
             "width": width,  # width in paperspace
             "height": height,  # height in paperspace
-            "status": 1,  # has by default the highest priority (stack order)
+            "status": status,
             "layer": "VIEWPORTS",
             # use separated layer to turn off for plotting
             "view_center_point": view_center_point,  # in modelspace
@@ -468,9 +467,15 @@ class Paperspace(Layout):
         }
         attribs.update(dxfattribs)
         viewport = cast("Viewport", self.new_entity("VIEWPORT", attribs))
-        viewport.dxf.id = viewport.get_next_viewport_id()
+        viewport.dxf.id = self.get_next_viewport_id()
         return viewport
 
+    def get_next_viewport_id(self):
+        viewports = self.viewports()
+        if viewports:
+            return max(vp.dxf.id for vp in viewports) + 1
+        return 2
+
     def reset_viewports(self) -> None:
         """Delete all existing viewports, and create a new main viewport."""
         # remove existing viewports
@@ -478,9 +483,7 @@ class Paperspace(Layout):
             self.delete_entity(viewport)
         self.add_new_main_viewport()
 
-    def reset_main_viewport(
-        self, center: UVec = None, size: UVec = None
-    ) -> "Viewport":
+    def reset_main_viewport(self, center: UVec = None, size: UVec = None) -> Viewport:
         """Reset the main viewport of this paper space layout to the given
         values, or reset them to the default values, deduced from the paper
         settings. Creates a new main viewport if none exist.
@@ -493,11 +496,9 @@ class Paperspace(Layout):
             size: viewport size as (width, height) tuple in paper space units
 
         """
-        viewports = self.viewports()
-        if len(viewports) == 0 or viewports[0].dxf.id != 1:
+        viewport = self.main_viewport()
+        if viewport is None:
             viewport = self.add_new_main_viewport()
-        else:
-            viewport = viewports[0]
         default_center, default_size = self.default_viewport_config()
         if center is None:
             center = default_center
@@ -512,7 +513,7 @@ class Paperspace(Layout):
 
     def default_viewport_config(
         self,
-    ) -> Tuple[Tuple[float, float], Tuple[float, float]]:
+    ) -> tuple[tuple[float, float], tuple[float, float]]:
         dxf = self.dxf_layout.dxf
         if dxf.plot_paper_units == 0:  # inches
             unit_factor = 25.4
@@ -534,14 +535,10 @@ class Paperspace(Layout):
 
         # printing area
         printable_width = (
-            paper_width
-            - paper_units(dxf.left_margin)
-            - paper_units(dxf.right_margin)
+            paper_width - paper_units(dxf.left_margin) - paper_units(dxf.right_margin)
         )
         printable_height = (
-            paper_height
-            - paper_units(dxf.bottom_margin)
-            - paper_units(dxf.top_margin)
+            paper_height - paper_units(dxf.bottom_margin) - paper_units(dxf.top_margin)
         )
 
         # AutoCAD viewport (window) size
@@ -555,7 +552,7 @@ class Paperspace(Layout):
         )
         return center, (vp_width, vp_height)
 
-    def add_new_main_viewport(self) -> "Viewport":
+    def add_new_main_viewport(self) -> Viewport:
         """Add a new main viewport."""
         center, size = self.default_viewport_config()
         vp_height = size[1]
@@ -565,20 +562,30 @@ class Paperspace(Layout):
             size=size,  # I don't get it, just use paper size!
             view_center_point=center,  # same as center
             view_height=vp_height,  # view height in paper space units
+            status=1,  # main viewport
         )
+        if len(self.entity_space) > 1:
+            # move main viewport to index 0 of entity space
+            _vp = self.entity_space.pop()
+            assert _vp is main_viewport
+            self.entity_space.insert(0, main_viewport)
+
         main_viewport.dxf.id = 1  # set as main viewport
         main_viewport.dxf.flags = 557088  # AutoCAD default value
-        self.dxf_layout.dxf.viewport_handle = main_viewport.dxf.handle
+        self.set_current_viewport_handle(main_viewport.dxf.handle)
         return main_viewport
 
+    def set_current_viewport_handle(self, handle: str) -> None:
+        self.dxf_layout.dxf.viewport_handle = handle
+
     def page_setup(
         self,
-        size: Tuple[float, float] = (297, 210),
-        margins: Tuple[float, float, float, float] = (10, 15, 10, 15),
+        size: tuple[float, float] = (297, 210),
+        margins: tuple[float, float, float, float] = (0, 0, 0, 0),
         units: str = "mm",
-        offset: Tuple[float, float] = (0, 0),
+        offset: tuple[float, float] = (0, 0),
         rotation: int = 0,
-        scale: Union[int, Tuple[float, float]] = 16,
+        scale: Union[int, tuple[float, float]] = 16,
         name: str = "ezdxf",
         device: str = "DWG to PDF.pc3",
     ) -> None:
@@ -616,9 +623,7 @@ class Paperspace(Layout):
             scale_num, scale_denom = scale
         elif isinstance(scale, int):
             standard_scale = scale
-            scale_num, scale_denom = const.STD_SCALES.get(
-                standard_scale, (1.0, 1.0)
-            )
+            scale_num, scale_denom = const.STD_SCALES.get(standard_scale, (1.0, 1.0))
         else:
             raise const.DXFTypeError(
                 "Scale has to be an int or a tuple(numerator, denominator)"
@@ -649,9 +654,7 @@ class Paperspace(Layout):
         self.use_standard_scale(False)  # works best, don't know why
         dxf.page_setup_name = ""
         dxf.plot_configuration_file = device
-        dxf.paper_size = (
-            f"{name}_({paper_width:.2f}_x_{paper_height:.2f}_{units})"
-        )
+        dxf.paper_size = f"{name}_({paper_width:.2f}_x_{paper_height:.2f}_{units})"
         dxf.left_margin = margin_left * unit_factor
         dxf.bottom_margin = margin_bottom * unit_factor
         dxf.right_margin = margin_right * unit_factor
@@ -699,7 +702,7 @@ class Paperspace(Layout):
         dxf.limmin = (-shift_x, -shift_y)  # paper space units
         dxf.limmax = (paper_width - shift_x, paper_height - shift_y)
 
-    def get_paper_limits(self) -> Tuple[Vec2, Vec2]:
+    def get_paper_limits(self) -> tuple[Vec2, Vec2]:
         """Returns paper limits in plot paper units, relative to the plot origin.
 
         plot origin = lower left corner of printable area + plot origin offset
@@ -713,14 +716,13 @@ class Paperspace(Layout):
 
     def page_setup_r12(
         self,
-        size: Tuple[int, int] = (297, 210),
-        margins: Tuple[int, int, int, int] = (0, 0, 0, 0),
+        size: tuple[float, float] = (297, 210),
+        margins: tuple[float, float, float, float] = (0, 0, 0, 0),
         units: str = "mm",
-        offset: Tuple[int, int] = (0, 0),
+        offset: tuple[float, float] = (0, 0),
         rotation: float = 0,
-        scale: Union[int, Tuple[float, float]] = 16,
+        scale: Union[int, tuple[float, float]] = 16,
     ) -> None:
-
         # remove existing viewports
         for viewport in self.viewports():
             self.delete_entity(viewport)
@@ -757,30 +759,20 @@ class Paperspace(Layout):
         def paper_units(value):
             return value * scale_factor
 
-        # TODO: don't know how paper setup in DXF R12 works
-        paper_width, paper_height = size
-
-        # TODO: don't know how margins setup in DXF R12 works
-        margin_top, margin_right, margin_bottom, margin_left = margins
-
+        margin_top = paper_units(margins[0])
+        margin_right = paper_units(margins[1])
+        margin_bottom = paper_units(margins[2])
+        margin_left = paper_units(margins[3])
         paper_width = paper_units(size[0])
         paper_height = paper_units(size[1])
-
-        plimmin = self.doc.header["$PLIMMIN"] = (0, 0)
-        plimmax = self.doc.header["$PLIMMAX"] = (paper_width, paper_height)
-
-        # TODO: don't know how paper setup in DXF R12 works
-
-        pextmin = self.doc.header["$PEXTMIN"] = (0, 0, 0)
-        pextmax = self.doc.header["$PEXTMAX"] = (paper_width, paper_height, 0)
+        self.doc.header["$PLIMMIN"] = (0, 0)
+        self.doc.header["$PLIMMAX"] = (paper_width, paper_height)
+        self.doc.header["$PEXTMIN"] = (0, 0, 0)
+        self.doc.header["$PEXTMAX"] = (paper_width, paper_height, 0)
 
         # printing area
-        printable_width = (
-            paper_width - paper_units(margin_left) - paper_units(margin_right)
-        )
-        printable_height = (
-            paper_height - paper_units(margin_bottom) - paper_units(margin_top)
-        )
+        printable_width = paper_width - margin_left - margin_right
+        printable_height = paper_height - margin_bottom - margin_top
 
         # AutoCAD viewport (window) size
         vp_width = paper_width * 1.1
@@ -795,12 +787,12 @@ class Paperspace(Layout):
             size=(vp_width, vp_height),  # I don't get it, just use paper size!
             view_center_point=center,  # same as center
             view_height=vp_height,  # view height in paper space units
+            status=1,  # main viewport
         )
         main_viewport.dxf.id = 1  # set as main viewport
-        main_viewport.dxf.status = 2  # AutoCAD default value
         main_viewport.dxf.render_mode = 1000  # AutoDesk default (view mode?)
 
-    def get_paper_limits_r12(self) -> Tuple[Vec2, Vec2]:
+    def get_paper_limits_r12(self) -> tuple[Vec2, Vec2]:
         """Returns paper limits in plot paper units."""
         limmin = self.doc.header.get("$PLIMMIN", (0, 0))
         limmax = self.doc.header.get("$PLIMMAX", (0, 0))
