@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2021, Manfred Moitzi
+# Copyright (c) 2014-2022, Manfred Moitzi
 # License: MIT License
 """
 DXF Types
@@ -12,10 +12,11 @@ Required DXF tag interface:
     - :meth:`clone`: returns a deep copy of tag
 
 """
+from __future__ import annotations
 from typing import (
     Union,
-    Tuple,
     Iterable,
+    Sequence,
     Type,
     Any,
 )
@@ -67,6 +68,11 @@ GROUP_MARKERS = {
 BINARY_FLAGS = {70, 90}
 HANDLE_CODES = {5, 105}
 POINTER_CODES = set(chain(range(320, 370), range(390, 400), (480, 481, 1005)))
+
+# pointer group codes 320-329 are not translated during INSERT and XREF operations
+TRANSLATABLE_POINTER_CODES = set(
+    chain(range(330, 370), range(390, 400), (480, 481, 1005))
+)
 HEX_HANDLE_CODES = set(chain(HANDLE_CODES, POINTER_CODES))
 BINARY_DATA = {310, 311, 312, 313, 314, 315, 316, 317, 318, 319, 1004}
 EMBEDDED_OBJ_STR = "Embedded Object"
@@ -228,6 +234,49 @@ def is_embedded_object_marker(tag: DXFTag) -> bool:
     return tag.code == EMBEDDED_OBJ_MARKER and tag.value == EMBEDDED_OBJ_STR
 
 
+def is_arbitrary_pointer(tag: DXFTag) -> bool:
+    """Arbitrary object handles; handle values that are taken "as is".
+    They are not translated during INSERT and XREF operations.
+    """
+    return 319 < tag.code < 330
+
+
+def is_soft_pointer(tag: DXFTag) -> bool:
+    """Soft-pointer handle; arbitrary soft pointers to other objects within same DXF
+    file or drawing. Translated during INSERT and XREF operations.
+    """
+    return 329 < tag.code < 340 or tag.code == 1005
+
+
+def is_hard_pointer(tag: DXFTag) -> bool:
+    """Hard-pointer handle; arbitrary hard pointers to other objects within same DXF
+    file or drawing. Translated during INSERT and XREF operations. Hard pointers
+    protect an object from being purged.
+    """
+    code = tag.code
+    return 339 < code < 350 or 389 < code < 400 or 479 < code < 482
+
+
+def is_soft_owner(tag: DXFTag) -> bool:
+    """Soft-owner handle; arbitrary soft ownership links to other objects within same
+    DXF file or drawing. Translated during INSERT and XREF operations.
+    """
+    return 349 < tag.code < 360
+
+
+def is_hard_owner(tag: DXFTag) -> bool:
+    """Hard-owner handle; arbitrary hard ownership links to other objects within same
+    DXF file or drawing. Translated during INSERT and XREF operations. Hard owner handle
+    protect an object from being purged.
+    """
+    return 359 < tag.code < 370
+
+
+def is_translatable_pointer(tag: DXFTag) -> bool:
+    # pointer group codes 320-329 are not translated during INSERT and XREF operations
+    return tag.code in TRANSLATABLE_POINTER_CODES
+
+
 class DXFVertex(DXFTag):
     """Represents a 2D or 3D vertex, stores only the group code of the
     x-component of the vertex, because the y-group-code is x-group-code + 10
@@ -257,15 +306,14 @@ class DXFVertex(DXFTag):
         return hash((self.code, x, y, z))
 
     @property
-    def value(self) -> Tuple[float, ...]:
+    def value(self) -> tuple[float, ...]:
         return tuple(self._value)  # type: ignore
 
     def dxftags(self) -> Iterable[DXFTag]:
         """Returns all vertex components as single :class:`DXFTag` objects."""
         c = self.code
         return (
-            DXFTag(code, value)
-            for code, value in zip((c, c + 10, c + 20), self.value)
+            DXFTag(code, value) for code, value in zip((c, c + 10, c + 20), self.value)
         )
 
     def dxfstr(self) -> str:
@@ -316,7 +364,7 @@ def dxftag(code: int, value: Any) -> DXFTag:
         return DXFTag(code, cast_tag_value(code, value))
 
 
-def tuples_to_tags(iterable: Iterable[Tuple[int, Any]]) -> Iterable[DXFTag]:
+def tuples_to_tags(iterable: Iterable[tuple[int, Any]]) -> Iterable[DXFTag]:
     """Returns an iterable if :class:`DXFTag` or inherited, accepts an
     iterable of (code, value) tuples as input.
     """
@@ -352,7 +400,7 @@ def is_point_code(code: int) -> bool:
     return code in POINT_CODES
 
 
-def is_point_tag(tag: Tuple) -> bool:
+def is_point_tag(tag: Sequence) -> bool:
     return tag[0] in POINT_CODES
 
 
@@ -364,7 +412,7 @@ def tag_type(code: int) -> Type:
     return TYPE_TABLE.get(code, str)
 
 
-def strtag(tag: Union[DXFTag, Tuple[int, Any]]) -> str:
+def strtag(tag: Union[DXFTag, tuple[int, Any]]) -> str:
     return TAG_STRING_FORMAT % tuple(tag)
 
 
