@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2022, Manfred Moitzi
+# Copyright (c) 2011-2022, Manfred Moitzi
 # License: MIT License
 from typing import Iterable
 # Using * imports to simplify namespace imports, therefore every module
@@ -40,8 +40,9 @@ def close_vectors(a: Iterable[AnyVec], b: Iterable[UVec], *,
 
 
 def xround(value: float, rounding: float = 0.) -> float:
-    """
-    Extended rounding function, argument `rounding` defines the rounding limit:
+    """Extended rounding function.
+
+    The argument `rounding` defines the rounding limit:
 
     ======= ======================================
     0       remove fraction
