@@ -1,9 +1,8 @@
-# Copyright (c) 2019-2022, Manfred Moitzi
+# Copyright (c) 2019-2023, Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
-from typing import Iterable, Tuple, Optional, List, Iterator
+from typing import Iterable, Optional, Iterator, Sequence
 import abc
-import warnings
 
 from ezdxf.math import Vec3, Vec2, UVec, AnyVec
 
@@ -14,7 +13,7 @@ __all__ = ["BoundingBox2d", "BoundingBox", "AbstractBoundingBox"]
 class AbstractBoundingBox:
     __slots__ = ("extmin", "extmax")
 
-    def __init__(self, vertices: Iterable[UVec] = None):
+    def __init__(self, vertices: Optional[Iterable[UVec]] = None):
         self.extmax: Optional[AnyVec] = None
         self.extmin: Optional[AnyVec] = None
         if vertices is not None:
@@ -48,7 +47,7 @@ class AbstractBoundingBox:
     @abc.abstractmethod
     def extends_detector(
         self, vertices: Iterable[UVec]
-    ) -> Tuple[AnyVec, AnyVec]:
+    ) -> tuple[AnyVec, AnyVec]:
         pass
 
     @property
@@ -61,24 +60,22 @@ class AbstractBoundingBox:
         ...
 
     @abc.abstractmethod
-    def has_intersection(self, other: "AbstractBoundingBox") -> bool:
+    def has_intersection(self, other: AbstractBoundingBox) -> bool:
         ...
 
     @abc.abstractmethod
-    def has_overlap(self, other: "AbstractBoundingBox") -> bool:
+    def has_overlap(self, other: AbstractBoundingBox) -> bool:
         ...
 
     @abc.abstractmethod
     def intersection(
-        self, other: "AbstractBoundingBox"
-    ) -> "AbstractBoundingBox":
+        self, other: AbstractBoundingBox
+    ) -> AbstractBoundingBox:
         ...
 
-    def contains(self, other: "AbstractBoundingBox") -> bool:
+    def contains(self, other: AbstractBoundingBox) -> bool:
         """Returns ``True`` if the `other` bounding box is completely inside
-        of this bounding box.
-
-        .. versionadded:: 0.17.2
+        this bounding box.
 
         """
         return self.inside(other.extmin) and self.inside(other.extmax)
@@ -136,18 +133,18 @@ class AbstractBoundingBox:
             v.extend([self.extmin, self.extmax])
         self.extmin, self.extmax = self.extends_detector(v)
 
-    def union(self, other: "AbstractBoundingBox"):
+    def union(self, other: AbstractBoundingBox):
         """Returns a new bounding box as union of this and `other` bounding
         box.
         """
-        vertices: List[AnyVec] = []
+        vertices: list[AnyVec] = []
         if self.has_data:
             vertices.extend(self)
         if other.has_data:
             vertices.extend(other)
         return self.__class__(vertices)
 
-    def rect_vertices(self) -> Tuple[Vec2, ...]:
+    def rect_vertices(self) -> Sequence[Vec2]:
         """Returns the corners of the bounding box in the xy-plane as
         :class:`Vec2` objects.
         """
@@ -187,8 +184,6 @@ class BoundingBox(AbstractBoundingBox):
         """Returns ``True`` if the bounding box is empty or the bounding box
         has a size of 0 in any or all dimensions or is undefined.
 
-        .. versionchanged:: 0.18
-
         """
         if self.has_data:
             sx, sy, sz = self.size
@@ -197,7 +192,7 @@ class BoundingBox(AbstractBoundingBox):
 
     def extends_detector(
         self, vertices: Iterable[UVec]
-    ) -> Tuple[Vec3, Vec3]:
+    ) -> tuple[Vec3, Vec3]:
         return extends3d(vertices)
 
     def inside(self, vertex: UVec) -> bool:
@@ -214,7 +209,7 @@ class BoundingBox(AbstractBoundingBox):
             (xmin <= x <= xmax) and (ymin <= y <= ymax) and (zmin <= z <= zmax)
         )
 
-    def has_intersection(self, other: "AbstractBoundingBox") -> bool:
+    def has_intersection(self, other: AbstractBoundingBox) -> bool:
         """Returns ``True`` if this bounding box intersects with `other` but does
         not include touching bounding boxes, see also :meth:`has_overlap`::
 
@@ -222,8 +217,6 @@ class BoundingBox(AbstractBoundingBox):
             bbox2 = BoundingBox([(1, 1, 1), (2, 2, 2)])
             assert bbox1.has_intersection(bbox2) is False
 
-        .. versionadded: 0.18
-
         """
         # Source: https://gamemath.com/book/geomtests.html#intersection_two_aabbs
         # Check for a separating axis:
@@ -252,14 +245,7 @@ class BoundingBox(AbstractBoundingBox):
             return False
         return True
 
-    def intersect(self, other: "AbstractBoundingBox") -> bool:
-        warnings.warn(
-            "intersect() is deprecated, replaced by has_intersection()",
-            DeprecationWarning,
-        )
-        return self.has_intersection(other)
-
-    def has_overlap(self, other: "AbstractBoundingBox") -> bool:
+    def has_overlap(self, other: AbstractBoundingBox) -> bool:
         """Returns ``True`` if this bounding box intersects with `other` but
         in contrast to :meth:`has_intersection` includes touching bounding boxes too::
 
@@ -267,8 +253,6 @@ class BoundingBox(AbstractBoundingBox):
             bbox2 = BoundingBox([(1, 1, 1), (2, 2, 2)])
             assert bbox1.has_overlap(bbox2) is True
 
-        .. versionadded: 0.18
-
         """
         # Source: https://gamemath.com/book/geomtests.html#intersection_two_aabbs
         # Check for a separating axis:
@@ -296,14 +280,7 @@ class BoundingBox(AbstractBoundingBox):
             return False
         return True
 
-    def overlap(self, other: "AbstractBoundingBox") -> bool:
-        warnings.warn(
-            "overlap() is deprecated, replaced by has_overlap()",
-            DeprecationWarning,
-        )
-        return self.has_overlap(other)
-
-    def cube_vertices(self) -> Tuple[Vec3, ...]:
+    def cube_vertices(self) -> Sequence[Vec3]:
         """Returns the 3D corners of the bounding box as :class:`Vec3` objects."""
         if self.extmin is not None and self.extmax is not None:
             x0, y0, z0 = self.extmin
@@ -321,13 +298,11 @@ class BoundingBox(AbstractBoundingBox):
         else:
             raise ValueError("empty bounding box")
 
-    def intersection(self, other: "AbstractBoundingBox") -> "BoundingBox":
+    def intersection(self, other: AbstractBoundingBox) -> BoundingBox:
         """Returns the bounding box of the intersection cube of both
         3D bounding boxes. Returns an empty bounding box if the intersection
         volume is 0.
 
-        .. versionadded: 0.18
-
         """
         new_bbox = self.__class__()
         if not self.has_intersection(other):
@@ -354,7 +329,7 @@ class BoundingBox(AbstractBoundingBox):
 
 
 class BoundingBox2d(AbstractBoundingBox):
-    """Optimized 2D bounding box.
+    """2D bounding box.
 
     Args:
         vertices: iterable of ``(x, y[, z])`` tuples or :class:`Vec3` objects
@@ -374,7 +349,7 @@ class BoundingBox2d(AbstractBoundingBox):
 
     def extends_detector(
         self, vertices: Iterable[UVec]
-    ) -> Tuple[Vec2, Vec2]:
+    ) -> tuple[Vec2, Vec2]:
         return extends2d(vertices)
 
     def inside(self, vertex: UVec) -> bool:
@@ -389,7 +364,7 @@ class BoundingBox2d(AbstractBoundingBox):
         max_ = self.extmax
         return (min_.x <= v.x <= max_.x) and (min_.y <= v.y <= max_.y)
 
-    def has_intersection(self, other: "AbstractBoundingBox") -> bool:
+    def has_intersection(self, other: AbstractBoundingBox) -> bool:
         """Returns ``True`` if this bounding box intersects with `other` but does
         not include touching bounding boxes, see also :meth:`has_overlap`::
 
@@ -417,14 +392,7 @@ class BoundingBox2d(AbstractBoundingBox):
             return False
         return True
 
-    def intersect(self, other: "AbstractBoundingBox") -> bool:
-        warnings.warn(
-            "intersect() is deprecated, replaced by has_intersection()",
-            DeprecationWarning,
-        )
-        return self.has_intersection(other)
-
-    def intersection(self, other: "AbstractBoundingBox") -> "BoundingBox2d":
+    def intersection(self, other: AbstractBoundingBox) -> BoundingBox2d:
         """Returns the bounding box of the intersection rectangle of both
         2D bounding boxes. Returns an empty bounding box if the intersection
         area is 0.
@@ -444,7 +412,7 @@ class BoundingBox2d(AbstractBoundingBox):
         )
         return new_bbox
 
-    def has_overlap(self, other: "AbstractBoundingBox") -> bool:
+    def has_overlap(self, other: AbstractBoundingBox) -> bool:
         """Returns ``True`` if this bounding box intersects with `other` but
         in contrast to :meth:`has_intersection` includes touching bounding boxes too::
 
@@ -452,8 +420,6 @@ class BoundingBox2d(AbstractBoundingBox):
             bbox2 = BoundingBox2d([(1, 1), (2, 2)])
             assert bbox1.has_overlap(bbox2) is True
 
-        .. versionadded:: 0.17.2
-
         """
         # Source: https://gamemath.com/book/geomtests.html#intersection_two_aabbs
         if (
@@ -474,15 +440,8 @@ class BoundingBox2d(AbstractBoundingBox):
             return False
         return True
 
-    def overlap(self, other: "AbstractBoundingBox") -> bool:
-        warnings.warn(
-            "overlap() is deprecated, replaced by has_overlap()",
-            DeprecationWarning,
-        )
-        return self.has_overlap(other)
 
-
-def extends3d(vertices: Iterable[UVec]) -> Tuple[Vec3, Vec3]:
+def extends3d(vertices: Iterable[UVec]) -> tuple[Vec3, Vec3]:
     minx, miny, minz = None, None, None
     maxx, maxy, maxz = None, None, None
     for v in vertices:
@@ -509,7 +468,7 @@ def extends3d(vertices: Iterable[UVec]) -> Tuple[Vec3, Vec3]:
     return Vec3(minx, miny, minz), Vec3(maxx, maxy, maxz)
 
 
-def extends2d(vertices: Iterable[UVec]) -> Tuple[Vec2, Vec2]:
+def extends2d(vertices: Iterable[UVec]) -> tuple[Vec2, Vec2]:
     minx, miny = None, None
     maxx, maxy = None, None
     for v in vertices:
@@ -530,5 +489,5 @@ def extends2d(vertices: Iterable[UVec]) -> Tuple[Vec2, Vec2]:
             elif y > maxy:
                 maxy = y
     if minx is None:
-        raise ValueError("No vertices give.")
+        raise ValueError("no vertices given")
     return Vec2(minx, miny), Vec2(maxx, maxy)
