@@ -1,6 +1,7 @@
 # Copyright (c) 2010-2022 Manfred Moitzi
 # License: MIT License
-from typing import Iterable, Tuple, Sequence
+from __future__ import annotations
+from typing import Iterable, Sequence
 from functools import lru_cache
 import math
 from ezdxf.math import Vec3, NULLVEC, Matrix44, UVec
@@ -72,7 +73,9 @@ continuous to any degree.
 
 
 class Bezier:
-    """A `Bézier curve`_ is a parametric curve used in computer graphics and
+    """Generic `Bézier curve`_ of any degree.
+
+    A `Bézier curve`_ is a parametric curve used in computer graphics and
     related fields. Bézier curves are used to model smooth curves that can be
     scaled indefinitely. "Paths", as they are commonly referred to in image
     manipulation programs, are combinations of linked Bézier curves.
@@ -81,7 +84,8 @@ class Bezier:
 
     This is a generic implementation which works with any count of definition
     points greater than 2, but it is a simple and slow implementation. For more
-    performance look at the specialized :class:`Bezier4P` class.
+    performance look at the specialized :class:`Bezier4P` and :class:`Bezier3P`
+    classes.
 
     Objects are immutable.
 
@@ -117,8 +121,6 @@ class Bezier:
                 subdivided.
             segments: minimum segment count
 
-        .. versionadded:: 0.15
-
         """
 
         def subdiv(start_point, end_point, start_t: float, end_t: float):
@@ -175,7 +177,7 @@ class Bezier:
         for u in t:
             yield self.point(u)
 
-    def derivative(self, t: float) -> Tuple[Vec3, Vec3, Vec3]:
+    def derivative(self, t: float) -> tuple[Vec3, Vec3, Vec3]:
         """Returns (point, 1st derivative, 2nd derivative) tuple for parameter `t`
         in range [0, 1] as :class:`Vec3` objects.
         """
@@ -215,7 +217,7 @@ class Bezier:
 
     def derivatives(
         self, t: Iterable[float]
-    ) -> Iterable[Tuple[Vec3, Vec3, Vec3]]:
+    ) -> Iterable[tuple[Vec3, Vec3, Vec3]]:
         """Returns multiple (point, 1st derivative, 2nd derivative) tuples for
         parameter vector  `t` as :class:`Vec3` objects.
         Parameters in range [0, 1]
@@ -223,11 +225,11 @@ class Bezier:
         for u in t:
             yield self.derivative(u)
 
-    def reverse(self) -> "Bezier":
+    def reverse(self) -> Bezier:
         """Returns a new Bèzier-curve with reversed control point order."""
         return Bezier(list(reversed(self.control_points)))
 
-    def transform(self, m: Matrix44) -> "Bezier":
+    def transform(self, m: Matrix44) -> Bezier:
         """General transformation interface, returns a new :class:`Bezier` curve.
 
         Args:
