@@ -18,14 +18,12 @@ https://books.google.at/books/about/The_NURBS_Book.html?id=7dqY5dyAwWkC&redir_es
 """
 from __future__ import annotations
 from typing import (
-    List,
     Iterable,
     Iterator,
     Sequence,
     TYPE_CHECKING,
-    Dict,
-    Tuple,
     Union,
+    Optional,
 )
 import math
 from ezdxf.math import (
@@ -36,7 +34,6 @@ from ezdxf.math import (
     Evaluator,
     create_t_vector,
     estimate_end_tangent_magnitude,
-    estimate_tangents,
     linspace,
     distance_point_line_3d,
     arc_angle_span_deg,
@@ -74,6 +71,7 @@ __all__ = [
     # Low level interpolation function:
     "unconstrained_global_bspline_interpolation",
     "global_bspline_interpolation_end_tangents",
+    "cad_fit_point_interpolation",
     "global_bspline_interpolation_first_derivatives",
     "local_cubic_bspline_interpolation_from_tangents",
     # Low level knot parametrization functions:
@@ -94,70 +92,40 @@ __all__ = [
 
 def fit_points_to_cad_cv(
     fit_points: Iterable[UVec],
-    tangents: Iterable[UVec] = None,
-    estimate: str = "5-p",
-) -> "BSpline":
+    tangents: Optional[Iterable[UVec]] = None,
+) -> BSpline:
     """Returns a cubic :class:`BSpline` from fit points as close as possible
     to common CAD applications like BricsCAD.
 
     There exist infinite numerical correct solution for this setup, but some
     facts are known:
 
-    - Global curve interpolation with start- and end derivatives, e.g. 6 fit points
-      creates 8 control vertices in BricsCAD
-    - Degree of B-spline is always 3, the stored degree is ignored,
-      this is only valid for B-splines defined by fit points
+    - CAD applications use the global curve interpolation with start- and end
+      derivatives if the end tangents are defined otherwise the equation system will
+      be completed by setting the second derivatives of the start and end point to 0,
+      for more information read this answer on stackoverflow: https://stackoverflow.com/a/74863330/6162864
+    - The degree of the B-spline is always 3 regardless which degree is stored in the
+      SPLINE entity, this is only valid for B-splines defined by fit points
     - Knot parametrization method is "chord"
     - Knot distribution is "natural"
 
-    The last missing parameter is the start- and end tangents estimation method
-    used by BricsCAD, if these tangents are stored in the DXF file provide them
-    as argument `tangents` as 2-tuple (start, end) and the interpolated control
-    vertices will match the BricsCAD calculation, except for floating point
-    imprecision.
-
-    If the end tangents are not given, the start- and ent tangent directions
-    will be estimated. The argument `estimate` lets choose from different
-    estimation methods (first 3 letters are significant):
-
-    - "3-points": 3 point interpolation
-    - "5-points": 5 point interpolation
-    - "bezier": tangents from an interpolated cubic bezier curve
-    - "diff": finite difference
-
-    The estimation method "5-p" yields the closest match to the BricsCAD
-    rendering, but sometimes "bez" creates a better result.
-
-    If I figure out how BricsCAD estimates the end tangents directions, the
-    argument `estimate` gets an additional value for that case. The existing
-    estimation methods will perform the same way as now, except for bug fixes.
-    But the default value may change, therefore set argument `estimate` to
-    specific value to always get the same result in the future.
-
     Args:
         fit_points: points the spline is passing through
         tangents: start- and end tangent, default is autodetect
-        estimate: tangent direction estimation method
-
-    .. versionchanged:: 0.16
-        removed unused arguments `degree` and `method`
 
     """
     # See also Spline class in ezdxf/entities/spline.py:
-    # degree has no effect. A spline with degree=3 is always constructed when
-    # interpolating a series of fit points.
     points = Vec3.list(fit_points)
     if len(points) < 2:
         raise ValueError("two or more points required ")
-    m1, m2 = estimate_end_tangent_magnitude(points, method="chord")
+
     if tangents is None:
-        t = estimate_tangents(points, method=estimate, normalize=False)
-        start_tangent = t[0].normalize(m1)
-        end_tangent = t[-1].normalize(m2)
-    else:
-        t = Vec3.list(tangents)
-        start_tangent = t[0].normalize(m1)
-        end_tangent = t[-1].normalize(m2)
+        control_points, knots = cad_fit_point_interpolation(points)
+        return BSpline(control_points, order=4, knots=knots)
+    t = Vec3.list(tangents)
+    m1, m2 = estimate_end_tangent_magnitude(points, method="chord")
+    start_tangent = t[0].normalize(m1)
+    end_tangent = t[-1].normalize(m2)
 
     return global_bspline_interpolation(
         points,
@@ -167,7 +135,7 @@ def fit_points_to_cad_cv(
     )
 
 
-def fit_points_to_cubic_bezier(fit_points: Iterable[UVec]) -> "BSpline":
+def fit_points_to_cubic_bezier(fit_points: Iterable[UVec]) -> BSpline:
     """Returns a cubic :class:`BSpline` from fit points **without** end
     tangents.
 
@@ -179,8 +147,6 @@ def fit_points_to_cubic_bezier(fit_points: Iterable[UVec]) -> "BSpline":
     Args:
         fit_points: points the spline is passing through
 
-    .. versionadded:: 0.16
-
     """
     points = Vec3.list(fit_points)
     if len(points) < 2:
@@ -195,9 +161,9 @@ def fit_points_to_cubic_bezier(fit_points: Iterable[UVec]) -> "BSpline":
 def global_bspline_interpolation(
     fit_points: Iterable[UVec],
     degree: int = 3,
-    tangents: Iterable[UVec] = None,
+    tangents: Optional[Iterable[UVec]] = None,
     method: str = "chord",
-) -> "BSpline":
+) -> BSpline:
     """`B-spline`_ interpolation by the `Global Curve Interpolation`_.
     Given are the fit points and the degree of the B-spline.
     The function provides 3 methods for generating the parameter vector t:
@@ -281,8 +247,8 @@ def global_bspline_interpolation(
 def local_cubic_bspline_interpolation(
     fit_points: Iterable[UVec],
     method: str = "5-points",
-    tangents: Iterable[UVec] = None,
-) -> "BSpline":
+    tangents: Optional[Iterable[UVec]] = None,
+) -> BSpline:
     """`B-spline`_ interpolation by 'Local Cubic Curve Interpolation', which
     creates B-spline from fit points and estimated tangent direction at start-,
     end- and passing points.
@@ -321,7 +287,7 @@ def local_cubic_bspline_interpolation(
 
 
 def required_knot_values(count: int, order: int) -> int:
-    """Returns the count of required knot values for a B-spline of `order` and
+    """Returns the count of required knot-values for a B-spline of `order` and
     `count` control points.
 
     Args:
@@ -375,14 +341,14 @@ def required_control_points(order: int) -> int:
     return max(order, 2)
 
 
-def normalize_knots(knots: Sequence[float]) -> List[float]:
+def normalize_knots(knots: Sequence[float]) -> list[float]:
     """Normalize knot vector into range [0, 1]."""
     min_val = knots[0]
     max_val = knots[-1] - min_val
     return [(v - min_val) / max_val for v in knots]
 
 
-def uniform_knot_vector(count: int, order: int, normalize=False) -> List[float]:
+def uniform_knot_vector(count: int, order: int, normalize=False) -> list[float]:
     """Returns an uniform knot vector for a B-spline of `order` and `count`
     control points.
 
@@ -401,9 +367,7 @@ def uniform_knot_vector(count: int, order: int, normalize=False) -> List[float]:
     return [knot_value / max_value for knot_value in range(count + order)]
 
 
-def open_uniform_knot_vector(
-    count: int, order: int, normalize=False
-) -> List[float]:
+def open_uniform_knot_vector(count: int, order: int, normalize=False) -> list[float]:
     """Returns an open (clamped) uniform knot vector for a B-spline of `order`
     and `count` control points.
 
@@ -431,7 +395,7 @@ def open_uniform_knot_vector(
 
 def knots_from_parametrization(
     n: int, p: int, t: Iterable[float], method="average", constrained=False
-) -> List[float]:
+) -> list[float]:
     """Returns a 'clamped' knot vector for B-splines. All knot values are
     normalized in the range [0, 1].
 
@@ -448,12 +412,10 @@ def knots_from_parametrization(
     """
     order = int(p + 1)
     if order > (n + 1):
-        raise DXFValueError(
-            "Invalid n/p combination, more fit points required."
-        )
+        raise DXFValueError("Invalid n/p combination, more fit points required.")
 
     t = [float(v) for v in t]
-    if t[0] != 0.0 or t[-1] != 1.0:
+    if t[0] != 0.0 or not math.isclose(t[-1], 1.0):
         raise ValueError("Parametrization vector t has to be normalized.")
 
     if method == "average":
@@ -472,9 +434,7 @@ def knots_from_parametrization(
         raise ValueError(f"Unknown knot generation method: {method}")
 
 
-def averaged_knots_unconstrained(
-    n: int, p: int, t: Sequence[float]
-) -> List[float]:
+def averaged_knots_unconstrained(n: int, p: int, t: Sequence[float]) -> list[float]:
     """Returns an averaged knot vector from parametrization vector `t` for an
     unconstrained B-spline.
 
@@ -485,7 +445,7 @@ def averaged_knots_unconstrained(
 
     """
     assert t[0] == 0.0
-    assert t[-1] == 1.0
+    assert math.isclose(t[-1], 1.0)
 
     knots = [0.0] * (p + 1)
     knots.extend(sum(t[j : j + p]) / p for j in range(1, n - p + 1))
@@ -495,9 +455,7 @@ def averaged_knots_unconstrained(
     return knots
 
 
-def averaged_knots_constrained(
-    n: int, p: int, t: Sequence[float]
-) -> List[float]:
+def averaged_knots_constrained(n: int, p: int, t: Sequence[float]) -> list[float]:
     """Returns an averaged knot vector from parametrization vector `t` for a
     constrained B-spline.
 
@@ -508,7 +466,7 @@ def averaged_knots_constrained(
 
     """
     assert t[0] == 0.0
-    assert t[-1] == 1.0
+    assert math.isclose(t[-1], 1.0)
 
     knots = [0.0] * (p + 1)
     knots.extend(sum(t[j : j + p - 1]) / p for j in range(n - p))
@@ -516,9 +474,7 @@ def averaged_knots_constrained(
     return knots
 
 
-def natural_knots_unconstrained(
-    n: int, p: int, t: Sequence[float]
-) -> List[float]:
+def natural_knots_unconstrained(n: int, p: int, t: Sequence[float]) -> list[float]:
     """Returns a 'natural' knot vector from parametrization vector `t` for an
     unconstrained B-spline.
 
@@ -529,7 +485,7 @@ def natural_knots_unconstrained(
 
     """
     assert t[0] == 0.0
-    assert t[-1] == 1.0
+    assert math.isclose(t[-1], 1.0)
 
     knots = [0.0] * (p + 1)
     knots.extend(t[2 : n - p + 2])
@@ -537,9 +493,7 @@ def natural_knots_unconstrained(
     return knots
 
 
-def natural_knots_constrained(
-    n: int, p: int, t: Sequence[float]
-) -> List[float]:
+def natural_knots_constrained(n: int, p: int, t: Sequence[float]) -> list[float]:
     """Returns a 'natural' knot vector from parametrization vector `t` for a
     constrained B-spline.
 
@@ -550,7 +504,7 @@ def natural_knots_constrained(
 
     """
     assert t[0] == 0.0
-    assert t[-1] == 1.0
+    assert math.isclose(t[-1], 1.0)
 
     knots = [0.0] * (p + 1)
     knots.extend(t[1 : n - p + 1])
@@ -558,7 +512,7 @@ def natural_knots_constrained(
     return knots
 
 
-def double_knots(n: int, p: int, t: Sequence[float]) -> List[float]:
+def double_knots(n: int, p: int, t: Sequence[float]) -> list[float]:
     """Returns a knot vector from parametrization vector `t` for B-spline
     constrained by first derivatives at all fit points.
 
@@ -570,7 +524,7 @@ def double_knots(n: int, p: int, t: Sequence[float]) -> List[float]:
 
     """
     assert t[0] == 0.0
-    assert t[-1] == 1.0
+    assert math.isclose(t[-1], 1.0)
 
     u = [0.0] * (p + 1)
     prev_t = 0.0
@@ -596,7 +550,7 @@ def double_knots(n: int, p: int, t: Sequence[float]) -> List[float]:
     return u
 
 
-def _get_best_solver(matrix: Union[List, linalg.Matrix], degree: int):
+def _get_best_solver(matrix: Union[list, linalg.Matrix], degree: int):
     """Returns best suited linear equation solver depending on matrix
     configuration and python interpreter.
     """
@@ -622,7 +576,7 @@ def unconstrained_global_bspline_interpolation(
     degree: int,
     t_vector: Sequence[float],
     knot_generation_method: str = "average",
-) -> Tuple[List[Vec3], List[float]]:
+) -> tuple[list[Vec3], list[float]]:
     """Interpolates the control points for a B-spline by global interpolation
     from fit points without any constraints.
 
@@ -656,15 +610,15 @@ def unconstrained_global_bspline_interpolation(
 
 
 def global_bspline_interpolation_end_tangents(
-    fit_points: List[Vec3],
+    fit_points: list[Vec3],
     start_tangent: Vec3,
     end_tangent: Vec3,
     degree: int,
     t_vector: Sequence[float],
     knot_generation_method: str = "average",
-) -> Tuple[List[Vec3], List[float]]:
-    """Interpolates the control points for a B-spline by global interpolation
-    from fit points and 1st derivatives for start- and end point as constraints.
+) -> tuple[list[Vec3], list[float]]:
+    """Calculates the control points for a B-spline by global interpolation
+    from fit points and the 1st derivative of the start- and end point as constraints.
     These 'tangents' are 1st derivatives and not unit vectors, if an estimation
     of the magnitudes is required use the :func:`estimate_end_tangent_magnitude`
     function.
@@ -709,12 +663,77 @@ def global_bspline_interpolation_end_tangents(
     return Vec3.list(control_points.rows()), knots
 
 
+def cad_fit_point_interpolation(
+    fit_points: list[Vec3],
+) -> tuple[list[Vec3], list[float]]:
+    """Calculates the control points for a B-spline by global interpolation
+    from fit points without any constraints in the same way as AutoCAD and BricsCAD.
+
+    Source: https://stackoverflow.com/a/74863330/6162864
+
+    Args:
+        fit_points: points the B-spline has to pass
+
+    Returns:
+        2-tuple of control points as list of Vec3 objects and the knot vector
+        as list of floats
+
+    """
+
+    def coefficients1() -> list[float]:
+        """Returns the coefficients for equation [1]."""
+        # Piegl & Tiller: "The NURBS Book" formula (3.9)
+        up1 = knots[p + 1]
+        up2 = knots[p + 2]
+        f = p * (p - 1) / up1
+        return [
+            f / up1,  # P0
+            -f * (up1 + up2) / (up1 * up2),  # P1
+            f / up2,  # P2
+        ]
+
+    def coefficients2() -> list[float]:
+        """Returns the coefficients for equation [n-1]."""
+        # Piegl & Tiller: "The NURBS Book" formula (3.10)
+        m = len(knots) - 1
+        ump1 = knots[m - p - 1]
+        ump2 = knots[m - p - 2]
+        f = p * (p - 1) / (1.0 - ump1)
+        return [
+            f / (1.0 - ump2),  # Pn-2
+            -f * (2.0 - ump1 - ump2) / (1.0 - ump1) / (1.0 - ump2),  # Pn-1
+            f / (1.0 - ump1),  # Pn
+        ]
+
+    t_vector = list(create_t_vector(fit_points, "chord"))
+    n = len(fit_points) - 1
+    p = 3
+    knots = knots_from_parametrization(
+        n + 2, p, t_vector, method="natural", constrained=True
+    )
+
+    N = Basis(knots=knots, order=p + 1, count=n + 3)
+    rows = [N.basis_vector(u) for u in t_vector]
+    spacing = [0.0] * n
+    rows.insert(1, coefficients1() + spacing)
+    rows.insert(-1, spacing + coefficients2())
+
+    # C"(0) == 0
+    fit_points.insert(1, Vec3(0, 0, 0))
+    # C"(1) == 0
+    fit_points.insert(-1, Vec3(0, 0, 0))
+
+    solver = _get_best_solver(rows, p)
+    control_points = solver.solve_matrix(fit_points)
+    return Vec3.list(control_points.rows()), knots
+
+
 def global_bspline_interpolation_first_derivatives(
-    fit_points: List[Vec3],
-    derivatives: List[Vec3],
+    fit_points: list[Vec3],
+    derivatives: list[Vec3],
     degree: int,
     t_vector: Sequence[float],
-) -> Tuple[List[Vec3], List[float]]:
+) -> tuple[list[Vec3], list[float]]:
     """Interpolates the control points for a B-spline by a global
     interpolation from fit points and 1st derivatives as constraints.
 
@@ -757,7 +776,7 @@ def global_bspline_interpolation_first_derivatives(
     A.append([0.0] * (count - 1) + [+1.0])  # Qn
 
     # Build right handed matrix B
-    B: List[Vec3] = []
+    B: list[Vec3] = []
     for rows in zip(fit_points, derivatives):
         B.extend(rows)  # Qi, Di
 
@@ -773,8 +792,8 @@ def global_bspline_interpolation_first_derivatives(
 
 
 def local_cubic_bspline_interpolation_from_tangents(
-    fit_points: List[Vec3], tangents: List[Vec3]
-) -> Tuple[List[Vec3], List[float]]:
+    fit_points: list[Vec3], tangents: list[Vec3]
+) -> tuple[list[Vec3], list[float]]:
     """Interpolates the control points for a cubic B-spline by local
     interpolation from fit points and tangents as unit vectors for each fit
     point. Use the :func:`estimate_tangents` function to estimate end tangents.
@@ -830,8 +849,10 @@ def local_cubic_bspline_interpolation_from_tangents(
 
 
 class BSpline:
-    """Representation of a `B-spline`_ curve. The default configuration of
-    the knot vector is a uniform open `knot`_ vector ("clamped").
+    """B-spline construction tool.
+
+    Internal representation of a `B-spline`_ curve. The default configuration
+    of the knot vector is a uniform open `knot`_ vector ("clamped").
 
     Factory functions:
 
@@ -859,15 +880,15 @@ class BSpline:
         self,
         control_points: Iterable[UVec],
         order: int = 4,
-        knots: Iterable[float] = None,
-        weights: Iterable[float] = None,
+        knots: Optional[Iterable[float]] = None,
+        weights: Optional[Iterable[float]] = None,
     ):
         self._control_points = Vec3.tuple(control_points)
         count = len(self._control_points)
         order = int(order)
         if order > count:
             raise DXFValueError(
-                f"Invalid need more control points for order {order}"
+                f"got {count} control points, need {order} or more for order of {order}"
             )
 
         if knots is None:
@@ -882,9 +903,7 @@ class BSpline:
             if knots[0] != 0.0:
                 knots = normalize_knots(knots)
         self._basis = Basis(knots, order, count, weights=weights)
-        self._clamped = (
-            len(set(knots[:order])) == 1 and len(set(knots[-order:])) == 1
-        )
+        self._clamped = len(set(knots[:order])) == 1 and len(set(knots[-order:])) == 1
 
     def __str__(self):
         return (
@@ -933,16 +952,12 @@ class BSpline:
         return self._clamped
 
     @staticmethod
-    def from_fit_points(
-        points: Iterable[UVec], degree=3, method="chord"
-    ) -> "BSpline":
+    def from_fit_points(points: Iterable[UVec], degree=3, method="chord") -> BSpline:
         """Returns :class:`BSpline` defined by fit points."""
         return global_bspline_interpolation(points, degree, method=method)
 
     @staticmethod
-    def ellipse_approximation(
-        ellipse: "ConstructionEllipse", num: int = 16
-    ) -> "BSpline":
+    def ellipse_approximation(ellipse: ConstructionEllipse, num: int = 16) -> BSpline:
         """Returns an ellipse approximation as :class:`BSpline` with `num`
         control points.
 
@@ -952,17 +967,15 @@ class BSpline:
         )
 
     @staticmethod
-    def arc_approximation(arc: "ConstructionArc", num: int = 16) -> "BSpline":
+    def arc_approximation(arc: ConstructionArc, num: int = 16) -> BSpline:
         """Returns an arc approximation as :class:`BSpline` with `num`
         control points.
 
         """
-        return global_bspline_interpolation(
-            arc.vertices(arc.angles(num)), degree=2
-        )
+        return global_bspline_interpolation(arc.vertices(arc.angles(num)), degree=2)
 
     @staticmethod
-    def from_ellipse(ellipse: "ConstructionEllipse") -> "BSpline":
+    def from_ellipse(ellipse: ConstructionEllipse) -> BSpline:
         """Returns the ellipse as :class:`BSpline` of 2nd degree with as few
         control points as possible.
 
@@ -970,7 +983,7 @@ class BSpline:
         return rational_bspline_from_ellipse(ellipse, segments=1)
 
     @staticmethod
-    def from_arc(arc: "ConstructionArc") -> "BSpline":
+    def from_arc(arc: ConstructionArc) -> BSpline:
         """Returns the arc as :class:`BSpline` of 2nd degree with as few control
         points as possible.
 
@@ -980,7 +993,7 @@ class BSpline:
         )
 
     @staticmethod
-    def from_nurbs_python_curve(curve) -> "BSpline":
+    def from_nurbs_python_curve(curve) -> BSpline:
         """Interface to the `NURBS-Python <https://pypi.org/project/geomdl/>`_
         package.
 
@@ -995,7 +1008,7 @@ class BSpline:
             weights=curve.weights,
         )
 
-    def reverse(self) -> "BSpline":
+    def reverse(self) -> BSpline:
         """Returns a new :class:`BSpline` object with reversed control point
         order.
 
@@ -1012,14 +1025,14 @@ class BSpline:
             weights=reversed(self.weights()) if self.is_rational else None,
         )
 
-    def knots(self) -> Tuple[float, ...]:
+    def knots(self) -> Sequence[float]:
         """Returns a tuple of `knot`_ values as floats, the knot vector
         **always** has order + count values (n + p + 2 in text book notation).
 
         """
         return self._basis.knots
 
-    def weights(self) -> Tuple[float, ...]:
+    def weights(self) -> Sequence[float]:
         """Returns a tuple of weights values as floats, one for each control
         point or an empty tuple.
 
@@ -1052,8 +1065,6 @@ class BSpline:
                 segment chord.
             segments: minimum segment count between two knots
 
-        .. versionadded:: 0.15
-
         """
 
         def subdiv(s: Vec3, e: Vec3, start_t: float, end_t: float):
@@ -1070,7 +1081,7 @@ class BSpline:
                 yield from subdiv(m, e, mid_t, end_t)
 
         evaluator = self.evaluator
-        knots: List[float] = self.knots()  # type: ignore
+        knots: list[float] = self.knots()  # type: ignore
         if self.is_clamped:
             lower_bound = 0.0
         else:
@@ -1111,7 +1122,7 @@ class BSpline:
         """
         return self.evaluator.points(t)
 
-    def derivative(self, t: float, n: int = 2) -> List[Vec3]:
+    def derivative(self, t: float, n: int = 2) -> list[Vec3]:
         """Return point and derivatives up to `n` <= degree for parameter `t`.
 
         e.g. n=1 returns point and 1st derivative.
@@ -1126,9 +1137,7 @@ class BSpline:
         """
         return self.evaluator.derivative(t, n)
 
-    def derivatives(
-        self, t: Iterable[float], n: int = 2
-    ) -> Iterable[List[Vec3]]:
+    def derivatives(self, t: Iterable[float], n: int = 2) -> Iterable[list[Vec3]]:
         """Yields points and derivatives up to `n` <= degree for parameter
         vector `t`.
 
@@ -1144,7 +1153,7 @@ class BSpline:
         """
         return self.evaluator.derivatives(t, n)
 
-    def insert_knot(self, t: float) -> "BSpline":
+    def insert_knot(self, t: float) -> BSpline:
         """Insert an additional knot, without altering the shape of the curve.
         Returns a new :class:`BSpline` object.
 
@@ -1174,7 +1183,7 @@ class BSpline:
         knots.insert(k + 1, t)  # knot[k] <= t < knot[k+1]
         return BSpline(cpoints, self.order, knots)
 
-    def knot_refinement(self, u: Iterable[float]) -> "BSpline":
+    def knot_refinement(self, u: Iterable[float]) -> BSpline:
         """Insert multiple knots, without altering the shape of the curve.
         Returns a new :class:`BSpline` object.
 
@@ -1187,7 +1196,7 @@ class BSpline:
             spline = spline.insert_knot(t)
         return spline
 
-    def transform(self, m: "Matrix44") -> "BSpline":
+    def transform(self, m: Matrix44) -> BSpline:
         """Returns a new :class:`BSpline` object transformed by a
         :class:`Matrix44` transformation matrix.
 
@@ -1195,23 +1204,7 @@ class BSpline:
         cpoints = m.transform_vertices(self.control_points)
         return BSpline(cpoints, self.order, self.knots(), self.weights())
 
-    def to_nurbs_python_curve(self):
-        """Returns a :class:`geomdl.BSpline.Curve` object, if the
-        `NURBS-Python <https://pypi.org/project/geomdl/>`_ package is installed.
-
-        """
-        if self._basis.is_rational:
-            from geomdl.NURBS import Curve
-        else:
-            from geomdl.BSpline import Curve
-        curve = Curve()
-        curve.degree = self.degree
-        curve.ctrlpts = [v.xyz for v in self.control_points]
-        curve.knotvector = self.knots()
-        curve.weights = self.weights()
-        return curve
-
-    def bezier_decomposition(self) -> Iterable[List[Vec3]]:
+    def bezier_decomposition(self) -> Iterable[list[Vec3]]:
         """Decompose a non-rational B-spline into multiple Bézier curves.
 
         This is the preferred method to represent the most common non-rational
@@ -1257,9 +1250,9 @@ class BSpline:
                     s = mult + j
                     for k in range(p, s - 1, -1):
                         alpha = alphas[k - s]
-                        bezier_points[k] = bezier_points[
-                            k
-                        ] * alpha + bezier_points[k - 1] * (1.0 - alpha)
+                        bezier_points[k] = bezier_points[k] * alpha + bezier_points[
+                            k - 1
+                        ] * (1.0 - alpha)
                     if b < m:
                         next_bezier_points[save] = bezier_points[p]
             yield bezier_points
@@ -1272,8 +1265,8 @@ class BSpline:
                 bezier_points = next_bezier_points
 
     def cubic_bezier_approximation(
-        self, level: int = 3, segments: int = None
-    ) -> Iterable["Bezier4P"]:
+        self, level: int = 3, segments: Optional[int] = None
+    ) -> Iterable[Bezier4P]:
         """Approximate arbitrary B-splines (degree != 3 and/or rational) by
         multiple segments of cubic Bézier curves. The choice of cubic Bézier
         curves is based on the widely support of this curves by many render
@@ -1309,7 +1302,7 @@ class BSpline:
 
         return cubic_bezier_interpolation(points)
 
-    def approximation_params(self, level: int = 3) -> List[float]:
+    def approximation_params(self, level: int = 3) -> Sequence[float]:
         """Returns an educated guess, the first level of approximation
         segments is based on the count of control points and their distribution
         along the B-spline, every additional level is a subdivision of the
@@ -1320,6 +1313,8 @@ class BSpline:
 
         """
         params = list(create_t_vector(self._control_points, "chord"))
+        if len(params) == 0:
+            return params
         if self.max_t != 1.0:
             max_t = self.max_t
             params = [p * max_t for p in params]
@@ -1328,7 +1323,7 @@ class BSpline:
         return params
 
 
-def subdivide_params(p: List[float]) -> Iterable[float]:
+def subdivide_params(p: list[float]) -> Iterable[float]:
     for i in range(len(p) - 1):
         yield p[i]
         yield (p[i] + p[i + 1]) / 2.0
@@ -1338,7 +1333,7 @@ def subdivide_params(p: List[float]) -> Iterable[float]:
 def open_uniform_bspline(
     control_points: Iterable[UVec],
     order: int = 4,
-    weights: Iterable[float] = None,
+    weights: Optional[Iterable[float]] = None,
 ) -> BSpline:
     """Creates an open uniform (periodic) `B-spline`_ curve (`open curve`_).
 
@@ -1360,9 +1355,9 @@ def open_uniform_bspline(
 def closed_uniform_bspline(
     control_points: Iterable[UVec],
     order: int = 4,
-    weights: Iterable[float] = None,
+    weights: Optional[Iterable[float]] = None,
 ) -> BSpline:
-    """Creates an closed uniform (periodic) `B-spline`_ curve (`open curve`_).
+    """Creates a closed uniform (periodic) `B-spline`_ curve (`open curve`_).
 
     This B-spline does not pass any of the control points.
 
@@ -1403,12 +1398,8 @@ def rational_bspline_from_arc(
     radius = float(radius)
 
     start_rad = math.radians(start_angle % 360)
-    end_rad = start_rad + math.radians(
-        arc_angle_span_deg(start_angle, end_angle)
-    )
-    control_points, weights, knots = nurbs_arc_parameters(
-        start_rad, end_rad, segments
-    )
+    end_rad = start_rad + math.radians(arc_angle_span_deg(start_angle, end_angle))
+    control_points, weights, knots = nurbs_arc_parameters(start_rad, end_rad, segments)
     return BSpline(
         control_points=(center + (p * radius) for p in control_points),
         weights=weights,
@@ -1421,7 +1412,7 @@ PI_2 = math.pi / 2.0
 
 
 def rational_bspline_from_ellipse(
-    ellipse: "ConstructionEllipse", segments: int = 1
+    ellipse: ConstructionEllipse, segments: int = 1
 ) -> BSpline:
     """Returns a rational B-splines for an elliptic arc.
 
@@ -1453,9 +1444,7 @@ def rational_bspline_from_ellipse(
     )
 
 
-def nurbs_arc_parameters(
-    start_angle: float, end_angle: float, segments: int = 1
-):
+def nurbs_arc_parameters(start_angle: float, end_angle: float, segments: int = 1):
     """Returns a rational B-spline parameters for a circular 2D arc with center
     at (0, 0) and a radius of 1.
 
@@ -1504,16 +1493,12 @@ def nurbs_arc_parameters(
     while g < 1.0:
         knots.extend((g, g))
         g += step
-    knots.extend(
-        [1.0] * (required_knot_values(len(control_points), 3) - len(knots))
-    )
+    knots.extend([1.0] * (required_knot_values(len(control_points), 3) - len(knots)))
 
     return control_points, weights, knots
 
 
-def bspline_basis(
-    u: float, index: int, degree: int, knots: Sequence[float]
-) -> float:
+def bspline_basis(u: float, index: int, degree: int, knots: Sequence[float]) -> float:
     """B-spline basis_vector function.
 
     Simple recursive implementation for testing and comparison.
@@ -1528,7 +1513,7 @@ def bspline_basis(
         float: basis_vector value N_i,p(u)
 
     """
-    cache: Dict[Tuple[int, int], float] = {}
+    cache: dict[tuple[int, int], float] = {}
     u = float(u)
 
     def N(i: int, p: int) -> float:
@@ -1539,11 +1524,7 @@ def bspline_basis(
                 retval = 1 if knots[i] <= u < knots[i + 1] else 0.0
             else:
                 dominator = knots[i + p] - knots[i]
-                f1 = (
-                    (u - knots[i]) / dominator * N(i, p - 1)
-                    if dominator
-                    else 0.0
-                )
+                f1 = (u - knots[i]) / dominator * N(i, p - 1) if dominator else 0.0
 
                 dominator = knots[i + p + 1] - knots[i + 1]
                 f2 = (
@@ -1561,7 +1542,7 @@ def bspline_basis(
 
 def bspline_basis_vector(
     u: float, count: int, degree: int, knots: Sequence[float]
-) -> List[float]:
+) -> list[float]:
     """Create basis_vector vector at parameter u.
 
     Used with the bspline_basis() for testing and comparison.
@@ -1573,13 +1554,13 @@ def bspline_basis_vector(
         knots: knot vector
 
     Returns:
-        List[float]: basis_vector vector, len(basis_vector) == count
+        list[float]: basis_vector vector, len(basis_vector) == count
 
     """
     assert len(knots) == (count + degree + 1)
-    basis = [
+    basis: list[float] = [
         bspline_basis(u, index, degree, knots) for index in range(count)
-    ]  # type: List[float]
+    ]
     # pick up last point ??? why is this necessary ???
     if math.isclose(u, knots[-1]):
         basis[-1] = 1.0
