@@ -1,11 +1,10 @@
-# Copyright (c) 2010-2022, Manfred Moitzi
+# Copyright (c) 2011-2023, Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
-from typing import Iterable, List, Tuple, Sequence
+from typing import Iterable, Sequence, Iterator
 
 from functools import partial
 import math
-import warnings
 from ezdxf.math import (
     Vec3,
     Vec2,
@@ -24,7 +23,6 @@ RADIANS_270 = RADIANS_90 * 3.0
 RADIANS_360 = 2.0 * math.pi
 
 __all__ = [
-    "is_close_points",
     "closest_point",
     "convex_hull_2d",
     "distance_point_line_2d",
@@ -35,8 +33,6 @@ __all__ = [
     "point_to_line_relation",
     "linspace",
     "enclosing_angles",
-    "reflect_angle_x_deg",
-    "reflect_angle_y_deg",
     "sign",
     "area",
     "circle_radius_3p",
@@ -47,19 +43,9 @@ __all__ = [
 ]
 
 
-def is_close_points(p1: UVec, p2: UVec, abs_tol=TOLERANCE) -> bool:
-    """Deprecated function will be removed in v0.18! Use Vec(p1).isclose(p2)."""
-    warnings.warn(
-        "Deprecated function will be removed in v0.18! "
-        "Use Vec3(p1).isclose(p2).",
-        DeprecationWarning,
-    )
-    return Vec3(p1).isclose(p2, abs_tol=abs_tol)
-
-
 def linspace(
     start: float, stop: float, num: int, endpoint=True
-) -> Iterable[float]:
+) -> Iterator[float]:
     """Return evenly spaced numbers over a specified interval, like
     numpy.linspace().
 
@@ -88,29 +74,7 @@ def sign(f: float) -> float:
     return -1.0 if f < 0.0 else +1.0
 
 
-def reflect_angle_x_deg(a: float) -> float:
-    """Returns reflected angle of `a` in x-direction in degrees.
-    Angles are counter-clockwise orientated and +x-axis is at 0 degrees.
-
-    Args:
-        a: angle to reflect in degrees
-
-    """
-    return (180.0 - (a % 360.0)) % 360.0
-
-
-def reflect_angle_y_deg(a: float) -> float:
-    """Returns reflected angle of `a` in y-direction in degrees.
-    Angles are counter-clockwise orientated and +y-axis is at 90 degrees.
-
-    Args:
-        a: angle to reflect in degrees
-
-    """
-    return (360.0 - (a % 360.0)) % 360.0
-
-
-def decdeg2dms(value: float) -> Tuple[float, float, float]:
+def decdeg2dms(value: float) -> tuple[float, float, float]:
     """Return decimal degrees as tuple (Degrees, Minutes, Seconds)."""
     mnt, sec = divmod(value * 3600.0, 60.0)
     deg, mnt = divmod(mnt, 60.0)
@@ -134,8 +98,8 @@ def ellipse_param_span(start_param: float, end_param: float) -> float:
     return arc_angle_span_rad(float(start_param), float(end_param))
 
 
-def closest_point(base: UVec, points: Iterable[UVec]) -> "Vec3":
-    """Returns the closest point to `base`.
+def closest_point(base: UVec, points: Iterable[UVec]) -> Vec3:
+    """Returns the closest point to a give `base` point.
 
     Args:
         base: base point as :class:`Vec3` compatible object
@@ -154,9 +118,10 @@ def closest_point(base: UVec, points: Iterable[UVec]) -> "Vec3":
     return found
 
 
-def convex_hull_2d(points: Iterable[UVec]) -> List[Vec2]:
-    """Returns 2D convex hull for `points` as list of :class:`Vec2`.
-    Returns a closed polyline, first vertex == last vertex.
+def convex_hull_2d(points: Iterable[UVec]) -> list[Vec2]:
+    """Returns the 2D convex hull of given `points`.
+
+    Returns a closed polyline, first vertex is equal to the last vertex.
 
     Args:
         points: iterable of points, z-axis is ignored
@@ -174,7 +139,7 @@ def convex_hull_2d(points: Iterable[UVec]) -> List[Vec2]:
         )
 
     n: int = len(vertices)
-    hull: List[Vec2] = [Vec2()] * (2 * n)
+    hull: list[Vec2] = [Vec2()] * (2 * n)
     k: int = 0
     i: int
     for i in range(n):
@@ -401,7 +366,7 @@ def has_matrix_2d_stretching(m: Matrix44) -> bool:
 
 
 def is_convex_polygon_2d(
-    polygon: List[Vec2], *, strict=False, epsilon=1e-6
+    polygon: list[Vec2], *, strict=False, epsilon=1e-6
 ) -> bool:
     """Returns ``True`` if the 2D `polygon` is convex. This function works with
     open and closed polygons and clockwise or counter-clockwise vertex
@@ -416,7 +381,7 @@ def is_convex_polygon_2d(
     if len(polygon) < 3:
         return False
 
-    signs: List[int] = []
+    signs: list[int] = []
     prev = polygon[-1]
     prev_prev = polygon[-2]
     for vertex in polygon:
