@@ -1,7 +1,7 @@
 # Copyright (c) 2020-2022, Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
-from typing import TYPE_CHECKING, Iterable, Dict, Tuple
+from typing import TYPE_CHECKING, Iterable, Any
 import math
 from ezdxf.math import (
     Vec3,
@@ -18,7 +18,8 @@ from ezdxf.math import (
 )
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import BaseLayout, Ellipse
+    from ezdxf.layouts import BaseLayout
+    from ezdxf.entities import Ellipse
 
 __all__ = [
     "ConstructionEllipse",
@@ -31,7 +32,7 @@ HALF_PI = math.pi / 2.0
 
 
 class ConstructionEllipse:
-    """This is a helper class to create parameters for 3D ellipses.
+    """Construction tool for 3D ellipsis.
 
     Args:
         center: 3D center point
@@ -40,7 +41,7 @@ class ConstructionEllipse:
         ratio: ratio of minor axis to major axis
         start_param: start param in radians
         end_param: end param in radians
-        ccw: is counter clockwise flag - swaps start- and end param if ``False``
+        ccw: is counter-clockwise flag - swaps start- and end param if ``False``
 
     """
 
@@ -66,9 +67,7 @@ class ConstructionEllipse:
         self.end_param = float(end_param)
         if not ccw:
             self.start_param, self.end_param = self.end_param, self.start_param
-        self.minor_axis = minor_axis(
-            self.major_axis, self.extrusion, self.ratio
-        )
+        self.minor_axis = minor_axis(self.major_axis, self.extrusion, self.ratio)
 
     @classmethod
     def from_arc(
@@ -79,7 +78,7 @@ class ConstructionEllipse:
         start_angle: float = 0,
         end_angle: float = 360,
         ccw: bool = True,
-    ) -> "ConstructionEllipse":
+    ) -> ConstructionEllipse:
         """Returns :class:`ConstructionEllipse` from arc or circle.
 
         Arc and Circle parameters defined in OCS.
@@ -145,7 +144,7 @@ class ConstructionEllipse:
             self.ratio,
         )
 
-    def dxfattribs(self) -> Dict:
+    def dxfattribs(self) -> dict[str, Any]:
         """Returns required DXF attributes to build an ELLIPSE entity.
 
         Entity ELLIPSE has always a ratio in range from 1e-6 to 1.
@@ -205,14 +204,10 @@ class ConstructionEllipse:
             # New normal vector:
             new_extrusion = new_major_axis.cross(new_minor_axis).normalize()
             # Calculate exact minor axis:
-            new_minor_axis = minor_axis(
-                new_major_axis, new_extrusion, new_ratio
-            )
+            new_minor_axis = minor_axis(new_major_axis, new_extrusion, new_ratio)
             adjust_params = False
 
-        if adjust_params and not math.isclose(
-            start_param, end_param, abs_tol=1e-9
-        ):
+        if adjust_params and not math.isclose(start_param, end_param, abs_tol=1e-9):
             # open ellipse, adjusting start- and end parameter
             x_axis = new_major_axis.normalize()
             y_axis = new_minor_axis.normalize()
@@ -255,9 +250,7 @@ class ConstructionEllipse:
                 # TODO: use ellipse_param_span()?
                 #  2021-01-28 this is possibly the source of errors!
                 new_param_span = (end_param - start_param) % math.tau
-                if not math.isclose(
-                    old_param_span, new_param_span, abs_tol=1e-9
-                ):
+                if not math.isclose(old_param_span, new_param_span, abs_tol=1e-9):
                     start_param, end_param = end_param, start_param
             else:  # param span is exact pi (180 deg)
                 # expensive but it seem to work:
@@ -281,17 +274,10 @@ class ConstructionEllipse:
                     start_param, end_param = end_param, start_param
 
         if new_ratio > 1:
-            new_major_axis = minor_axis(
-                new_major_axis, new_extrusion, new_ratio
-            )
+            new_major_axis = minor_axis(new_major_axis, new_extrusion, new_ratio)
             new_ratio = 1.0 / new_ratio
-            new_minor_axis = minor_axis(
-                new_major_axis, new_extrusion, new_ratio
-            )
-            if not (
-                math.isclose(start_param, 0)
-                and math.isclose(end_param, math.tau)
-            ):
+            new_minor_axis = minor_axis(new_major_axis, new_extrusion, new_ratio)
+            if not (math.isclose(start_param, 0) and math.isclose(end_param, math.tau)):
                 start_param -= HALF_PI
                 end_param -= HALF_PI
 
@@ -313,14 +299,14 @@ class ConstructionEllipse:
 
     @property
     def param_span(self) -> float:
-        """Returns the counter clockwise params span from start- to end param,
+        """Returns the counter-clockwise params span from start- to end param,
         see also :func:`ezdxf.math.ellipse_param_span` for more information.
 
         """
         return arc_angle_span_rad(self.start_param, self.end_param)
 
     def params(self, num: int) -> Iterable[float]:
-        """Returns `num` params from start- to end param in counter clockwise
+        """Returns `num` params from start- to end param in counter-clockwise
         order.
 
         All params are normalized in the range from [0, 2π).
@@ -333,7 +319,7 @@ class ConstructionEllipse:
 
         Args:
             params: param values in the range from [0, 2π) in radians,
-                param goes counter clockwise around the extrusion vector,
+                param goes counter-clockwise around the extrusion vector,
                 major_axis = local x-axis = 0 rad.
 
         """
@@ -361,8 +347,6 @@ class ConstructionEllipse:
                 segment chord.
             segments: minimum segment count
 
-        .. versionadded:: 0.15
-
         """
 
         def vertex_(p: float) -> Vec3:
@@ -411,12 +395,10 @@ class ConstructionEllipse:
             param = next_end_param
             start_point = end_point
 
-    def params_from_vertices(
-        self, vertices: Iterable[UVec]
-    ) -> Iterable[float]:
+    def params_from_vertices(self, vertices: Iterable[UVec]) -> Iterable[float]:
         """Yields ellipse params for all given `vertices`.
 
-        The vertex don't has to be exact on the ellipse curve or in the range
+        The vertex don't have to be exact on the ellipse curve or in the range
         from start- to end param or even in the ellipse plane. Param is
         calculated from the intersection point of the ray projected on the
         ellipse plane from the center of the ellipse through the vertex.
@@ -442,7 +424,7 @@ class ConstructionEllipse:
 
         Args:
             params: param values in the range from [0, 2π] in radians, param
-                goes counter clockwise around the extrusion vector,
+                goes counter-clockwise around the extrusion vector,
                 major_axis = local x-axis = 0 rad.
 
         """
@@ -460,9 +442,7 @@ class ConstructionEllipse:
         self.major_axis = self.minor_axis
         ratio = 1.0 / self.ratio
         self.ratio = max(ratio, 1e-6)
-        self.minor_axis = minor_axis(
-            self.major_axis, self.extrusion, self.ratio
-        )
+        self.minor_axis = minor_axis(self.major_axis, self.extrusion, self.ratio)
 
         start_param = self.start_param
         end_param = self.end_param
@@ -471,9 +451,7 @@ class ConstructionEllipse:
         self.start_param = (start_param - HALF_PI) % math.tau
         self.end_param = (end_param - HALF_PI) % math.tau
 
-    def add_to_layout(
-        self, layout: "BaseLayout", dxfattribs=None
-    ) -> "Ellipse":
+    def add_to_layout(self, layout: BaseLayout, dxfattribs=None) -> Ellipse:
         """Add ellipse as DXF :class:`~ezdxf.entities.Ellipse` entity to a
         layout.
 
@@ -491,7 +469,7 @@ class ConstructionEllipse:
         layout.add_entity(e)
         return e
 
-    def to_ocs(self) -> "ConstructionEllipse":
+    def to_ocs(self) -> ConstructionEllipse:
         """Returns ellipse parameters as OCS representation.
 
         OCS elevation is stored in :attr:`center.z`.
@@ -500,7 +478,7 @@ class ConstructionEllipse:
         ocs = OCS(self.extrusion)
         return self.__class__(
             center=ocs.from_wcs(self.center),
-            major_axis=ocs.from_wcs(self.major_axis).replace(z=0),  # type: ignore
+            major_axis=ocs.from_wcs(self.major_axis).replace(z=0.0),  # type: ignore
             ratio=self.ratio,
             start_param=self.start_param,
             end_param=self.end_param,
@@ -530,7 +508,7 @@ def vertex(
 
 
 def get_params(start: float, end: float, num: int) -> Iterable[float]:
-    """Returns `num` params from start- to end param in counter clockwise order.
+    """Returns `num` params from start- to end param in counter-clockwise order.
 
     All params are normalized in the range from [0, 2π).
 
@@ -574,7 +552,7 @@ def param_to_angle(ratio: float, param: float) -> float:
     return math.atan2(math.sin(param) * ratio, math.cos(param))
 
 
-def rytz_axis_construction(d1: Vec3, d2: Vec3) -> Tuple[Vec3, Vec3, float]:
+def rytz_axis_construction(d1: Vec3, d2: Vec3) -> tuple[Vec3, Vec3, float]:
     """The Rytz’s axis construction is a basic method of descriptive Geometry
     to find the axes, the semi-major axis and semi-minor axis, starting from two
     conjugated half-diameters.
@@ -596,9 +574,7 @@ def rytz_axis_construction(d1: Vec3, d2: Vec3) -> Tuple[Vec3, Vec3, float]:
     """
     Q = Vec3(d1)  # vector CQ
     # calculate vector CP', location P'
-    if math.isclose(d1.z, 0, abs_tol=1e-9) and math.isclose(
-        d2.z, 0, abs_tol=1e-9
-    ):
+    if math.isclose(d1.z, 0, abs_tol=1e-9) and math.isclose(d2.z, 0, abs_tol=1e-9):
         # Vec3.orthogonal() works only for vectors in the xy-plane!
         P1 = Vec3(d2).orthogonal(ccw=False)
     else:
@@ -614,9 +590,7 @@ def rytz_axis_construction(d1: Vec3, d2: Vec3) -> Tuple[Vec3, Vec3, float]:
         raise ArithmeticError("Conjugated axis required, invalid source data.")
     major_axis_length = (A - Q).magnitude
     minor_axis_length = (B - Q).magnitude
-    if math.isclose(major_axis_length, 0.0) or math.isclose(
-        minor_axis_length, 0.0
-    ):
+    if math.isclose(major_axis_length, 0.0) or math.isclose(minor_axis_length, 0.0):
         raise ArithmeticError("Conjugated axis required, invalid source data.")
     ratio = minor_axis_length / major_axis_length
     major_axis = B.normalize(major_axis_length)
