@@ -22,7 +22,7 @@ ABS_TOL = 1e-12
 
 
 class ConstructionRay:
-    """Infinite 2D construction ray as immutable object.
+    """Construction tool for infinite 2D rays.
 
     Args:
         p1: definition point 1
@@ -31,7 +31,9 @@ class ConstructionRay:
 
     """
 
-    def __init__(self, p1: UVec, p2: UVec = None, angle: float = None):
+    def __init__(
+        self, p1: UVec, p2: Optional[UVec] = None, angle: Optional[float] = None
+    ):
         self._location = Vec2(p1)
         self._angle: Optional[float]
         self._slope: Optional[float]
@@ -194,9 +196,11 @@ class ConstructionRay:
 
 
 class ConstructionLine:
-    """2D ConstructionLine is similar to :class:`ConstructionRay`, but has a
-    start- and endpoint. The direction of line goes from start- to endpoint,
-    "left of line" is always in relation to this line direction.
+    """Construction tool for 2D lines.
+
+    The :class:`ConstructionLine` class is similar to :class:`ConstructionRay`,
+    but has a start- and endpoint. The direction of line goes from start- to
+    endpoint, "left of line" is always in relation to this line direction.
 
     Args:
         start: start point of line as :class:`Vec2` compatible object
