@@ -1,4 +1,4 @@
-# Copyright (c) 2020-2022, Manfred Moitzi
+# Copyright (c) 2020-2023, Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
 from typing import (
@@ -6,15 +6,13 @@ from typing import (
     Iterable,
     Iterator,
     Union,
-    Tuple,
     Optional,
-    Dict,
     Callable,
     Type,
     TypeVar,
 )
+from typing_extensions import TypeAlias
 from functools import singledispatch, partial
-import enum
 from ezdxf.math import (
     ABS_TOL,
     Vec2,
@@ -58,6 +56,7 @@ from ezdxf.entities import (
     EllipseEdge,
     SplineEdge,
 )
+from ezdxf.entities.polygon import DXFPolygon
 from .path import Path
 from .commands import Command
 from . import tools
@@ -74,16 +73,11 @@ __all__ = [
     "to_bsplines_and_vertices",
     "to_splines_and_polylines",
     "from_hatch",
+    "from_hatch_ocs",
     "from_hatch_boundary_path",
     "from_hatch_edge_path",
     "from_hatch_polyline_path",
     "from_vertices",
-    "from_matplotlib_path",
-    "from_qpainter_path",
-    "to_matplotlib_path",
-    "to_qpainter_path",
-    "multi_path_from_matplotlib_path",
-    "multi_path_from_qpainter_path",
 ]
 
 MAX_DISTANCE = 0.01
@@ -115,7 +109,7 @@ def make_path(entity, segments: int = 1, level: int = 4) -> Path:
 
 
 @make_path.register(LWPolyline)
-def _from_lwpolyline(lwpolyline: LWPolyline, **kwargs) -> "Path":
+def _from_lwpolyline(lwpolyline: LWPolyline, **kwargs) -> Path:
     path = Path()
     tools.add_2d_polyline(
         path,
@@ -123,12 +117,13 @@ def _from_lwpolyline(lwpolyline: LWPolyline, **kwargs) -> "Path":
         close=lwpolyline.closed,
         ocs=lwpolyline.ocs(),
         elevation=lwpolyline.dxf.elevation,
+        segments=kwargs.get("segments", 1),
     )
     return path
 
 
 @make_path.register(Polyline)
-def _from_polyline(polyline: Polyline, **kwargs) -> "Path":
+def _from_polyline(polyline: Polyline, **kwargs) -> Path:
     if polyline.is_polygon_mesh or polyline.is_poly_face_mesh:
         raise TypeError("Unsupported DXF type PolyMesh or PolyFaceMesh")
 
@@ -144,8 +139,8 @@ def _from_polyline(polyline: Polyline, **kwargs) -> "Path":
     if polyline.dxf.hasattr("elevation"):
         elevation = Vec3(polyline.dxf.elevation).z
     else:
-        # Elevation attribute is mandatory, but you never know,
-        # take elevation from first vertex.
+        # the elevation attribute is mandatory, but if it's missing
+        # take the elevation value of the first vertex.
         elevation = Vec3(polyline.vertices[0].dxf.location).z
     tools.add_2d_polyline(
         path,
@@ -153,13 +148,14 @@ def _from_polyline(polyline: Polyline, **kwargs) -> "Path":
         close=polyline.is_closed,
         ocs=ocs,
         elevation=elevation,
+        segments=kwargs.get("segments", 1),
     )
     return path
 
 
 @make_path.register(Helix)
 @make_path.register(Spline)
-def _from_spline(spline: Spline, **kwargs) -> "Path":
+def _from_spline(spline: Spline, **kwargs) -> Path:
     level = kwargs.get("level", 4)
     path = Path()
     tools.add_spline(path, spline.construction_tool(), level=level, reset=True)
@@ -167,24 +163,22 @@ def _from_spline(spline: Spline, **kwargs) -> "Path":
 
 
 @make_path.register(Ellipse)
-def _from_ellipse(ellipse: Ellipse, **kwargs) -> "Path":
+def _from_ellipse(ellipse: Ellipse, **kwargs) -> Path:
     segments = kwargs.get("segments", 1)
     path = Path()
-    tools.add_ellipse(
-        path, ellipse.construction_tool(), segments=segments, reset=True
-    )
+    tools.add_ellipse(path, ellipse.construction_tool(), segments=segments, reset=True)
     return path
 
 
 @make_path.register(Line)
-def _from_line(line: Line, **kwargs) -> "Path":
+def _from_line(line: Line, **kwargs) -> Path:
     path = Path(line.dxf.start)
     path.line_to(line.dxf.end)
     return path
 
 
 @make_path.register(Arc)
-def _from_arc(arc: Arc, **kwargs) -> "Path":
+def _from_arc(arc: Arc, **kwargs) -> Path:
     segments = kwargs.get("segments", 1)
     path = Path()
     radius = abs(arc.dxf.radius)
@@ -201,7 +195,7 @@ def _from_arc(arc: Arc, **kwargs) -> "Path":
 
 
 @make_path.register(Circle)
-def _from_circle(circle: Circle, **kwargs) -> "Path":
+def _from_circle(circle: Circle, **kwargs) -> Path:
     segments = kwargs.get("segments", 1)
     path = Path()
     radius = abs(circle.dxf.radius)
@@ -218,7 +212,7 @@ def _from_circle(circle: Circle, **kwargs) -> "Path":
 @make_path.register(Face3d)
 @make_path.register(Trace)
 @make_path.register(Solid)
-def _from_quadrilateral(solid: "Solid", **kwargs) -> "Path":
+def _from_quadrilateral(solid: "Solid", **kwargs) -> Path:
     vertices = solid.wcs_vertices()
     return from_vertices(vertices, close=True)
 
@@ -260,22 +254,30 @@ def _from_hatch(hatch: Hatch, **kwargs) -> Path:
     return tools.to_multi_path(paths)
 
 
-def from_hatch(hatch: Hatch) -> Iterator[Path]:
-    """Yield all HATCH boundary paths as separated :class:`Path` objects.
-
-
-    .. versionadded:: 0.16
-
-    .. versionchanged:: 17.1
-
-        Attaches the boundary state to each path as
-        :class:`ezdxf.lldxf.const.BoundaryPathState`.
-
+def from_hatch(hatch: DXFPolygon, offset: Vec3 = NULLVEC) -> Iterator[Path]:
+    """Yield all HATCH/MPOLYGON boundary paths as separated :class:`Path` objects in WCS
+    coordinates.
     """
     ocs = hatch.ocs()
     elevation = hatch.dxf.elevation.z
     for boundary in hatch.paths:
-        p = from_hatch_boundary_path(boundary, ocs, elevation)
+        p = from_hatch_boundary_path(boundary, ocs, elevation=elevation, offset=offset)
+        if p.has_sub_paths:
+            yield from p.sub_paths()
+        else:
+            yield p
+
+
+def from_hatch_ocs(hatch: DXFPolygon, offset: Vec3 = NULLVEC) -> Iterator[Path]:
+    """Yield all HATCH/MPOLYGON boundary paths as separated :class:`Path` objects in OCS
+    coordinates. Elevation and offset is applied to all vertices.
+
+    .. versionadded:: 1.1
+
+    """
+    elevation = hatch.dxf.elevation.z
+    for boundary in hatch.paths:
+        p = from_hatch_boundary_path(boundary, elevation=elevation, offset=offset)
         if p.has_sub_paths:
             yield from p.sub_paths()
         else:
@@ -284,18 +286,12 @@ def from_hatch(hatch: Hatch) -> Iterator[Path]:
 
 def from_hatch_boundary_path(
     boundary: AbstractBoundaryPath,
-    ocs: OCS = None,
+    ocs: Optional[OCS] = None,
     elevation: float = 0,
     offset: Vec3 = NULLVEC,  # ocs offset!
-) -> "Path":
+) -> Path:
     """Returns a :class:`Path` object from a :class:`~ezdxf.entities.Hatch`
     polyline- or edge path.
-
-    .. versionchanged:: 17.1
-
-        Attaches the boundary state to each path as
-        :class:`ezdxf.lldxf.const.BoundaryPathState`.
-
     """
     if isinstance(boundary, EdgePath):
         p = from_hatch_edge_path(boundary, ocs, elevation)
@@ -315,7 +311,7 @@ def from_hatch_boundary_path(
 
 
 def from_hatch_polyline_path(
-    polyline: PolylinePath, ocs: OCS = None, elevation: float = 0
+    polyline: PolylinePath, ocs: Optional[OCS] = None, elevation: float = 0
 ) -> Path:
     """Returns a :class:`Path` object from a :class:`~ezdxf.entities.Hatch`
     polyline path.
@@ -323,7 +319,7 @@ def from_hatch_polyline_path(
     path = Path()
     tools.add_2d_polyline(
         path,
-        polyline.vertices,  # List[(x, y, bulge)]
+        polyline.vertices,  # list[(x, y, bulge)]
         close=polyline.is_closed,
         ocs=ocs or OCS(),
         elevation=elevation,
@@ -333,7 +329,7 @@ def from_hatch_polyline_path(
 
 def from_hatch_edge_path(
     edges: EdgePath,
-    ocs: OCS = None,
+    ocs: Optional[OCS] = None,
     elevation: float = 0,
 ) -> Path:
     """Returns a :class:`Path` object from a :class:`~ezdxf.entities.Hatch`
@@ -372,7 +368,7 @@ def from_hatch_edge_path(
         # clockwise ellipses at the loading stage!
         # See: ezdxf.entities.boundary_paths.EllipseEdge.load_tags()
         ellipse = ConstructionEllipse(
-            center=wcs(ocs_ellipse.center.replace(z=elevation)),
+            center=wcs(ocs_ellipse.center.replace(z=float(elevation))),
             major_axis=wcs_tangent(ocs_ellipse.major_axis),
             ratio=ocs_ellipse.ratio,
             extrusion=extrusion,
@@ -419,11 +415,11 @@ def from_hatch_edge_path(
             weights=edge.weights if edge.weights else None,
         )
 
-    def wcs(vertex: Vec2) -> Vec3:
-        return _wcs(Vec3(vertex.x, vertex.y, elevation))
+    def wcs(vertex: UVec) -> Vec3:
+        return _wcs(Vec3(vertex[0], vertex[1], elevation))
 
-    def wcs_tangent(vertex: Vec2) -> Vec3:
-        return _wcs(Vec3(vertex.x, vertex.y, 0))
+    def wcs_tangent(vertex: UVec) -> Vec3:
+        return _wcs(Vec3(vertex[0], vertex[1], 0))
 
     def _wcs(vec3: Vec3) -> Vec3:
         if ocs and ocs.transform:
@@ -523,8 +519,6 @@ def to_lwpolylines(
     Returns:
         iterable of :class:`~ezdxf.entities.LWPolyline` objects
 
-    .. versionadded:: 0.16
-
     """
     if isinstance(paths, Path):
         paths = [paths]
@@ -532,9 +526,8 @@ def to_lwpolylines(
         paths = list(paths)
     if len(paths) == 0:
         return []
-
     extrusion = Vec3(extrusion)
-    reference_point = paths[0].start
+    reference_point = Vec3(paths[0].start)
     dxfattribs = dict(dxfattribs or {})
     if not Z_AXIS.isclose(extrusion):
         ocs, elevation = _get_ocs(extrusion, reference_point)
@@ -544,14 +537,14 @@ def to_lwpolylines(
     elif reference_point.z != 0:
         dxfattribs["elevation"] = reference_point.z
 
-    for path in tools.single_paths(paths):
+    for path in tools.single_paths(paths):  # type: ignore
         if len(path) > 0:
             p = LWPolyline.new(dxfattribs=dxfattribs)
             p.append_points(path.flattening(distance, segments), format="xy")  # type: ignore
             yield p
 
 
-def _get_ocs(extrusion: Vec3, reference_point: Vec3) -> Tuple[OCS, float]:
+def _get_ocs(extrusion: Vec3, reference_point: Vec3) -> tuple[OCS, float]:
     ocs = OCS(extrusion)
     elevation = ocs.from_wcs(reference_point).z  # type: ignore
     return ocs, elevation
@@ -582,8 +575,6 @@ def to_polylines2d(
     Returns:
         iterable of 2D :class:`~ezdxf.entities.Polyline` objects
 
-    .. versionadded:: 0.16
-
     """
     if isinstance(paths, Path):
         paths = [paths]
@@ -591,9 +582,8 @@ def to_polylines2d(
         paths = list(paths)
     if len(paths) == 0:
         return []
-
     extrusion = Vec3(extrusion)
-    reference_point = paths[0].start
+    reference_point = Vec3(paths[0].start)
     dxfattribs = dict(dxfattribs or {})
     if not Z_AXIS.isclose(extrusion):
         ocs, elevation = _get_ocs(extrusion, reference_point)
@@ -603,10 +593,11 @@ def to_polylines2d(
     elif reference_point.z != 0:
         dxfattribs["elevation"] = Vec3(0, 0, reference_point.z)
 
-    for path in tools.single_paths(paths):
+    for path in tools.single_paths(paths):  # type: ignore
         if len(path) > 0:
             p = Polyline.new(dxfattribs=dxfattribs)
             p.append_vertices(path.flattening(distance, segments))
+            p.new_seqend()
             yield p
 
 
@@ -641,8 +632,6 @@ def to_hatches(
     Returns:
         iterable of :class:`~ezdxf.entities.Hatch` objects
 
-    .. versionadded:: 0.16
-
     """
     boundary_factory: BoundaryFactory
     if edge_path:
@@ -688,8 +677,6 @@ def to_mpolygons(
     Returns:
         iterable of :class:`~ezdxf.entities.MPolygon` objects
 
-    .. versionadded:: 0.17
-
     """
     # noinspection PyTypeChecker
     boundary_factory: BoundaryFactory = partial(
@@ -761,7 +748,7 @@ def _polygon_converter(
 
     extrusion = Vec3(extrusion)
     reference_point = paths[0].start
-    _dxfattribs: Dict = dict(dxfattribs or {})
+    _dxfattribs: dict = dict(dxfattribs or {})
     if not Z_AXIS.isclose(extrusion):
         ocs, elevation = _get_ocs(extrusion, reference_point)
         paths = tools.transform_paths_to_ocs(paths, ocs)
@@ -780,10 +767,10 @@ def _polygon_converter(
         boundaries = polygon.paths
         external = group[0]
         external.close()
-        add_boundary(boundaries, external, 1)
+        add_boundary(boundaries, external, 1)  # type: ignore
         for hole in group[1:]:
             hole.close()
-            add_boundary(boundaries, hole, 0)
+            add_boundary(boundaries, hole, 0)  # type: ignore
         yield polygon
 
 
@@ -806,18 +793,17 @@ def to_polylines3d(
     Returns:
         iterable of 3D :class:`~ezdxf.entities.Polyline` objects
 
-    .. versionadded:: 0.16
-
     """
     if isinstance(paths, Path):
         paths = [paths]
 
     dxfattribs = dict(dxfattribs or {})
     dxfattribs["flags"] = const.POLYLINE_3D_POLYLINE
-    for path in tools.single_paths(paths):
+    for path in tools.single_paths(paths):  # type: ignore
         if len(path) > 0:
             p = Polyline.new(dxfattribs=dxfattribs)
             p.append_vertices(path.flattening(distance, segments))
+            p.new_seqend()
             yield p
 
 
@@ -839,14 +825,12 @@ def to_lines(
     Returns:
         iterable of :class:`~ezdxf.entities.Line` objects
 
-    .. versionadded:: 0.16
-
     """
     if isinstance(paths, Path):
         paths = [paths]
     dxfattribs = dict(dxfattribs or {})
     prev_vertex = None
-    for path in tools.single_paths(paths):
+    for path in tools.single_paths(paths):  # type: ignore
         if len(path) == 0:
             continue
         for vertex in path.flattening(distance, segments):
@@ -860,12 +844,10 @@ def to_lines(
         prev_vertex = None
 
 
-PathParts = Union[BSpline, List[Vec3]]
+PathParts: TypeAlias = Union[BSpline, List[Vec3]]
 
 
-def to_bsplines_and_vertices(
-    path: Path, g1_tol: float = G1_TOL
-) -> Iterator[PathParts]:
+def to_bsplines_and_vertices(path: Path, g1_tol: float = G1_TOL) -> Iterator[PathParts]:
     """Convert a :class:`Path` object into multiple cubic B-splines and
     polylines as lists of vertices. Breaks adjacent Bèzier without G1
     continuity into separated B-splines.
@@ -877,8 +859,6 @@ def to_bsplines_and_vertices(
     Returns:
         :class:`~ezdxf.math.BSpline` and lists of :class:`~ezdxf.math.Vec3`
 
-    .. versionadded:: 0.16
-
     """
     from ezdxf.math import bezier_to_bspline
 
@@ -903,7 +883,7 @@ def to_bsplines_and_vertices(
             yield bezier_to_bspline(_g1_continuity_curves)
 
     curves = []
-    for path in tools.single_paths([path]):
+    for path in tools.single_paths([path]):  # type: ignore
         prev = path.start
         for cmd in path:
             if cmd.type == Command.CURVE3_TO:
@@ -917,8 +897,8 @@ def to_bsplines_and_vertices(
             curves.append(curve)
             prev = cmd.end
 
-    bezier: List = []
-    polyline: List = []
+    bezier: list = []
+    polyline: list = []
     for curve in curves:
         if isinstance(curve, tuple):
             if bezier:
@@ -954,14 +934,12 @@ def to_splines_and_polylines(
     Returns:
         iterable of :class:`~ezdxf.entities.Line` objects
 
-    .. versionadded:: 0.16
-
     """
     if isinstance(paths, Path):
         paths = [paths]
     dxfattribs = dict(dxfattribs or {})
 
-    for path in tools.single_paths(paths):
+    for path in tools.single_paths(paths):  # type: ignore
         for data in to_bsplines_and_vertices(path, g1_tol):
             if isinstance(data, BSpline):
                 spline = Spline.new(dxfattribs=dxfattribs)
@@ -972,224 +950,5 @@ def to_splines_and_polylines(
                 attribs["flags"] = const.POLYLINE_3D_POLYLINE
                 polyline = Polyline.new(dxfattribs=dxfattribs)
                 polyline.append_vertices(data)
+                polyline.new_seqend()
                 yield polyline
-
-
-# Interface to Matplotlib.path.Path
-
-
-@enum.unique
-class MplCmd(enum.IntEnum):
-    CLOSEPOLY = 79
-    CURVE3 = 3
-    CURVE4 = 4
-    LINETO = 2
-    MOVETO = 1
-    STOP = 0
-
-
-def multi_path_from_matplotlib_path(mpath, curves=True) -> Path:
-    """Returns a :class:`Path` object from a Matplotlib `Path`_
-    (`TextPath`_)  object. (requires Matplotlib). Returns a multi-path object
-    if necessary.
-
-    .. versionadded:: 0.17
-
-    .. _TextPath: https://matplotlib.org/3.1.1/api/textpath_api.html
-    .. _Path: https://matplotlib.org/3.1.1/api/path_api.html#matplotlib.path.Path
-
-    """
-    path = Path()
-    current_polyline_start = Vec3()
-    for vertices, cmd in mpath.iter_segments(curves=curves):
-        cmd = MplCmd(cmd)
-        if cmd == MplCmd.MOVETO:
-            # vertices = [x0, y0]
-            current_polyline_start = Vec3(vertices)
-            path.move_to(vertices)
-        elif cmd == MplCmd.LINETO:
-            # vertices = [x0, y0]
-            path.line_to(vertices)
-        elif cmd == MplCmd.CURVE3:
-            # vertices = [x0, y0, x1, y1]
-            path.curve3_to(vertices[2:], vertices[0:2])
-        elif cmd == MplCmd.CURVE4:
-            # vertices = [x0, y0, x1, y1, x2, y2]
-            path.curve4_to(vertices[4:], vertices[0:2], vertices[2:4])
-        elif cmd == MplCmd.CLOSEPOLY:
-            # vertices = [0, 0]
-            if not path.end.isclose(current_polyline_start):
-                path.line_to(current_polyline_start)
-        elif cmd == MplCmd.STOP:  # not used
-            pass
-    return path
-
-
-def from_matplotlib_path(mpath, curves=True) -> Iterator[Path]:
-    """Yields multiple :class:`Path` objects from a Matplotlib `Path`_
-    (`TextPath`_)  object. (requires Matplotlib)
-
-    .. versionadded:: 0.16
-
-    .. _TextPath: https://matplotlib.org/3.1.1/api/textpath_api.html
-    .. _Path: https://matplotlib.org/3.1.1/api/path_api.html#matplotlib.path.Path
-
-    """
-    path = multi_path_from_matplotlib_path(mpath, curves=curves)
-    if path.has_sub_paths:
-        return path.sub_paths()
-    else:
-        return iter([path])
-
-
-def to_matplotlib_path(paths: Iterable[Path], extrusion: UVec = Z_AXIS):
-    """Convert the given `paths` into a single :class:`matplotlib.path.Path`
-    object.
-    The `extrusion` vector is applied to all paths, all vertices are projected
-    onto the plane normal to this extrusion vector.The default extrusion vector
-    is the WCS z-axis. The Matplotlib :class:`Path` is a 2D object with
-    :ref:`OCS` coordinates and the z-elevation is lost. (requires Matplotlib)
-
-    Args:
-        paths: iterable of :class:`Path` objects
-        extrusion: extrusion vector for all paths
-
-    Returns:
-        matplotlib `Path`_ in OCS!
-
-    .. versionadded:: 0.16
-
-    """
-    from matplotlib.path import Path as MatplotlibPath
-
-    if not Z_AXIS.isclose(extrusion):
-        paths = tools.transform_paths_to_ocs(paths, OCS(extrusion))
-    else:
-        paths = list(paths)
-    if len(paths) == 0:
-        raise ValueError("one or more paths required")
-
-    def add_command(code: MplCmd, point: Vec3):
-        codes.append(code)
-        vertices.append((point.x, point.y))
-
-    vertices: List[Tuple[float, float]] = []
-    codes: List[MplCmd] = []
-    for path in paths:
-        add_command(MplCmd.MOVETO, path.start)
-        for cmd in path.commands():
-            if cmd.type == Command.LINE_TO:
-                add_command(MplCmd.LINETO, cmd.end)
-            elif cmd.type == Command.MOVE_TO:
-                add_command(MplCmd.MOVETO, cmd.end)
-            elif cmd.type == Command.CURVE3_TO:
-                add_command(MplCmd.CURVE3, cmd.ctrl)  # type: ignore
-                add_command(MplCmd.CURVE3, cmd.end)
-            elif cmd.type == Command.CURVE4_TO:
-                add_command(MplCmd.CURVE4, cmd.ctrl1)  # type: ignore
-                add_command(MplCmd.CURVE4, cmd.ctrl2)  # type: ignore
-                add_command(MplCmd.CURVE4, cmd.end)
-
-    # STOP command is currently not required
-    assert len(vertices) == len(codes)
-    return MatplotlibPath(vertices, codes)
-
-
-# Interface to QtGui.QPainterPath
-
-
-def multi_path_from_qpainter_path(qpath) -> Path:
-    """Returns a :class:`Path` objects from a `QPainterPath`_.
-    Returns a multi-path object if necessary. (requires Qt bindings)
-
-    .. versionadded:: 0.17
-
-    .. _QPainterPath: https://doc.qt.io/qt-5/qpainterpath.html
-
-    """
-    # QPainterPath stores only cubic Bèzier curves
-    path = Path()
-    vertices: List[Vec3] = []
-    for index in range(qpath.elementCount()):
-        element = qpath.elementAt(index)
-        cmd = element.type
-        v = Vec3(element.x, element.y)
-
-        if cmd == 0:  # MoveTo
-            assert len(vertices) == 0
-            path.move_to(v)
-        elif cmd == 1:  # LineTo
-            assert len(vertices) == 0
-            path.line_to(v)
-        elif cmd == 2:  # CurveTo
-            assert len(vertices) == 0
-            vertices.append(v)
-        elif cmd == 3:  # CurveToDataElement
-            if len(vertices) == 2:
-                path.curve4_to(v, vertices[0], vertices[1])
-                vertices.clear()
-            else:
-                vertices.append(v)
-    return path
-
-
-def from_qpainter_path(qpath) -> Iterator[Path]:
-    """Yields multiple :class:`Path` objects from a `QPainterPath`_.
-    (requires Qt bindings)
-
-    .. versionadded:: 0.16
-
-    .. _QPainterPath: https://doc.qt.io/qt-5/qpainterpath.html
-
-    """
-    # QPainterPath stores only cubic Bèzier curves
-    path = multi_path_from_qpainter_path(qpath)
-    if path.has_sub_paths:
-        return path.sub_paths()
-    else:
-        return iter([path])
-
-
-def to_qpainter_path(paths: Iterable[Path], extrusion: UVec = Z_AXIS):
-    """Convert the given `paths` into a :class:`QtGui.QPainterPath`
-    object.
-    The `extrusion` vector is applied to all paths, all vertices are projected
-    onto the plane normal to this extrusion vector. The default extrusion vector
-    is the WCS z-axis. The :class:`QPainterPath` is a 2D object with :ref:`OCS`
-    coordinates and the z-elevation is lost. (requires Qt bindings)
-
-    Args:
-        paths: iterable of :class:`Path` objects
-        extrusion: extrusion vector for all paths
-
-    Returns:
-        `QPainterPath`_ in OCS!
-
-    .. versionadded:: 0.16
-
-    """
-    from ezdxf.addons.xqt import QPainterPath, QPointF
-
-    if not Z_AXIS.isclose(extrusion):
-        paths = tools.transform_paths_to_ocs(paths, OCS(extrusion))
-    else:
-        paths = list(paths)
-    if len(paths) == 0:
-        raise ValueError("one or more paths required")
-
-    def qpnt(v: Vec3):
-        return QPointF(v.x, v.y)
-
-    qpath = QPainterPath()
-    for path in paths:
-        qpath.moveTo(qpnt(path.start))
-        for cmd in path.commands():
-            if cmd.type == Command.LINE_TO:
-                qpath.lineTo(qpnt(cmd.end))
-            elif cmd.type == Command.MOVE_TO:
-                qpath.moveTo(qpnt(cmd.end))
-            elif cmd.type == Command.CURVE3_TO:
-                qpath.quadTo(qpnt(cmd.ctrl), qpnt(cmd.end))  # type: ignore
-            elif cmd.type == Command.CURVE4_TO:
-                qpath.cubicTo(qpnt(cmd.ctrl1), qpnt(cmd.ctrl2), qpnt(cmd.end))  # type: ignore
-    return qpath
