@@ -1,16 +1,14 @@
-# Copyright (c) 2020-2022, Manfred Moitzi
+# Copyright (c) 2020-2023, Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
 from typing import (
     TYPE_CHECKING,
-    List,
     Iterable,
     Iterator,
-    Tuple,
     Optional,
     Sequence,
+    TypeVar,
 )
-
 import math
 from ezdxf.math import (
     Vec2,
@@ -29,6 +27,7 @@ from ezdxf.math import (
     reverse_bezier_curves,
     bulge_to_arc,
     linear_vertex_spacing,
+    linspace,
     inscribe_circle_tangent_length,
     cubic_bezier_arc_parameters,
     cubic_bezier_bbox,
@@ -42,7 +41,9 @@ from .commands import Command
 from . import converter, nesting
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import Layout, EntityQuery
+    from ezdxf.query import EntityQuery
+    from ezdxf.eztypes import GenericLayoutType
+
 
 __all__ = [
     "bbox",
@@ -84,9 +85,6 @@ IS_CLOSE_TOL = 1e-10
 def to_multi_path(paths: Iterable[Path]) -> Path:
     """Returns a multi-path object from all given paths and their sub-paths.
     Ignores paths without any commands (empty paths).
-
-    .. versionadded:: 0.17
-
     """
     multi_path = Path()
     for p in paths:
@@ -95,11 +93,7 @@ def to_multi_path(paths: Iterable[Path]) -> Path:
 
 
 def single_paths(paths: Iterable[Path]) -> Iterable[Path]:
-    """Yields all given paths and their sub-paths as single path objects.
-
-    .. versionadded:: 0.17
-
-    """
+    """Yields all given paths and their sub-paths as single path objects."""
     for p in paths:
         if p.has_sub_paths:
             yield from p.sub_paths()
@@ -107,24 +101,24 @@ def single_paths(paths: Iterable[Path]) -> Iterable[Path]:
             yield p
 
 
-def transform_paths(paths: Iterable[Path], m: Matrix44) -> List[Path]:
-    """Transform multiple :class:`Path` objects at once by transformation
-    matrix `m`. Returns a list of the transformed :class:`Path` objects.
+def transform_paths(paths: Iterable[Path], m: Matrix44) -> list[Path]:
+    """Transform multiple path objects at once by transformation
+    matrix `m`. Returns a list of the transformed path objects.
 
     Args:
-        paths: iterable of :class:`Path` objects
+        paths: iterable of :class:`Path` or :class:`Path2d` objects
         m: transformation matrix of type :class:`~ezdxf.math.Matrix44`
 
     """
     return [p.transform(m) for p in paths]
 
 
-def transform_paths_to_ocs(paths: Iterable[Path], ocs: OCS) -> List[Path]:
+def transform_paths_to_ocs(paths: Iterable[Path], ocs: OCS) -> list[Path]:
     """Transform multiple :class:`Path` objects at once from WCS to OCS.
     Returns a list of the transformed :class:`Path` objects.
 
     Args:
-        paths: iterable of :class:`Path` objects
+        paths: iterable of :class:`Path` or :class:`Path2d` objects
         ocs: OCS transformation of type :class:`~ezdxf.math.OCS`
 
     """
@@ -137,17 +131,11 @@ def bbox(paths: Iterable[Path], *, fast=False) -> BoundingBox:
     """Returns the :class:`~ezdxf.math.BoundingBox` for the given paths.
 
     Args:
-        paths: iterable of :class:`~ezdxf.path.Path` objects
+        paths: iterable of :class:`Path` or :class:`Path2d` objects
         fast: calculates the precise bounding box of Bèzier curves if
             ``False``, otherwise uses the control points of Bézier curves to
             determine their bounding box.
 
-    .. versionchanged:: 0.18
-
-        Uses a new algorithm to determine exact Bézier curve bounding boxes.
-        Added argument `fast` and removed the arguments `flatten` and
-        `segments`.
-
     """
     box = BoundingBox()
     for p in paths:
@@ -161,15 +149,12 @@ def bbox(paths: Iterable[Path], *, fast=False) -> BoundingBox:
 
 
 def precise_bbox(path: Path) -> BoundingBox:
-    """Returns the precise :class:`~ezdxf.math.BoundingBox` for the given
-    :class:`~ezdxf.path.Path`.
-
-    """
+    """Returns the precise :class:`~ezdxf.math.BoundingBox` for the given paths."""
     if len(path) == 0:  # empty path
         return BoundingBox()
     start = path.start
-    points: List[Vec3] = [start]
-    for cmd in path:
+    points: list[Vec3] = [start]
+    for cmd in path.commands():
         if cmd.type == Command.LINE_TO:
             points.append(cmd.end)
         elif cmd.type == Command.CURVE4_TO:
@@ -191,10 +176,10 @@ def precise_bbox(path: Path) -> BoundingBox:
 
 def fit_paths_into_box(
     paths: Iterable[Path],
-    size: Tuple[float, float, float],
+    size: tuple[float, float, float],
     uniform: bool = True,
-    source_box: BoundingBox = None,
-) -> List[Path]:
+    source_box: Optional[BoundingBox] = None,
+) -> list[Path]:
     """Scale the given `paths` to fit into a box of the given `size`,
     so that all path vertices are inside these borders.
     If `source_box` is ``None`` the default source bounding box is calculated
@@ -272,7 +257,7 @@ def _get_non_uniform_scaling(current_size: Vec3, target_size: Vec3):
 
 
 def render_lwpolylines(
-    layout: "Layout",
+    layout: GenericLayoutType,
     paths: Iterable[Path],
     *,
     distance: float = MAX_DISTANCE,
@@ -289,7 +274,7 @@ def render_lwpolylines(
 
     Args:
         layout: the modelspace, a paperspace layout or a block definition
-        paths: iterable of :class:`Path` objects
+        paths: iterable of :class:`Path` or :class:`Path2d` objects
         distance:  maximum distance, see :meth:`Path.flattening`
         segments: minimum segment count per Bézier curve
         extrusion: extrusion vector for all paths
@@ -298,8 +283,6 @@ def render_lwpolylines(
     Returns:
         created entities in an :class:`~ezdxf.query.EntityQuery` object
 
-    .. versionadded:: 0.16
-
     """
     lwpolylines = list(
         converter.to_lwpolylines(
@@ -316,7 +299,7 @@ def render_lwpolylines(
 
 
 def render_polylines2d(
-    layout: "Layout",
+    layout: GenericLayoutType,
     paths: Iterable[Path],
     *,
     distance: float = 0.01,
@@ -333,7 +316,7 @@ def render_polylines2d(
 
     Args:
         layout: the modelspace, a paperspace layout or a block definition
-        paths: iterable of :class:`Path` objects
+        paths: iterable of :class:`Path` or :class:`Path2d` objects
         distance:  maximum distance, see :meth:`Path.flattening`
         segments: minimum segment count per Bézier curve
         extrusion: extrusion vector for all paths
@@ -342,8 +325,6 @@ def render_polylines2d(
     Returns:
         created entities in an :class:`~ezdxf.query.EntityQuery` object
 
-    .. versionadded:: 0.16
-
     """
     polylines2d = list(
         converter.to_polylines2d(
@@ -360,7 +341,7 @@ def render_polylines2d(
 
 
 def render_hatches(
-    layout: "Layout",
+    layout: GenericLayoutType,
     paths: Iterable[Path],
     *,
     edge_path: bool = True,
@@ -379,7 +360,7 @@ def render_hatches(
 
     Args:
         layout: the modelspace, a paperspace layout or a block definition
-        paths: iterable of :class:`Path` objects
+        paths: iterable of :class:`Path` or :class:`Path2d`  objects
         edge_path: ``True`` for edge paths build of LINE and SPLINE edges,
             ``False`` for only LWPOLYLINE paths as boundary paths
         distance:  maximum distance, see :meth:`Path.flattening`
@@ -391,8 +372,6 @@ def render_hatches(
     Returns:
         created entities in an :class:`~ezdxf.query.EntityQuery` object
 
-    .. versionadded:: 0.16
-
     """
     hatches = list(
         converter.to_hatches(
@@ -411,7 +390,7 @@ def render_hatches(
 
 
 def render_mpolygons(
-    layout: "Layout",
+    layout: GenericLayoutType,
     paths: Iterable[Path],
     *,
     distance: float = MAX_DISTANCE,
@@ -430,7 +409,7 @@ def render_mpolygons(
 
     Args:
         layout: the modelspace, a paperspace layout or a block definition
-        paths: iterable of :class:`Path` objects
+        paths: iterable of :class:`Path` or :class:`Path2d` objects
         distance:  maximum distance, see :meth:`Path.flattening`
         segments: minimum segment count per Bézier curve to flatten polyline paths
         extrusion: extrusion vector for all paths
@@ -439,8 +418,6 @@ def render_mpolygons(
     Returns:
         created entities in an :class:`~ezdxf.query.EntityQuery` object
 
-    .. versionadded:: 0.17
-
     """
     polygons = list(
         converter.to_mpolygons(
@@ -457,7 +434,7 @@ def render_mpolygons(
 
 
 def render_polylines3d(
-    layout: "Layout",
+    layout: GenericLayoutType,
     paths: Iterable[Path],
     *,
     distance: float = MAX_DISTANCE,
@@ -469,7 +446,7 @@ def render_polylines3d(
 
     Args:
         layout: the modelspace, a paperspace layout or a block definition
-        paths: iterable of :class:`Path` objects
+        paths: iterable of :class:`Path`or :class:`Path2d` objects
         distance:  maximum distance, see :meth:`Path.flattening`
         segments: minimum segment count per Bézier curve
         dxfattribs: additional DXF attribs
@@ -477,8 +454,6 @@ def render_polylines3d(
     Returns:
         created entities in an :class:`~ezdxf.query.EntityQuery` object
 
-    .. versionadded:: 0.16
-
     """
 
     polylines3d = list(
@@ -495,7 +470,7 @@ def render_polylines3d(
 
 
 def render_lines(
-    layout: "Layout",
+    layout: GenericLayoutType,
     paths: Iterable[Path],
     *,
     distance: float = MAX_DISTANCE,
@@ -507,7 +482,7 @@ def render_lines(
 
     Args:
         layout: the modelspace, a paperspace layout or a block definition
-        paths: iterable of :class:`Path` objects
+        paths: iterable of :class:`Path`or :class:`Path2d` objects
         distance:  maximum distance, see :meth:`Path.flattening`
         segments: minimum segment count per Bézier curve
         dxfattribs: additional DXF attribs
@@ -515,8 +490,6 @@ def render_lines(
     Returns:
         created entities in an :class:`~ezdxf.query.EntityQuery` object
 
-    .. versionadded:: 0.16
-
     """
     lines = list(
         converter.to_lines(
@@ -532,7 +505,7 @@ def render_lines(
 
 
 def render_splines_and_polylines(
-    layout: "Layout",
+    layout: GenericLayoutType,
     paths: Iterable[Path],
     *,
     g1_tol: float = G1_TOL,
@@ -543,15 +516,13 @@ def render_splines_and_polylines(
 
     Args:
         layout: the modelspace, a paperspace layout or a block definition
-        paths: iterable of :class:`Path` objects
+        paths: iterable of :class:`Path`or :class:`Path2d` objects
         g1_tol: tolerance for G1 continuity check
         dxfattribs: additional DXF attribs
 
     Returns:
         created entities in an :class:`~ezdxf.query.EntityQuery` object
 
-    .. versionadded:: 0.16
-
     """
     entities = list(
         converter.to_splines_and_polylines(
@@ -577,7 +548,7 @@ def add_ellipse(
     the path end point to the ellipse start point will be added automatically
     (see :func:`add_bezier4p`).
 
-    By default the start of an **empty** path is set to the start point of
+    By default, the start of an **empty** path is set to the start point of
     the ellipse, setting argument `reset` to ``False`` prevents this
     behavior.
 
@@ -605,10 +576,6 @@ def add_bezier4p(path: Path, curves: Iterable[Bezier4P]) -> None:
     the path end point to the start point of the first curve will be added
     automatically.
 
-    .. versionchanged:: 0.16.2
-
-        add linear Bézier curve segments as LINE_TO commands
-
     """
     rel_tol = 1e-15
     abs_tol = 0.0
@@ -626,9 +593,9 @@ def add_bezier4p(path: Path, curves: Iterable[Bezier4P]) -> None:
             path.line_to(start)
 
         # add linear bezier segments as LINE_TO commands
-        if start.isclose(
-            ctrl1, rel_tol=rel_tol, abs_tol=abs_tol
-        ) and end.isclose(ctrl2, rel_tol=rel_tol, abs_tol=abs_tol):
+        if start.isclose(ctrl1, rel_tol=rel_tol, abs_tol=abs_tol) and end.isclose(
+            ctrl2, rel_tol=rel_tol, abs_tol=abs_tol
+        ):
             path.line_to(end)
         else:
             path.curve4_to(end, ctrl1, ctrl2)
@@ -642,10 +609,6 @@ def add_bezier3p(path: Path, curves: Iterable[Bezier3P]) -> None:
     the path end point to the start point of the first curve will be added
     automatically.
 
-    .. versionchanged:: 0.16.2
-
-        add linear Bézier curve segments as LINE_TO commands
-
     """
     rel_tol = 1e-15
     abs_tol = 0.0
@@ -676,24 +639,35 @@ def add_2d_polyline(
     close: bool,
     ocs: OCS,
     elevation: float,
+    segments: int = 1,
 ) -> None:
     """Internal API to add 2D polylines which may include bulges to an
     **empty** path.
 
     """
 
-    def bulge_to(p1: Vec3, p2: Vec3, bulge: float):
+    def bulge_to(p1: Vec3, p2: Vec3, bulge: float, segments: int):
         if p1.isclose(p2, rel_tol=IS_CLOSE_TOL, abs_tol=0):
             return
+        # each cubic_bezier adds 3 segments, need 1 minimum
+        num_bez = math.ceil(segments / 3)
         center, start_angle, end_angle, radius = bulge_to_arc(p1, p2, bulge)
-        ellipse = ConstructionEllipse.from_arc(
-            center,
-            radius,
-            Z_AXIS,
-            math.degrees(start_angle),
-            math.degrees(end_angle),
-        )
-        curves = list(cubic_bezier_from_ellipse(ellipse))
+        # normalize angles into range 0 .. 2pi
+        start_angle = start_angle % math.tau
+        end_angle = end_angle % math.tau
+        if start_angle > end_angle:
+            end_angle += math.tau
+        angles = list(linspace(start_angle, end_angle, num_bez + 1))
+        curves = []
+        for i in range(num_bez):
+            ellipse = ConstructionEllipse.from_arc(
+                center,
+                radius,
+                Z_AXIS,
+                math.degrees(angles[i]),
+                math.degrees(angles[i + 1]),
+            )
+            curves.extend(list(cubic_bezier_from_ellipse(ellipse)))
         curve0 = curves[0]
         cp0 = curve0.control_points[0]
         if cp0.isclose(p2, rel_tol=IS_CLOSE_TOL, abs_tol=0):
@@ -717,17 +691,15 @@ def add_2d_polyline(
             continue
 
         if prev_bulge:
-            bulge_to(prev_point, point, prev_bulge)
+            bulge_to(prev_point, point, prev_bulge, segments)
         else:
             path.line_to(point)
         prev_point = point
         prev_bulge = bulge
 
-    if close and not path.start.isclose(
-        path.end, rel_tol=IS_CLOSE_TOL, abs_tol=0
-    ):
+    if close and not path.start.isclose(path.end, rel_tol=IS_CLOSE_TOL, abs_tol=0):
         if prev_bulge:
-            bulge_to(path.end, path.start, prev_bulge)
+            bulge_to(path.end, path.start, prev_bulge, segments)
         else:
             path.line_to(path.start)
 
@@ -739,7 +711,7 @@ def add_spline(path: Path, spline: BSpline, level=4, reset=True) -> None:
     """Add a B-spline as multiple cubic Bèzier-curves.
 
     Non-rational B-splines of 3rd degree gets a perfect conversion to
-    cubic bezier curves with a minimal count of curve segments, all other
+    cubic Bézier curves with a minimal count of curve segments, all other
     B-spline require much more curve segments for approximation.
 
     Auto-detect the connection point to the given `path`, if neither the start-
@@ -747,7 +719,7 @@ def add_spline(path: Path, spline: BSpline, level=4, reset=True) -> None:
     from the path end point to the start point of the B-spline will be added
     automatically. (see :meth:`add_bezier4p`).
 
-    By default the start of an **empty** path is set to the start point of
+    By default, the start of an **empty** path is set to the start point of
     the spline, setting argument `reset` to ``False`` prevents this
     behavior.
 
@@ -771,11 +743,7 @@ def add_spline(path: Path, spline: BSpline, level=4, reset=True) -> None:
 def have_close_control_vertices(
     a: Path, b: Path, *, rel_tol=1e-9, abs_tol=1e-12
 ) -> bool:
-    """Returns ``True`` if the control vertices of given paths are close.
-
-    .. versionadded:: 0.16.5
-
-    """
+    """Returns ``True`` if the control vertices of given paths are close."""
     return all(
         cp_a.isclose(cp_b, rel_tol=rel_tol, abs_tol=abs_tol)
         for cp_a, cp_b in zip(a.control_vertices(), b.control_vertices())
@@ -834,7 +802,7 @@ def _all_lines_to_curve(path: Path, count: int = 4) -> Path:
     return new_path
 
 
-def _get_local_fillet_ucs(p0, p1, p2, radius) -> Tuple[Vec3, float, UCS]:
+def _get_local_fillet_ucs(p0, p1, p2, radius) -> tuple[Vec3, float, UCS]:
     dir1 = (p0 - p1).normalize()
     dir2 = (p2 - p1).normalize()
     if dir1.isclose(dir2) or dir1.isclose(-dir2):
@@ -858,16 +826,13 @@ def _get_local_fillet_ucs(p0, p1, p2, radius) -> Tuple[Vec3, float, UCS]:
 
 
 def fillet(points: Sequence[Vec3], radius: float) -> Path:
-    """
-    Returns a :class:`Path` with circular fillets of given `radius` between
+    """Returns a :class:`Path` with circular fillets of given `radius` between
     straight line segments.
 
     Args:
         points: coordinates of the line segments
         radius: fillet radius
 
-    .. versionadded:: 0.18
-
     """
     if len(points) < 3:
         raise ValueError("at least 3 not coincident points required")
@@ -897,9 +862,7 @@ def _segment_count(angle: float, count: int) -> int:
     return max(int(angle / (math.tau / count)), 1)
 
 
-def polygonal_fillet(
-    points: Sequence[Vec3], radius: float, count: int = 32
-) -> Path:
+def polygonal_fillet(points: Sequence[Vec3], radius: float, count: int = 32) -> Path:
     """
     Returns a :class:`Path` with polygonal fillets of given `radius` between
     straight line segments. The `count` argument defines the vertex count of the
@@ -910,8 +873,6 @@ def polygonal_fillet(
         radius: fillet radius
         count: polygon vertex count for a full circle, minimum is 4
 
-    .. versionadded:: 0.18
-
     """
     if len(points) < 3:
         raise ValueError("at least 3 not coincident points required")
@@ -945,8 +906,6 @@ def chamfer(points: Sequence[Vec3], length: float) -> Path:
         points: coordinates of the line segments
         length: chamfer length
 
-    .. versionadded:: 0.18
-
     """
     if len(points) < 3:
         raise ValueError("at least 3 not coincident points required")
@@ -980,8 +939,6 @@ def chamfer2(points: Sequence[Vec3], a: float, b: float) -> Path:
         a: distance of the chamfer start point to the segment point
         b: distance of the chamfer end point to the segment point
 
-    .. versionadded:: 0.18
-
     """
     if len(points) < 3:
         raise ValueError("at least 3 non-coincident points required")
@@ -1004,17 +961,22 @@ def chamfer2(points: Sequence[Vec3], a: float, b: float) -> Path:
 
 
 def triangulate(
-    paths: Iterable[Path], max_flattening_distance: float = 0.01
+    paths: Iterable[Path], max_sagitta: float = 0.01, min_segments: int = 16
 ) -> Iterator[Sequence[Vec2]]:
     """Tessellate nested 2D paths into triangle-faces. For 3D paths the
     projection onto the xy-plane will be triangulated.
 
-    .. versionadded:: 0.18.1
+    Args:
+        paths: iterable of nested Path instances
+        max_sagitta: maximum distance from the center of the curve to the
+            center of the line segment between two approximation points to determine if
+            a segment should be subdivided.
+        min_segments: minimum segment count per Bézier curve
 
     """
     for polygon in nesting.group_paths(single_paths(paths)):
-        exterior = polygon[0].flattening(max_flattening_distance)
-        holes = [p.flattening(max_flattening_distance) for p in polygon[1:]]
+        exterior = polygon[0].flattening(max_sagitta, min_segments)
+        holes = [p.flattening(max_sagitta, min_segments) for p in polygon[1:]]
         yield from mapbox_earcut_2d(exterior, holes)
 
 
