@@ -1,13 +1,11 @@
-# Copyright (c) 2020-2021, Manfred Moitzi
+# Copyright (c) 2020-2023, Manfred Moitzi
 # License: MIT License
+from __future__ import annotations
 from typing import (
     TYPE_CHECKING,
     Optional,
     Iterable,
-    Tuple,
-    List,
-    Set,
-    Dict,
+    Iterator,
     cast,
     Sequence,
     Any,
@@ -22,6 +20,7 @@ from ezdxf.tools.binarydata import bytes_to_hexstr, ByteStream, BitStream
 from ezdxf import colors
 from ezdxf.math import (
     Vec3,
+    Vec2,
     Matrix44,
     Z_AXIS,
     ConstructionCircle,
@@ -34,10 +33,10 @@ from ezdxf.entities import factory
 import logging
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        Tags,
-        TagWriter,
-        Drawing,
+    from ezdxf.document import Drawing
+    from ezdxf.lldxf.tags import Tags
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.entities import (
         DXFGraphic,
         Polymesh,
         Polyface,
@@ -56,7 +55,7 @@ class ProxyGraphicError(Exception):
 
 
 def load_proxy_graphic(
-    tags: "Tags", length_code: int = 160, data_code: int = 310
+    tags: Tags, length_code: int = 160, data_code: int = 310
 ) -> Optional[bytes]:
     binary_data = [
         tag.value
@@ -68,7 +67,7 @@ def load_proxy_graphic(
 
 def export_proxy_graphic(
     data: bytes,
-    tagwriter: "TagWriter",
+    tagwriter: AbstractTagWriter,
     length_code: int = 160,
     data_code: int = 310,
 ) -> None:
@@ -131,7 +130,7 @@ TRAIT_TESTER = {
 
 def read_prim_traits(
     bs: ByteStream, types: Sequence[str], prim_flags: int, count: int
-) -> Dict:
+) -> dict:
     def read_float_list():
         return [bs.read_long() for _ in range(count)]
 
@@ -156,21 +155,21 @@ def read_mesh_traits(
 ):
     # Traits data format:
     # all entries are optional
-    # traits: Dict[str, Dict]
-    #     "edges": Dict[str, List]
-    #         "colors": List[int]
-    #         "layers": List[int] as layer ids
-    #         "linetypes": List[int] as linetype ids
-    #         "markers": List[int]
-    #         "visibilities": List[int]
-    #     "faces": Dict[str, List]
-    #         "colors": List[int]
-    #         "layers": List[int] as layer ids
-    #         "markers": List[int]
-    #         "normals": List[Vec3]
-    #         "visibilities": List[int]
-    #     "vertices": Dict
-    #         "normals": List[Vec3]
+    # traits: dict[str, dict]
+    #     "edges": dict[str, list]
+    #         "colors": list[int]
+    #         "layers": list[int] as layer ids
+    #         "linetypes": list[int] as linetype ids
+    #         "markers": list[int]
+    #         "visibilities": list[int]
+    #     "faces": dict[str, list]
+    #         "colors": list[int]
+    #         "layers": list[int] as layer ids
+    #         "markers": list[int]
+    #         "normals": list[Vec3]
+    #         "visibilities": list[int]
+    #     "vertices": dict
+    #         "normals": list[Vec3]
     #         "orientation": bool
     traits = dict()
     edge_flags = bs.read_long()
@@ -245,15 +244,18 @@ class ProxyGraphicTypes(IntEnum):
     UNICODE_TEXT = 36
     UNKNOWN_37 = 37
     UNICODE_TEXT2 = 38
+    ELLIPTIC_ARC = 44  # found in test data of issue #832
 
 
 class ProxyGraphic:
-    def __init__(self, data: bytes, doc: "Drawing" = None):
+    def __init__(
+        self, data: bytes, doc: Optional[Drawing] = None, *, dxfversion=const.DXF2000
+    ):
         self._doc = doc
         self._factory = factory.new
         self._buffer: bytes = data
         self._index: int = 8
-        self.dxfversion = doc.dxfversion if doc else "AC1015"
+        self.dxfversion = doc.dxfversion if doc else dxfversion
         self.color: int = const.BYLAYER
         self.layer: str = "0"
         self.linetype: str = "BYLAYER"
@@ -264,24 +266,22 @@ class ProxyGraphic:
         self.ltscale: float = 1.0
         self.thickness: float = 0.0
         # Layer list in storage order
-        self.layers: List[str] = []
+        self.layers: list[str] = []
         # Linetypes list in storage order
-        self.linetypes: List[str] = []
+        self.linetypes: list[str] = []
         # List of text styles, with font name as key
-        self.textstyles: Dict[str, str] = dict()
-        self.required_fonts: Set[str] = set()
-        self.matrices: List[Matrix44] = []
+        self.textstyles: dict[str, str] = dict()
+        self.required_fonts: set[str] = set()
+        self.matrices: list[Matrix44] = []
 
         if self._doc:
             self.layers = list(layer.dxf.name for layer in self._doc.layers)
-            self.linetypes = list(
-                linetype.dxf.name for linetype in self._doc.linetypes
-            )
+            self.linetypes = list(linetype.dxf.name for linetype in self._doc.linetypes)
             self.textstyles = {
                 style.dxf.font: style.dxf.name for style in self._doc.styles
             }
 
-    def info(self) -> Iterable[Tuple[int, int, str]]:
+    def info(self) -> Iterable[tuple[int, int, str]]:
         index = self._index
         buffer = self._buffer
         while index < len(buffer):
@@ -293,17 +293,17 @@ class ProxyGraphic:
             yield index, size, name
             index += size
 
-    def virtual_entities(self) -> Iterable["DXFGraphic"]:
+    def virtual_entities(self) -> Iterator[DXFGraphic]:
         return self.__virtual_entities__()
 
-    def __virtual_entities__(self) -> Iterable["DXFGraphic"]:
+    def __virtual_entities__(self) -> Iterator[DXFGraphic]:
         """Implements the SupportsVirtualEntities protocol."""
         try:
             yield from self.unsafe_virtual_entities()
         except Exception as e:
             raise ProxyGraphicError(f"Proxy graphic error: {str(e)}")
 
-    def unsafe_virtual_entities(self) -> Iterable["DXFGraphic"]:
+    def unsafe_virtual_entities(self) -> Iterable[DXFGraphic]:
         def transform(entity):
             if self.matrices:
                 return entity.transform(self.matrices[-1])
@@ -363,8 +363,14 @@ class ProxyGraphic:
     def attribute_linetype(self, data: bytes):
         if self._doc:
             index = struct.unpack("<L", data)[0]
-            if index < len(self.linetypes):
-                self.linetype = self.linetypes[index]
+            try:
+                # first two entries ByLayer and ByBlock are not included in CAD applications:
+                self.linetype = self.linetypes[index + 2]
+            except IndexError:
+                if index == 32766:
+                    self.linetype = "BYBLOCK"
+                else:  # index is 32767 or invalid
+                    self.linetype = "BYLAYER"
 
     def attribute_marker(self, data: bytes):
         self.marker_index = struct.unpack("<L", data)[0]
@@ -396,9 +402,18 @@ class ProxyGraphic:
     def circle(self, data: bytes):
         bs = ByteStream(data)
         attribs = self._build_dxf_attribs()
-        attribs["center"] = Vec3(bs.read_vertex())
+        center = Vec3(bs.read_vertex())
         attribs["radius"] = bs.read_float()
-        attribs["extrusion"] = bs.read_vertex()
+        normal = Vec3(bs.read_vertex())
+        attribs["extrusion"] = normal
+        if not normal.isclose(Z_AXIS):
+            # TODO: (issue 873) circle has a normal vector but center is in WCS
+            #  It's not clear if all coordinates in proxy graphics are WCS coordinates
+            #  even for OCS entities - the ODA DWG documentation contains no information
+            #  about that.
+            ocs = OCS(normal)
+            center = ocs.from_wcs(center)
+        attribs["center"] = center
         return self._factory("CIRCLE", dxfattribs=attribs)
 
     def circle_3p(self, data: bytes):
@@ -460,54 +475,82 @@ class ProxyGraphic:
         attribs["end_angle"] = arc.end_angle
         return self._factory("ARC", dxfattribs=attribs)
 
+    def elliptic_arc(self, data: bytes):
+        bs = ByteStream(data)
+        attribs = self._build_dxf_attribs()
+        attribs["center"] = Vec3(bs.read_vertex())
+        extrusion = Vec3(bs.read_vertex())
+        attribs["extrusion"] = extrusion
+        major_axis_length = bs.read_float()
+        minor_axis_length = bs.read_float()
+        attribs["ratio"] = minor_axis_length / major_axis_length
+        start_param = bs.read_float()
+        end_param = bs.read_float()
+        major_axis_angle = bs.read_float()
+
+        ocs = OCS(extrusion)
+        major_axis = ocs.to_wcs(Vec3.from_angle(major_axis_angle, major_axis_length))
+        attribs["major_axis"] = major_axis
+        attribs["start_param"] = start_param
+        attribs["end_param"] = end_param
+        return self._factory("ELLIPSE", dxfattribs=attribs)
+
     def _filled_polygon(self, vertices, attribs):
         hatch = cast("Hatch", self._factory("HATCH", dxfattribs=attribs))
-        hatch.paths.add_polyline_path(vertices, is_closed=True)
+        elevation = _get_elevation(vertices)
+        hatch.paths.add_polyline_path(Vec2.generate(vertices), is_closed=True)
+        if elevation:
+            hatch.dxf.elevation = Vec3(0, 0, elevation)
         return hatch
 
-    def _polyline(self, vertices, *, close=False, normal=Z_AXIS):
+    def _polyline(self, vertices: list[Vec3], *, close=False, normal=Z_AXIS):
         # Polyline without bulge values!
         # Current implementation ignores the normal vector!
+        # Polyline ignores the filled flag, see #906
         attribs = self._build_dxf_attribs()
         count = len(vertices)
         if count == 1 or (count == 2 and vertices[0].isclose(vertices[1])):
             attribs["location"] = vertices[0]
             return self._factory("POINT", dxfattribs=attribs)
-
-        if self.fill and count > 2:
-            polyline = self._filled_polygon(vertices, attribs)
-        else:
+        if not is_2d_polyline(vertices):
             attribs["flags"] = const.POLYLINE_3D_POLYLINE
-            polyline = cast(
-                "Polyline", self._factory("POLYLINE", dxfattribs=attribs)
-            )
-            polyline.append_vertices(vertices)
-            if close:
-                polyline.close()
+        polyline = cast("Polyline", self._factory("POLYLINE", dxfattribs=attribs))
+        polyline.append_vertices(vertices)
+        if close:
+            polyline.close()
+        polyline.new_seqend()
         return polyline
 
     def polyline_with_normals(self, data: bytes):
         # Polyline without bulge values!
+        # Polyline ignores the filled flag, see #906
         vertices, normal = self._load_vertices(data, load_normal=True)
         return self._polyline(vertices, normal=normal)
 
     def polyline(self, data: bytes):
         # Polyline without bulge values!
+        # Polyline ignores the filled flag, see #906
         vertices, normal = self._load_vertices(data, load_normal=False)
         return self._polyline(vertices)
 
     def polygon(self, data: bytes):
         # Polyline without bulge values!
         vertices, normal = self._load_vertices(data, load_normal=False)
+        if self.fill:
+            return self._filled_polygon(vertices, self._build_dxf_attribs())
         return self._polyline(vertices, close=True)
 
     def lwpolyline(self, data: bytes):
         # OpenDesign Specs LWPLINE: 20.4.85 Page 211
-        # TODO: MLEADER exploration example "explore_mleader_block.dxf" has
-        #  LWPOLYLINE proxy graphic and raises an exception!
-        bs = BitStream(data)
-        flag: int = bs.read_bit_short()
         attribs = self._build_dxf_attribs()
+        num_bulges = 0
+        num_vertex_ids = 0
+        num_width = 0
+        is_closed = False
+        bs = BitStream(data)
+
+        num_data_bytes: int = bs.read_unsigned_long()
+        flag: int = bs.read_bit_short()
         if flag & 4:
             attribs["const_width"] = bs.read_bit_double()
         if flag & 8:
@@ -516,50 +559,44 @@ class ProxyGraphic:
             attribs["thickness"] = bs.read_bit_double()
         if flag & 1:
             attribs["extrusion"] = Vec3(bs.read_bit_double(3))
-
+        if flag & 512:  # todo: is this correct? not documented by the ODA DWG ref.
+            is_closed = True
         num_points = bs.read_bit_long()
+        if num_points <= 0:
+            return None  # ignored in method unsafe_virtual_entities()
+
         if flag & 16:
             num_bulges = bs.read_bit_long()
-        else:
-            num_bulges = 0
 
         if self.dxfversion >= "AC1024":  # R2010+
-            vertex_id_count = bs.read_bit_long()
-        else:
-            vertex_id_count = 0
-
-        if flag & 32:
-            num_width = bs.read_bit_long()
-        else:
-            num_width = 0
+            if flag & 1024:
+                num_vertex_ids = bs.read_bit_long()
+            if flag & 32:
+                num_width = bs.read_bit_long()
         # ignore DXF R13/14 special vertex order
 
-        vertices: List[Tuple[float, float]] = [bs.read_raw_double(2)]  # type: ignore
+        vertices: list[tuple[float, float]] = [bs.read_raw_double(2)]  # type: ignore
         prev_point = vertices[-1]
         for _ in range(num_points - 1):
             x = bs.read_bit_double_default(default=prev_point[0])  # type: ignore
             y = bs.read_bit_double_default(default=prev_point[1])  # type: ignore
             prev_point = (x, y)
             vertices.append(prev_point)
-        bulges: List[float] = [bs.read_bit_double() for _ in range(num_bulges)]
-        vertex_ids: List[int] = [
-            bs.read_bit_long() for _ in range(vertex_id_count)
-        ]
-        widths: List[Tuple[float, float]] = [
-            (bs.read_bit_double(), bs.read_bit_double())
-            for _ in range(num_width)
+        bulges: list[float] = [bs.read_bit_double() for _ in range(num_bulges)]
+        vertex_ids: list[int] = [bs.read_bit_long() for _ in range(num_vertex_ids)]
+        widths: list[tuple[float, float]] = [
+            (bs.read_bit_double(), bs.read_bit_double()) for _ in range(num_width)
         ]
         if len(bulges) == 0:
             bulges = list(repeat(0, num_points))
         if len(widths) == 0:
             widths = list(repeat((0, 0), num_points))
-        points: List[Sequence[float]] = []
+        points: list[Sequence[float]] = []
         for v, w, b in zip(vertices, widths, bulges):
             points.append((v[0], v[1], w[0], w[1], b))
-        lwpolyline = cast(
-            "LWPolyline", self._factory("LWPOLYLINE", dxfattribs=attribs)
-        )
+        lwpolyline = cast("LWPolyline", self._factory("LWPOLYLINE", dxfattribs=attribs))
         lwpolyline.set_points(points)
+        lwpolyline.closed = is_closed
         return lwpolyline
 
     def mesh(self, data: bytes):
@@ -582,9 +619,7 @@ class ProxyGraphic:
                 bs, total_edge_count, total_face_count, vertex_count=0
             )
         except struct.error:
-            logger.error(
-                "Structure error while parsing traits for MESH proxy graphic"
-            )
+            logger.error("Structure error while parsing traits for MESH proxy graphic")
         if traits:
             # apply traits
             pass
@@ -594,9 +629,7 @@ class ProxyGraphic:
         attribs["m_count"] = rows
         attribs["n_count"] = columns
         attribs["flags"] = const.POLYLINE_3D_POLYMESH
-        polymesh = cast(
-            "Polymesh", self._factory("POLYLINE", dxfattribs=attribs)
-        )
+        polymesh = cast("Polymesh", self._factory("POLYLINE", dxfattribs=attribs))
         polymesh.append_vertices(vertices)
         return polymesh
 
@@ -607,9 +640,7 @@ class ProxyGraphic:
         bs = ByteStream(data)
         attribs = self._build_dxf_attribs()
         attribs["flags"] = const.POLYLINE_POLYFACE
-        polyface = cast(
-            "Polyface", self._factory("POLYLINE", dxfattribs=attribs)
-        )
+        polyface = cast("Polyface", self._factory("POLYLINE", dxfattribs=attribs))
         total_vertex_count = bs.read_long()
         vertices = [Vec3(bs.read_vertex()) for _ in range(total_vertex_count)]
         face_entry_count = bs.read_long()
@@ -632,9 +663,7 @@ class ProxyGraphic:
                 bs, total_edge_count, total_face_count, vertex_count=0
             )
         except struct.error:
-            logger.error(
-                "Structure error while parsing traits for SHELL proxy graphic"
-            )
+            logger.error("Structure error while parsing traits for SHELL proxy graphic")
         polyface.append_faces(faces)
         if traits:
             face_traits = traits.get("faces")
@@ -753,9 +782,7 @@ class ProxyGraphic:
         return self._xline(data, "RAY")
 
     def _xline(self, data: bytes, type_: str):
-        logger.warning(
-            "Untested proxy graphic entity: RAY/XLINE - Need examples!"
-        )
+        logger.warning("Untested proxy graphic entity: RAY/XLINE - Need examples!")
         bs = ByteStream(data)
         attribs = self._build_dxf_attribs()
         start_point = Vec3(bs.read_vertex())
@@ -777,13 +804,14 @@ class ProxyGraphic:
                 self.textstyles[font] = style
         return style
 
-    def _load_vertices(self, data: bytes, load_normal=False):
+    @staticmethod
+    def _load_vertices(data: bytes, load_normal=False) -> tuple[list[Vec3], bool]:
         normal = Z_AXIS
         bs = ByteStream(data)
         count = bs.read_long()
         if load_normal:
             count += 1
-        vertices = []
+        vertices: list[Vec3] = []
         while count > 0:
             vertices.append(Vec3(bs.read_struct("<3d")))
             count -= 1
@@ -791,8 +819,8 @@ class ProxyGraphic:
             normal = vertices.pop()
         return vertices, normal
 
-    def _build_dxf_attribs(self) -> Dict:
-        attribs: Dict[str, Any] = dict()
+    def _build_dxf_attribs(self) -> dict[str, Any]:
+        attribs: dict[str, Any] = dict()
         if self.layer != "0":
             attribs["layer"] = self.layer
         if self.color != const.BYLAYER:
@@ -809,7 +837,7 @@ class ProxyGraphic:
 
 
 class ProxyGraphicDebugger(ProxyGraphic):
-    def __init__(self, data: bytes, doc: "Drawing" = None, debug_stream=None):
+    def __init__(self, data: bytes, doc: Optional[Drawing] = None, debug_stream=None):
         super(ProxyGraphicDebugger, self).__init__(data, doc)
         if debug_stream is None:
             debug_stream = sys.stdout
@@ -886,7 +914,7 @@ class ProxyGraphicDebugger(ProxyGraphic):
         self.log_state()
 
 
-def _apply_face_colors(polyface: "Polyface", colors: List[int]) -> None:
+def _apply_face_colors(polyface: Polyface, colors: list[int]) -> None:
     color_count: int = len(colors)
     if color_count == 0:
         return
@@ -898,3 +926,16 @@ def _apply_face_colors(polyface: "Polyface", colors: List[int]) -> None:
             index += 1
             if index >= color_count:
                 return
+
+
+def _get_elevation(vertices) -> float:
+    if vertices:
+        return vertices[0].z
+    return 0.0
+
+
+def is_2d_polyline(vertices: list[Vec3]) -> bool:
+    if len(vertices) < 1:
+        return True
+    z = vertices[0].z
+    return all(math.isclose(z, v.z) for v in vertices)
