@@ -1,16 +1,17 @@
-#  Copyright (c) 2020-2022, Manfred Moitzi
+#  Copyright (c) 2020-2023, Manfred Moitzi
 #  License: MIT License
 from __future__ import annotations
+
+import string
 import typing
 from typing import (
     TYPE_CHECKING,
     BinaryIO,
     Iterable,
-    List,
+    Iterator,
     Callable,
-    Tuple,
-    Dict,
     Union,
+    Optional,
 )
 import itertools
 import re
@@ -23,6 +24,8 @@ from ezdxf.lldxf import repair
 from ezdxf.lldxf.encoding import (
     has_dxf_unicode,
     decode_dxf_unicode,
+    has_mif_encoding,
+    decode_mif_to_unicode,
 )
 from ezdxf.lldxf.types import (
     DXFTag,
@@ -39,7 +42,8 @@ from ezdxf.tools.codepage import toencoding
 from ezdxf.audit import Auditor, AuditError
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import Drawing, SectionDict
+    from ezdxf.document import Drawing
+    from ezdxf.eztypes import SectionDict
 
 __all__ = ["read", "readfile"]
 
@@ -57,9 +61,9 @@ logger = logging.getLogger("ezdxf")
 
 def readfile(
     filename: Union[str, Path], errors: str = "surrogateescape"
-) -> Tuple["Drawing", "Auditor"]:
+) -> tuple[Drawing, Auditor]:
     """Read a DXF document from file system similar to :func:`ezdxf.readfile`,
-    but this function will repair as much flaws as possible, runs the required
+    but this function will repair as many flaws as possible, runs the required
     audit process automatically the DXF document and the :class:`Auditor`.
 
     Args:
@@ -82,12 +86,10 @@ def readfile(
     return doc, auditor
 
 
-def read(
-    stream: BinaryIO, errors: str = "surrogateescape"
-) -> Tuple["Drawing", "Auditor"]:
+def read(stream: BinaryIO, errors: str = "surrogateescape") -> tuple[Drawing, Auditor]:
     """Read a DXF document from a binary-stream similar to :func:`ezdxf.read`,
     but this function will detect the text encoding automatically and repair
-    as much flaws as possible, runs the required audit process afterwards
+    as many flaws as possible, runs the required audit process afterwards
     and returns the DXF document and the :class:`Auditor`.
 
     Args:
@@ -109,12 +111,12 @@ def read(
 
 def explore(
     filename: Union[str, Path], errors: str = "ignore"
-) -> Tuple["Drawing", "Auditor"]:
+) -> tuple[Drawing, Auditor]:
     """Read a DXF document from file system similar to :func:`readfile`,
-    but this function will use a special tag loader, which synchronise the tag
-    stream if invalid tags occur. This function is intended to load corrupted
-    DXF files and should only be used to explore such files, data loss is very
-    likely.
+    but this function will use a special tag loader, which tries to recover the
+    tag stream if invalid tags occur.  This function is intended to load
+    corrupted DXF files and should only be used to explore such files, data loss
+    is very likely.
 
     Args:
         filename: file-system name of the DXF document to load
@@ -131,15 +133,13 @@ def explore(
     """
     filename = str(filename)
     with open(filename, mode="rb") as fp:
-        recover_tool = Recover.run(
-            fp, errors=errors, loader=synced_bytes_loader
-        )
+        recover_tool = Recover.run(fp, errors=errors, loader=synced_bytes_loader)
         doc, auditor = _load_and_audit_document(recover_tool)
     doc.filename = filename
     return doc, auditor
 
 
-def _load_and_audit_document(recover_tool) -> Tuple["Drawing", "Auditor"]:
+def _load_and_audit_document(recover_tool) -> tuple[Drawing, Auditor]:
     from ezdxf.document import Drawing
 
     doc = Drawing()
@@ -158,7 +158,7 @@ def _load_and_audit_document(recover_tool) -> Tuple["Drawing", "Auditor"]:
 class Recover:
     """Loose coupled recovering tools."""
 
-    def __init__(self, loader: Callable = None):
+    def __init__(self, loader: Optional[Callable] = None):
         # different tag loading strategies can be used:
         #  - bytes_loader(): expects a valid low level structure
         #  - synced_bytes_loader(): loads everything which looks like a tag
@@ -169,8 +169,8 @@ class Recover:
         self.section_dict: "SectionDict" = dict()
 
         # Store error messages from low level processes
-        self.errors: List[Tuple[int, str]] = []
-        self.fixes: List[Tuple[int, str]] = []
+        self.errors: list[tuple[int, str]] = []
+        self.fixes: list[tuple[int, str]] = []
 
         # Detected DXF version
         self.dxfversion = const.DXF12
@@ -179,9 +179,9 @@ class Recover:
     def run(
         cls,
         stream: BinaryIO,
-        loader: Callable = None,
+        loader: Optional[Callable] = None,
         errors: str = "surrogateescape",
-    ) -> "Recover":
+    ) -> Recover:
         """Execute the recover process."""
         recover_tool = Recover(loader)
         tags = recover_tool.load_tags(stream, errors)
@@ -202,12 +202,12 @@ class Recover:
 
         return recover_tool
 
-    def load_tags(self, stream: BinaryIO, errors: str) -> Iterable[DXFTag]:
+    def load_tags(self, stream: BinaryIO, errors: str) -> Iterator[DXFTag]:
         return safe_tag_loader(
             stream, self.tag_loader, messages=self.errors, errors=errors
         )
 
-    def rebuild_sections(self, tags: Iterable[DXFTag]) -> List[List[DXFTag]]:
+    def rebuild_sections(self, tags: Iterable[DXFTag]) -> list[list[DXFTag]]:
         """Collect tags between SECTION and ENDSEC or next SECTION tag
         as list of DXFTag objects, collects tags outside of sections
         as an extra section.
@@ -278,9 +278,9 @@ class Recover:
                 )
                 orphans.append(tag)
 
-        orphans: List[DXFTag] = []
-        sections: List[List[DXFTag]] = []
-        collector: List[DXFTag] = []
+        orphans: list[DXFTag] = []
+        sections: list[list[DXFTag]] = []
+        collector: list[DXFTag] = []
         inside_section = False
         for tag in tags:
             code, value = tag
@@ -292,7 +292,7 @@ class Recover:
         sections.append(orphans)
         return sections
 
-    def load_section_dict(self, sections: List[List[DXFTag]]) -> None:
+    def load_section_dict(self, sections: list[list[DXFTag]]) -> None:
         """Merge sections of same type."""
 
         def add_section(name: str, tags) -> None:
@@ -301,12 +301,12 @@ class Recover:
             else:
                 section_dict[name] = tags
 
-        def _build_section_dict(d: Dict) -> None:
+        def _build_section_dict(d: dict) -> None:
             for name, section in d.items():
                 if name in const.MANAGED_SECTIONS:
                     self.section_dict[name] = list(group_tags(section, 0))
 
-        def _remove_unsupported_sections(d: Dict):
+        def _remove_unsupported_sections(d: dict):
             for name in ("CLASSES", "OBJECTS", "ACDSDATA"):
                 if name in d:
                     del d[name]
@@ -350,7 +350,7 @@ class Recover:
             _remove_unsupported_sections(section_dict)
         _build_section_dict(section_dict)
 
-    def rebuild_tables(self, tables: List[Tags]) -> List[Tags]:
+    def rebuild_tables(self, tables: list[Tags]) -> list[Tags]:
         """Rebuild TABLES section."""
 
         # Note: the recover module does not report invalid placed table entries,
@@ -404,7 +404,7 @@ class Recover:
         return tables
 
     def rescue_orphaned_header_vars(
-        self, header: List[DXFTag], orphans: Iterable[DXFTag]
+        self, header: list[DXFTag], orphans: Iterable[DXFTag]
     ) -> None:
         var_name = None
         for tag in orphans:
@@ -416,7 +416,7 @@ class Recover:
                 header.append(tag)
                 var_name = None
 
-    def check_entities(self, entities: List[Tags]) -> Iterable[Tags]:
+    def check_entities(self, entities: list[Tags]) -> Iterator[Tags]:
         for entity in entities:
             _, dxftype = entity[0]
             if dxftype in EXCLUDE_STRUCTURE_CHECK:
@@ -449,7 +449,7 @@ class Recover:
             )
 
 
-def _detect_dxf_version(header: List) -> str:
+def _detect_dxf_version(header: list) -> str:
     next_is_dxf_version = False
     for tag in header:
         if next_is_dxf_version:
@@ -470,7 +470,7 @@ def _is_rootdict(entity: Tags) -> bool:
     return any(tag == (3, "ACAD_GROUP") for tag in entity)
 
 
-def _find_rootdict(objects: List[Tags]) -> Tuple[int, Tags]:
+def _find_rootdict(objects: list[Tags]) -> tuple[int, Tags]:
     for index, entity in enumerate(objects):
         if _is_rootdict(entity):
             return index, entity
@@ -479,10 +479,10 @@ def _find_rootdict(objects: List[Tags]) -> Tuple[int, Tags]:
 
 def safe_tag_loader(
     stream: BinaryIO,
-    loader: Callable = None,
-    messages: List = None,
+    loader: Optional[Callable] = None,
+    messages: Optional[list] = None,
     errors: str = "surrogateescape",
-) -> Iterable[DXFTag]:
+) -> Iterator[DXFTag]:
     """Yields :class:``DXFTag`` objects from a bytes `stream`
     (untrusted external  source), skips all comment tags (group code == 999).
 
@@ -550,7 +550,7 @@ def _search_float(s: Union[str, bytes]) -> float:
 
 
 @typing.no_type_check
-def bytes_loader(stream: BinaryIO) -> Iterable[DXFTag]:
+def bytes_loader(stream: BinaryIO) -> Iterator[DXFTag]:
     """Yields :class:``DXFTag`` objects from a bytes `stream`
     (untrusted external  source), skips all comment tags (group code == 999).
 
@@ -595,7 +595,7 @@ def bytes_loader(stream: BinaryIO) -> Iterable[DXFTag]:
             return
 
 
-def synced_bytes_loader(stream: BinaryIO) -> Iterable[DXFTag]:
+def synced_bytes_loader(stream: BinaryIO) -> Iterator[DXFTag]:
     """Yields :class:``DXFTag`` objects from a bytes `stream`
     (untrusted external source), skips all comment tags (group code == 999).
 
@@ -639,6 +639,11 @@ DWGCODEPAGE = b"$DWGCODEPAGE"
 ACADVER = b"$ACADVER"
 
 
+def _strip_whitespace(s: str) -> str:
+    ws = set(string.whitespace)
+    return "".join([c for c in s if c not in ws])
+
+
 def detect_encoding(tags: Iterable[DXFTag]) -> str:
     """Detect text encoding from header variables $DWGCODEPAGE and $ACADVER
     out of a stream of DXFTag objects.
@@ -680,9 +685,9 @@ def detect_encoding(tags: Iterable[DXFTag]) -> str:
 def byte_tag_compiler(
     tags: Iterable[DXFTag],
     encoding=const.DEFAULT_ENCODING,
-    messages: List = None,
+    messages: Optional[list] = None,
     errors: str = "surrogateescape",
-) -> Iterable[DXFTag]:
+) -> Iterator[DXFTag]:
     """Compiles DXF tag values imported by bytes_loader() into Python types.
 
     Raises DXFStructureError() for invalid float values and invalid coordinate
@@ -723,6 +728,7 @@ def byte_tag_compiler(
     def recover_float(s: Union[str, bytes]) -> float:
         if isinstance(s, bytes):
             s = s.decode(encoding="utf8", errors="ignore")
+        s = _strip_whitespace(s)
         value = _search_float(s)
         msg = f'recovered invalid floating point value "{s}" near line {line} as "{value}"'
         messages.append((AuditError.INVALID_FLOATING_POINT_VALUE, msg))
@@ -816,11 +822,14 @@ def byte_tag_compiler(
                             )
                         )
 
-                    # Convert DXF unicode notation "\U+xxxx" to unicode,
-                    # but exclude structure tags (code>0):
-                    if code and has_dxf_unicode(str_):
-                        str_ = decode_dxf_unicode(str_)
-
+                    # exclude structure tags (code == 0):
+                    if code:
+                        # Convert DXF-Unicode notation "\U+xxxx" to unicode
+                        if has_dxf_unicode(str_):
+                            str_ = decode_dxf_unicode(str_)
+                        # Convert MIF notation "\M+cxxxx" to unicode
+                        elif has_mif_encoding(str_):
+                            str_ = decode_mif_to_unicode(str_)
                     yield DXFTag(code, str_)
                 else:
                     try:
