@@ -3,12 +3,9 @@
 from __future__ import annotations
 from typing import (
     TYPE_CHECKING,
-    Tuple,
     Iterable,
     Optional,
-    Dict,
     Any,
-    List,
     cast,
 )
 import math
@@ -31,11 +28,9 @@ from ezdxf.render.arrows import ARROWS
 from ezdxf.entities import DimStyleOverride, Dimension
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        Drawing,
-        GenericLayoutType,
-        Textstyle,
-    )
+    from ezdxf.document import Drawing
+    from ezdxf.entities import Textstyle
+    from ezdxf.eztypes import GenericLayoutType
 
 
 class TextBox(ConstructionBox):
@@ -50,9 +45,7 @@ class TextBox(ConstructionBox):
         hgap: float = 0.0,  # horizontal gap - width
         vgap: float = 0.0,  # vertical gap - height
     ):
-        super().__init__(
-            center, width + 2.0 * hgap, height + 2.0 * vgap, angle
-        )
+        super().__init__(center, width + 2.0 * hgap, height + 2.0 * vgap, angle)
 
 
 PLUS_MINUS = "±"
@@ -80,7 +73,7 @@ def sign_char(value: float) -> str:
 
 def format_text(
     value: float,
-    dimrnd: float = None,
+    dimrnd: Optional[float] = None,
     dimdec: int = 2,
     dimzin: int = 0,
     dimdsep: str = ".",
@@ -182,9 +175,7 @@ class Tolerance:  # and Limits
         self.text_width: float = 0.0
         self.text_upper: str = ""
         self.text_lower: str = ""
-        self.char_height: float = (
-            cap_height * self.text_scale_factor * self.dim_scale
-        )
+        self.char_height: float = cap_height * self.text_scale_factor * self.dim_scale
         if self.has_tolerance:
             self.init_tolerance()
         elif self.has_limits:
@@ -207,21 +198,15 @@ class Tolerance:  # and Limits
             self.text_width = self.get_text_width(self.text, self.text)
         else:  # 2 stacked values: +upper tolerance <above> -lower tolerance
             # requires 2 text lines
-            self.text_height = self.char_height + (
-                self.text_height * self.line_spacing
-            )
-            self.text_width = self.get_text_width(
-                self.text_upper, self.text_lower
-            )
+            self.text_height = self.char_height + (self.text_height * self.line_spacing)
+            self.text_width = self.get_text_width(self.text_upper, self.text_lower)
         self.update_tolerance_text(self.maximum, self.minimum)
 
     def update_tolerance_text(self, tol_upper: float, tol_lower: float):
         if tol_upper == tol_lower:
             self.text = PLUS_MINUS + self.format_text(abs(tol_upper))
         else:
-            self.text_upper = sign_char(tol_upper) + self.format_text(
-                abs(tol_upper)
-            )
+            self.text_upper = sign_char(tol_upper) + self.format_text(abs(tol_upper))
             self.text_lower = sign_char(tol_lower * -1) + self.format_text(
                 abs(tol_lower)
             )
@@ -230,9 +215,7 @@ class Tolerance:  # and Limits
         # self.text is always an empty string (default value)
         # Limit text are always 2 stacked numbers and requires the actual
         # measurement!
-        self.text_height = self.char_height + (
-            self.text_height * self.line_spacing
-        )
+        self.text_height = self.char_height + (self.text_height * self.line_spacing)
 
     def format_text(self, value: float) -> str:
         """Rounding and text formatting of tolerance `value`, removes leading
@@ -292,9 +275,7 @@ class Tolerance:  # and Limits
 class ExtensionLines:
     default_lineweight: int = const.LINEWEIGHT_BYBLOCK
 
-    def __init__(
-        self, dim_style: DimStyleOverride, default_color: int, scale: float
-    ):
+    def __init__(self, dim_style: DimStyleOverride, default_color: int, scale: float):
         get = dim_style.get
         self.color: int = get("dimclre", default_color)  # ACI
         self.linetype1: str = get("dimltex1", "")
@@ -319,9 +300,9 @@ class ExtensionLines:
         # Length below the dimension line:
         self.length_below: float = get("dimfxl", self.extension_above) * scale
 
-    def dxfattribs(self, num: int = 1) -> Dict[str, Any]:
+    def dxfattribs(self, num: int = 1) -> Any:
         """Returns default dimension line DXF attributes as dict."""
-        attribs: Dict[str, Any] = {"color": self.color}
+        attribs: dict[str, Any] = {"color": self.color}
         if num == 1:
             linetype = self.linetype1
         elif num == 2:
@@ -339,9 +320,7 @@ class ExtensionLines:
 class DimensionLine:
     default_lineweight: int = const.LINEWEIGHT_BYBLOCK
 
-    def __init__(
-        self, dim_style: DimStyleOverride, default_color: int, scale: float
-    ):
+    def __init__(self, dim_style: DimStyleOverride, default_color: int, scale: float):
         get = dim_style.get
         self.color: int = get("dimclrd", default_color)  # ACI
 
@@ -367,9 +346,9 @@ class DimensionLine:
         # not supported yet - ezdxf behaves like option 1
         self.has_dim_line_if_text_outside: bool = bool(get("dimtofl", 1))
 
-    def dxfattribs(self) -> Dict[str, Any]:
+    def dxfattribs(self) -> Any:
         """Returns default dimension line DXF attributes as dict."""
-        attribs: Dict[str, Any] = {"color": self.color}
+        attribs: dict[str, Any] = {"color": self.color}
         if self.linetype:
             attribs["linetype"] = self.linetype
         if self.lineweight != self.default_lineweight:
@@ -380,7 +359,7 @@ class DimensionLine:
 class Arrows:
     def __init__(self, dim_style: DimStyleOverride, color: int, scale: float):
         get = dim_style.get
-        self.color: int = color
+        self.color: int = get("dimclrd", color)
         self.tick_size: float = get("dimtsz", 0.0) * scale
         self.arrow1_name: str = ""  # empty string is a closed filled arrow
         self.arrow2_name: str = ""  # empty string is a closed filled arrow
@@ -404,7 +383,7 @@ class Arrows:
     def has_ticks(self) -> bool:
         return self.tick_size > 0.0
 
-    def dxfattribs(self) -> Dict[str, Any]:
+    def dxfattribs(self) -> Any:
         return {"color": self.color}
 
 
@@ -478,9 +457,7 @@ class Measurement:
         self.text_gap: float = get("dimgap", 0.625) * scale
 
         # User defined text rotation - overrides everything:
-        self.user_text_rotation: float = dimension.get_dxf_attrib(
-            "text_rotation", None
-        )
+        self.user_text_rotation: float = dimension.get_dxf_attrib("text_rotation", None)
         # calculated text rotation
         self.text_rotation: float = self.user_text_rotation
         self.text_color: int = get("dimclrt", color)  # ACI
@@ -659,9 +636,7 @@ class Measurement:
         if self.text_valign == 0:
             return self.text_height * self.text_vertical_position
         else:
-            return (
-                self.text_height / 2.0 + self.text_gap
-            ) * self.vertical_placement
+            return (self.text_height / 2.0 + self.text_gap) * self.vertical_placement
 
     def text_width(self, text: str) -> float:
         """
@@ -685,9 +660,7 @@ class Measurement:
         else:  # user override
             return text
 
-    def location_override(
-        self, location: UVec, leader=False, relative=False
-    ) -> None:
+    def location_override(self, location: UVec, leader=False, relative=False) -> None:
         """Set user defined dimension text location. ezdxf defines a user
         defined location per definition as 'outside'.
 
@@ -703,7 +676,7 @@ class Measurement:
         self.relative_user_location = relative
         self.text_is_outside = True
 
-    def dxfattribs(self) -> Dict[str, Any]:
+    def dxfattribs(self) -> Any:
         return {"color": self.text_color}
 
     @abc.abstractmethod
@@ -792,7 +765,7 @@ class Geometry:
         insert: Vec2,
         size: float,
         rotation: float,
-        dxfattribs: Dict[str, Any],
+        dxfattribs,
     ) -> None:
         # OCS of the arrow blocks is defined by the DIMENSION entity!
         # Therefore remove OCS elevation, the elevation is defined by the
@@ -807,7 +780,7 @@ class Geometry:
         name: str,
         insert: Vec2,
         rotation: float,
-        dxfattribs: Dict[str, Any],
+        dxfattribs,
     ) -> None:
         # OCS of the arrow blocks is defined by the DIMENSION entity!
         # Therefore remove OCS elevation, the elevation is defined by the
@@ -817,20 +790,19 @@ class Geometry:
         dxfattribs["rotation"] = self.ucs.to_ocs_angle_deg(rotation)
         self.layout.add_blockref(name, insert, dxfattribs)
 
-    def add_text(
-        self, text: str, pos: Vec2, rotation: float, dxfattribs: Dict[str, Any]
-    ) -> None:
+    def add_text(self, text: str, pos: Vec2, rotation: float, dxfattribs) -> None:
         dxfattribs["rotation"] = self.ucs.to_ocs_angle_deg(rotation)
         entity = self.layout.add_text(text, dxfattribs=dxfattribs)
         # OCS of the measurement text is defined by the DIMENSION entity!
         # Therefore remove OCS elevation, the elevation is defined by the
         # DIMENSION 'text_midpoint' (group code 11) and do not set 'extrusion'
         # either!
-        entity.set_placement(self.ucs.to_ocs(Vec3(pos)).vec2, align=TextEntityAlignment.MIDDLE_CENTER)
+        entity.set_placement(
+            self.ucs.to_ocs(Vec3(pos)).vec2,
+            align=TextEntityAlignment.MIDDLE_CENTER,
+        )
 
-    def add_mtext(
-        self, text: str, pos: Vec2, rotation: float, dxfattribs: Dict[str, Any]
-    ) -> None:
+    def add_mtext(self, text: str, pos: Vec2, rotation: float, dxfattribs) -> None:
         # OCS of the measurement text is defined by the DIMENSION entity!
         # Therefore remove OCS elevation, the elevation is defined by the
         # DIMENSION 'text_midpoint' (group code 11) and do not set 'extrusion'
@@ -847,14 +819,14 @@ class Geometry:
             # Despite the fact that the POINT entity has WCS coordinates,
             # the coordinates of defpoints in DIMENSION entities have OCS
             # coordinates.
-            location = self.ucs.to_ocs(Vec3(point)).replace(z=0)
+            location = self.ucs.to_ocs(Vec3(point)).replace(z=0.0)
             self.layout.add_point(location, dxfattribs=attribs)
 
     def add_line(
         self,
         start: Vec2,
         end: Vec2,
-        dxfattribs: Dict[str, Any],
+        dxfattribs,
         remove_hidden_lines=False,
     ) -> None:
         """Add a LINE entity to the geometry layout. Removes parts of the line
@@ -877,7 +849,7 @@ class Geometry:
                 dxfattribs=dxfattribs,
             )
 
-        def order(a: Vec2, b: Vec2) -> Tuple[Vec2, Vec2]:
+        def order(a: Vec2, b: Vec2) -> tuple[Vec2, Vec2]:
             if (start - a).magnitude < (start - b).magnitude:
                 return a, b
             else:
@@ -892,9 +864,7 @@ class Geometry:
             if inside == 2:  # start and end inside text_box
                 return  # do not draw line
             elif inside == 1:  # one point inside text_box or on a border line
-                intersection_points = text_box.intersect(
-                    ConstructionLine(start, end)
-                )
+                intersection_points = text_box.intersect(ConstructionLine(start, end))
                 if len(intersection_points) == 1:
                     # one point inside one point outside -> one intersection point
                     p1 = intersection_points[0]
@@ -905,14 +875,10 @@ class Geometry:
                 add_line_to_block(p1, p2)
                 return
             else:
-                intersection_points = text_box.intersect(
-                    ConstructionLine(start, end)
-                )
+                intersection_points = text_box.intersect(ConstructionLine(start, end))
                 if len(intersection_points) == 2:
                     # sort intersection points by distance to start point
-                    p1, p2 = order(
-                        intersection_points[0], intersection_points[1]
-                    )
+                    p1, p2 = order(intersection_points[0], intersection_points[1])
                     # line[start-p1] - gap - line[p2-end]
                     add_line_to_block(start, p1)
                     add_line_to_block(p2, end)
@@ -926,7 +892,7 @@ class Geometry:
         radius: float,
         start_angle: float,
         end_angle: float,
-        dxfattribs: dict = None,
+        dxfattribs=None,
         remove_hidden_lines=False,
     ) -> None:
         """Add a ARC entity to the geometry layout. Removes parts of the arc
@@ -978,8 +944,8 @@ class BaseDimensionRenderer:
     def __init__(
         self,
         dimension: Dimension,
-        ucs: UCS = None,
-        override: DimStyleOverride = None,
+        ucs: Optional[UCS] = None,
+        override: Optional[DimStyleOverride] = None,
     ):
         self.dimension: Dimension = dimension
         self.geometry = self.init_geometry(dimension, ucs)
@@ -1011,15 +977,11 @@ class BaseDimensionRenderer:
         # >0 = Center marks are drawn
         self.dim_center_marks: int = get("dimcen", 0)
 
-        self.measurement = self.init_measurement(
-            self.default_color, self.dim_scale
-        )
+        self.measurement = self.init_measurement(self.default_color, self.dim_scale)
         self.dimension_line: DimensionLine = self.init_dimension_line(
             self.default_color, self.dim_scale
         )
-        self.arrows: Arrows = self.init_arrows(
-            self.default_color, self.dim_scale
-        )
+        self.arrows: Arrows = self.init_arrows(self.default_color, self.dim_scale)
         # Suppress arrow rendering - only rendering is suppressed (rendering
         # effect).
         # All placing related calculations are done without this settings.
@@ -1032,23 +994,19 @@ class BaseDimensionRenderer:
             self.default_color, self.dim_scale
         )
         # tolerances have to be initialized after measurement:
-        self.tol: Tolerance = self.init_tolerance(
-            self.dim_scale, self.measurement
-        )
+        self.tol: Tolerance = self.init_tolerance(self.dim_scale, self.measurement)
 
         # Update text height
         self.measurement.text_height = max(
             self.measurement.text_height, self.tol.text_height
         )
 
-    def init_geometry(self, dimension: Dimension, ucs: UCS = None):
+    def init_geometry(self, dimension: Dimension, ucs: Optional[UCS] = None):
         from ezdxf.layouts import VirtualLayout
 
         return Geometry(dimension, ucs or PassTroughUCS(), VirtualLayout())
 
-    def init_tolerance(
-        self, scale: float, measurement: Measurement
-    ) -> Tolerance:
+    def init_tolerance(self, scale: float, measurement: Measurement) -> Tolerance:
         return Tolerance(
             self.dim_style,
             cap_height=measurement.text_height,
@@ -1108,16 +1066,14 @@ class BaseDimensionRenderer:
                     width += self.tol.text_width  # type: ignore
         return width
 
-    def default_attributes(self) -> Dict[str, Any]:
+    def default_attributes(self) -> dict[str, Any]:
         """Returns default DXF attributes as dict."""
         return {
             "layer": self.default_layer,
             "color": self.default_color,
         }
 
-    def location_override(
-        self, location: UVec, leader=False, relative=False
-    ) -> None:
+    def location_override(self, location: UVec, leader=False, relative=False) -> None:
         """Set user defined dimension text location. ezdxf defines a user
         defined location per definition as 'outside'.
 
@@ -1135,7 +1091,7 @@ class BaseDimensionRenderer:
         self,
         start: Vec2,
         end: Vec2,
-        dxfattribs: Dict[str, Any],
+        dxfattribs,
         remove_hidden_lines=False,
     ) -> None:
         """Add a LINE entity to the dimension BLOCK. Remove parts of the line
@@ -1161,7 +1117,7 @@ class BaseDimensionRenderer:
         radius: float,
         start_angle: float,
         end_angle: float,
-        dxfattribs: Dict[str, Any] = None,
+        dxfattribs=None,
         remove_hidden_lines=False,
     ) -> None:
         """Add a ARC entity to the geometry layout. Remove parts of the arc
@@ -1190,7 +1146,7 @@ class BaseDimensionRenderer:
         insert: Vec2,
         rotation: float,
         scale: float,
-        dxfattribs: Dict[str, Any],
+        dxfattribs,
     ) -> None:
         """
         Add block references and standard arrows to the dimension BLOCK.
@@ -1209,9 +1165,7 @@ class BaseDimensionRenderer:
 
         if name in ARROWS:
             # generates automatically BLOCK definitions for arrows if needed
-            self.geometry.add_arrow_blockref(
-                name, insert, scale, rotation, attribs
-            )
+            self.geometry.add_arrow_blockref(name, insert, scale, rotation, attribs)
         else:
             if name is None or not self.geometry.has_block(name):
                 raise DXFUndefinedBlockError(f'Undefined block: "{name}"')
@@ -1220,9 +1174,7 @@ class BaseDimensionRenderer:
                 attribs["yscale"] = scale
             self.geometry.add_blockref(name, insert, rotation, attribs)
 
-    def add_text(
-        self, text: str, pos: Vec2, rotation: float, dxfattribs: Dict[str, Any]
-    ) -> None:
+    def add_text(self, text: str, pos: Vec2, rotation: float, dxfattribs) -> None:
         """
         Add TEXT (DXF R12) or MTEXT (DXF R2000+) entity to the dimension BLOCK.
 
@@ -1287,7 +1239,7 @@ class BaseDimensionRenderer:
             self.dimension.dxf.extrusion = self.geometry.extrusion
 
 
-def order_leader_points(p1: Vec2, p2: Vec2, p3: Vec2) -> Tuple[Vec2, Vec2]:
+def order_leader_points(p1: Vec2, p2: Vec2, p3: Vec2) -> tuple[Vec2, Vec2]:
     if (p1 - p2).magnitude > (p1 - p3).magnitude:
         return p3, p2
     else:
@@ -1296,7 +1248,7 @@ def order_leader_points(p1: Vec2, p2: Vec2, p3: Vec2) -> Tuple[Vec2, Vec2]:
 
 def get_center_leader_points(
     target_point: Vec2, text_box: TextBox, leg_length: float
-) -> Tuple[Vec2, Vec2]:
+) -> tuple[Vec2, Vec2]:
     """Returns the leader points of the "leg" for a vertical centered leader."""
     c0, c1, c2, c3 = text_box.corners
     #              c3-------c2
@@ -1329,7 +1281,7 @@ def visible_arcs(
     start_angle: float,
     end_angle: float,
     box: ConstructionBox,
-) -> List[Tuple[float, float]]:
+) -> list[tuple[float, float]]:
     """Returns the visible parts of an arc intersecting with a construction box
     as (start angle, end angle) tuples.
 
@@ -1342,7 +1294,7 @@ def visible_arcs(
 
     """
 
-    intersection_angles: List[float] = []  # angles are in the range 0 to 2pi
+    intersection_angles: list[float] = []  # angles are in the range 0 to 2pi
     start_angle %= math.tau
     end_angle %= math.tau
     arc = ConstructionArc(
@@ -1360,8 +1312,7 @@ def visible_arcs(
     if len(intersection_angles) == 2:
         if start_angle > end_angle:  # arc passes 0 degrees
             intersection_angles = [
-                (a if a >= start_angle else a + math.tau)
-                for a in intersection_angles
+                (a if a >= start_angle else a + math.tau) for a in intersection_angles
             ]
         intersection_angles.sort()
         return [
@@ -1384,9 +1335,7 @@ def get_text_style(doc: "Drawing", name: str) -> Textstyle:
     return cast("Textstyle", style)
 
 
-def get_char_height(
-    dim_style: DimStyleOverride, text_style: Textstyle
-) -> float:
+def get_char_height(dim_style: DimStyleOverride, text_style: Textstyle) -> float:
     """Unscaled character height defined by text style or DIMTXT."""
     height: float = text_style.dxf.get("height", 0.0)
     if height == 0.0:  # variable text height (not fixed)
