@@ -1,6 +1,6 @@
-#  Copyright (c) 2020-2022, Manfred Moitzi
-#  License: MIT License
-from typing import Iterable
+# Copyright (c) 2020-2022, Manfred Moitzi
+# License: MIT License
+from typing import Iterable, Iterator
 import ezdxf
 from ezdxf.math import UVec
 from ._linetypes import _LineTypeRenderer, LineSegment
@@ -13,7 +13,7 @@ if ezdxf.options.use_c_ext:
 
 
 class LineTypeRenderer(_LineTypeRenderer):
-    def line_segments(self, vertices: Iterable[UVec]) -> Iterable[LineSegment]:
+    def line_segments(self, vertices: Iterable[UVec]) -> Iterator[LineSegment]:
         last = None
         for vertex in vertices:
             if last is not None:
