@@ -2,42 +2,38 @@
 # License: MIT License
 from __future__ import annotations
 from typing import (
-    List,
-    Sequence,
-    Tuple,
+    Dict,
     Iterable,
     Iterator,
-    TYPE_CHECKING,
-    Union,
-    Set,
-    Dict,
-    TypeVar,
-    Type,
     NamedTuple,
+    Optional,
+    Sequence,
+    TYPE_CHECKING,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
 )
+from typing_extensions import TypeAlias
 from ezdxf.math import (
-    Matrix44,
-    Vec3,
     BoundingBox,
-    UVec,
+    Matrix44,
     NULLVEC,
+    OCS,
+    UCS,
+    UVec,
+    Vec3,
+    area,
     is_planar_face,
-    subdivide_face,
     normal_vector_3p,
     safe_normal_vector,
+    subdivide_face,
     subdivide_ngons,
-    area,
-    OCS,
 )
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        UCS,
-        Polyface,
-        Polymesh,
-        GenericLayoutType,
-        Mesh,
-    )
+    from ezdxf.entities import Polyface, Polymesh, Mesh
+    from ezdxf.eztypes import GenericLayoutType
 
 T = TypeVar("T")
 
@@ -49,9 +45,9 @@ class EdgeStat(NamedTuple):
     balance: int
 
 
-Face = Sequence[int]
-Edge = Tuple[int, int]
-EdgeStats = Dict[Edge, EdgeStat]
+Face: TypeAlias = Sequence[int]
+Edge: TypeAlias = Tuple[int, int]
+EdgeStats: TypeAlias = Dict[Edge, EdgeStat]
 
 
 class MeshBuilderError(Exception):
@@ -88,7 +84,7 @@ def open_faces(faces: Iterable[Face]) -> Iterator[Face]:
 
 
 def normalize_faces(
-    faces: List[Sequence[int]],
+    faces: list[Sequence[int]],
     *,
     close=False,
 ) -> Iterator[Sequence[int]]:
@@ -241,7 +237,7 @@ class MeshDiagnose:
         self._mesh = mesh
         self._edge_stats: EdgeStats = {}
         self._bbox = BoundingBox()
-        self._face_normals: List[Vec3] = []
+        self._face_normals: list[Vec3] = []
 
     @property
     def vertices(self) -> Sequence[Vec3]:
@@ -439,17 +435,13 @@ class MeshBuilder:
 
     """
 
-    def __init__(self):
-        self.vertices: List[Vec3] = []
+    def __init__(self) -> None:
+        self.vertices: list[Vec3] = []
         # face storage, each face is a tuple of vertex indices (v0, v1, v2, v3, ....)
-        self.faces: List[Sequence[int]] = []
+        self.faces: list[Sequence[int]] = []
 
     def bbox(self) -> BoundingBox:
-        """Returns the :class:`~ezdxf.math.BoundingBox` of the mesh.
-
-        .. versionadded:: 0.18
-
-        """
+        """Returns the :class:`~ezdxf.math.BoundingBox` of the mesh."""
         return BoundingBox(self.vertices)
 
     def copy(self):
@@ -457,19 +449,12 @@ class MeshBuilder:
         return self.from_builder(self)
 
     def diagnose(self) -> MeshDiagnose:
-        """Returns the :class:`MeshDiagnose` object for this mesh.
-
-        .. versionadded:: 0.18
-
-        """
+        """Returns the :class:`MeshDiagnose` object for this mesh."""
         return MeshDiagnose(self)
 
     def get_face_vertices(self, index: int) -> Sequence[Vec3]:
         """Returns the face `index` as sequence of :class:`~ezdxf.math.Vec3`
         objects.
-
-        .. versionadded:: 0.18
-
         """
         vertices = self.vertices
         return tuple(vertices[vi] for vi in self.faces[index])
@@ -477,9 +462,6 @@ class MeshBuilder:
     def get_face_normal(self, index: int) -> Vec3:
         """Returns the normal vector of the face `index` as :class:`~ezdxf.math.Vec3`,
         returns the ``NULLVEC`` instance for degenerated  faces.
-
-        .. versionadded:: 0.18
-
         """
         face = self.get_face_vertices(index)
         try:
@@ -490,9 +472,6 @@ class MeshBuilder:
     def face_normals(self) -> Iterator[Vec3]:
         """Yields all face normals, yields the ``NULLVEC`` instance for degenerated
         faces.
-
-        .. versionadded:: 0.18
-
         """
         for face in self.faces_as_vertices():
             try:
@@ -500,7 +479,7 @@ class MeshBuilder:
             except (ValueError, ZeroDivisionError):
                 yield NULLVEC
 
-    def faces_as_vertices(self) -> Iterator[List[Vec3]]:
+    def faces_as_vertices(self) -> Iterator[list[Vec3]]:
         """Yields all faces as list of vertices."""
         v = self.vertices
         for face in self.faces:
@@ -509,9 +488,6 @@ class MeshBuilder:
     def open_faces(self) -> Iterator[Face]:
         """Yields all faces as sequence of integers where the first vertex
         is not coincident with the last vertex.
-
-        .. versionadded:: 0.18
-
         """
         yield from open_faces(self.faces)
 
@@ -551,8 +527,8 @@ class MeshBuilder:
 
     def add_mesh(
         self,
-        vertices: List[Vec3] = None,
-        faces: List[Face] = None,
+        vertices: Optional[list[Vec3]] = None,
+        faces: Optional[list[Face]] = None,
         mesh=None,
     ) -> None:
         """Add another mesh to this mesh.
@@ -584,8 +560,8 @@ class MeshBuilder:
         self,
         layout: GenericLayoutType,
         dxfattribs=None,
-        matrix: Matrix44 = None,
-        ucs: UCS = None,
+        matrix: Optional[Matrix44] = None,
+        ucs: Optional[UCS] = None,
     ):
         """Render mesh as :class:`~ezdxf.entities.Mesh` entity into `layout`.
 
@@ -610,8 +586,6 @@ class MeshBuilder:
             data.faces = list(self.faces)  # type: ignore
         return mesh
 
-    render = render_mesh  # TODO: 2021-02-10 - compatibility alias
-
     def render_normals(
         self,
         layout: GenericLayoutType,
@@ -697,8 +671,8 @@ class MeshBuilder:
         self,
         layout: GenericLayoutType,
         dxfattribs=None,
-        matrix: Matrix44 = None,
-        ucs: UCS = None,
+        matrix: Optional[Matrix44] = None,
+        ucs: Optional[UCS] = None,
     ):
         """Render mesh as :class:`~ezdxf.entities.Polyface` entity into
         `layout`.
@@ -709,11 +683,6 @@ class MeshBuilder:
             matrix: transformation matrix of type :class:`~ezdxf.math.Matrix44`
             ucs: transform vertices by :class:`~ezdxf.math.UCS` to :ref:`WCS`
 
-        .. versionchanged:: 0.18
-
-            Uses the :meth:`tessellation` method to process ngons with more the
-            4 vertices which can handle concave faces.
-
         """
         dxfattribs = dict(dxfattribs) if dxfattribs else {}
         polyface = layout.add_polyface(dxfattribs=dxfattribs)
@@ -732,8 +701,8 @@ class MeshBuilder:
         self,
         layout: GenericLayoutType,
         dxfattribs=None,
-        matrix: Matrix44 = None,
-        ucs: UCS = None,
+        matrix: Optional[Matrix44] = None,
+        ucs: Optional[UCS] = None,
     ):
         """Render mesh as :class:`~ezdxf.entities.Face3d` entities into
         `layout`.
@@ -744,11 +713,6 @@ class MeshBuilder:
             matrix: transformation matrix of type :class:`~ezdxf.math.Matrix44`
             ucs: transform vertices by :class:`~ezdxf.math.UCS` to :ref:`WCS`
 
-        .. versionchanged:: 0.18
-
-            Uses the :meth:`tessellation` method to process ngons with more the
-            4 vertices which can handle concave faces.
-
         """
         dxfattribs = dict(dxfattribs) if dxfattribs else {}
         t = MeshTransformer.from_builder(self)
@@ -784,8 +748,6 @@ class MeshBuilder:
 
         The current implementation is not very capable!
 
-        .. versionadded:: 0.18
-
         """
         mesh = self
         for _ in range(passes):
@@ -811,9 +773,6 @@ class MeshBuilder:
         merged together and all faces are open faces (first vertex != last
         vertex). Uses internally the :class:`MeshVertexMerger` class to merge
         vertices.
-
-        .. versionadded:: 0.18
-
         """
         mesh = MeshVertexMerger(precision=precision)
         mesh.add_mesh(mesh=self)
@@ -826,9 +785,6 @@ class MeshBuilder:
         In contrast to the :meth:`tessellation` method, creates this method a
         new vertex in the centroid of the face. This can create a more regular
         tessellation but only works reliable for convex faces!
-
-        .. versionadded:: 0.18
-
         """
         yield from subdivide_ngons(self.faces_as_vertices(), max_vertex_count)
 
@@ -839,9 +795,6 @@ class MeshBuilder:
         each face has no more vertices than the given `max_vertex_count`. This
         method uses the "ear clipping" algorithm which works with concave faces
         too and does not create any additional vertices.
-
-        .. versionadded:: 0.18
-
         """
         from ezdxf.math.triangulation import mapbox_earcut_3d
 
@@ -857,9 +810,6 @@ class MeshBuilder:
 
         The `fast` mode uses a shortcut for faces with less than 6 vertices
         which may not work for concave faces!
-
-        .. versionadded:: 0.18
-
         """
         mesh = MeshVertexMerger()
         for face in self.tessellation(max_vertex_count=max_vertex_count):
@@ -868,19 +818,13 @@ class MeshBuilder:
 
     def flip_normals(self) -> None:
         """Flips the normals of all faces by reversing the vertex order inplace.
-
-        .. versionadded:: 0.18
-
         """
         self.faces = list(flip_face_normals(self.faces))
 
-    def separate_meshes(self) -> List[MeshTransformer]:
+    def separate_meshes(self) -> list[MeshTransformer]:
         """A single :class:`MeshBuilder` instance can store multiple separated
         meshes. This function returns this separated meshes as multiple
         :class:`MeshTransformer` instances.
-
-        .. versionadded:: 0.18
-
         """
         return list(separate_meshes(self))
 
@@ -888,9 +832,6 @@ class MeshBuilder:
         """Removes duplicated vertex indices from faces and stores all faces as
         open faces, where the last vertex is not coincident with the first
         vertex.
-
-        .. versionadded:: 0.18
-
         """
         self.faces = list(normalize_faces(self.faces, close=False))
 
@@ -909,7 +850,7 @@ class MeshBuilder:
         return FaceOrientationDetector(self, reference=reference)
 
     def unify_face_normals(
-        self, *, fod: FaceOrientationDetector = None
+        self, *, fod: Optional[FaceOrientationDetector] = None
     ) -> MeshTransformer:
         """Returns a new :class:`MeshTransformer` object with unified
         face normal vectors of all faces.
@@ -926,8 +867,6 @@ class MeshBuilder:
             NonManifoldError: non-manifold mesh
             MultipleMeshesError: the :class:`MeshBuilder` object contains multiple disconnected meshes
 
-        .. versionadded:: 0.18
-
         """
         return unify_face_normals_by_majority(self, fod=fod)
 
@@ -936,7 +875,7 @@ class MeshBuilder:
         reference: int = 0,
         *,
         force_outwards=False,
-        fod: FaceOrientationDetector = None,
+        fod: Optional[FaceOrientationDetector] = None,
     ) -> MeshTransformer:
         """Returns a new :class:`MeshTransformer` object with unified
         face normal vectors of all faces.
@@ -968,8 +907,6 @@ class MeshBuilder:
             ValueError: non-manifold mesh or the :class:`MeshBuilder` object
                 contains multiple disconnected meshes
 
-        .. versionadded:: 0.18
-
         """
         mesh = unify_face_normals_by_reference(
             self, reference=reference, fod=fod
@@ -1130,7 +1067,7 @@ class MeshVertexMerger(MeshBuilder):
 
         """
         super().__init__()
-        self.ledger: Dict[Vec3, int] = {}
+        self.ledger: dict[Vec3, int] = {}
         self.precision: int = precision
 
     def add_vertices(self, vertices: Iterable[UVec]) -> Face:
@@ -1202,8 +1139,8 @@ class MeshAverageVertexMerger(MeshBuilder):
     # can not support vertex transformation
     def __init__(self, precision: int = 6):
         super().__init__()
-        self.ledger: Dict[
-            Vec3, Tuple[int, int]
+        self.ledger: dict[
+            Vec3, tuple[int, int]
         ] = {}  # each key points to a tuple (vertex index, vertex count)
         self.precision: int = precision
 
@@ -1285,10 +1222,10 @@ class _XFace:
 
 
 def _merge_adjacent_coplanar_faces(
-    vertices: List[Vec3], faces: List[Face], precision: int = 4
+    vertices: list[Vec3], faces: list[Face], precision: int = 4
 ) -> MeshVertexMerger:
-    oriented_faces: Dict[Vec3, List[_XFace]] = {}
-    extended_faces: List[_XFace] = []
+    oriented_faces: dict[Vec3, list[_XFace]] = {}
+    extended_faces: list[_XFace] = []
     for face in faces:
         if len(face) < 3:
             raise ValueError("found invalid face count < 3")
@@ -1342,7 +1279,7 @@ def remove_colinear_face_vertices(vertices: Sequence[Vec3]) -> Iterator[Vec3]:
         return
 
     # remove duplicated vertices
-    _vertices: List[Vec3] = [vertices[0]]
+    _vertices: list[Vec3] = [vertices[0]]
     for v in vertices[1:]:
         if not v.isclose(_vertices[-1]):
             _vertices.append(v)
@@ -1436,8 +1373,8 @@ def merge_full_patch(path: Sequence[int], patch: Sequence[int]):
 
 class Lump:
     def __init__(self, face: Face):
-        self.edges: Set[Edge] = set()
-        self.faces: List[Face] = [face]
+        self.edges: set[Edge] = set()
+        self.faces: list[Face] = [face]
         for a, b in face_edges(face):
             # sort vertex indices to guarantee: edge a,b == edge b,a
             self.edges.add((a, b) if a <= b else (b, a))
@@ -1450,7 +1387,7 @@ class Lump:
         self.edges.update(other.edges)
 
 
-def merge_lumps(lumps: Iterable[Lump]) -> List[Lump]:
+def merge_lumps(lumps: Iterable[Lump]) -> list[Lump]:
     merged_lumps = _merge_lumps(lumps)
     prior_len = 0
     while 1 < len(merged_lumps) != prior_len:
@@ -1459,8 +1396,8 @@ def merge_lumps(lumps: Iterable[Lump]) -> List[Lump]:
     return merged_lumps
 
 
-def _merge_lumps(lumps: Iterable[Lump]) -> List[Lump]:
-    merged_lumps: List[Lump] = []
+def _merge_lumps(lumps: Iterable[Lump]) -> list[Lump]:
+    merged_lumps: list[Lump] = []
     for lump in lumps:
         for base in merged_lumps:
             if lump.is_connected(base):
@@ -1532,8 +1469,8 @@ def face_normals_after_transformation(m: Matrix44) -> bool:
     return have_away_pointing_normals(bottom, top)
 
 
-def _make_edge_mapping(faces: Iterable[Face]) -> Dict[Edge, List[Face]]:
-    mapping: Dict[Edge, List[Face]] = {}
+def _make_edge_mapping(faces: Iterable[Face]) -> dict[Edge, list[Face]]:
+    mapping: dict[Edge, list[Face]] = {}
     for face in faces:
         for edge in face_edges(face):
             mapping.setdefault(edge, []).append(face)
@@ -1557,19 +1494,17 @@ class FaceOrientationDetector:
         mesh: source mesh as :class:`MeshBuilder` object
         reference: index of the reference face
 
-    .. versionadded:: 0.18
-
     """
 
     def __init__(self, mesh: MeshBuilder, reference: int = 0):
         self._mesh = mesh
-        self.edge_mapping: Dict[Edge, List[Face]] = _make_edge_mapping(
+        self.edge_mapping: dict[Edge, list[Face]] = _make_edge_mapping(
             mesh.faces
         )
         self.reference = reference
         self.is_manifold = True  # 2-manifold is meant
-        self.forward: Dict[int, Face] = dict()
-        self.backward: Dict[int, Face] = dict()
+        self.forward: dict[int, Face] = dict()
+        self.backward: dict[int, Face] = dict()
         self.classify_faces(reference)
 
     @property
@@ -1594,7 +1529,7 @@ class FaceOrientationDetector:
         return self.all_reachable
 
     @property
-    def count(self) -> Tuple[int, int]:
+    def count(self) -> tuple[int, int]:
         """Returns the count of forward and backward oriented faces."""
         return len(self.forward), len(self.backward)
 
@@ -1652,10 +1587,10 @@ class FaceOrientationDetector:
         self.reference = int(reference)
         self.is_manifold = True
         edge_mapping = self.edge_mapping
-        forward: Dict[int, Face] = dict()
-        backward: Dict[int, Face] = dict()
+        forward: dict[int, Face] = dict()
+        backward: dict[int, Face] = dict()
         # the reference face defines the forward orientation
-        process_faces: List[Tuple[Face, bool]] = [
+        process_faces: list[tuple[Face, bool]] = [
             (self._mesh.faces[reference], True)
         ]
         while len(process_faces):
@@ -1682,7 +1617,7 @@ class FaceOrientationDetector:
         """
         if not self.is_manifold:
             return False
-        empty: List[Face] = []
+        empty: list[Face] = []
         edge_mapping = self.edge_mapping
         # For a closed surface all edges have to connect exact 2 faces.
         for edge in edge_mapping.keys():
@@ -1712,7 +1647,7 @@ class FaceOrientationDetector:
 def unify_face_normals_by_reference(
     mesh: MeshBuilder,
     *,
-    fod: FaceOrientationDetector = None,
+    fod: Optional[FaceOrientationDetector] = None,
     reference: int = 0,
 ) -> MeshTransformer:
     """Unifies the orientation of all faces of a :class:`MeshBuilder` object.
@@ -1748,7 +1683,7 @@ def unify_face_normals_by_reference(
 def unify_face_normals_by_majority(
     mesh: MeshBuilder,
     *,
-    fod: FaceOrientationDetector = None,
+    fod: Optional[FaceOrientationDetector] = None,
 ) -> MeshTransformer:
     """Unifies the orientation of all faces of a :class:`MeshBuilder` object.
     The forward orientation is defined by the orientation of the majority of
