@@ -1,15 +1,16 @@
-# Copyright (c) 2011-2021, Manfred Moitzi
+# Copyright (c) 2011-2022, Manfred Moitzi
 # License: MIT License
+from __future__ import annotations
 from typing import (
-    TYPE_CHECKING,
-    Iterator,
-    Optional,
-    List,
-    Dict,
-    cast,
-    Union,
     Generic,
+    Iterator,
+    List,
+    Optional,
+    TYPE_CHECKING,
     TypeVar,
+    Union,
+    cast,
+    Sequence,
 )
 from collections import OrderedDict
 import logging
@@ -17,24 +18,26 @@ import logging
 from ezdxf.audit import Auditor, AuditError
 from ezdxf.lldxf import const, validator
 from ezdxf.entities.table import TableHead
-from ezdxf.entities import factory
+from ezdxf.entities import (
+    factory,
+    DXFEntity,
+    Layer,
+    Linetype,
+    Textstyle,
+    VPort,
+    View,
+    AppID,
+    UCSTableEntry,
+    BlockRecord,
+    DimStyle,
+    is_graphic_entity,
+)
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import (
-        TagWriter,
-        EntityDB,
-        Drawing,
-        DXFEntity,
-        Layer,
-        Linetype,
-        Textstyle,
-        VPort,
-        View,
-        AppID,
-        UCSTableEntry,
-        BlockRecord,
-        DimStyle,
-    )
+    from ezdxf.document import Drawing
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.entitydb import EntityDB
+
 
 logger = logging.getLogger("ezdxf")
 
@@ -44,21 +47,19 @@ T = TypeVar("T", bound="DXFEntity")
 class Table(Generic[T]):
     TABLE_TYPE = "UNKNOWN"
 
-    def __init__(self):
-        self.doc: Optional["Drawing"] = None
-        self.entries: Dict[str, T] = OrderedDict()
+    def __init__(self) -> None:
+        self.doc: Optional[Drawing] = None
+        self.entries: dict[str, T] = OrderedDict()
         self._head = TableHead()
 
-    def load(self, doc: "Drawing", entities: Iterator["DXFEntity"]) -> None:
+    def load(self, doc: Drawing, entities: Iterator[DXFEntity]) -> None:
         """Loading interface. (internal API)"""
         self.doc = doc
         table_head = next(entities)
         if isinstance(table_head, TableHead):
             self._head = table_head
         else:
-            raise const.DXFStructureError(
-                "Critical structure error in TABLES section."
-            )
+            raise const.DXFStructureError("Critical structure error in TABLES section.")
         expected_entry_dxftype = self.TABLE_TYPE
         for table_entry in entities:
             if table_entry.dxftype() == expected_entry_dxftype:
@@ -69,13 +70,13 @@ class Table(Generic[T]):
                     f" in {self.TABLE_TYPE} table."
                 )
 
-    def reset(self, doc: "Drawing", handle: str) -> None:
+    def reset(self, doc: Drawing, handle: str) -> None:
         """Reset table. (internal API)"""
         self.doc = doc
         self._set_head(self.TABLE_TYPE, handle)
         self.entries.clear()
 
-    def _set_head(self, name: str, handle: str = None) -> None:
+    def _set_head(self, name: str, handle: Optional[str] = None) -> None:
         self._head = TableHead.new(
             handle, owner="0", dxfattribs={"name": name}, doc=self.doc
         )
@@ -95,7 +96,7 @@ class Table(Generic[T]):
         return validator.make_table_key(name)
 
     def has_entry(self, name: str) -> bool:
-        """Returns ``True`` if an table entry `name` exist."""
+        """Returns ``True`` if a table entry `name` exist."""
         return self.key(name) in self.entries
 
     __contains__ = has_entry
@@ -110,11 +111,11 @@ class Table(Generic[T]):
             if e.is_alive:
                 yield e
 
-    def new(self, name: str, dxfattribs: dict = None) -> T:
+    def new(self, name: str, dxfattribs=None) -> T:
         """Create a new table entry `name`.
 
         Args:
-            name: name of table entry, case insensitive
+            name: name of table entry
             dxfattribs: additional DXF attributes for table entry
 
         """
@@ -128,19 +129,54 @@ class Table(Generic[T]):
         return self.new_entry(dxfattribs)
 
     def get(self, name: str) -> T:
-        """Get table entry `name` (case insensitive).
-        Raises :class:`DXFValueError` if table entry does not exist.
+        """Returns table entry `name`.
+
+        Args:
+            name: name of table entry, case-insensitive
+
+        Raises:
+            DXFTableEntryError: table entry does not exist
+
         """
-        key = self.key(name)
-        entry = self.entries.get(key, None)
+        entry = self.entries.get(self.key(name))
         if entry:
             return entry
         else:
             raise const.DXFTableEntryError(name)
 
+    def get_entry_by_handle(self, handle: str) -> Optional[T]:
+        """Returns table entry by handle or ``None`` if entry does not exist.
+
+        (internal API)
+        """
+        entry = self.doc.entitydb.get(handle)  # type: ignore
+        if entry and entry.dxftype() == self.TABLE_TYPE:
+            return entry  # type: ignore
+        return None
+
+    def get_handle_of_entry(self, name: str) -> str:
+        """Returns the handle of table entry by `name`, returns an empty string if no
+        entry for the given name exist.
+
+        Args:
+            name: name of table entry, case-insensitive
+
+        (internal API)
+        """
+        entry = self.entries.get(self.key(name))
+        if entry is not None:
+            return entry.dxf.handle
+        return ""
+
     def remove(self, name: str) -> None:
-        """Removes table entry `name`. Raises :class:`DXFValueError`
-        if table-entry does not exist.
+        """Removes table entry `name`.
+
+        Args:
+            name: name of table entry, case-insensitive
+
+        Raises:
+            DXFTableEntryError: table entry does not exist
+
         """
         key = self.key(name)
         entry = self.get(name)
@@ -151,8 +187,12 @@ class Table(Generic[T]):
         """Returns a new table entry `new_name` as copy of `name`,
         replaces entry `new_name` if already exist.
 
+        Args:
+            name: name of table entry, case-insensitive
+            new_name: name of duplicated table entry
+
         Raises:
-             DXFValueError: `name` does not exist
+            DXFTableEntryError: table entry does not exist
 
         """
         entry = self.get(name)
@@ -167,7 +207,13 @@ class Table(Generic[T]):
         return entry
 
     def discard(self, name: str) -> None:
-        """Remove table entry without destroying object. (internal API)"""
+        """Remove table entry without destroying object.
+
+        Args:
+            name: name of table entry, case-insensitive
+
+        (internal API)
+        """
         del self.entries[self.key(name)]
 
     def replace(self, name: str, entry: T) -> None:
@@ -176,19 +222,17 @@ class Table(Generic[T]):
         self._append(entry)
 
     @property
-    def entitydb(self) -> "EntityDB":
+    def entitydb(self) -> EntityDB:
         return self.doc.entitydb  # type: ignore
 
-    def new_entry(self, dxfattribs: dict) -> T:
+    def new_entry(self, dxfattribs) -> T:
         """Create and add new table-entry of type 'self.entry_dxftype'.
 
         Does not check if an entry dxfattribs['name'] already exists!
         Duplicate entries are possible for Viewports.
         """
-        assert self.doc is not None, "valid DXF document expected"
-        entry = cast(
-            T, factory.create_db_entry(self.TABLE_TYPE, dxfattribs, self.doc)
-        )
+        assert self.doc is not None, "valid DXF document required"
+        entry = cast(T, factory.create_db_entry(self.TABLE_TYPE, dxfattribs, self.doc))
 
         self._append(entry)
         return entry
@@ -214,35 +258,25 @@ class Table(Generic[T]):
             raise const.DXFTableEntryError(
                 f"{self._head.dxf.name} {name} already exists!"
             )
-        entry.doc = self.doc
+        if self.doc:
+            factory.bind(entry, self.doc)
         entry.dxf.owner = self._head.dxf.handle
         self._append(entry)
 
-    def export_dxf(self, tagwriter: "TagWriter") -> None:
+    def export_dxf(self, tagwriter: AbstractTagWriter) -> None:
         """Export DXF representation. (internal API)"""
 
-        def prologue():
-            self.update_owner_handles()
-            # The table head itself has no owner and is therefore always '0':
-            self._head.dxf.owner = "0"
-            self._head.dxf.count = len(self)
-            self._head.export_dxf(tagwriter)
+        self.update_owner_handles()
+        # The table head itself has no owner and is therefore always '0':
+        self._head.dxf.owner = "0"
+        self._head.dxf.count = len(self)
+        self._head.export_dxf(tagwriter)
+        self.export_table_entries(tagwriter)
+        tagwriter.write_tag2(0, "ENDTAB")
 
-        def content():
-            for entry in self.entries.values():
-                # VPORT
-                if isinstance(entry, list):
-                    for e in entry:
-                        e.export_dxf(tagwriter)
-                else:
-                    entry.export_dxf(tagwriter)
-
-        def epilogue():
-            tagwriter.write_tag2(0, "ENDTAB")
-
-        prologue()
-        content()
-        epilogue()
+    def export_table_entries(self, tagwriter: AbstractTagWriter) -> None:
+        for entry in self.entries.values():
+            entry.export_dxf(tagwriter)
 
     def update_owner_handles(self) -> None:
         owner_handle = self._head.dxf.handle
@@ -309,11 +343,11 @@ class Table(Generic[T]):
         self.update_owner_handles()
 
 
-class LayerTable(Table["Layer"]):
+class LayerTable(Table[Layer]):
     TABLE_TYPE = "LAYER"
 
-    def new_entry(self, dxfattribs: dict) -> "Layer":
-        layer = cast("Layer", super().new_entry(dxfattribs))
+    def new_entry(self, dxfattribs) -> Layer:
+        layer = cast(Layer, super().new_entry(dxfattribs))
         if self.doc:
             layer.set_required_attributes()
         return layer
@@ -323,13 +357,13 @@ class LayerTable(Table["Layer"]):
         name: str,
         *,
         color: int = const.BYLAYER,
-        true_color: int = None,
+        true_color: Optional[int] = None,
         linetype: str = "Continuous",
         lineweight: int = const.LINEWEIGHT_BYLAYER,
         plot: bool = True,
         transparency: Optional[float] = None,
-        dxfattribs: Dict = None,
-    ) -> "Layer":
+        dxfattribs=None,
+    ) -> Layer:
         """Add a new :class:`~ezdxf.entities.Layer`.
 
         Args:
@@ -344,8 +378,6 @@ class LayerTable(Table["Layer"]):
                 100% transparent and 0 is opaque
             dxfattribs (dict): additional DXF attributes
 
-        .. versionadded:: 0.17
-
         """
         dxfattribs = dict(dxfattribs or {})
         if validator.is_valid_aci_color(color):
@@ -365,26 +397,38 @@ class LayerTable(Table["Layer"]):
             layer.transparency = transparency
         return layer
 
+    def create_referenced_layers(self) -> None:
+        """Create for all referenced layers table entries if not exist."""
+        if self.doc is None:
+            return
+        for e in self.doc.entitydb.values():
+            if not is_graphic_entity(e):
+                continue
+            layer_name = e.dxf.get("layer", "")
+            if layer_name and not self.has_entry(layer_name):
+                # create layer table entry with default settings
+                self.add(layer_name)
 
-class LinetypeTable(Table["Linetype"]):
+
+class LinetypeTable(Table[Linetype]):
     TABLE_TYPE = "LTYPE"
 
-    def new_entry(self, dxfattribs: dict) -> T:
+    def new_entry(self, dxfattribs) -> Linetype:
         pattern = dxfattribs.pop("pattern", [0.0])
         length = dxfattribs.pop("length", 0)  # required for complex types
-        ltype = cast("Linetype", super().new_entry(dxfattribs))
+        ltype = cast(Linetype, super().new_entry(dxfattribs))
         ltype.setup_pattern(pattern, length)
-        return cast(T, ltype)
+        return ltype
 
     def add(
         self,
         name: str,
-        pattern: Union[List[float], str],
+        pattern: Union[Sequence[float], str],
         *,
         description: str = "",
         length: float = 0.0,
-        dxfattribs: Dict = None,
-    ) -> "Linetype":
+        dxfattribs=None,
+    ) -> Linetype:
         """Add a new line type entry. The simple line type pattern is a list of
         floats :code:`[total_pattern_length, elem1, elem2, ...]`
         where an element > 0 is a line, an element < 0 is a gap and  an
@@ -407,8 +451,6 @@ class LinetypeTable(Table["Linetype"]):
             length (float): total pattern length, only for complex line types required
             dxfattribs (dict): additional DXF attributes
 
-        .. versionadded:: 0.17
-
         """
         dxfattribs = dict(dxfattribs or {})
         dxfattribs.update(
@@ -422,12 +464,34 @@ class LinetypeTable(Table["Linetype"]):
         return self.new_entry(dxfattribs)  # type: ignore
 
 
-class TextstyleTable(Table["Textstyle"]):
+class TextstyleTable(Table[Textstyle]):
     TABLE_TYPE = "STYLE"
 
-    def add(
-        self, name: str, *, font: str, dxfattribs: Dict = None
-    ) -> "Textstyle":
+    def __init__(self) -> None:
+        super().__init__()
+        self.shx_files: dict[str, Textstyle] = dict()
+
+    def export_table_entries(self, tagwriter: AbstractTagWriter) -> None:
+        super().export_table_entries(tagwriter)
+        for shx_file in self.shx_files.values():
+            shx_file.export_dxf(tagwriter)
+
+    def _append(self, entry: Textstyle) -> None:
+        """Add a table entry, replaces existing entries with same name.
+        (internal API).
+        """
+        if entry.dxf.name == "" and (entry.dxf.flags & 1):  # shx shape file
+            self.shx_files[self.key(entry.dxf.font)] = entry
+        else:
+            self.entries[self.key(entry.dxf.name)] = entry
+
+    def update_owner_handles(self) -> None:
+        super().update_owner_handles()
+        owner_handle = self._head.dxf.handle
+        for entry in self.shx_files.values():
+            entry.dxf.owner = owner_handle
+
+    def add(self, name: str, *, font: str, dxfattribs=None) -> Textstyle:
         """Add a new text style entry for TTF fonts. The entry must not yet
         exist, otherwise an :class:`DXFTableEntryError` exception will be
         raised.
@@ -438,12 +502,10 @@ class TextstyleTable(Table["Textstyle"]):
         Args:
             name (str): text style name
             font (str): TTF font file name like "Arial.ttf", the real font file
-                name from the file system is required and remember only Windows
-                is case insensitive.
+                name from the file system is required and only the Windows filesystem
+                is case-insensitive.
             dxfattribs (dict): additional DXF attributes
 
-        .. versionadded:: 0.17
-
         """
         dxfattribs = dict(dxfattribs or {})
         dxfattribs.update(
@@ -455,25 +517,23 @@ class TextstyleTable(Table["Textstyle"]):
         )
         return self.new_entry(dxfattribs)
 
-    def add_shx(self, shx_file: str, *, dxfattribs: Dict = None) -> "Textstyle":
+    def add_shx(self, shx_file_name: str, *, dxfattribs=None) -> Textstyle:
         """Add a new shape font (SHX file) entry. These are special text style
         entries and have no name. The entry must not yet exist, otherwise an
         :class:`DXFTableEntryError` exception will be raised.
 
-        Finding the SHX files is the task of the DXF viewer and each
+        Locating the SHX files in the filesystem is the task of the DXF viewer and each
         viewer is different (hint: support files).
 
         Args:
-            shx_file (str): shape file name like "gdt.shx"
+            shx_file_name (str): shape file name like "gdt.shx"
             dxfattribs (dict): additional DXF attributes
 
-        .. versionadded:: 0.17
-
         """
-        if self.find_shx(shx_file) is not None:
+        if self.find_shx(shx_file_name) is not None:
             raise const.DXFTableEntryError(
                 f"{self._head.dxf.name} shape file entry for "
-                f"'{shx_file}' already exists!"
+                f"'{shx_file_name}' already exists!"
             )
 
         dxfattribs = dict(dxfattribs or {})
@@ -481,58 +541,73 @@ class TextstyleTable(Table["Textstyle"]):
             {
                 "name": "",  # shape file entry has no name
                 "flags": 1,  # shape file flag
-                "font": shx_file,
+                "font": shx_file_name,
                 "last_height": 2.5,  # maybe required by AutoCAD
             }
         )
         return self.new_entry(dxfattribs)
 
-    def get_shx(self, shx_file: str) -> "Textstyle":
+    def get_shx(self, shx_file_name: str) -> Textstyle:
         """Get existing entry for a shape file (SHX file), or create a new
         entry.
 
-        Finding the SHX files is the task of the DXF viewer and each
+        Locating the SHX files in the filesystem is the task of the DXF viewer and each
         viewer is different (hint: support files).
 
         Args:
-            shx_file (str): shape file name like "gdt.shx"
+            shx_file_name (str): shape file name like "gdt.shx"
 
         """
-        shape_file = self.find_shx(shx_file)
+        shape_file = self.find_shx(shx_file_name)
         if shape_file is None:
-            return self.add_shx(shx_file)
+            return self.add_shx(shx_file_name)
         return shape_file
 
-    def find_shx(self, shx_file: str) -> Optional["Textstyle"]:
-        """Find the shape file (SHX file) text style table entry, by a case
-        insensitive search.
+    def find_shx(self, shx_file_name: str) -> Optional[Textstyle]:
+        """Find the shape file (SHX file) text style table entry, by a
+        case-insensitive search.
 
         A shape file table entry has no name, so you have to search by the
         font attribute.
 
         Args:
-            shx_file (str): shape file name like "gdt.shx"
+            shx_file_name (str): shape file name like "gdt.shx"
 
         """
-        lower_name = shx_file.lower()
-        for entry in iter(self):
-            if entry.dxf.font.lower() == lower_name:
-                return entry
-        return None
+        return self.shx_files.get(self.key(shx_file_name))
+
+    def discard_shx(self, shx_file_name: str) -> None:
+        """Discard the shape file (SHX file) text style table entry. Does not raise an
+        exception if the entry does not exist.
+
+        Args:
+            shx_file_name (str): shape file name like "gdt.shx"
+
+        """
+        try:
+            del self.shx_files[self.key(shx_file_name)]
+        except KeyError:
+            pass
 
 
-class ViewportTable(Table["VPort"]):
+class ViewportTable(Table[VPort]):
     TABLE_TYPE = "VPORT"
     # Viewport-Table can have multiple entries with same name
     # each table entry is a list of VPORT entries
 
-    def new(self, name: str, dxfattribs: dict = None) -> T:
+    def export_table_entries(self, tagwriter: AbstractTagWriter) -> None:
+        for entry in self.entries.values():
+            assert isinstance(entry, list)
+            for e in entry:
+                e.export_dxf(tagwriter)
+
+    def new(self, name: str, dxfattribs=None) -> VPort:
         """Create a new table entry."""
         dxfattribs = dxfattribs or {}
         dxfattribs["name"] = name
         return self.new_entry(dxfattribs)
 
-    def add(self, name: str, *, dxfattribs: Dict = None) -> "VPort":
+    def add(self, name: str, *, dxfattribs=None) -> VPort:
         """Add a new modelspace viewport entry. A modelspace viewport
         configuration can consist of multiple viewport entries with the same
         name.
@@ -541,8 +616,6 @@ class ViewportTable(Table["VPort"]):
             name (str): viewport name, multiple entries possible
             dxfattribs (dict): additional DXF attributes
 
-        .. versionadded:: 0.17
-
         """
         dxfattribs = dict(dxfattribs or {})
         dxfattribs["name"] = name
@@ -551,24 +624,24 @@ class ViewportTable(Table["VPort"]):
     def remove(self, name: str) -> None:
         """Remove table-entry from table and entitydb by name."""
         key = self.key(name)
-        entries = cast(List["DXFEntity"], self.get(name))
+        entries = cast(List[DXFEntity], self.get(name))
         for entry in entries:
             self.entitydb.delete_entity(entry)
         del self.entries[key]
 
-    def __iter__(self) -> Iterator[T]:
+    def __iter__(self) -> Iterator[VPort]:
         for entries in self.entries.values():
             yield from iter(entries)  # type: ignore
 
-    def _flatten(self) -> Iterator[T]:
+    def _flatten(self) -> Iterator[VPort]:
         for entries in self.entries.values():
             yield from iter(entries)  # type: ignore
 
-    def __len__(self):
+    def __len__(self) -> int:
         # calling __iter__() invokes recursion!
         return len(list(self._flatten()))
 
-    def new_entry(self, dxfattribs: dict) -> T:
+    def new_entry(self, dxfattribs) -> VPort:
         """Create and add new table-entry of type 'self.entry_dxftype'.
 
         Does not check if an entry dxfattribs['name'] already exists!
@@ -576,12 +649,13 @@ class ViewportTable(Table["VPort"]):
         """
         assert self.doc is not None, "valid DXF document expected"
         entry = cast(
-            T, factory.create_db_entry(self.TABLE_TYPE, dxfattribs, self.doc)
+            VPort,
+            factory.create_db_entry(self.TABLE_TYPE, dxfattribs, self.doc),
         )
         self._append(entry)
         return entry
 
-    def duplicate_entry(self, name: str, new_name: str) -> T:
+    def duplicate_entry(self, name: str, new_name: str) -> VPort:
         raise NotImplementedError()
 
     def _append(self, entry: T) -> None:
@@ -591,13 +665,25 @@ class ViewportTable(Table["VPort"]):
         else:
             self.entries[key] = [entry]  # type: ignore # store list of VPORT
 
+    def replace(self, name: str, entry: T) -> None:
+        self.discard(name)
+        config: list[T]
+        if isinstance(entry, list):
+            config = entry
+        else:
+            config = [entry]
+        if not config:
+            return
+        key = self.key(config[0].dxf.name)
+        self.entries[key] = config  # type: ignore
+
     def update_owner_handles(self) -> None:
         owner_handle = self._head.dxf.handle
         for entries in self.entries.values():
             for entry in entries:  # type: ignore
                 entry.dxf.owner = owner_handle
 
-    def get_config(self, name: str) -> List["VPort"]:
+    def get_config(self, name: str) -> list[VPort]:
         """Returns a list of :class:`~ezdxf.entities.VPort` objects, for
         the multi-viewport configuration `name`.
         """
@@ -613,90 +699,80 @@ class ViewportTable(Table["VPort"]):
         self.remove(name)
 
 
-class AppIDTable(Table["AppID"]):
+class AppIDTable(Table[AppID]):
     TABLE_TYPE = "APPID"
 
-    def add(self, name: str, *, dxfattribs: Dict = None) -> "AppID":
+    def add(self, name: str, *, dxfattribs=None) -> AppID:
         """Add a new appid table entry.
 
         Args:
             name (str): appid name
             dxfattribs (dict): DXF attributes
 
-        .. versionadded:: 0.17
-
         """
         dxfattribs = dict(dxfattribs or {})
         dxfattribs["name"] = name
         return self.new_entry(dxfattribs)
 
 
-class ViewTable(Table["View"]):
+class ViewTable(Table[View]):
     TABLE_TYPE = "VIEW"
 
-    def add(self, name: str, *, dxfattribs: Dict = None) -> "View":
+    def add(self, name: str, *, dxfattribs=None) -> View:
         """Add a new view table entry.
 
         Args:
             name (str): view name
             dxfattribs (dict): DXF attributes
 
-        .. versionadded:: 0.17
-
         """
         dxfattribs = dict(dxfattribs or {})
         dxfattribs["name"] = name
         return self.new_entry(dxfattribs)
 
 
-class BlockRecordTable(Table["BlockRecord"]):
+class BlockRecordTable(Table[BlockRecord]):
     TABLE_TYPE = "BLOCK_RECORD"
 
-    def add(self, name: str, *, dxfattribs: Dict = None) -> "BlockRecord":
+    def add(self, name: str, *, dxfattribs=None) -> BlockRecord:
         """Add a new block record table entry.
 
         Args:
             name (str): block record name
             dxfattribs (dict): DXF attributes
 
-        .. versionadded:: 0.17
-
         """
         dxfattribs = dict(dxfattribs or {})
         dxfattribs["name"] = name
         return self.new_entry(dxfattribs)
 
 
-class DimStyleTable(Table["DimStyle"]):
+class DimStyleTable(Table[DimStyle]):
     TABLE_TYPE = "DIMSTYLE"
 
-    def add(self, name: str, *, dxfattribs: Dict = None) -> "DimStyle":
+    def add(self, name: str, *, dxfattribs=None) -> DimStyle:
         """Add a new dimension style table entry.
 
         Args:
             name (str): dimension style name
             dxfattribs (dict): DXF attributes
 
-        .. versionadded:: 0.17
-
         """
         dxfattribs = dict(dxfattribs or {})
         dxfattribs["name"] = name
         return self.new_entry(dxfattribs)
 
 
-class UCSTable(Table["UCSTableEntry"]):
+class UCSTable(Table[UCSTableEntry]):
     TABLE_TYPE = "UCS"
 
-    def add(self, name: str, *, dxfattribs: Dict = None) -> "UCSTableEntry":
+    def add(self, name: str, *, dxfattribs=None) -> UCSTableEntry:
         """Add a new UCS table entry.
 
         Args:
             name (str): UCS name
             dxfattribs (dict): DXF attributes
 
-        .. versionadded:: 0.17
-
         """
         dxfattribs = dict(dxfattribs or {})
         dxfattribs["name"] = name
