@@ -173,9 +173,7 @@ def test_load_from_text(entity):
     assert entity.dxf.defpoint == (0, 0, 0)
 
 
-@pytest.mark.parametrize(
-    "txt,ver", [(ENTITY_R2000, DXF2000), (ENTITY_R12, DXF12)]
-)
+@pytest.mark.parametrize("txt,ver", [(ENTITY_R2000, DXF2000), (ENTITY_R12, DXF12)])
 def test_write_dxf(txt, ver):
     expected = basic_tags_from_text(txt)
     dimension = TEST_CLASS.from_text(txt)
@@ -253,9 +251,9 @@ def test_dimension_transform_interface():
     dim.dxf.angle = 45
 
     dim.transform(Matrix44.translate(1, 2, 3))
-    assert dim.dxf.insert == (2, 2, 3)
-    assert dim.dxf.defpoint == (1, 3, 3)
-    assert dim.dxf.angle == 45
+    assert Vec3(2, 2, 3).isclose(dim.dxf.insert)
+    assert Vec3(1, 3, 3).isclose(dim.dxf.defpoint)
+    assert math.isclose(dim.dxf.angle, 45)
 
     dim.transform(Matrix44.z_rotate(math.radians(45)))
     assert math.isclose(dim.dxf.angle, 90)
@@ -286,3 +284,15 @@ def test_audit_invalid_geometry_block(doc):
     assert len(auditor.fixes) == 1
     assert dim.dimension.is_alive is False, "DIMENSION should be destroyed"
 
+
+def test_audit_fake_dimensional_constraints(doc):
+    msp = doc.modelspace()
+    dim = msp.add_linear_dim((5, 2), (0, 0), (10, 0))
+    # fake a dimensional constraint
+    dimension = dim.dimension
+    dimension.dxf.layer = "*ADSK_CONSTRAINTS"
+    # no geometry block was created without calling dim.render()
+
+    auditor = doc.audit()
+    assert len(auditor.fixes) == 0
+    assert dim.dimension.is_alive is True, "dimensional constraint should be preserved"
