@@ -4,6 +4,7 @@
 import pytest
 import ezdxf
 from ezdxf.entities import get_font_name, DXFEntity
+from ezdxf.lldxf import const
 
 
 @pytest.fixture(scope="module")
@@ -13,9 +14,9 @@ def msp():
     return doc.modelspace()
 
 
-def test_get_font_name_for_entity_without_font_usage():
+def test_get_font_name_for_entity_without_font_support():
     e = DXFEntity()
-    assert get_font_name(e) == "txt", "should return the default font"
+    assert get_font_name(e) == const.DEFAULT_TEXT_FONT, "should return the default font"
 
 
 def test_get_font_name_for_text(msp):
@@ -23,10 +24,15 @@ def test_get_font_name_for_text(msp):
     assert get_font_name(text) == "arial.ttf"
 
 
+def test_undefined_text_style(msp):
+    text = msp.add_text("abc", dxfattribs={"style": "UNDEFINED"})
+    assert get_font_name(text) == const.DEFAULT_TEXT_FONT
+
+
 def test_get_font_name_for_mtext(msp):
     mtext = msp.add_mtext("abc", dxfattribs={"style": "ARIAL"})
     assert get_font_name(mtext) == "arial.ttf"
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     pytest.main([__file__])
